@@ -142,10 +142,6 @@ static int file_get_line (ExecState *s)
 	tailstrip(linebak);
     }
 
-    if (!strncmp(line, "noecho", 6)) {
-	set_gretl_echo(0);
-    }
-
     if (gretl_echo_on() && s->cmd->ci == RUN && batch && *line == '(') {
 	printf("%s", line);
 	*linebak = 0;
@@ -177,30 +173,47 @@ static void nls_init (void)
 static int cli_clear_data (CMD *cmd, double ***pZ, DATAINFO *pdinfo,
 			   MODEL **models)
 {
+    gretlopt clearopt = 0;
     int err = 0;
 
-    *datafile = 0;
-
-    if (pZ != NULL && *pZ != NULL) {
-	err = restore_full_sample(pZ, pdinfo, NULL); 
-	free_Z(*pZ, pdinfo); 
-	*pZ = NULL;
+    if (cmd->ci == CLEAR) {
+	clearopt = cmd->opt;
+    } else {
+	if (cmd->opt & OPT_P) {
+	    /* --preserve: clear dataset only */
+	    clearopt = OPT_D;
+	}
     }
 
-    clear_datainfo(pdinfo, CLEAR_FULL);
+    if (!(clearopt & OPT_O)) {
+	/* OPT_O means "other", i.e. don't scrub the dataset */
+	*datafile = 0;
 
-    data_status = 0;
+	if (pZ != NULL && *pZ != NULL) {
+	    err = restore_full_sample(pZ, pdinfo, NULL); 
+	    free_Z(*pZ, pdinfo); 
+	    *pZ = NULL;
+	}
+
+	clear_datainfo(pdinfo, CLEAR_FULL);
+	data_status = 0;
+    }
 
     clear_model(models[0]);
     clear_model(models[1]);
 
-    if (cmd->opt & OPT_P) {
-	libgretl_session_cleanup(SESSION_PRESERVE_MATRICES);
+    if (clearopt & OPT_D) {
+	libgretl_session_cleanup(SESSION_CLEAR_DATASET);
+    } else if (clearopt & OPT_O) {
+	/* only supported by the "clear" command:
+	   clear stuff _other than_ the dataset
+	*/
+	libgretl_session_cleanup(SESSION_CLEAR_OTHER);
     } else {
-	libgretl_session_cleanup(SESSION_CLEAR_FULL);
+	libgretl_session_cleanup(SESSION_CLEAR_ALL);
     }
 
-    reset_model_count();
+    set_model_count(0);
     gretl_cmd_destroy_context(cmd);
 
     return err;
@@ -562,6 +575,9 @@ int main (int argc, char *argv[])
     if (batch || runit) {
 	/* re-initialize: will be incremented by "run" cmd */
 	runit = 0;
+	if (makepkg) {
+	    set_gretl_echo(0);
+	} 
 	sprintf(line, "run %s\n", runfile);
 	err = exec_line(&state, &Z, datainfo);
     }
@@ -831,15 +847,21 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     }
 
     if (err) {
+	int catch = 0;
+
 	gretl_exec_state_uncomment(s);
-	errmsg(err, prn);
-	return err;
+	if (err != E_ALLOC && (cmd->flags & CMD_CATCH)) {
+	    set_gretl_errno(err);
+	    catch = 1;
+	}	
+        errmsg(err, prn);
+	return (catch)? 0 : err;
     }
 
     gretl_exec_state_transcribe_flags(s, cmd);
 
     /* if in batch mode, echo comments from input */
-    if (batch && cmd->ci == CMD_COMMENT && gretl_echo_on()) {
+    if (batch && runit < 2 && cmd->ci == CMD_COMMENT && gretl_echo_on()) {
 	printline(linebak);
     }
 
@@ -881,8 +903,12 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     if (gretl_echo_on()) {
 	/* visual feedback, not recording */
-	eflag = (batch || runit)? CMD_BATCH_MODE : CMD_CLI;
-	echo_cmd(cmd, pdinfo, line, eflag, prn);
+	if (cmd->ci == FUNC && runit > 1) {
+	    ; /* don't echo */
+	} else {
+	    eflag = (batch || runit)? CMD_BATCH_MODE : CMD_CLI;
+	    echo_cmd(cmd, pdinfo, line, eflag, prn);
+	}
     }
 
     check_for_loop_only_options(cmd->ci, cmd->opt, prn);
@@ -992,14 +1018,14 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    pputs(prn, _("Command is malformed\n"));
 	    break;
 	}
-	if (gretl_messages_on()) {
-	    pprintf(prn, " %s\n", runfile);
-	}
 	if (cmd->ci == INCLUDE && gretl_is_xml_file(runfile)) {
 	    err = load_user_XML_file(runfile);
 	    if (err) {
 		pprintf(prn, _("Error reading %s\n"), runfile);
 	    } else {
+		if (gretl_messages_on()) {
+		    pprintf(prn, " %s\n", runfile);
+		}
 		pprintf(cmdprn, "include \"%s\"\n", runfile);
 	    }
 	    break;
@@ -1013,14 +1039,15 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    push_input_file(fb);
 	}
 	if ((fb = fopen(runfile, "r")) == NULL) {
-	    fprintf(stderr, _("Couldn't open script \"%s\"\n"), runfile);
+	    pprintf(prn, _("Error reading %s\n"), runfile);
+	    err = 1;
 	    fb = pop_input_file();
 	} else {
+	    if (gretl_messages_on()) {
+		pprintf(prn, " %s\n", runfile);
+	    }	    
 	    gretl_set_current_dir(runfile);
 	    strcpy(s->runfile, runfile);
-	    if (libset_get_bool(VERBOSE_INCLUDE)) {
-		pprintf(prn, _("%s opened OK\n"), runfile);
-	    }
 	    if (cmd->ci == INCLUDE) {
 		pprintf(cmdprn, "include \"%s\"\n", runfile);
 	    } else {
@@ -1030,6 +1057,10 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	}
 	break;
 
+    case CLEAR:
+	err = cli_clear_data(cmd, pZ, pdinfo, models);
+	break;
+
     case DATAMOD:
 	if (cmd->aux == DS_CLEAR) {
 	    err = cli_clear_data(cmd, pZ, pdinfo, models);
@@ -1054,7 +1085,7 @@ static int exec_line (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     if (err) {
 	gretl_exec_state_uncomment(s);
     }
-
+    
     return err;
 }
 
