@@ -255,7 +255,7 @@ enum {
 #define F_NORM   "normal(x,mu,s)=1/(s*sqrt(2*pi))*exp(-(x-mu)**2/(2*s*s))"
 #define F_BINV   "Binv(p,q)=exp(lgamma(p+q)-lgamma(p)-lgamma(q))"
 #define F_CHI    "chi(x,m)=x<0?0.0/0.0:x**(0.5*m-1.0)*exp(-0.5*x)/gamma(0.5*m)/2**(0.5*m)"
-#define F_ALTCHI "bigchi(x,m)=x<0?0.0/0.0:exp((0.5*m-1.0)*log(x)-0.5*x-lgamma(0.5*m)-df1*0.5*log(2.0))"
+#define F_ALTCHI "bigchi(x,m)=x<0?0.0/0.0:exp((0.5*m-1.0)*log(x)-0.5*x-lgamma(0.5*m)-m*0.5*log(2.0))"
 #define F_F      "f(x,m,n)=x<0?0.0/0.0:Binv(0.5*m,0.5*n)*(m/n)**(0.5*m)*" \
                  "x**(0.5*m-1.0)/(1.0+m/n*x)**(0.5*(m+n))"
 #define F_STUD   "stud(x,m)=Binv(0.5*m,0.5)/sqrt(m)*(1.0+(x*x)/m)**(-0.5*(m+1.0))"
@@ -1527,7 +1527,7 @@ static void do_h_test (test_t *test, double *x, int n1, int n2)
 	pprintf(prn, _("Test statistic: chi-square(%d) = %d * %g/%g = %g\n"), 
 		n1-1, n1-1, x[0], x[1], ts);
 
-	if (x[0] > x[1]) {
+	if (x[0] >= x[1]) {
 	    pv = chisq_cdf_comp(n1 - 1, ts);
 	} else {
 	    pv = chisq_cdf(n1 - 1, ts);
@@ -2138,7 +2138,7 @@ static void populate_stats (GtkWidget *w, gpointer p)
 	return;
     } 
 
-    buf = gtk_combo_box_get_active_text(GTK_COMBO_BOX(p));
+    buf = combo_box_get_active_text(p);
     if (buf == NULL || *buf == '\0') {
 	g_free(buf);
 	return;
@@ -2240,13 +2240,13 @@ static int var_is_ok (int i, int code)
     return ret;
 }
 
-static void add_vars_to_combo (GtkWidget *w, int code, int pos)
+static void add_vars_to_combo (GtkWidget *box, int code, int pos)
 {
     int i, vmin = (pos > 0)? 2 : 1;
 
     for (i=vmin; i<datainfo->v; i++) {
 	if (var_is_ok(i, code)) {
-	    gtk_combo_box_append_text(GTK_COMBO_BOX(w), datainfo->varname[i]);
+	    combo_box_append_text(box, datainfo->varname[i]);
 	}
     }
 
@@ -2254,13 +2254,13 @@ static void add_vars_to_combo (GtkWidget *w, int code, int pos)
 	/* add first variable at the end of the list */
 	for (i=1; i<datainfo->v; i++) {
 	    if (var_is_ok(i, code)) {
-		gtk_combo_box_append_text(GTK_COMBO_BOX(w), datainfo->varname[i]);
+		combo_box_append_text(box, datainfo->varname[i]);
 		break;
 	    }
 	}
     }
 
-    gtk_combo_box_set_active(GTK_COMBO_BOX(w), 0);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(box), 0);
 }
 
 static void toggle_combo_ok (GtkWidget *toggle, gpointer p)
@@ -2323,7 +2323,7 @@ static void add_test_var_selector (GtkWidget *tbl, gint *row,
 			      label, 0, 1, *row - 1, *row);
     gtk_widget_show(label);
 
-    tmp = gtk_combo_box_entry_new_text();
+    tmp = combo_box_text_new_with_entry();
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
 			      tmp, 1, 2, *row - 1, *row);
     gtk_widget_show(tmp);
@@ -2361,7 +2361,7 @@ static void add_test_combo (GtkWidget *tbl, gint *rows,
 			      button, 0, 1, *rows - 1, *rows);
     gtk_widget_show(button);
 
-    tmp = gtk_combo_box_entry_new_text();
+    tmp = combo_box_text_new_with_entry();
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
 			      tmp, 1, 2, *rows - 1, *rows);
     gtk_widget_show(tmp);
@@ -3155,7 +3155,7 @@ static void plot_curve (void)
     static struct curve_plotter plotter;
     GtkWidget *dialog, *hbox, *vbox;
     GtkWidget *button, *tmp;
-    GtkObject *adj;
+    GtkAdjustment *adj;
 
     if (plotter.dlg != NULL) {
 	gtk_window_present(GTK_WINDOW(plotter.dlg));
@@ -3191,8 +3191,8 @@ static void plot_curve (void)
     hbox = gtk_hbox_new(FALSE, 5);
     tmp = gtk_label_new(_("x minimum"));
     gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
-    adj = gtk_adjustment_new(plotter.xmin, -100, 100, 1, 0, 0);
-    tmp = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(plotter.xmin, -100, 100, 1, 0, 0);
+    tmp = gtk_spin_button_new(adj, 1, 0);
     g_signal_connect(tmp, "value-changed", 
 		     G_CALLBACK(set_double_from_spinner), &plotter.xmin);
     gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 0);
@@ -3200,8 +3200,8 @@ static void plot_curve (void)
     gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
     tmp = gtk_label_new(_("x range"));
     gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
-    adj = gtk_adjustment_new(plotter.xrange, 1, 1000, 1, 0, 0);
-    tmp = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1, 0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(plotter.xrange, 1, 1000, 1, 0, 0);
+    tmp = gtk_spin_button_new(adj, 1, 0);
     g_signal_connect(tmp, "value-changed", 
 		     G_CALLBACK(set_double_from_spinner), &plotter.xrange);
     gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 0); 
