@@ -176,7 +176,7 @@ static void display_dbdata (const double **dbZ, DATAINFO *dbinfo)
 		NULL); 
 }
 
-static void graph_dbdata (double ***dbZ, DATAINFO *dbinfo)
+static void graph_dbdata (const double **dbZ, DATAINFO *dbinfo)
 {
     int *list;
     int err;
@@ -190,7 +190,7 @@ static void graph_dbdata (double ***dbZ, DATAINFO *dbinfo)
     if (dbinfo->structure == CROSS_SECTION) {
 	err = boxplots(list, dbZ, dbinfo, OPT_NONE);
     } else {
-	err = gnuplot(list, NULL, (const double **) *dbZ, dbinfo,
+	err = gnuplot(list, NULL, dbZ, dbinfo,
 		      OPT_G | OPT_O | OPT_T);
     }
 
@@ -337,7 +337,7 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 	const char *cstr = NULL;
 	int v, dbv, start, stop;
 	int pad1 = 0, pad2 = 0;
-	int expand = 0, compact = 0;
+	int compact = 0;
 	int interpol = 0;
 
 	sinfo = &dw->sinfo[i];
@@ -380,7 +380,6 @@ add_db_series_to_dataset (windata_t *vwin, double **dbZ, dbwrapper *dw)
 
 	if (sinfo->pd < datainfo->pd) {
 	    /* the series needs to be expanded */
-	    expand = 1;
 	    xvec = expand_db_series(dbZ[v], sinfo, datainfo->pd, interpol);
 	} else if (sinfo->pd > datainfo->pd) {
 	    /* the series needs to be compacted */
@@ -732,7 +731,7 @@ static void gui_get_db_series (windata_t *vwin, int cmd)
     if (cmd == DB_DISPLAY) {
 	display_dbdata((const double **) dbZ, dbinfo);
     } else if (cmd == DB_GRAPH) {
-	graph_dbdata(&dbZ, dbinfo);
+	graph_dbdata((const double **) dbZ, dbinfo);
     } else if (cmd == DB_IMPORT) { 
 	add_dbdata(vwin, dbZ, dbinfo, dw, &freeit);
     }
@@ -916,7 +915,12 @@ maybe_adjust_descrip_column (windata_t *vwin)
     w1 = gtk_tree_view_column_get_width(col);
 
     window = gtk_widget_get_window(vwin->listbox);
+
+#if GTK_MAJOR_VERSION >= 3
+    lw = gdk_window_get_width(window);
+#else
     gdk_drawable_get_size(window, &lw, NULL);
+#endif
 
     w1max = lw - w0 - 140;
 
@@ -1173,7 +1177,7 @@ static int add_remote_db_series_list (windata_t *vwin, char *buf)
     GtkTreeIter iter;  
     gchar *row[3];
     char sername[VNAMELEN];
-    char line1[150], line2[150];
+    char line1[256], line2[256];
     int offset = 0;
     int n, err = 0;
 
@@ -1691,8 +1695,8 @@ void open_remote_db_index (GtkWidget *w, gpointer data)
 
 #define INFOLEN 100
 
-static int parse_db_header (const char *buf, size_t *idxlen, 
-			    size_t *datalen, size_t *cblen)
+static int parse_db_header (const char *buf, unsigned *idxlen, 
+			    unsigned *datalen, unsigned *cblen)
 {
     char *p;
     int err = 0;
@@ -1733,8 +1737,8 @@ static int ggz_extract (char *ggzname)
 {
     gzFile fgz = NULL;
     FILE *fidx = NULL, *fbin = NULL, *fcbk = NULL;
-    size_t idxlen, datalen, bytesleft, bgot;
-    size_t cblen = 0;
+    unsigned idxlen, datalen, cblen = 0;
+    int bgot, bytesleft;
     char idxname[MAXLEN], binname[MAXLEN], cbname[MAXLEN];
     char gzbuf[GRETL_BUFSIZE];
 #if G_BYTE_ORDER == G_BIG_ENDIAN
@@ -1885,7 +1889,8 @@ enum {
    data files, or function package 
 */
 
-static char *get_writable_target (int code, int op, char *objname)
+static char *get_writable_target (int code, int op, char *objname,
+				  gboolean zipfile)
 {
     const char *ext;
     FILE *fp;
@@ -1900,7 +1905,7 @@ static char *get_writable_target (int code, int op, char *objname)
     if (code == REMOTE_DB) {
 	ext = ".ggz";
     } else if (code == REMOTE_FUNC_FILES) {
-	ext = ".gfn";
+	ext = zipfile ? ".zip" : ".gfn";
     } else {
 	ext = ".tar.gz";
     }
@@ -1979,6 +1984,48 @@ static int unpack_book_data (const char *fname)
     return err;
 }
 
+int unzip_package_file (const char *zipname, const char *path)
+{
+    char *p, dirname[FILENAME_MAX];
+    int (*gretl_unzip_file) (const char *, GError **);
+    void *handle = NULL;
+    GError *gerr = NULL;
+    int err = 0;
+
+    strcpy(dirname, path);
+    p = strrchr(dirname, SLASH);
+    if (p != NULL) {
+	*p = '\0';
+    }
+
+    err = gretl_chdir(dirname);
+    if (err) {
+	return err;
+    }
+
+    gretl_unzip_file = gui_get_plugin_function("gretl_unzip_file", 
+					       &handle);
+    if (gretl_unzip_file == NULL) {
+        return 1;
+    }
+
+    err = (*gretl_unzip_file)(zipname, &gerr);
+    if (gerr != NULL) {
+	fprintf(stderr, "gretl_unzip_file: '%s'\n", gerr->message);
+	if (!err) {
+	    err = 1;
+	}
+	g_error_free(gerr);
+    } else if (err) {
+	fprintf(stderr, "gretl_unzip_file: err = %d\n", err);
+    }
+
+    close_plugin(handle);
+    gretl_remove(zipname);
+
+    return err;
+}
+
 /* note : 'vwin' here is the source viewer window displaying the
    remote file (database or datafiles package or function package)
    that is being installed onto the local machine.
@@ -1988,6 +2035,7 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 {
     gchar *objname = NULL;
     char *path = NULL;
+    gboolean zipfile = FALSE;
     int err = 0;
 
     if (vwin->role == REMOTE_DB) {
@@ -2003,21 +2051,35 @@ void install_file_from_server (GtkWidget *w, windata_t *vwin)
 	/* remote datafiles or function package */
 	tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			     vwin->active_var, 0, &objname);
+	if (vwin->role == REMOTE_FUNC_FILES) {
+	    tree_view_get_bool(GTK_TREE_VIEW(vwin->listbox), 
+			       vwin->active_var, 4, &zipfile);
+	}
     }
 
     if (objname == NULL || *objname == '\0') {
 	g_free(objname);
 	return;
-    }    
+    } 
 
-    path = get_writable_target(vwin->role, REAL_INSTALL, objname);
+    path = get_writable_target(vwin->role, REAL_INSTALL, objname, zipfile);
     if (path == NULL) {
 	g_free(objname);
 	return;
     }
 
     if (vwin->role == REMOTE_FUNC_FILES) {
-	err = retrieve_remote_function_package(objname, path);
+	if (zipfile) {
+	    gchar *zipname = g_strdup_printf("%s.zip", objname);
+
+	    err = retrieve_remote_function_package(zipname, path);
+	    if (!err) {
+		err = unzip_package_file(zipname, path);
+	    }
+	    g_free(zipname);
+	} else {
+	    err = retrieve_remote_function_package(objname, path);
+	}
     } else if (vwin->role == REMOTE_DATA_PKGS) {
 	err = retrieve_remote_datafiles_package(objname, path);
     } else {
@@ -2082,7 +2144,7 @@ void pkg_info_from_server (GtkWidget *w, windata_t *vwin)
     tree_view_get_string(GTK_TREE_VIEW(vwin->listbox), 
 			 vwin->active_var, 0, &objname);
 
-    path = get_writable_target(vwin->role, TMP_INSTALL, objname);
+    path = get_writable_target(vwin->role, TMP_INSTALL, objname, FALSE);
     if (path == NULL) {
 	g_free(objname);
 	return;
@@ -2572,8 +2634,6 @@ gint populate_remote_func_list (windata_t *vwin)
     char fname[32];
     const char *status;
     char *basename;
-    char *descrip;
-    char *version;
     time_t remtime;
     int n, err = 0;
 
@@ -2584,6 +2644,10 @@ gint populate_remote_func_list (windata_t *vwin)
 	return err;
     }
 
+#if 0
+    fprintf(stderr, "getbuf: '%s'\n", getbuf);
+#endif
+
     store = GTK_LIST_STORE(gtk_tree_view_get_model 
 			   (GTK_TREE_VIEW(vwin->listbox)));
     gtk_list_store_clear(store);
@@ -2593,8 +2657,9 @@ gint populate_remote_func_list (windata_t *vwin)
     bufgets_init(getbuf);
 
     while (bufgets(line, sizeof line, getbuf)) {
-	descrip = NULL;
-	version = NULL;
+	char *descrip = NULL;
+	char *version = NULL;
+	gboolean zipfile;
 
 	if (read_remote_filetime(line, fname, &remtime, NULL)) {
 	    continue;
@@ -2602,6 +2667,7 @@ gint populate_remote_func_list (windata_t *vwin)
 
 	status = "";
 	get_local_object_status(fname, vwin->role, &status, remtime);
+	zipfile = has_suffix(fname, ".zip");
 	basename = strip_extension(fname);
 
 	if (bufgets(line, sizeof line, getbuf)) {
@@ -2626,6 +2692,7 @@ gint populate_remote_func_list (windata_t *vwin)
 			   1, version,
 			   2, descrip, 
 			   3, _(status),
+			   4, zipfile,
 			   -1);
 
 	free(descrip);
@@ -2659,7 +2726,6 @@ gint populate_remote_data_pkg_list (windata_t *vwin)
     char line[256];
     char fname[32];
     char tstr[16];
-    const char *status;
     char *basename;
     char *descrip;
     int n, err = 0;
@@ -2688,7 +2754,6 @@ gint populate_remote_data_pkg_list (windata_t *vwin)
 	    continue;
 	}
 
-	status = "";
 	basename = strip_extension(fname);
 
 	if (bufgets(line, sizeof line, getbuf)) {
