@@ -28,6 +28,7 @@
 #include "gretl_www.h"
 #include "menustate.h"
 #include "fnsave.h"
+#include "gretl_bundle.h"
 #include "fncall.h"
 #include "treeutils.h"
 #include "selector.h"
@@ -44,7 +45,7 @@
 #include <errno.h>
 #include <time.h>
 
-#define N_BROWSER_TYPES 5
+#define N_BROWSER_TYPES 6
 #define BROWSER_BUSY    1
 #define BROWSER_OK      0
 
@@ -590,7 +591,7 @@ char *strip_extension (char *s)
 	if (p != NULL && 
 	    (!strcmp(p, ".gdt") || !strcmp(p, ".inp") ||
 	     !strcmp(p, ".bin") || !strcmp(p, ".gfn") ||
-	     !strcmp(p, ".bn7"))) {
+	     !strcmp(p, ".bn7") || !strcmp(p, ".zip"))) {
 	    *p = '\0';
 	}
     }
@@ -738,7 +739,7 @@ static gint enter_opens_file (GtkWidget *w, GdkEventKey *key,
 
 static int gui_load_user_functions (const char *fname)
 {
-    int err = load_function_package_from_file(fname);
+    int err = load_function_package_by_filename(fname);
 
     if (err) {
 	gui_errmsg(err);
@@ -924,7 +925,7 @@ static void browser_functions_handler (windata_t *vwin, int task)
 	strcpy(path, pkgname);
     }
 
-#if 0
+#if 1
     fprintf(stderr, "browser_functions_handler: path='%s'\n", path);
 #endif
 
@@ -939,7 +940,7 @@ static void browser_functions_handler (windata_t *vwin, int task)
     } else if (task == EDIT_FN_PKG) {
 	edit_function_package(path);
     } else if (task == CALL_FN_PKG) {
-	call_function_package(path, vwin->main, &err);
+	call_function_package(path, vwin, &err);
     }
 
     g_free(pkgname);
@@ -1632,7 +1633,7 @@ static int fn_file_is_duplicate (const char *fname,
 
     model = GTK_TREE_MODEL(store);
 
-    if (!gtk_tree_model_get_iter_first(model, &iter)) {
+    if (imax == 0 || !gtk_tree_model_get_iter_first(model, &iter)) {
 	return 0;
     }
 
@@ -1657,6 +1658,53 @@ static int fn_file_is_duplicate (const char *fname,
     return ret;
 }
 
+static int is_functions_dir (const char *path)
+{
+    int n = strlen(path) - 9;
+
+    return n > 0 && !strcmp(path + n, "functions");
+}
+
+static int ok_gfn_path (const char *fullname, 
+			const char *shortname,
+			const char *dirname,
+			GtkListStore *store, 
+			GtkTreeIter *iter,
+			int imax, 
+			int *maxlen)
+{
+    char *descrip = NULL, *version = NULL;
+    int err, ok = 0;
+
+    err = get_func_info(fullname, &descrip, &version);
+
+    if (!err && !fn_file_is_duplicate(shortname, version, store, imax)) {
+	if (iter != NULL) {
+	    gchar *fname = g_strdup(shortname);
+	    int n = strlen(fname) - 4;
+
+	    fname[n] = '\0'; /* chop off ".gfn" */
+	    if (n > *maxlen) {
+		*maxlen = n;
+	    }
+	    gtk_list_store_append(store, iter);
+	    gtk_list_store_set(store, iter, 
+			       0, fname, 
+			       1, version,
+			       2, descrip, 
+			       3, function_package_is_loaded(fullname), 
+			       4, dirname, -1);
+	    g_free(fname);
+	}
+	ok = 1;
+    }
+
+    free(descrip);
+    free(version);
+
+    return ok;
+}
+
 /* Read (or just count) the .gfn files in a given directory.
    The signal to count rather than read is that the @iter
    argument is NULL.
@@ -1669,43 +1717,62 @@ read_fn_files_in_dir (DIR *dir, const char *path,
 {
     struct dirent *dirent;
     char fullname[MAXLEN];
-    int n, imax = *nfn;
-    int err;
+    int imax = *nfn;
+
+    /* look first for the new-style setup: gfn file in its own
+       subdir, e.g. functions/gig/gig.gfn 
+    */
+
+    if (is_functions_dir(path)) {
+	while ((dirent = readdir(dir)) != NULL) {
+	    const char *basename = dirent->d_name;
+	
+	    if (!strcmp(basename, ".") ||
+		!strcmp(basename, "..")) {
+		continue;
+	    }
+
+	    build_path(fullname, path, basename, NULL);
+	    if (gretl_isdir(fullname)) {
+		gchar *realbase, *realpath;
+		FILE *fp;
+
+		strcat(fullname, SLASHSTR);
+		strcat(fullname, basename);
+		strcat(fullname, ".gfn");
+		fp = gretl_fopen(fullname, "r");
+		if (fp != NULL) {
+		    fclose(fp);
+		    realbase = g_strdup_printf("%s.gfn", basename);
+		    realpath = g_strdup_printf("%s%c%s", path, SLASH, basename);
+		    *nfn += ok_gfn_path(fullname, realbase, realpath,
+					store, iter, imax, maxlen);
+		    g_free(realbase);
+		    g_free(realpath);
+		} else {
+		    gretl_error_clear();
+		}	
+	    }
+	}
+    
+	imax = *nfn;
+	rewinddir(dir);
+    }
+
+    /* then look for "plain" .gfn files */
 
     while ((dirent = readdir(dir)) != NULL) {
-	if (!strcmp(dirent->d_name, ".") ||
-	    !strcmp(dirent->d_name, "..")) {
+	const char *basename = dirent->d_name;
+
+	if (!strcmp(basename, ".") ||
+	    !strcmp(basename, "..")) {
 	    continue;
 	}
 
-	if (has_suffix(dirent->d_name, ".gfn")) {
-	    char *descrip = NULL, *version = NULL;
-
-	    build_path(fullname, path, dirent->d_name, NULL);
-	    err = get_func_info(fullname, &descrip, &version);
-
-	    if (!err && !fn_file_is_duplicate(dirent->d_name, version, store, imax)) {
-		if (iter != NULL) {
-		    gchar *fname = g_strdup(dirent->d_name);
-
-		    gtk_list_store_append(store, iter);
-		    n = strlen(fname) - 4;
-		    fname[n] = '\0';
-		    if (n > *maxlen) {
-			*maxlen = n;
-		    }
-		    gtk_list_store_set(store, iter, 
-				       0, fname, 
-				       1, version,
-				       2, descrip, 
-				       3, function_package_is_loaded(fullname), 
-				       4, path, -1);
-		    g_free(fname);
-		}
-		*nfn += 1;
-	    }
-	    free(descrip);
-	    free(version);
+	if (has_suffix(basename, ".gfn")) {
+	    build_path(fullname, path, basename, NULL);
+	    *nfn += ok_gfn_path(fullname, basename, path,
+				store, iter, imax, maxlen);
 	}
     }
 }
@@ -1804,31 +1871,9 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
 	free_strings_array(dnames, ndirs);
     } else {
 	char fndir[FILENAME_MAX];
+	int i = 0;
 
-	for (i=0; i<5; i++) {
-	    *fndir = '\0';
-
-	    if (i == 0) {
-		/* pick up any function files in system dir */
-		build_path(fndir, gretl_home(), "functions", NULL);
-	    } else if (i == 1) {
-		/* plus any function files in the user's working dir... */
-		strcpy(fndir, gretl_workdir());
-	    } else if (i == 2) {
-		/* and in any "functions" subdir thereof */
-		build_path(fndir, gretl_workdir(), "functions", NULL);
-	    } else if (i == 3) {
-		/* plus any in the user's dotdir */
-		build_path(fndir, gretl_dotdir(), "functions", NULL);
-	    } else if (i == 4) {
-		/* plus any in the default working dir, if not already searched */
-		const char *wdir = gretl_default_workdir();
-
-		if (wdir != NULL) {
-		    build_path(fndir, wdir, "functions", NULL);
-		} 
-	    }
-
+	while (get_plausible_functions_dir(fndir, i++)) {
 	    if (*fndir != '\0') {
 		dir = opendir(fndir);
 		if (dir != NULL) {
@@ -1863,54 +1908,6 @@ gint populate_func_list (windata_t *vwin, struct fpkg_response *fresp)
     return 0;
 }
 
-#ifndef NAME_MAX
-# define NAME_MAX 255
-#endif
-
-/* Given the name of a package (its internal name, not a path, e.g.
-   "gig"), search some plausible directories and see if we can
-   find the corresponding gfn file. If so, return an allocated copy 
-   of the absolute path, otherwise return NULL.
-*/
-
-gchar *gretl_function_package_get_path (const char *name)
-{
-    gchar *path = NULL;
-    char dirname[FILENAME_MAX];
-    DIR *dir;
-    int i, found = 0;
-
-    for (i=0; i<3 && !found; i++) {
-	if (i == 0) {
-	    build_path(dirname, gretl_home(), "functions", NULL);
-	} else if (i == 1) {
-	    strcpy(dirname, gretl_workdir());
-	} else if (i == 2) {
-	    build_path(dirname, gretl_workdir(), "functions", NULL);
-	} 
-
-	if ((dir = opendir(dirname)) != NULL) {
-	    struct dirent *dirent;
-	    char *p, test[NAME_MAX+1];
-
-	    while ((dirent = readdir(dir)) != NULL && !found) {
-		if (has_suffix(dirent->d_name, ".gfn")) {
-		    strcpy(test, dirent->d_name);
-		    p = strrchr(test, '.');
-		    *p = '\0';
-		    if (!strcmp(test, name)) {
-			path = g_strdup_printf("%s%c%s", dirname, SLASH, dirent->d_name);
-			found = 1;
-		    }
-		}
-	    }
-	    closedir(dir);
-	}
-    }
-
-    return path;
-}
-
 static void revise_loaded_status (GtkWidget *lbox)
 {
     char fullname[MAXLEN];
@@ -2031,18 +2028,19 @@ static GtkWidget *files_window (windata_t *vwin)
 	G_TYPE_STRING,
 	G_TYPE_STRING
     };
-    GType types_4[] = {
-	G_TYPE_STRING,
-	G_TYPE_STRING,
-	G_TYPE_STRING,
-	G_TYPE_STRING
-    };
-    GType types_5[] = {
+    GType types_5s[] = {
 	G_TYPE_STRING,
 	G_TYPE_STRING,
 	G_TYPE_STRING,
 	G_TYPE_BOOLEAN,
-	G_TYPE_STRING
+	G_TYPE_STRING   /* hidden string: directory */
+    };
+    GType types_5b[] = {
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_STRING,
+	G_TYPE_BOOLEAN  /* hidden boolean: zipfile? */
     };
 
     const char **titles = data_titles;
@@ -2081,15 +2079,16 @@ static GtkWidget *files_window (windata_t *vwin)
     case FUNC_FILES:
 	titles = func_titles;
 	cols = 5;
-	types = types_5;
+	types = types_5s;
 	hidden_col = TRUE;
 	full_width = 560;
 	file_height = 320;
 	break;
     case REMOTE_FUNC_FILES:
 	titles = remote_func_titles;
-	cols = 4;
-	types = types_4;
+	cols = 5;
+	types = types_5b;
+	hidden_col = TRUE;
 	full_width = 580;
 	break;
     default:
@@ -2113,7 +2112,7 @@ static GtkWidget *files_window (windata_t *vwin)
 }
 
 static void 
-switch_file_page_callback (GtkNotebook *notebook, GtkNotebookPage *page,
+switch_file_page_callback (GtkNotebook *notebook, GtkWidget *page,
 			   guint page_num, windata_t *vwin)
 {
     gpointer p = g_object_get_data(G_OBJECT(notebook), "browse_ptr");
