@@ -486,9 +486,9 @@ static void add_poly_weights_options (GtkWidget *vbox, filter_info *finfo)
     hbox = gtk_hbox_new(FALSE, 5);
     label = gtk_label_new(_("Weights:"));
     gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 5);
-    combo = gtk_combo_box_new_text();
+    combo = gtk_combo_box_text_new();
     for (i=0; i<4; i++) {
-	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _(wstrs[i]));
+	combo_box_append_text(combo, _(wstrs[i]));
     }
     gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
     g_signal_connect(GTK_COMBO_BOX(combo), "changed",
@@ -1045,7 +1045,7 @@ static void weights_shape_graph (GtkWidget *button, filter_info *finfo)
 	err = private_matrix_add(W, "$Weights");
 
 	if (!err) {
-	    err = umatrix_set_colnames_from_string(W, "weight t");
+	    err = umatrix_set_names_from_string(W, "weight t", 0);
 	}
 
 	if (!err) {
