@@ -20,13 +20,23 @@
 #ifndef FNCALL_H
 #define FNCALL_H
 
-void call_function_package (const char *fname, GtkWidget *w,
+void call_function_package (const char *fname, windata_t *vwin,
 			    int *loaderr);
 
 void function_call_cleanup (void);
 
 void gui_define_list (void);
 
+void fncall_register_genr (int addv, gpointer p);
+
+gchar *get_bundle_plot_function (gretl_bundle *b);
+
+int exec_bundle_plot_function (gretl_bundle *b, const char *funname);
+
+int try_exec_bundle_print_function (gretl_bundle *b, PRN *prn);
+
+void maybe_add_packages_to_model_menus (windata_t *vwin);
+
 void maybe_add_packages_to_menus (windata_t *vwin);
 
 #endif /* FNCALL_H */
