@@ -53,19 +53,22 @@ struct gpt_range_t {
 #define MAX_AXES 3
 #define NTITLES  4
 
-/* apparatus for handling the case where lines and/or labels are
+/* apparatus for handling the case where plot components are
    added or removed via the gui dialog, but these changes are
    discarded (the user doesn't click "OK" or "Apply")
 */
 
 #define old_lines_init(e) (e->old_n_lines = -1, e->old_lines = NULL)
 #define old_labels_init(e) (e->old_n_labels = -1, e->old_labels = NULL)
+#define old_arrows_init(e) (e->old_n_arrows = -1, e->old_arrows = NULL)
 
 #define lines_not_synced(e) (e->old_n_lines < 0)
 #define labels_not_synced(e) (e->old_n_labels < 0)
+#define arrows_not_synced(e) (e->old_n_arrows < 0)
 
 #define restore_lines(e) (e->old_n_lines >= 0)
 #define restore_labels(e) (e->old_n_labels >= 0)
+#define restore_arrows(e) (e->old_n_arrows >= 0)
 
 typedef struct plot_editor_ plot_editor;
 
@@ -89,6 +92,7 @@ struct plot_editor_ {
     GtkWidget **labeltext;
     GtkWidget **labeljust;
     GtkWidget **labelpos;
+    GtkWidget **arrowpos;
 
     GtkWidget *keycombo;
     GtkWidget *fitcombo;
@@ -103,23 +107,27 @@ struct plot_editor_ {
 
     int gui_nlines;
     int gui_nlabels;
+    int gui_narrows;
 
     struct gpt_range_t axis_range[MAX_AXES];
     struct gpt_titles_t gpt_titles[NTITLES];
 
     int old_n_lines;
     int old_n_labels;
+    int old_n_arrows;
 
     GPT_LINE *old_lines;
     GPT_LABEL *old_labels;
+    GPT_ARROW *old_arrows;
 };
 
-#define PLOT_LABEL_POS_LEN 32
+#define PLOT_POSITION_LEN 32
 
 const gchar *fittype_strings[] = {
     N_("none"),
     N_("linear: y = a + b*x"),
     N_("quadratic: y = a + b*x + c*x^2"),
+    N_("cubic: y = a + b*x + c*x^2 + d*x^3"),
     N_("inverse: y = a + b*(1/x)"),
     N_("loess (locally weighted fit)"),
     NULL
@@ -127,14 +135,17 @@ const gchar *fittype_strings[] = {
 
 enum {
     GUI_LINE,
-    GUI_LABEL
+    GUI_LABEL,
+    GUI_ARROW
 };
 
 static const char *get_font_filename (const char *showname);
 static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins);
 static void gpt_tab_labels (plot_editor *ed, GPT_SPEC *spec, int ins);
+static void gpt_tab_arrows (plot_editor *ed, GPT_SPEC *spec, int ins);
 static int add_line_widget (plot_editor *ed);
 static int add_label_widget (plot_editor *ed);
+static int add_arrow_widget (plot_editor *ed);
 static void plot_editor_set_fontname (plot_editor *ed, const char *name);
 
 /* graph color selection apparatus */
@@ -480,7 +491,7 @@ static void combo_to_gp_style (GtkWidget *w, int *sty)
 
     g_return_if_fail(GTK_IS_COMBO_BOX(w));
 
-    s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(w));
+    s = combo_box_get_active_text(w);
 
     if (s != NULL && *s != '\0') {
 	*sty = gp_style_index_from_display_name(s);
@@ -500,7 +511,8 @@ static void fittype_from_combo (GtkComboBox *box, GPT_SPEC *spec)
     }
 
     if (f == PLOT_FIT_OLS || f == PLOT_FIT_QUADRATIC || 
-	f == PLOT_FIT_INVERSE || f == PLOT_FIT_LOESS) {
+	f == PLOT_FIT_CUBIC || f == PLOT_FIT_INVERSE || 
+	f == PLOT_FIT_LOESS) {
 	plotspec_add_fit(spec, f);
 	spec->flags &= ~GPT_FIT_HIDDEN;
     } else if (f == PLOT_FIT_NONE) {
@@ -540,6 +552,9 @@ static gboolean fit_type_changed (GtkComboBox *box, plot_editor *ed)
     } else if (f == PLOT_FIT_QUADRATIC) {
 	title = g_strdup_printf(_("%s versus %s (with quadratic fit)"),
 		s1, s2);
+    } else if (f == PLOT_FIT_CUBIC) {
+	title = g_strdup_printf(_("%s versus %s (with cubic fit)"),
+		s1, s2);
     } else if (f == PLOT_FIT_INVERSE) {
 	title = g_strdup_printf(_("%s versus %s (with inverse fit)"),
 		s1, s2);
@@ -639,8 +654,7 @@ static void entry_to_gp_double (GtkWidget *w, double *val)
 #define gp_string_to_entry(w,s) do { \
 	gtk_entry_set_text(GTK_ENTRY(w),s); } while (0)
 
-static int
-get_label_pos_from_entry (GtkWidget *w, double *pos)
+static int get_label_pos_from_entry (GtkWidget *w, double *pos)
 {
     int err = 0;
 
@@ -650,7 +664,7 @@ get_label_pos_from_entry (GtkWidget *w, double *pos)
     
 	chk = sscanf(s, "%lf %lf", &pos[0], &pos[1]);
 	if (chk != 2) {
-	    errbox(_("Invalid label position, must be X Y"));
+	    errbox(_("Invalid position, must be X Y"));
 	    gtk_editable_select_region(GTK_EDITABLE(w), 0, strlen(s));
 	    pos[0] = pos[1] = NADBL;
 	    err = 1;
@@ -662,6 +676,61 @@ get_label_pos_from_entry (GtkWidget *w, double *pos)
     return err;
 }
 
+static int get_arrow_spec_from_entries (plot_editor *ed, int i, 
+					GPT_ARROW *arrow)
+{
+    int j, err = 0;
+
+    for (j=0; j<2 && !err; j++) {
+	GtkWidget *entry, *b1 = NULL, *b2 = NULL;
+	double *x, *y;
+
+	if (j == 0) {
+	    entry = ed->arrowpos[i];
+	    b1 = g_object_get_data(G_OBJECT(entry), "arrow_check");
+	    b2 = g_object_get_data(G_OBJECT(entry), "dots_check");
+	    x = &arrow->x0;
+	    y = &arrow->y0;
+	} else {
+	    entry = g_object_get_data(G_OBJECT(ed->arrowpos[i]), "pos_2");
+	    x = &arrow->x1;
+	    y = &arrow->y1;
+	}
+
+	if (GTK_IS_ENTRY(entry)) {
+	    const gchar *s = gtk_entry_get_text(GTK_ENTRY(entry));
+	    int n;
+    
+	    n = sscanf(s, "%lf %lf", x, y);
+	    if (n != 2) {
+		errbox(_("Invalid position, must be X Y"));
+		gtk_editable_select_region(GTK_EDITABLE(entry), 0, strlen(s));
+		*x = *y = NADBL;
+		err = 1;
+	    } 
+	} else {
+	    err = 1;
+	}
+
+	if (b1 != NULL) {
+	    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(b1))) {
+		arrow->flags |= GP_ARROW_HEAD;
+	    } else {
+		arrow->flags &= ~GP_ARROW_HEAD;
+	    }
+	}
+	if (b2 != NULL) {
+	    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(b2))) {
+		arrow->flags |= GP_ARROW_DOTS;
+	    } else {
+		arrow->flags &= ~GP_ARROW_DOTS;
+	    }
+	}
+    }
+
+    return err;
+}
+
 static int validate_range (double *r)
 {
     int err = 0;
@@ -724,11 +793,14 @@ static void plot_editor_sync (plot_editor *ed)
 
     free(ed->old_labels);
     old_labels_init(ed);
+
+    free(ed->old_arrows);
+    old_arrows_init(ed);
 }
 
 static void set_graph_font_from_widgets (plot_editor *ed)
 {
-    gchar *tmp = gtk_combo_box_get_active_text(GTK_COMBO_BOX(ed->ttfcombo));
+    gchar *tmp = combo_box_get_active_text(ed->ttfcombo);
     int ptsize = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(ed->ttfspin));
 
     if (tmp != NULL && *tmp != '\0') {
@@ -783,7 +855,7 @@ static gboolean maybe_append_user_bars_file (GtkComboBox *combo,
 	const char *p = strrchr(ed->barsfile, SLASH);
 	const char *show = (p != NULL)? (p+1) : ed->barsfile;
 
-	gtk_combo_box_append_text(combo, show);
+	combo_box_append_text(combo, show);
 	return TRUE;
     } else {
 	return FALSE;
@@ -852,7 +924,7 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
     }
 
     if (ed->keycombo != NULL) {
-        s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(ed->keycombo));
+        s = combo_box_get_active_text(ed->keycombo);
 	spec->keyspec = gp_keypos_from_display_name(s);
 	g_free(s);
     }
@@ -867,7 +939,7 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 	line = &spec->lines[i];
 	line->yaxis = 1;
 	if (!suppress_y2 && ed->yaxiscombo[i] != NULL) {
-	    s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(ed->yaxiscombo[i]));
+	    s = combo_box_get_active_text(ed->yaxiscombo[i]);
 	    if (!strcmp(s, _("right"))) {
 		line->yaxis = 2;	
 	    }
@@ -960,6 +1032,10 @@ static void apply_gpt_changes (GtkWidget *w, plot_editor *ed)
 	}
     } 
 
+    for (i=0; i<ed->gui_narrows && !err; i++) {
+	err = get_arrow_spec_from_entries(ed, i, &spec->arrows[i]);
+    } 
+
     if (!err && ed->border_check != NULL) {
 	if (button_is_active(ed->border_check)) {
 	    /* full border */
@@ -1440,9 +1516,9 @@ void set_plotbars_filename (const char *fname, gpointer data)
 
 	g_free(ed->barsfile);
 	ed->barsfile = g_strdup(fname);
-	gtk_combo_box_remove_text(box, 1);
-	gtk_combo_box_append_text(box, show);
-	gtk_combo_box_append_text(box, _("other..."));
+	combo_box_remove(box, 1);
+	combo_box_append_text(box, show);
+	combo_box_append_text(box, _("other..."));
 	gtk_combo_box_set_active(box, 1);
     }
 }
@@ -1477,7 +1553,9 @@ static void plot_bars_changed (GtkComboBox *box, plot_editor *ed)
 					  NULL);
 
 	gretl_dialog_add_message(dlg, _(msg));
+#if GTK_MAJOR_VERSION < 3
 	gtk_dialog_set_has_separator(GTK_DIALOG(dlg), FALSE);
+#endif
 	gtk_window_set_keep_above(GTK_WINDOW(dlg), TRUE);  
 	ret = gtk_dialog_run(GTK_DIALOG(dlg));
 	gtk_widget_destroy(dlg);
@@ -1577,12 +1655,12 @@ static void add_gd_font_selector (plot_editor *ed, GtkWidget *tbl, int *rows)
 
     /* FIXME max length of font name? */
 
-    ed->ttfcombo = gtk_combo_box_entry_new_text();
+    ed->ttfcombo = combo_box_text_new_with_entry();
     entry = gtk_bin_get_child(GTK_BIN(ed->ttfcombo));
     gtk_entry_set_max_length(GTK_ENTRY(entry), 15);
     gtk_table_attach_defaults(GTK_TABLE(tbl), ed->ttfcombo, 1, 2, 
 			      *rows-1, *rows);
-    set_combo_box_strings_from_list(GTK_COMBO_BOX(ed->ttfcombo), fontnames); 
+    set_combo_box_strings_from_list(ed->ttfcombo, fontnames); 
     gtk_entry_set_text(GTK_ENTRY(entry), default_font);
     gtk_entry_set_width_chars(GTK_ENTRY(entry), 15);
     g_signal_connect(G_OBJECT(entry), "activate", 
@@ -1668,7 +1746,7 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 			      label, 0, 1, rows-1, rows);
     gtk_widget_show(label);
 
-    ed->keycombo = gtk_combo_box_new_text();
+    ed->keycombo = gtk_combo_box_text_new();
     gtk_table_attach_defaults(GTK_TABLE(tbl), 
 			      ed->keycombo, 1, TAB_MAIN_COLS, rows-1, rows);
     for (i=0; ; i++) {
@@ -1677,7 +1755,7 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 	if (kp == NULL) {
 	    break;
 	}
-	gtk_combo_box_append_text(GTK_COMBO_BOX(ed->keycombo), _(kp->str));
+	combo_box_append_text(ed->keycombo, _(kp->str));
 	if (kp->id == spec->keyspec) {
 	    kactive = i;
 	}
@@ -1694,7 +1772,7 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 				  label, 0, 1, rows-1, rows);
 	gtk_widget_show(label);
 
-	ed->fitcombo = gtk_combo_box_new_text();
+	ed->fitcombo = gtk_combo_box_text_new();
 	gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				  ed->fitcombo, 1, TAB_MAIN_COLS, rows-1, rows);
 
@@ -1707,12 +1785,11 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 		}
 		strcpy(tmp, _(fittype_strings[i]));
 		charsub(tmp, 'x', 't');
-		gtk_combo_box_append_text(GTK_COMBO_BOX(ed->fitcombo), tmp);
+		combo_box_append_text(ed->fitcombo, tmp);
 	    }	    
 	} else {
 	    for (i=0; fittype_strings[i] != NULL; i++) {
-		gtk_combo_box_append_text(GTK_COMBO_BOX(ed->fitcombo), 
-					  _(fittype_strings[i]));
+		combo_box_append_text(ed->fitcombo, _(fittype_strings[i]));
 	    }
 	}
 
@@ -1795,10 +1872,10 @@ static void gpt_tab_main (plot_editor *ed, GPT_SPEC *spec)
 	gtk_widget_show(ed->bars_check);
 
 	/* plus combo selector */
-	ed->barscombo = combo = gtk_combo_box_new_text();
-	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("NBER recessions"));
+	ed->barscombo = combo = gtk_combo_box_text_new();
+	combo_box_append_text(combo, _("NBER recessions"));
 	userbars = maybe_append_user_bars_file(GTK_COMBO_BOX(combo), ed);
-	gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("other..."));
+	combo_box_append_text(combo, _("other..."));
 	gtk_combo_box_set_active(GTK_COMBO_BOX(combo), userbars ? 1 : 0);
 	gtk_widget_set_sensitive(combo, spec->nbars > 0);
 	g_signal_connect(G_OBJECT(combo), "changed", 
@@ -1878,13 +1955,12 @@ static void gpt_tab_new_line (plot_editor *ed, new_line_info *nlinfo)
 {
     GtkWidget *label, *tbl;
     GtkWidget *vbox, *hbox;
-    int tbl_len, tbl_num, tbl_col;
+    int tbl_len = 1;
     char label_text[32];
 
     vbox = gtk_dialog_get_content_area(GTK_DIALOG(nlinfo->dlg));
     hbox = gtk_hbox_new(FALSE, 5);
 
-    tbl_len = 1;
     tbl = gtk_table_new(tbl_len, 3, FALSE);
     gtk_table_set_row_spacings(GTK_TABLE(tbl), 5);
     gtk_table_set_col_spacings(GTK_TABLE(tbl), 5);
@@ -1893,8 +1969,6 @@ static void gpt_tab_new_line (plot_editor *ed, new_line_info *nlinfo)
     gtk_widget_show(hbox);
     gtk_widget_show(tbl);
    
-    tbl_num = tbl_col = 0;
-
     /* identifier and formula text */
     tbl_len++;
     gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
@@ -1920,7 +1994,7 @@ static void gpt_tab_new_line (plot_editor *ed, new_line_info *nlinfo)
     gtk_widget_show(nlinfo->formula_entry);
 }
 
-/* back up exiting spec lines / labels in case changes are not
+/* back up existing spec components in case changes are not
    applied and we need to restore the prior state */
 
 static int make_labels_backup (plot_editor *ed)
@@ -1937,6 +2011,20 @@ static int make_labels_backup (plot_editor *ed)
     return err;
 }
 
+static int make_arrows_backup (plot_editor *ed)
+{
+    int err = 0;
+
+    if (arrows_not_synced(ed)) {
+	ed->old_arrows = plotspec_clone_arrows(ed->spec, &err);
+	if (!err) {
+	    ed->old_n_arrows = ed->spec->n_arrows;
+	}
+    }
+
+    return err;
+}
+
 static int make_lines_backup (plot_editor *ed)
 {
     int err = 0;
@@ -1975,6 +2063,30 @@ static int allocate_label (plot_editor *ed)
     return err;
 }
 
+static int allocate_arrow (plot_editor *ed)
+{
+    int err = 0;
+
+    err = make_arrows_backup(ed);
+
+    if (!err) {
+	err = plotspec_add_arrow(ed->spec);
+    }
+
+    if (!err) {
+	err = add_arrow_widget(ed);
+	if (err) {
+	    ed->spec->n_arrows -= 1;
+	}
+    }
+
+    if (err) {
+	nomem();
+    }
+
+    return err;
+}
+
 static int allocate_line (plot_editor *ed)
 {
     int err = make_lines_backup(ed);
@@ -1999,20 +2111,32 @@ static int allocate_line (plot_editor *ed)
 
 static void add_label_callback (GtkWidget *w, plot_editor *ed)
 {
-    gint pgnum;
-    int err;
-
-    err = allocate_label(ed);
+    int err = allocate_label(ed);
 
     if (!err) {
 	/* re-fill the "labels" notebook page */
-	pgnum = widget_get_int(ed->notebook, "labels_page");
+	gint pgnum = widget_get_int(ed->notebook, "labels_page");
+
 	gtk_notebook_remove_page(GTK_NOTEBOOK(ed->notebook), pgnum);
 	gpt_tab_labels(ed, ed->spec, pgnum);
 	gtk_notebook_set_current_page(GTK_NOTEBOOK(ed->notebook), pgnum);
     }
 }
 
+static void add_arrow_callback (GtkWidget *w, plot_editor *ed)
+{
+    int err = allocate_arrow(ed);
+ 
+    if (!err) {
+	/* re-fill the "arrows" notebook page */
+	gint pgnum = widget_get_int(ed->notebook, "arrows_page");
+
+	gtk_notebook_remove_page(GTK_NOTEBOOK(ed->notebook), pgnum);
+	gpt_tab_arrows(ed, ed->spec, pgnum);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(ed->notebook), pgnum);
+    }
+}
+
 static void set_gp_formula (char *targ, const char *s)
 {
     int decom = (get_local_decpoint() == ',');
@@ -2153,6 +2277,23 @@ static void remove_label (GtkWidget *w, plot_editor *ed)
     }
 }
 
+static void remove_arrow (GtkWidget *w, plot_editor *ed)
+{
+    if (make_arrows_backup(ed) != 0) {
+	nomem();
+    } else {
+	int pgnum = widget_get_int(ed->notebook, "arrows_page");
+
+	plotspec_delete_arrow(ed->spec, widget_get_int(w, "arrownum"));
+	ed->gui_narrows -= 1;
+
+	/* refresh the associated notebook page */
+	gtk_notebook_remove_page(GTK_NOTEBOOK(ed->notebook), pgnum);
+	gpt_tab_arrows(ed, ed->spec, pgnum);
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(ed->notebook), pgnum);
+    }
+}
+
 static void item_remove_button (GtkWidget *tbl, int row, 
 				plot_editor *ed, int i,
 				int j)
@@ -2160,11 +2301,13 @@ static void item_remove_button (GtkWidget *tbl, int row,
     GtkWidget *button; 
     const gchar *key[] = {
 	"linenum",
-	"labelnum"
+	"labelnum",
+	"arrownum"
     };
     GCallback cb[] = {
 	G_CALLBACK(remove_line),
-	G_CALLBACK(remove_label)
+	G_CALLBACK(remove_label),
+	G_CALLBACK(remove_arrow)
     };
 
     button = gtk_button_new_with_label(_("Remove"));
@@ -2305,7 +2448,7 @@ static int line_get_point_style (GPT_LINE *line, int i)
 static void flip_pointsel (GtkWidget *box, GtkWidget *targ)
 {
     GtkWidget *label = g_object_get_data(G_OBJECT(targ), "label");
-    gchar *s = gtk_combo_box_get_active_text(GTK_COMBO_BOX(box));
+    gchar *s = combo_box_get_active_text(box);
     int hp = has_point(gp_style_index_from_display_name(s));
 
     gtk_widget_set_sensitive(targ, hp);
@@ -2362,14 +2505,15 @@ static int gp_style_index (int sty, GList *list)
     return 0;
 }
 
-void set_combo_box_strings_from_stylist (GtkComboBox *box, GList *list)
+static void set_combo_box_strings_from_stylist (GtkWidget *box, 
+						GList *list)
 {
     gp_style_spec *spec;
     GList *mylist = list;
 
     while (mylist != NULL) {
 	spec = (gp_style_spec *) mylist->data;
-	gtk_combo_box_append_text(box, _(spec->trname));
+	combo_box_append_text(box, _(spec->trname));
 	mylist = mylist->next;
     }
 }
@@ -2390,7 +2534,7 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
     GtkWidget *label, *tbl;
     GtkWidget *vbox, *hbox, *sep;
     GtkWidget *page;
-    int i, tbl_len, tbl_num, tbl_col;
+    int i, tbl_len;
     GList *stylist = NULL;
     int axis_chooser;
     int pgnum = -1;
@@ -2442,8 +2586,6 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
     tbl = gp_dialog_table(tbl_len, 3, vbox);
     gtk_widget_show(tbl);
    
-    tbl_num = tbl_col = 0;
-
     for (i=0; i<ed->gui_nlines; i++) {
 	GPT_LINE *line = &spec->lines[i];
 	int label_done = 0;
@@ -2521,7 +2663,7 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 	gtk_widget_show(label);
 
 	hbox = gtk_hbox_new(FALSE, 5);
-	ed->stylecombo[i] = gtk_combo_box_new_text();
+	ed->stylecombo[i] = gtk_combo_box_text_new();
 	gtk_box_pack_start(GTK_BOX(hbox), ed->stylecombo[i], FALSE, FALSE, 0);
 
 	/* the errorbars and filledcurves styles are not exchangeable
@@ -2532,8 +2674,7 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 
 	    altsty = add_style_spec(altsty, GP_STYLE_ERRORBARS); 
 	    altsty = add_style_spec(altsty, GP_STYLE_FILLEDCURVE);
-	    set_combo_box_strings_from_stylist(GTK_COMBO_BOX(ed->stylecombo[i]), 
-					       altsty);
+	    set_combo_box_strings_from_stylist(ed->stylecombo[i], altsty);
 	    gtk_combo_box_set_active(GTK_COMBO_BOX(ed->stylecombo[i]),
 				     gp_style_index(line->style, altsty));
 	    g_list_free(altsty);
@@ -2542,8 +2683,7 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 	    int lt = gp_style_index(line->style, stylist);
 	    int hp, pt = line_get_point_style(line, i);
 
-	    set_combo_box_strings_from_stylist(GTK_COMBO_BOX(ed->stylecombo[i]), 
-					       stylist);
+	    set_combo_box_strings_from_stylist(ed->stylecombo[i], stylist);
 	    gtk_combo_box_set_active(GTK_COMBO_BOX(ed->stylecombo[i]), lt);
 	    hp = has_point(line->style);
 	    label = gtk_label_new(_("point"));
@@ -2589,11 +2729,11 @@ static void gpt_tab_lines (plot_editor *ed, GPT_SPEC *spec, int ins)
 	    gtk_table_resize(GTK_TABLE(tbl), tbl_len, 3);
 	    print_field_label(tbl, tbl_len, _("y axis"));
 
-	    ed->yaxiscombo[i] = gtk_combo_box_new_text();
+	    ed->yaxiscombo[i] = gtk_combo_box_text_new();
 	    gtk_table_attach_defaults(GTK_TABLE(tbl), 
 				      ed->yaxiscombo[i], 2, 3, tbl_len-1, tbl_len);
-	    gtk_combo_box_append_text(GTK_COMBO_BOX(ed->yaxiscombo[i]), _("left"));
-	    gtk_combo_box_append_text(GTK_COMBO_BOX(ed->yaxiscombo[i]), _("right"));
+	    combo_box_append_text(ed->yaxiscombo[i], _("left"));
+	    combo_box_append_text(ed->yaxiscombo[i], _("right"));
 	    gtk_combo_box_set_active(GTK_COMBO_BOX(ed->yaxiscombo[i]), 
 				     (line->yaxis == 1)? 0 : 1);
 	    gtk_widget_show(ed->yaxiscombo[i]);
@@ -2707,7 +2847,7 @@ static void gpt_tab_labels (plot_editor *ed, GPT_SPEC *spec, int ins)
     GtkWidget *notebook = ed->notebook;
     GtkWidget *label, *tbl;
     GtkWidget *vbox, *page, *sep;
-    int i, j, tbl_len, tbl_num, tbl_col;
+    int i, j, tbl_len;
     png_plot *plot = (png_plot *) spec->ptr;
     int pgnum = -1;
 
@@ -2735,8 +2875,6 @@ static void gpt_tab_labels (plot_editor *ed, GPT_SPEC *spec, int ins)
     tbl = gp_dialog_table(tbl_len, 3, vbox);
     gtk_widget_show(tbl);
    
-    tbl_num = tbl_col = 0;
-
     for (i=0; i<ed->gui_nlabels; i++) {
 	GtkWidget *hbox, *button, *image;
 	GdkPixbuf *icon;
@@ -2769,7 +2907,7 @@ static void gpt_tab_labels (plot_editor *ed, GPT_SPEC *spec, int ins)
 
 	/* entry for coordinates */
 	ed->labelpos[i] = gtk_entry_new();
-	gtk_entry_set_max_length(GTK_ENTRY(ed->labelpos[i]), PLOT_LABEL_POS_LEN);
+	gtk_entry_set_max_length(GTK_ENTRY(ed->labelpos[i]), PLOT_POSITION_LEN);
 	label_pos_to_entry(spec->labels[i].pos, ed->labelpos[i]);
 	g_signal_connect(G_OBJECT(ed->labelpos[i]), "activate", 
 			 G_CALLBACK(apply_gpt_changes), 
@@ -2780,9 +2918,9 @@ static void gpt_tab_labels (plot_editor *ed, GPT_SPEC *spec, int ins)
 	if (plot_is_mouseable(plot)) {
 	    /* button to invoke mouse-assisted placement */
 	    button = gtk_button_new();
-	    g_object_set_data(G_OBJECT(button), "labelpos_entry", ed->labelpos[i]);
+	    g_object_set_data(G_OBJECT(button), "pos_entry", ed->labelpos[i]);
 	    g_signal_connect(G_OBJECT(button), "clicked",
-			     G_CALLBACK(plot_label_position_click), spec->ptr);
+			     G_CALLBACK(plot_position_click), spec->ptr);
 	    icon = gdk_pixbuf_new_from_xpm_data((const char **) mini_mouse_xpm);
 	    image = gtk_image_new_from_pixbuf(icon);
 	    gtk_widget_set_size_request(button, 32, 26);
@@ -2801,10 +2939,10 @@ static void gpt_tab_labels (plot_editor *ed, GPT_SPEC *spec, int ins)
 
 	print_field_label(tbl, tbl_len, _("justification"));
 
-	ed->labeljust[i] = gtk_combo_box_new_text();
+	ed->labeljust[i] = gtk_combo_box_text_new();
 	for (j=0; j<3; j++) {
-	    gtk_combo_box_append_text(GTK_COMBO_BOX(ed->labeljust[i]),
-				      _(gp_justification_string(j)));
+	    combo_box_append_text(ed->labeljust[i],
+				  _(gp_justification_string(j)));
 	}
 	gtk_combo_box_set_active(GTK_COMBO_BOX(ed->labeljust[i]), 
 				 spec->labels[i].just);
@@ -2839,6 +2977,140 @@ static void gpt_tab_labels (plot_editor *ed, GPT_SPEC *spec, int ins)
     }
 }
 
+static void arrow_pos_to_entry (GPT_ARROW *arrow, GtkWidget *w, int j)
+{
+    double x = (j == 0)? arrow->x0 : arrow->x1;
+    double y = (j == 0)? arrow->y0 : arrow->y1;
+
+    if (!na(x) && !na(y)) {
+	gchar *s = g_strdup_printf("%.10g %.10g", x, y);
+
+	gtk_entry_set_text(GTK_ENTRY(w), s);
+	g_free(s);
+    } else {
+	gtk_entry_set_text(GTK_ENTRY(w), "");
+    }
+}
+
+#define ARROW_MAX 6
+
+static void gpt_tab_arrows (plot_editor *ed, GPT_SPEC *spec, int ins) 
+{
+    GtkWidget *notebook = ed->notebook;
+    GtkWidget *label, *tbl;
+    GtkWidget *vbox, *page, *sep;
+    int i, j, r, tbl_len;
+    png_plot *plot = (png_plot *) spec->ptr;
+    int nsep = 0, pgnum = -1;
+
+    vbox = gp_dialog_vbox();
+    gtk_widget_show(vbox);
+
+    label = gtk_label_new(_("Arrows"));
+    gtk_widget_show(label);
+    page = vbox;
+
+    if (ins > 0) {
+	pgnum = gtk_notebook_insert_page(GTK_NOTEBOOK(notebook), page, label, ins); 
+    } else {
+	pgnum = gtk_notebook_append_page(GTK_NOTEBOOK(notebook), page, label);
+    }  
+
+    widget_set_int(notebook, "arrows_page", pgnum);
+
+    if (ed->gui_narrows > 1) {
+	nsep = ed->gui_narrows - 1;
+    }
+    tbl_len = 1 + 4 * ed->gui_narrows + nsep;
+    tbl = gp_dialog_table(tbl_len, 3, vbox);
+    r = 1;
+   
+    for (i=0; i<ed->gui_narrows; i++) {
+	GtkWidget *button, *image, *chk;
+	GdkPixbuf *icon;
+
+	item_remove_button(tbl, r, ed, i, GUI_ARROW);
+
+	/* entries and buttons for (start, stop) coordinates */
+	for (j=0; j<2; j++) {
+	    GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
+	    GtkWidget *apos;
+
+	    if (j == 0) {
+		label = gtk_label_new(_("from (X Y)"));
+	    } else {
+		label = gtk_label_new(_("to (X Y)"));
+	    }
+	    gtk_table_attach_defaults(GTK_TABLE(tbl), label, 1, 2, r-1, r);
+
+	    apos = gtk_entry_new();
+	    gtk_entry_set_max_length(GTK_ENTRY(apos), PLOT_POSITION_LEN);
+	    arrow_pos_to_entry(&spec->arrows[i], apos, j);
+	    g_signal_connect(G_OBJECT(apos), "activate", 
+			     G_CALLBACK(apply_gpt_changes), 
+			     ed);
+	    gtk_container_add(GTK_CONTAINER(hbox), apos);
+
+	    if (j == 0) {
+		ed->arrowpos[i] = apos;
+	    } else {
+		g_object_set_data(G_OBJECT(ed->arrowpos[i]), "pos_2", apos);
+	    }
+
+	    if (plot_is_mouseable(plot)) {
+		/* button to invoke mouse-assisted placement */
+		button = gtk_button_new();
+		g_object_set_data(G_OBJECT(button), "pos_entry", apos);
+		g_signal_connect(G_OBJECT(button), "clicked",
+				 G_CALLBACK(plot_position_click), spec->ptr);
+		icon = gdk_pixbuf_new_from_xpm_data((const char **) mini_mouse_xpm);
+		image = gtk_image_new_from_pixbuf(icon);
+		gtk_widget_set_size_request(button, 32, 26);
+		gtk_container_add(GTK_CONTAINER(button), image);
+		gtk_container_add(GTK_CONTAINER(hbox), button);
+		gtk_table_attach_defaults(GTK_TABLE(tbl), hbox, 2, 3, r-1, r);
+		r++;
+	    }
+	}
+
+	/* arrow head selector */
+	chk = gtk_check_button_new_with_label(_("arrow has head"));
+	g_object_set_data(G_OBJECT(ed->arrowpos[i]), "arrow_check", chk);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chk), 
+				     (spec->arrows[i].flags & GP_ARROW_HEAD)?
+				     TRUE : FALSE);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), chk, 2, 3, r-1, r);
+	r++;
+
+	/* dotted line selector */
+	chk = gtk_check_button_new_with_label(_("line is dotted"));
+	g_object_set_data(G_OBJECT(ed->arrowpos[i]), "dots_check", chk);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chk), 
+				     (spec->arrows[i].flags & GP_ARROW_DOTS)?
+				     TRUE : FALSE);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), chk, 2, 3, r-1, r);
+	r++;	
+
+	if (i < ed->gui_narrows - 1) {
+	    /* separator */
+	    sep = gtk_hseparator_new();
+	    gtk_table_attach_defaults(GTK_TABLE(tbl), sep, 0, 3, r-1, r);
+	    r++;
+	}
+    }
+
+    if (spec->n_arrows < ARROW_MAX) {
+	/* button for adding an arrow */
+	GtkWidget *button = gtk_button_new_with_label(_("Add..."));
+
+	g_signal_connect(G_OBJECT(button), "clicked", 
+			 G_CALLBACK(add_arrow_callback), ed);
+	gtk_table_attach_defaults(GTK_TABLE(tbl), button, 0, 1, r-1, r);
+    }
+
+    gtk_widget_show_all(tbl);
+}
+
 static void gpt_tab_XY (plot_editor *ed, GPT_SPEC *spec, gint axis) 
 {
     png_plot *plot = (png_plot *) spec->ptr;
@@ -3004,8 +3276,8 @@ static void gpt_tab_XY (plot_editor *ed, GPT_SPEC *spec, gint axis)
 				  tbl_len-1, tbl_len);
 	gtk_widget_show(b2);    
 
-	combo = gtk_combo_box_entry_new_text();
-	set_combo_box_strings_from_list(GTK_COMBO_BOX(combo), strs);
+	combo = combo_box_text_new_with_entry();
+	set_combo_box_strings_from_list(combo, strs);
 	g_list_free(strs);
 	gtk_table_attach_defaults(GTK_TABLE(tbl), combo, 1, 2, 
 				  tbl_len-1, tbl_len);
@@ -3023,6 +3295,7 @@ static void gpt_tab_XY (plot_editor *ed, GPT_SPEC *spec, gint axis)
 	    g_free(txt);
 	    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b2), TRUE);
 	} else {
+	    gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
 	    gtk_widget_set_sensitive(entry, FALSE);
 	}
 
@@ -3066,6 +3339,13 @@ static void plot_editor_destroy (plot_editor *ed)
 	ed->spec->n_labels = ed->old_n_labels;
     }  
 
+    if (restore_arrows(ed)) {
+	/* undo any unapplied changes to arrows */
+	free(ed->spec->arrows);
+	ed->spec->arrows = ed->old_arrows;
+	ed->spec->n_arrows = ed->old_n_arrows;
+    }     
+
     if (ed->barsfile != NULL) {
 	remember_user_bars_file(ed->barsfile);
 	g_free(ed->barsfile);
@@ -3083,6 +3363,8 @@ static void plot_editor_destroy (plot_editor *ed)
     free(ed->labelpos);
     free(ed->labeljust);
 
+    free(ed->arrowpos);
+
     free(ed);
 }
 
@@ -3193,6 +3475,34 @@ static int allocate_label_widgets (plot_editor *ed, int n)
     return err;
 }
 
+static int add_arrow_widget (plot_editor *ed)
+{
+    int n = ed->gui_narrows + 1;
+    int err = 0;
+
+    ed->arrowpos = widget_array_expand(&ed->arrowpos, n, &err);
+    
+    if (!err) {
+	ed->gui_narrows = n;
+    }
+
+    return err;
+}
+
+static int allocate_arrow_widgets (plot_editor *ed, int n)
+{
+    int err = 0;
+
+    if (n > 0) {
+	ed->arrowpos = widget_array_new(n, &err);
+	if (!err) {
+	    ed->gui_narrows = n;
+	}
+    }
+
+    return err;
+}
+
 static plot_editor *plot_editor_new (GPT_SPEC *spec)
 {
     plot_editor *ed;
@@ -3220,6 +3530,8 @@ static plot_editor *plot_editor_new (GPT_SPEC *spec)
     ed->labelpos = NULL;
     ed->labeljust = NULL;
 
+    ed->arrowpos = NULL;
+
     /* also set to NULL the widgets which may or may not
        actually be used in constructing the plot editor 
        dialog 
@@ -3237,7 +3549,7 @@ static plot_editor *plot_editor_new (GPT_SPEC *spec)
     ed->fontcheck = NULL;
     ed->barscombo = NULL;
 
-    ed->gui_nlines = ed->gui_nlabels = 0;
+    ed->gui_nlines = ed->gui_nlabels = ed->gui_narrows = 0;
 
     if (allocate_line_widgets(ed, spec->n_lines)) {
 	plot_editor_destroy(ed);
@@ -3249,6 +3561,11 @@ static plot_editor *plot_editor_new (GPT_SPEC *spec)
 	return NULL;
     }   
 
+    if (allocate_arrow_widgets(ed, spec->n_arrows)) {
+	plot_editor_destroy(ed);
+	return NULL;
+    }     
+
     for (i=0; i<MAX_AXES; i++) {
 	ed->axis_range[i].isauto = NULL;
 	ed->axis_range[i].lbase = NULL;
@@ -3263,6 +3580,7 @@ static plot_editor *plot_editor_new (GPT_SPEC *spec)
 
     old_lines_init(ed);
     old_labels_init(ed);
+    old_arrows_init(ed);
 
     return ed;
 } 
@@ -3285,7 +3603,9 @@ GtkWidget *plot_add_editor (png_plot *plot)
 			      GRETL_DLG_RESIZE);
     vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
     gtk_box_set_spacing(GTK_BOX(vbox), 2);
+#if GTK_MAJOR_VERSION < 3
     gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
+#endif
     
     if (plot != NULL) {
 	gtk_window_set_transient_for(GTK_WINDOW(dialog), 
@@ -3315,6 +3635,7 @@ GtkWidget *plot_add_editor (png_plot *plot)
     }
 
     gpt_tab_labels(editor, spec, 0);
+    gpt_tab_arrows(editor, spec, 0);
 
     if (!frequency_plot_code(spec->code)) {
 	gpt_tab_palette(notebook);
