@@ -24,6 +24,7 @@
 #include "gretl_func.h"
 #include "gretl_xml.h"
 #include "libset.h"
+#include "gretl_bundle.h"
 
 #include "treeutils.h"
 #include "ssheet.h"
@@ -98,9 +99,6 @@ mdata_handle_drag  (GtkWidget          *widget,
 
 static char *optrun, *optdb, *optwebdb, *optpkg;
 static int opteng, optbasque, optdump, optver;
-#ifndef OLD_GTK
-static int optswitch;
-#endif
 #ifdef G_OS_WIN32
 static int optdebug;
 #endif
@@ -209,8 +207,6 @@ static GOptionEntry options[] = {
 #endif
     { "version", 'v', 0, G_OPTION_ARG_NONE, &optver, 
       N_("print version information"), NULL }, 
-    { "switch", 's', 0, G_OPTION_ARG_INT, &optswitch,
-      N_("pass integer value to script"), "value" },
     { NULL, '\0', 0, 0, NULL, NULL, NULL },
 };
 
@@ -323,7 +319,7 @@ static void get_runfile (char *fname)
     strncat(tryfile, fname, MAXLEN - 1);
 #endif
 
-    if (addpath(tryfile, 1) == NULL) {
+    if (gretl_addpath(tryfile, 1) == NULL) {
 	fprintf(stderr, I_("Couldn't find script '%s'\n"), tryfile);
 	exit(EXIT_FAILURE);
     } else {
@@ -473,7 +469,7 @@ static void record_filearg (char *targ, const char *src)
 #endif
 
 /* callback from within potentially lengthy libgretl
-   operations: try to avoid haveing the GUI become
+   operations: try to avoid having the GUI become
    totally unresponsive
 */
 
@@ -522,9 +518,11 @@ int main (int argc, char **argv)
 	g_print("%s\n", opterr->message);
 	exit(EXIT_FAILURE);
     }
-    if (optswitch) {
-	set_script_switch(optswitch);
-    }
+#endif
+
+#ifdef G_OS_WIN32
+    /* let's call this before doing libgretl_init */
+    gretl_win32_debug_init(optdebug);
 #endif
 
     libgretl_init();
@@ -1064,13 +1062,10 @@ void mdata_select_last_var (void)
     GtkTreeIter iter, last;
     GtkTreeView *view;
     GtkTreeModel *model;
-    GtkTreeStore *store;
     GtkTreeSelection *select;
 
     view = GTK_TREE_VIEW(mdata->listbox);
     model = gtk_tree_view_get_model(view);
-    store = GTK_TREE_STORE(model);
-
     gtk_tree_model_get_iter_first(model, &iter);
 
     while (1) {
@@ -1332,7 +1327,7 @@ static GtkWidget *make_main_window (void)
 
 static void iconview_callback (void)
 {
-    view_session(NULL);
+    view_session();
 }
 
 GtkActionEntry main_entries[] = {
@@ -1530,6 +1525,7 @@ GtkActionEntry main_entries[] = {
     { "Gini", NULL, N_("_Gini coefficient"), NULL, NULL, G_CALLBACK(do_gini) },
     { "rmplot", NULL, N_("_Range-mean graph"), NULL, NULL, G_CALLBACK(do_range_mean) },
     { "VarTSPlot", NULL, N_("_Time series plot"), NULL, NULL, G_CALLBACK(ts_plot_callback) },
+    { "PanPlot", NULL, N_("_Panel plot..."), NULL, NULL, G_CALLBACK(ts_plot_callback) },
     { "URTests", NULL, N_("_Unit root tests"), NULL, NULL, NULL },
     { "adf", NULL, N_("_Augmented Dickey-Fuller test"), NULL, NULL, G_CALLBACK(ur_callback) },
     { "dfgls", NULL, N_("ADF-GLS test"), NULL, NULL, G_CALLBACK(ur_callback) },
@@ -1576,7 +1572,6 @@ GtkActionEntry main_entries[] = {
     { "PWE", NULL, N_("_Prais-Winsten..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "ar", NULL, N_("_Autoregressive estimation..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "arima", NULL, N_("ARI_MA..."), NULL, NULL, G_CALLBACK(model_callback) },
-    { "arch", NULL, N_("_ARCH..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "garch", NULL, N_("_GARCH..."), NULL, NULL, G_CALLBACK(model_callback) },
     { "var", NULL, N_("_Vector Autoregression..."), NULL, NULL, G_CALLBACK(selector_callback) },
     { "VLAGSEL", NULL, N_("VAR _lag selection..."), NULL, NULL, G_CALLBACK(selector_callback) },
@@ -1671,10 +1666,7 @@ static void add_conditional_items (windata_t *vwin)
 			      FALSE);
     }
 
-    /* not public yet */
-    if (getenv("GRETL_TEST_GFN") != NULL) {
-	maybe_add_packages_to_menus(vwin);
-    }
+    maybe_add_packages_to_menus(vwin);
 }
 
 /* retrieve the XML description of the main window menus */
