@@ -58,6 +58,7 @@ enum extra_cmds {
     EDIT_R,
     EDIT_OX,
     EDIT_OCTAVE,
+    EDIT_X12A,
     CLI_HELP,
     GUI_HELP,
     CLI_HELP_EN,
@@ -108,6 +109,7 @@ enum extra_cmds {
     COUNTMOD,
     BWFILTER,
     POLYWEIGHTS,
+    X12AHELP,
     GUI_CMD_MAX
 };
 
@@ -174,6 +176,7 @@ enum file_ops {
     SAVE_BOOT_DATA,
     SAVE_MARKERS,
     SAVE_LABELS,
+    SAVE_GFN_SPEC,
     END_SAVE_OTHER, /* marker for end of other user-file saving */
     EDIT_FUNCTIONS,
     SET_PROG,
