@@ -537,9 +537,9 @@ static void
 gtk_fontsel_hack_display_fonts (GtkFontselHack *fontsel)
 {
     /* Insert the (possibly revised) list of fonts. */
-    gtk_fontsel_hack_show_available_fonts (fontsel, HACK_REGULAR);
-    gtk_fontsel_hack_show_available_styles (fontsel);
-    gtk_fontsel_hack_show_available_sizes (fontsel, TRUE);
+    gtk_fontsel_hack_show_available_fonts(fontsel, HACK_REGULAR);
+    gtk_fontsel_hack_show_available_styles(fontsel);
+    gtk_fontsel_hack_show_available_sizes(fontsel, TRUE);
 }
 
 GtkWidget *
@@ -555,12 +555,8 @@ gtk_fontsel_hack_new (void)
 static void
 gtk_fontsel_hack_finalize (GObject *object)
 {
-    GtkFontselHack *fontsel;
-  
     g_return_if_fail(GTK_IS_FNTHACK(object));
   
-    fontsel = GTK_FNTHACK(object);
-
     (* G_OBJECT_CLASS (gtk_fontsel_hack_parent_class)->finalize) (object);
 }
 
@@ -570,10 +566,10 @@ gtk_fontsel_hack_screen_changed (GtkWidget *widget,
 {
     GtkFontselHack *fontsel = GTK_FNTHACK(widget);
 
-    if (gtk_widget_has_screen (GTK_WIDGET (fontsel))) {
-	gtk_fontsel_hack_show_available_fonts (fontsel, HACK_INIT);
-	gtk_fontsel_hack_show_available_sizes (fontsel, TRUE);
-	gtk_fontsel_hack_show_available_styles (fontsel);
+    if (gtk_widget_has_screen(GTK_WIDGET(fontsel))) {
+	gtk_fontsel_hack_show_available_fonts(fontsel, HACK_INIT);
+	gtk_fontsel_hack_show_available_sizes(fontsel, TRUE);
+	gtk_fontsel_hack_show_available_styles(fontsel);
     }
 }
 
@@ -641,7 +637,6 @@ gtk_fontsel_hack_select_font (GtkTreeSelection *selection,
     GtkFontselHack *fontsel;
     GtkTreeModel *model;
     GtkTreeIter iter;
-    const gchar *family_name;
   
     fontsel = GTK_FNTHACK (data);
 
@@ -651,7 +646,6 @@ gtk_fontsel_hack_select_font (GtkTreeSelection *selection,
 	gtk_tree_model_get (model, &iter, FAMILY_COLUMN, &family, -1);
 	if (fontsel->family != family) {
 	    fontsel->family = family;
-	    family_name = pango_font_family_get_name (fontsel->family);
 	    gtk_fontsel_hack_show_available_styles (fontsel);
 	    gtk_fontsel_hack_select_best_style (fontsel, TRUE);
 	}
@@ -695,7 +689,7 @@ static int create_font_test_rig (void)
 static void destroy_font_test_rig (void)
 {
     g_object_unref(G_OBJECT(font_test_context));
-    gtk_widget_destroy(font_test_widget);    
+    gtk_widget_destroy(font_test_widget); 
 }
 
 enum {
@@ -1074,12 +1068,10 @@ gtk_fontsel_hack_show_available_sizes (GtkFontselHack *fontsel,
 {
     size_t i;
     GtkListStore *model;
-    GtkTreeSelection *selection;
     gchar buffer[128];
     gchar *p;
       
     model = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (fontsel->size_list)));
-    selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (fontsel->size_list));
 
     /* Insert the standard font sizes */
     if (first_time) {
@@ -1110,6 +1102,7 @@ gtk_fontsel_hack_show_available_sizes (GtkFontselHack *fontsel,
 
 	if (!found) {
 	    GtkTreeSelection *selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (fontsel->size_list));
+
 	    gtk_tree_selection_unselect_all (selection);
 	}
     }
@@ -1225,19 +1218,24 @@ gtk_fontsel_hack_get_font_description (GtkFontselHack *fontsel)
 static void
 gtk_fontsel_hack_update_preview (GtkFontselHack *fontsel)
 {
+#if GTK_MAJOR_VERSION < 3
     GtkRcStyle *rc_style;
+#endif
     gint new_height;
     GtkRequisition req, old_requisition;
     GtkWidget *preview_entry = fontsel->preview_entry;
     const gchar *text;
 
     gtk_widget_get_child_requisition(preview_entry, &old_requisition);
-  
+
+#if GTK_MAJOR_VERSION >= 3
+    gtk_widget_modify_font(preview_entry, gtk_fontsel_hack_get_font_description(fontsel));
+#else  
     rc_style = gtk_rc_style_new();
     rc_style->font_desc = gtk_fontsel_hack_get_font_description(fontsel);
-  
     gtk_widget_modify_style(preview_entry, rc_style);
     g_object_unref(rc_style);
+#endif
 
     gtk_widget_size_request(preview_entry, NULL);
 
@@ -1309,21 +1307,20 @@ gtk_fontsel_hack_set_font_name (GtkFontselHack *fontsel,
     model = gtk_tree_view_get_model (GTK_TREE_VIEW (fontsel->family_list));
     for (valid = gtk_tree_model_get_iter_first (model, &iter);
 	 valid;
-	 valid = gtk_tree_model_iter_next (model, &iter))
-	{
-	    PangoFontFamily *family;
+	 valid = gtk_tree_model_iter_next (model, &iter)) {
+	PangoFontFamily *family;
       
-	    gtk_tree_model_get (model, &iter, FAMILY_COLUMN, &family, -1);
+	gtk_tree_model_get (model, &iter, FAMILY_COLUMN, &family, -1);
       
-	    if (g_ascii_strcasecmp (pango_font_family_get_name (family),
-				    pango_font_description_get_family (new_desc)) == 0)
-		new_family = family;
+	if (g_ascii_strcasecmp (pango_font_family_get_name (family),
+				pango_font_description_get_family (new_desc)) == 0)
+	    new_family = family;
       
-	    g_object_unref (family);
+	g_object_unref (family);
       
-	    if (new_family)
-		break;
-	}
+	if (new_family)
+	    break;
+    }
 
     if (!new_family)
 	return FALSE;
@@ -1531,9 +1528,9 @@ gtk_fontsel_hack_dialog_init (GtkFontselHackDialog *fontseldiag)
     gtk_widget_grab_default (fontseldiag->ok_button);
   
     gtk_window_set_title (GTK_WINDOW (fontseldiag), _("Font Selection"));
-
+#if GTK_MAJOR_VERSION < 3
     gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
-  
+#endif  
     gtk_widget_pop_composite_child ();
 }
 
