@@ -24,7 +24,8 @@
 				r == EDIT_GP ||	    \
 				r == EDIT_R ||	    \
 				r == EDIT_OX ||     \
-                                r == EDIT_OCTAVE)
+                                r == EDIT_OCTAVE || \
+				r == EDIT_X12A)
 
 #define vwin_editing_buffer(r) (r == EDIT_HEADER || \
 	                        r == EDIT_NOTES)
