@@ -551,13 +551,10 @@ static void begin_image_print (GtkPrintOperation *op,
 			       GtkPrintContext *context,
 			       char *pngname)
 {
-    GtkPageSetup *setup;
     cairo_surface_t *cs;
     cairo_t *cr;
     gdouble x, y;
 
-    setup = gtk_print_context_get_page_setup(context);
-
     x = gtk_print_context_get_width(context);
     y = gtk_print_context_get_height(context);
 
@@ -1864,8 +1861,8 @@ static int data_to_buf_as_rtf (const int *list, PRN *prn)
     int err;
 
     gretl_print_set_format(prn, GRETL_FORMAT_RTF);
-    err = print_data_sorted(list, NULL, (const double **) Z, 
-			    datainfo, prn);
+    err = print_data_in_columns(list, NULL, (const double **) Z, 
+				datainfo, prn);
     return err;
 }
 
@@ -1874,8 +1871,8 @@ static int data_to_buf_as_csv (const int *list, PRN *prn)
     int err;
 
     gretl_print_set_format(prn, GRETL_FORMAT_CSV);
-    err = print_data_sorted(list, NULL, (const double **) Z, 
-			    datainfo, prn);
+    err = print_data_in_columns(list, NULL, (const double **) Z, 
+				datainfo, prn);
     return err;
 }
 
@@ -1890,7 +1887,12 @@ static int real_csv_to_clipboard (const int *list)
 
     err = data_to_buf_as_csv(list, prn);
     if (!err) {
-	prn_to_clipboard(prn, GRETL_FORMAT_CSV);
+	err = prn_to_clipboard(prn, GRETL_FORMAT_CSV);
+	if (err) {
+	    fprintf(stderr, "prn_to_clipboard: err = %d\n", err);
+	}
+    } else {
+	fprintf(stderr, "data_to_buf_as_csv: err = %d\n", err);
     }
 
     gretl_print_destroy(prn);
