@@ -143,6 +143,7 @@ static struct gui_help_item gui_help_items[] = {
     { COUNTMOD,       "count-model" },
     { BWFILTER,       "bwfilter" },
     { POLYWEIGHTS,    "polyweights" },
+    { X12AHELP,       "x12a" },
     { -1,             NULL },
 };
 
@@ -958,10 +959,10 @@ static void toggle_search_all_help (GtkComboBox *box, GtkWidget *entry)
 
 static void finder_add_options (GtkWidget *hbox, GtkWidget *entry)
 {
-    GtkWidget *combo = gtk_combo_box_new_text();
+    GtkWidget *combo = gtk_combo_box_text_new();
 
-    gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("this page"));
-    gtk_combo_box_append_text(GTK_COMBO_BOX(combo), _("all pages"));
+    combo_box_append_text(combo, _("this page"));
+    combo_box_append_text(combo, _("all pages"));
     gtk_box_pack_end(GTK_BOX(hbox), combo, FALSE, FALSE, 5);
     gtk_combo_box_set_active(GTK_COMBO_BOX(combo), 0);
 
@@ -1901,7 +1902,7 @@ enum {
     GRETL_GUIDE = 1,
     GRETL_REF,
     GNUPLOT_REF,
-    GFN_DOC
+    X12A_REF
 };
 
 static int get_writable_path (char *path, const char *fname, int code)
@@ -1914,11 +1915,7 @@ static int get_writable_path (char *path, const char *fname, int code)
     int err = 0;
 
     if (sysdoc_writable == 1) {
-	if (code == GFN_DOC) {
-	    sprintf(path, "%sfunctions%cdocs%c%s", gretldir, SLASH, SLASH, fname);
-	} else {
-	    sprintf(path, "%sdoc%c%s", gretldir, SLASH, fname);
-	}
+	sprintf(path, "%sdoc%c%s", gretldir, SLASH, fname);
 	return 0;
     } else if (userdoc_writable == 1) {
 	sprintf(path, "%sdoc%c%s", dotdir, SLASH, fname);
@@ -1927,11 +1924,7 @@ static int get_writable_path (char *path, const char *fname, int code)
 
     if (sysdoc_writable < 0) {
 	sysdoc_writable = 0;
-	if (code == GFN_DOC) {
-	    sprintf(path, "%sfunctions%cdocs", gretldir, SLASH);
-	} else {
-	    sprintf(path, "%sdoc", gretldir);
-	}
+	sprintf(path, "%sdoc", gretldir);
 	if (gretl_mkdir(path) == 0) {
 	    strcat(path, SLASHSTR);
 	    strcat(path, fname);
@@ -1966,6 +1959,37 @@ static int get_writable_path (char *path, const char *fname, int code)
     return err;
 }
 
+static int get_x12a_doc_path (char *path, const char *fname)
+{
+    const char *x12a = gretl_x12_arima();
+    int ret = 0;
+
+    *path = '\0';
+
+    if (x12a != NULL && *x12a != '\0') {
+	char *p;
+
+	strcpy(path, x12a);
+	p = strrchr(path, SLASH);
+	if (p != NULL) {
+	    sprintf(p + 1, "docs%c%s", SLASH, fname);
+	    ret = 1;
+	} else {
+	    *path = '\0';
+	}
+
+#if !defined(G_OS_WIN32) && !defined(OSX_BUILD)
+	if (!ret) {
+	    /* using gretl x12a package? */
+	    sprintf(path, "/opt/x12arima/docs/%s", fname);
+	    ret = 1;
+	}
+#endif
+    }
+
+    return ret;
+}
+
 static int find_or_download_pdf (int code, int i, char *fullpath)
 {
     const char *guide_files[] = {
@@ -1996,26 +2020,32 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 	fname = ref_files[i];
     } else if (code == GNUPLOT_REF) {
 	fname = "gnuplot.pdf";
-    } else if (code == GFN_DOC) {
-	tmp = gretl_strdup(fullpath);
-	fname = tmp;
+    } else if (code == X12A_REF) {
+	fname = "x12adocV03.pdf";
     } else {
 	return E_DATA;
     }
 
     /* is the file available in public dir? */
-    if (code == GFN_DOC) {
-	sprintf(fullpath, "%sfunctions%cdocs%c%s", gretl_home(), SLASH, 
-		SLASH, fname);
-    } else {
-	sprintf(fullpath, "%sdoc%c%s", gretl_home(), SLASH, fname);
-    }
+    sprintf(fullpath, "%sdoc%c%s", gretl_home(), SLASH, fname);
+
     fp = gretl_fopen(fullpath, "r");
     if (fp != NULL) {
 	fclose(fp);
 	gotit = 1;
     }
 
+    if (!gotit && code == X12A_REF) {
+	get_x12a_doc_path(fullpath, fname);
+	if (*fullpath != '\0') {
+	    fp = gretl_fopen(fullpath, "r");
+	    if (fp != NULL) {
+		fclose(fp);
+		gotit = 1;
+	    }
+	}
+    }	
+
     if (!gotit) {
 	/* or maybe in user dir? */
 	sprintf(fullpath, "%sdoc%c%s", gretl_dotdir(), SLASH, fname);
@@ -2032,11 +2062,7 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
 
 	/* do actual download */
 	if (!err) {
-	    if (code == GFN_DOC) {
-		err = retrieve_gfndoc(fname, fullpath);
-	    } else {
-		err = retrieve_manfile(fname, fullpath);
-	    }
+	    err = retrieve_manfile(fname, fullpath);
 	}
 
 	if (err) {
@@ -2055,7 +2081,7 @@ static int find_or_download_pdf (int code, int i, char *fullpath)
     return err;
 }
 
-static void show_pdf (const char *fname)
+void gretl_show_pdf (const char *fname)
 {
 #if defined(G_OS_WIN32)
     win32_open_file(fname);
@@ -2081,7 +2107,7 @@ void display_pdf_help (GtkAction *action)
     err = find_or_download_pdf(code, get_manpref(), fname);
 
     if (!err) {
-	show_pdf(fname);
+	gretl_show_pdf(fname);
     }
 }
 
@@ -2093,21 +2119,19 @@ void display_gnuplot_help (void)
     err = find_or_download_pdf(GNUPLOT_REF, 0, fname);
 
     if (!err) {
-	show_pdf(fname);
+	gretl_show_pdf(fname);
     }
 }
 
-int display_gfn_help (const char *pdfname)
+void display_x12a_help (void)
 {
     char fname[FILENAME_MAX];
     int err;
 
-    strcpy(fname, pdfname);
-    err = find_or_download_pdf(GFN_DOC, 0, fname);
+    err = find_or_download_pdf(X12A_REF, 0, fname);
 
     if (!err) {
-	show_pdf(fname);
+	gretl_show_pdf(fname);
     }
-
-    return err;
 }
+
