@@ -195,6 +195,7 @@ void time_series_menu_state (gboolean s)
 {
     gboolean sx = extended_ts(datainfo);
     gboolean panel = dataset_is_panel(datainfo);
+    gboolean realpan = multi_unit_panel_sample(datainfo);
     gboolean ur = s;
 
     if (mdata->ui == NULL) {
@@ -211,7 +212,8 @@ void time_series_menu_state (gboolean s)
     /* Plots */
     flip(mdata->ui, "/menubar/View/GraphVars/TSPlot", sx);
     flip(mdata->ui, "/menubar/View/MultiPlots/MultiTS", sx);
-    flip(mdata->ui, "/menubar/Variable/VarTSPlot", sx);
+    flip(mdata->ui, "/menubar/Variable/VarTSPlot", sx && !realpan);
+    flip(mdata->ui, "/menubar/Variable/PanPlot", realpan);
 
     /* Variable menu */
     flip(mdata->ui, "/menubar/Variable/URTests", ur); 
@@ -246,7 +248,7 @@ void panel_menu_state (gboolean s)
 	flip(mdata->ui, "/menubar/Add/TimeDums", s);
 	flip(mdata->ui, "/menubar/Model/PanelModels", s);
 	if (s && datainfo->pd <= 2) {
-	    flip(mdata->ui, "/menubar/Model/PanelModels/arbond", 0);
+	    flip(mdata->ui, "/menubar/Model/PanelModels/dpanel", 0);
 	}
     }
 }
@@ -319,7 +321,7 @@ void check_var_labels_state (GtkMenuItem *item, gpointer p)
 {
     gboolean s = FALSE;
 
-    if (datainfo != NULL) {
+    if (datainfo != NULL && datainfo->v > 0) {
 	if (datainfo->v > 2 || strcmp(datainfo->varname[1], "index")) {
 	    s = TRUE;
 	}
@@ -339,10 +341,12 @@ static gint var_popup_click (GtkWidget *w, gpointer p)
 	do_menu_op(VAR_SUMMARY, NULL, OPT_NONE);
     else if (!strcmp(item, _("Time series plot"))) 
 	do_graph_var(v);
+    else if (!strcmp(item, _("Panel plot..."))) 
+	do_graph_var(v);
     else if (!strcmp(item, _("Frequency distribution"))) 
 	do_freq_dist();
     else if (!strcmp(item, _("Boxplot")))
-	do_boxplot_var(v);
+	do_boxplot_var(v, OPT_NONE);
     else if (!strcmp(item, _("Gini coefficient")))
 	do_gini();
     else if (!strcmp(item, _("Correlogram")))
@@ -427,6 +431,7 @@ GtkWidget *build_var_popup (void)
 	N_("Display values"),
 	N_("Descriptive statistics"),
 	N_("Time series plot"),
+	N_("Panel plot..."),
 	N_("Frequency distribution"),
 	N_("Boxplot"),
 	N_("Correlogram"),
@@ -441,6 +446,7 @@ GtkWidget *build_var_popup (void)
     GtkWidget *menu;
     GtkWidget *item;
     int i, n = G_N_ELEMENTS(items);
+    int real_panel = multi_unit_panel_sample(datainfo);
 
     menu = gtk_menu_new();
 
@@ -451,7 +457,15 @@ GtkWidget *build_var_popup (void)
 	    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
 	    continue;
 	}
-	if ((i == 5 || i == 6) && !dataset_is_time_series(datainfo)) {
+	if (real_panel && (i == 2 || i == 5)) {
+	    /* don't offer regular ts or boxplot */
+	    continue;
+	}
+	if (!real_panel && i == 3) {
+	    /* don't offer panel plot */
+	    continue;
+	}	
+	if ((i == 6 || i == 7) && !dataset_is_time_series(datainfo)) {
 	    continue;
 	}
 	if (i == 2 && !extended_ts(datainfo)) {
