@@ -257,8 +257,8 @@ static void multi_series_view_print_sorted (windata_t *vwin)
 	return;
     }
 
-    err = print_data_sorted(sview->list, obsvec, (const double **) Z, 
-			    datainfo, prn);
+    err = print_data_in_columns(sview->list, obsvec, (const double **) Z, 
+				datainfo, prn);
     if (err) {
 	gui_errmsg(err);
     } else {
@@ -333,8 +333,8 @@ PRN *vwin_print_sorted_with_format (windata_t *vwin, PrnFormat fmt)
     }
 
     gretl_print_set_format(prn, fmt);
-    err = print_data_sorted(sview->list, obsvec, (const double **) Z, 
-			    datainfo, prn);
+    err = print_data_in_columns(sview->list, obsvec, (const double **) Z, 
+				datainfo, prn);
     if (err) {
 	gui_errmsg(err);
 	gretl_print_destroy(prn);
@@ -442,7 +442,7 @@ void series_view_graph (GtkWidget *w, windata_t *vwin)
     if (dataset_is_time_series(datainfo)) {
 	do_graph_var(sview->varnum);
     } else {
-	do_boxplot_var(sview->varnum);
+	do_boxplot_var(sview->varnum, OPT_NONE);
     }
 }
 
@@ -660,9 +660,9 @@ static void real_view_format_dialog (GtkWidget *src, windata_t *vwin,
     gtk_box_pack_start(GTK_BOX(hbox), vt.spin, FALSE, FALSE, 0);
 
     /* and selector for digits / decimal places */
-    vt.combo = gtk_combo_box_new_text();
-    gtk_combo_box_append_text(GTK_COMBO_BOX(vt.combo), _("significant figures"));
-    gtk_combo_box_append_text(GTK_COMBO_BOX(vt.combo), _("decimal places"));
+    vt.combo = gtk_combo_box_text_new();
+    combo_box_append_text(vt.combo, _("significant figures"));
+    combo_box_append_text(vt.combo, _("decimal places"));
     if (sview->format == 'g') {
 	gtk_combo_box_set_active(GTK_COMBO_BOX(vt.combo), 0);
     } else {
