@@ -36,6 +36,7 @@
 #include "menustate.h"
 #include "toolbar.h"
 #include "winstack.h"
+#include "gretl_bundle.h"
 #include "fncall.h"
 #include "lib_private.h"
 
@@ -48,6 +49,7 @@
 #include "matrix_extra.h"
 #include "cmd_private.h"
 #include "gretl_scalar.h"
+#include "gretl_bundle.h"
 
 #include <sys/stat.h>
 #include <unistd.h>
@@ -70,6 +72,7 @@
 #include "../pixmaps/model_table.xpm"
 #include "../pixmaps/graph_page.xpm"
 #include "../pixmaps/matrix.xpm"
+#include "../pixmaps/bundle.xpm"
 
 #define SESSION_DEBUG 0
 
@@ -205,6 +208,12 @@ static char *matrix_items[] = {
     N_("Delete")
 };
 
+static char *bundle_items[] = {
+    N_("View"),
+    N_("Rename"),
+    N_("Delete")
+};
+
 /* file-scope globals */
 
 SESSION session;            /* hold models, graphs */
@@ -223,6 +232,7 @@ static GtkWidget *data_popup;
 static GtkWidget *scalars_popup;
 static GtkWidget *info_popup;
 static GtkWidget *matrix_popup;
+static GtkWidget *bundle_popup;
 static GtkWidget *save_item;
 
 static GList *iconlist;
@@ -241,6 +251,7 @@ static void data_popup_callback (GtkWidget *widget, gpointer data);
 static void scalars_popup_callback (GtkWidget *widget, gpointer data);
 static void info_popup_callback (GtkWidget *widget, gpointer data);
 static void matrix_popup_callback (GtkWidget *widget, gpointer data);
+static void bundle_popup_callback (GtkWidget *widget, gpointer data);
 static void session_delete_icon (gui_obj *obj);
 static void open_gui_graph (gui_obj *obj);
 static gboolean session_view_click (GtkWidget *widget, 
@@ -250,6 +261,7 @@ static int real_delete_model_from_session (SESSION_MODEL *model);
 static void make_short_label_string (char *targ, const char *src);
 static gui_obj *get_gui_obj_by_data (void *finddata);
 static void add_user_matrix_callback (void);
+static void add_bundle_callback (void);
 
 static int session_graph_count;
 static int commands_recorded;
@@ -682,7 +694,7 @@ void save_output_as_text_icon (windata_t *vwin)
 
 	    session_add_icon(text, GRETL_OBJ_TEXT, ICON_ADD_SINGLE);
     } else if (autoicon_on()) {
-	view_session(NULL);
+	view_session();
     }
 }
 
@@ -704,10 +716,10 @@ static int add_model_to_session (void *ptr, const char *name,
     } else if (iconlist != NULL) {
 	session_add_icon(mod, type, ICON_ADD_SINGLE);
     } else if (autoicon_on()) {
-	view_session(NULL);
+	view_session();
     }
 
-    return 0;
+    return err;
 }
 
 static int replace_session_graph (SESSION_GRAPH *graph,
@@ -745,7 +757,7 @@ real_add_graph_to_session (const char *fname, const char *grname,
     if (iconlist != NULL) {
 	session_add_icon(graph, type, ICON_ADD_SINGLE);
     } else if (autoicon_on()) {
-	view_session(NULL);
+	view_session();
     }
 
     return ADD_OBJECT_OK;
@@ -1077,6 +1089,49 @@ void model_add_as_icon (GtkAction *action, gpointer p)
     } 	
 }
 
+void bundle_add_as_icon (GtkAction *action, gpointer p)
+{
+    windata_t *vwin = (windata_t *) p;
+    gretl_bundle *bundle = vwin->data;
+    char vname[VNAMELEN];
+    char *defname;
+    const char *blurb;
+    int *pshow, show = 0;
+    int resp;
+
+    defname = get_bundle_default_name();
+    strcpy(vname, defname);
+    free(defname);
+    blurb = "Save bundle\nName (max. 15 characters):";
+    pshow = (iconlist == NULL)? &show : NULL;
+    resp = object_name_entry_dialog(vname, GRETL_TYPE_BUNDLE, blurb, pshow);
+
+    if (resp >= 0) {    
+	int err = gretl_bundle_set_name(bundle, vname);
+	int flipit = 1;
+
+	if (err) {
+	    gui_errmsg(err);
+	} else {
+	    if (iconlist != NULL) {
+		session_add_icon(bundle, GRETL_OBJ_BUNDLE, ICON_ADD_SINGLE);
+	    } else if (autoicon_on() || show) {
+		view_session();
+	    }
+	    mark_session_changed();
+	    if (close_on_add(action) && !window_is_busy(vwin)) {
+		gtk_widget_destroy(vwin->main);
+		flipit = 0;
+	    }
+	}
+
+	if (flipit) {
+	    flip(vwin->ui, "/menubar/File/SaveAsIcon", FALSE);
+	    flip(vwin->ui, "/menubar/File/SaveAndClose", FALSE);
+	}
+    }   
+}
+
 static void
 session_name_from_session_file (char *sname, const char *fname)
 {
@@ -1253,7 +1308,7 @@ void do_open_session (void)
     }
 
     /* close existing session, if any, and initialize */
-    close_session(NULL, &Z, datainfo, OPT_NONE);
+    close_session(OPT_NONE);
 
     fprintf(stderr, I_("\nReading session file %s\n"), tryfile);
 
@@ -1331,6 +1386,9 @@ void do_open_session (void)
     session_file_make_path(fname, "matrices.xml");
     err = maybe_read_matrix_file(fname);
 
+    session_file_make_path(fname, "bundles.xml");
+    err = maybe_read_bundles_file(fname);
+
     session_file_make_path(fname, "scalars.xml");
     err = maybe_read_scalars_file(fname);
 
@@ -1384,7 +1442,7 @@ void do_open_session (void)
 	/* sync gui with session */
 	session_menu_state(TRUE);
 
-	view_session(NULL);
+	view_session();
 	mark_session_saved();
 	session_switch_log_location(LOG_OPEN);
     }
@@ -1403,7 +1461,7 @@ void verify_clear_data (void)
 	return;
     }
 
-    close_session(NULL, &Z, datainfo, OPT_NONE); /* FIXME opt */
+    close_session(OPT_NONE); /* FIXME opt */
 }
 
 static void remove_session_dir (void)
@@ -1434,11 +1492,14 @@ void session_init (void)
     session.nmodels = 0;
     session.ngraphs = 0;
     session.ntexts = 0;
+
     *session.name = '\0';
     *session.dirname = '\0';
 
     winstack_init();
-    set_matrix_add_callback(add_user_matrix_callback);    
+
+    set_matrix_add_callback(add_user_matrix_callback); 
+    set_bundle_add_callback(add_bundle_callback);
 }
 
 void free_session (void)
@@ -1563,12 +1624,11 @@ static void session_clear_data (double ***pZ, DATAINFO *pdinfo)
     clear_model(models[2]);
 
     free_command_stack(); 
-    reset_model_count();
+    set_model_count(0);
     lib_cmd_destroy_context();
 }
 
-void close_session (ExecState *s, double ***pZ, DATAINFO *pdinfo,
-		    gretlopt opt)
+void close_session (gretlopt opt)
 {
     int logcode = LOG_NULL;
 
@@ -1576,9 +1636,9 @@ void close_session (ExecState *s, double ***pZ, DATAINFO *pdinfo,
     fprintf(stderr, "close_session: starting cleanup\n");
 #endif
 
-    if (pdinfo != NULL && pdinfo->v > 0) {
+    if (datainfo != NULL && datainfo->v > 0) {
 	logcode = LOG_CLOSE;
-	session_clear_data(pZ, pdinfo); 
+	session_clear_data(&Z, datainfo); 
     }
 
     free_session();
@@ -1604,13 +1664,12 @@ void close_session (ExecState *s, double ***pZ, DATAINFO *pdinfo,
     edit_dialog_special_get_text(NULL);
 
     if (opt & OPT_P) {
-	libgretl_session_cleanup(SESSION_PRESERVE_MATRICES);
+	libgretl_session_cleanup(SESSION_CLEAR_DATASET);
     } else {
-	libgretl_session_cleanup(SESSION_CLEAR_FULL);
+	libgretl_session_cleanup(SESSION_CLEAR_ALL);
     }
 
     session_graph_count = 0;
-
     reset_plot_count();
 
     set_session_log(NULL, logcode);
@@ -1861,6 +1920,7 @@ int save_session_commands (char *fname)
 	    gui_errmsg(err);
 	} else {
 	    fputs(s, fp);
+	    g_free(s);
 	}
 
 	fclose(fp);
@@ -2010,9 +2070,30 @@ void session_model_callback (void *ptr, int action)
 
 static void open_matrix (gui_obj *obj)
 {
-    user_matrix *um = (user_matrix *) obj->data;
+    user_matrix *u = (user_matrix *) obj->data;
+    const char *name = user_matrix_get_name(u);
 
-    edit_user_matrix_by_name(user_matrix_get_name(um));
+    edit_user_matrix_by_name(name);
+}
+
+static void open_bundle (gui_obj *obj)
+{
+    gretl_bundle *b = (gretl_bundle *) obj->data;
+    const char *name = gretl_bundle_get_name(b);
+    PRN *prn = NULL;
+    int done = 0;
+
+    if (bufopen(&prn)) {
+	return;
+    }
+
+    done = try_exec_bundle_print_function(b, prn);
+
+    if (!done) {
+ 	gretl_bundle_print(b, prn);
+    }
+
+    view_buffer(prn, 80, 400, name, VIEW_BUNDLE, b);
 }
 
 static void open_gui_text (gui_obj *obj)
@@ -2156,6 +2237,8 @@ static int delete_session_object (gui_obj *obj)
 	real_delete_text_from_session(obj->data);
     } else if (obj->sort == GRETL_OBJ_MATRIX) { 
 	user_matrix_destroy(obj->data);
+    } else if (obj->sort == GRETL_OBJ_BUNDLE) {
+	gretl_bundle_delete_by_name(obj->name, NULL);
     }
 
     session_delete_icon(obj);
@@ -2229,6 +2312,28 @@ int session_matrix_destroy_by_name (const char *name, PRN *prn)
     return err;
 }
 
+int session_bundle_destroy_by_name (const char *name, PRN *prn)
+{
+    gretl_bundle *b = get_gretl_bundle_by_name(name);
+    int err;
+
+    if (b == NULL) {
+	err = E_UNKVAR;
+    } else if (winstack_match_data(b)) {
+	errbox(_("Please close this object's window first"));
+	return 0;
+    } else {
+	gui_obj *obj = get_gui_obj_by_data(b);
+
+	if (obj != NULL) {
+	    session_delete_icon(obj);
+	}
+	err = gretl_bundle_delete_by_name(name, NULL);
+    } 
+    
+    return err;
+}
+
 static void rename_session_graph (SESSION_GRAPH *graph, const char *newname)
 {
     int i;
@@ -2284,6 +2389,8 @@ static int rename_session_object (gui_obj *obj, const char *newname)
 	}
     } else if (obj->sort == GRETL_OBJ_MATRIX) {
 	user_matrix_set_name(obj->data, newname);
+    } else if (obj->sort == GRETL_OBJ_BUNDLE) {
+	gretl_bundle_set_name(obj->data, newname);
     } else if (obj->sort == GRETL_OBJ_TEXT) {
 	SESSION_TEXT *st;
 
@@ -2334,11 +2441,19 @@ static void rename_object_callback (GtkWidget *widget, dialog_t *dlg)
 
 static void rename_object_dialog (gui_obj *obj) 
 {
-    edit_dialog(_("gretl: rename object"), 
-		_("Enter new name\n"
-		  "(max. 31 characters)"),
+    int maxlen = MAXSAVENAME - 1;
+    gchar *tmp;
+
+    if (obj->sort == GRETL_OBJ_MATRIX || obj->sort == GRETL_OBJ_BUNDLE) {
+	maxlen = VNAMELEN - 1;
+    }
+
+    tmp = g_strdup_printf(_("Enter new name\n(max. %d characters)"),
+			  maxlen);
+    edit_dialog(_("gretl: rename object"), tmp,
 		obj->name, rename_object_callback, obj, 
 		0, VARCLICK_NONE, NULL);
+    g_free(tmp);
 }
 
 void delete_text_from_session (void *p)
@@ -2425,6 +2540,19 @@ static void session_delete_icon (gui_obj *obj)
 
 /* apparatus for getting a white background */
 
+#if GTK_MAJOR_VERSION >= 3
+
+static void white_bg_style (GtkWidget *widget, gpointer data)
+{
+    GdkRGBA rgb = {1, 1, 1, 1};
+
+    gtk_widget_override_background_color(widget,
+					 GTK_STATE_NORMAL,
+					 &rgb);
+}
+
+#else
+
 static GdkColor *get_white (void)
 {
     GdkColormap *cmap;
@@ -2451,6 +2579,8 @@ static void white_bg_style (GtkWidget *widget, gpointer data)
     gtk_widget_modify_bg(widget, GTK_STATE_NORMAL, white);
 }
 
+#endif
+
 static void real_pack_icon (gui_obj *obj, int row, int col)
 {
     obj->row = row;
@@ -2525,7 +2655,23 @@ static void add_user_matrix_callback (void)
 			     ICON_ADD_SINGLE);
 	}
     } else if (autoicon_on()) {
-	view_session(NULL);
+	view_session();
+    }
+
+    mark_session_changed();
+}
+
+static void add_bundle_callback (void)
+{
+    if (iconview != NULL) {
+	int n = n_user_bundles();
+
+	if (n > 0) {
+	    session_add_icon(get_gretl_bundle_by_index(n-1), GRETL_OBJ_BUNDLE, 
+			     ICON_ADD_SINGLE);
+	}
+    } else if (autoicon_on()) {
+	view_session();
     }
 
     mark_session_changed();
@@ -2582,6 +2728,12 @@ static void add_all_icons (void)
 			 ICON_ADD_BATCH);
     }
 
+    n = n_user_bundles();
+    for (i=0; i<n; i++) {
+	session_add_icon(get_gretl_bundle_by_index(i), GRETL_OBJ_BUNDLE, 
+			 ICON_ADD_BATCH);
+    }    
+
     batch_pack_icons();
 }
 
@@ -2648,6 +2800,9 @@ static void object_popup_show (gui_obj *obj, GdkEventButton *event)
     case GRETL_OBJ_MATRIX: 
 	w = matrix_popup; 
 	break;
+    case GRETL_OBJ_BUNDLE: 
+	w = bundle_popup; 
+	break;
     default: 
 	break;
     }
@@ -2713,6 +2868,9 @@ static gboolean session_view_click (GtkWidget *widget,
 	case GRETL_OBJ_MATRIX:
 	    open_matrix(obj); 
 	    break;
+	case GRETL_OBJ_BUNDLE:
+	    open_bundle(obj); 
+	    break;
 	case GRETL_OBJ_INFO:
 	    dataset_info(); 
 	    break;
@@ -2743,7 +2901,8 @@ static gboolean session_view_click (GtkWidget *widget,
 	    obj->sort == GRETL_OBJ_INFO || obj->sort == GRETL_OBJ_GPAGE ||
 	    obj->sort == GRETL_OBJ_PLOT || obj->sort == GRETL_OBJ_MODTAB ||
 	    obj->sort == GRETL_OBJ_VAR  || obj->sort == GRETL_OBJ_SYS ||
-	    obj->sort == GRETL_OBJ_MATRIX || obj->sort == GRETL_OBJ_SCALARS) {
+	    obj->sort == GRETL_OBJ_MATRIX || obj->sort == GRETL_OBJ_BUNDLE ||
+	    obj->sort == GRETL_OBJ_SCALARS) {
 	    object_popup_show(obj, (GdkEventButton *) event);
 	}
 	return TRUE;
@@ -2809,6 +2968,20 @@ static void matrix_popup_callback (GtkWidget *widget, gpointer data)
     }
 }
 
+static void bundle_popup_callback (GtkWidget *widget, gpointer data)
+{
+    gchar *item = (gchar *) data;
+    gui_obj *obj = active_object;
+
+    if (!strcmp(item, _("View"))) {
+	open_bundle(obj);
+    } else if (!strcmp(item, _("Rename"))) {
+	rename_object_dialog(obj);
+    } else if (!strcmp(item, _("Delete"))) {
+	maybe_delete_session_object(obj);
+    }
+}
+
 static void data_popup_callback (GtkWidget *widget, gpointer data)
 {
     gchar *item = (gchar *) data;
@@ -3030,7 +3203,6 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
 {
     gui_obj *obj;
     gchar *name = NULL;
-    user_matrix *um = NULL;
     SESSION_GRAPH *graph = NULL;
     SESSION_MODEL *mod = NULL;
     SESSION_TEXT *text = NULL;
@@ -3077,8 +3249,10 @@ static gui_obj *session_add_icon (gpointer data, int sort, int mode)
 	name = g_strdup(_("Graph page"));
 	break;
     case GRETL_OBJ_MATRIX:
-	um = (user_matrix *) data;
-	name = g_strdup(user_matrix_get_name(um));
+	name = g_strdup(user_matrix_get_name((user_matrix *) data));
+	break;
+    case GRETL_OBJ_BUNDLE:
+	name = g_strdup(gretl_bundle_get_name((gretl_bundle *) data));
 	break;
     default:
 	break;
@@ -3247,6 +3421,15 @@ static void session_build_popups (void)
 			    matrix_popup_callback);	    
 	}
     }
+
+    if (bundle_popup == NULL) {
+	bundle_popup = gtk_menu_new();
+	n = G_N_ELEMENTS(bundle_items);
+	for (i=0; i<n; i++) {
+	    create_pop_item(bundle_popup, _(bundle_items[i]), 
+			    bundle_popup_callback);	    
+	}
+    }
 }
 
 static gboolean 
@@ -3277,13 +3460,7 @@ static void iconview_connect_signals (GtkWidget *iconview)
 		     G_CALLBACK(iconview_resize_callback), NULL);
 }
 
-/* @parent: this should be non-NULL only if want to embed the "icon
-   view" of a session inside another widget, which is something that
-   I've experimented with, briefly, but is not an active option as of
-   this writing. AC, 2009-05-02
-*/
-
-void view_session (GtkWidget *parent)
+void view_session (void)
 {
     GtkWidget *hbox, *scroller;
     gchar *title;
@@ -3418,6 +3595,7 @@ static gui_obj *gui_object_new (gchar *name, int sort, gpointer data)
     case GRETL_OBJ_MODTAB:  xpm = model_table_xpm; break;
     case GRETL_OBJ_GPAGE:   xpm = graph_page_xpm;  break;
     case GRETL_OBJ_MATRIX:  xpm = matrix_xpm;      break;
+    case GRETL_OBJ_BUNDLE:  xpm = bundle_xpm;      break;
     default: break;
     }
 
@@ -3570,7 +3748,7 @@ view_matrix_properties (const gretl_matrix *m, const char *name)
     if (m->rows == m->cols) {
 	double det;
 
-	print_double_formatted(_("Trace"), gretl_matrix_trace(m, &err), prn);
+	print_double_formatted(_("Trace"), gretl_matrix_trace(m), prn);
 	if (A == NULL) {
 	    A = gretl_matrix_copy(m);
 	} else {
