@@ -78,10 +78,14 @@ int add_model_to_session_callback (void *ptr, GretlObjType type);
 
 void session_model_callback (void *ptr, int action);
 
+void bundle_add_as_icon (GtkAction *action, gpointer p);
+
 void *get_session_object_by_name (const char *name, GretlObjType *type);
 
 int session_matrix_destroy_by_name (const char *name, PRN *prn);
 
+int session_bundle_destroy_by_name (const char *name, PRN *prn);
+
 void delete_text_from_session (void *p);
 
 void display_saved_text (void *p);
@@ -96,8 +100,7 @@ void gui_clear_dataset (void);
 
 void verify_clear_data (void);
 
-void close_session (ExecState *s, double ***pZ, DATAINFO *pdinfo,
-		    gretlopt opt);
+void close_session (gretlopt opt);
 
 void free_session (void);
 
@@ -105,7 +108,7 @@ int highest_numbered_variable_in_session (void);
 
 int is_session_model (void *p);
 
-void view_session (GtkWidget *parent);
+void view_session (void);
 
 void save_session_callback (GtkAction *action);
 
