@@ -416,12 +416,32 @@ void set_app_font (const char *fontname)
     }
 }
 
+static int write_OK (gchar *dirname)
+{
+    int ok = 0;
+
+    if (gretl_mkdir(dirname) == 0) {
+	gchar  *target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
+
+	if (target != NULL) {
+	    FILE *fp = gretl_try_fopen(target, "w");
+
+	    if (fp != NULL) {
+		ok = 1;
+		fclose(fp);
+		gretl_remove(target);
+	    }
+	    g_free(target);
+	}
+    }
+
+    return ok;
+}
+
 static void get_pkg_dir (char *dirname, int action)
 {
     const char *subdir = NULL;
-    char *target = NULL;
-    FILE *fp;
-    int err, ok = 0;
+    int ok = 0;
 
     if (action == SAVE_FUNCTIONS) {
 	subdir = "functions";
@@ -434,49 +454,23 @@ static void get_pkg_dir (char *dirname, int action)
     /* try 'system' location first */
 
     sprintf(dirname, "%s%s", gretl_home(), subdir);
-    err = gretl_mkdir(dirname);
-    if (!err) {
-	target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
-	if (target != NULL) {
-	    fp = gretl_fopen(target, "w");
-	    if (fp != NULL) {
-		ok = 1;
-		fclose(fp);
-		gretl_remove(target);
-	    }
-	    free(target);
-	}
-    } 
-    
-    if (ok) return;
+    ok = write_OK(dirname);
 
-    /* try user's directory */
+    if (!ok) {
+	/* try user's filespace */
+	const char *wdir = gretl_default_workdir();
 
-    if (action == SAVE_DATA_PKG) {
-	strcpy(dirname, gretl_workdir());
-	target = g_strdup_printf("%s%s", dirname, "wtest");
-    } else {
-	/* should we really use dotdir here? */
-	sprintf(dirname, "%s%s", gretl_dotdir(), subdir);
-	err = gretl_mkdir(dirname);
-	if (!err) {
-	    target = g_strdup_printf("%s%c%s", dirname, SLASH, "wtest");
+	if (wdir == NULL) {
+	    wdir = gretl_workdir();
 	}
-    } 
 
-    if (target != NULL) {
-	fp = gretl_fopen(target, "w");
-	if (fp != NULL) {
-	    ok = 1;
-	    fclose(fp);
-	    gretl_remove(target);
-	}
-	free(target);
+	sprintf(dirname, "%s%s", wdir, subdir);
+	ok = write_OK(dirname);
     }
 
-    if (ok) return;
-
-    *dirname = '\0';
+    if (!ok) {
+	*dirname = '\0';
+    }
 }
 
 void set_gretl_startdir (void)
@@ -854,7 +848,9 @@ int options_dialog (int page, const char *varname, GtkWidget *parent)
     }
 
     dialog = gretl_dialog_new(_("gretl: options"), parent, GRETL_DLG_BLOCK);
+#if GTK_MAJOR_VERSION < 3
     gtk_dialog_set_has_separator(GTK_DIALOG(dialog), FALSE);
+#endif
     vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
     gtk_box_set_spacing(GTK_BOX(vbox), 2);
 
@@ -973,7 +969,7 @@ static gboolean try_switch_locale (GtkComboBox *box, gpointer p)
 	return FALSE;
     }
 
-    langstr = gtk_combo_box_get_active_text(box);
+    langstr = combo_box_get_active_text(box);
     err = test_locale(langstr);
     g_free(langstr);
 
@@ -1323,7 +1319,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 	    } 
 	    gtk_widget_show(w);
 
-	    rc->widget = gtk_combo_box_new_text();
+	    rc->widget = gtk_combo_box_text_new();
 	    gtk_table_attach(GTK_TABLE(l_table), rc->widget, 
 			     1, 2, l_len - 1, l_len,
 			     0, 0, 0, 0);
@@ -1333,7 +1329,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 
 		for (j=LANG_AUTO; j<LANG_MAX; j++) {
 		    str = lang_string_from_id(j);
-		    gtk_combo_box_append_text(GTK_COMBO_BOX(rc->widget), str);
+		    combo_box_append_text(rc->widget, str);
 		    if (!strcmp(str, strvar)) {
 			active = j;
 		    }
@@ -1345,8 +1341,7 @@ static void make_prefs_tab (GtkWidget *notebook, int tab)
 		
 		strs = get_list_setting_strings(rc->var, &nopt);
 		for (j=0; j<nopt; j++) {
-		    gtk_combo_box_append_text(GTK_COMBO_BOX(rc->widget), 
-					      strs[j]);
+		    combo_box_append_text(rc->widget, strs[j]);
 		    if (!strcmp(strs[j], strvar)) {
 			active = j;
 		    }
@@ -1586,7 +1581,7 @@ static void apply_changes (GtkWidget *widget, gpointer data)
 
 		    rcvar_set_int(rcvar, ival, &changed);
 		} else {
-		    gchar *str = gtk_combo_box_get_active_text(box);
+		    gchar *str = combo_box_get_active_text(box);
 
 		    rcvar_set_string(rcvar, str, &changed);
 		    g_free(str);
@@ -1961,7 +1956,7 @@ int read_win32_config (int debug)
     int i, err = 0;
 
     if (chinese_locale()) {
-	strcpy(fixedfontname, "MS Gothic 10");
+	strcpy(fixedfontname, "NSimSun 10");
     }
 
     appdata = appdata_path();
@@ -2311,9 +2306,9 @@ add_wdir_content (GtkWidget *dialog, struct wdir_setter *wset)
     trim_slash(tmp);
  
     /* combo + browse button for current working dir */
-    w = gtk_combo_box_entry_new_text();
+    w = combo_box_text_new_with_entry();
     gtk_container_add(GTK_CONTAINER(hbox), w);
-    set_combo_box_strings_from_list(GTK_COMBO_BOX(w), list);
+    set_combo_box_strings_from_list(w, list);
     set_combo_box_default_text(GTK_COMBO_BOX(w), tmp);
     entry = gtk_bin_get_child(GTK_BIN(w));
     gtk_entry_set_width_chars(GTK_ENTRY(entry), 32);
@@ -2384,7 +2379,7 @@ apply_wdir_changes (GtkWidget *w, struct wdir_setter *wset)
     gchar *str;
     int err;
 
-    str = gtk_combo_box_get_active_text(GTK_COMBO_BOX(wset->wdir_combo));
+    str = combo_box_get_active_text(GTK_COMBO_BOX(wset->wdir_combo));
     *tmp = '\0';
     if (str != NULL) {
 	strncat(tmp, str, MAXLEN - 2);
