@@ -23,14 +23,22 @@
 #include "toolbar.h"
 #include "dlgutils.h"
 
+#if defined(USE_GTKSOURCEVIEW_2) || defined(USE_GTKSOURCEVIEW_3)
+# define NEWER_SOURCEVIEW
+#endif
+
 #include <gtksourceview/gtksourceview.h>
 #include <gtksourceview/gtksourcelanguage.h>
-#ifdef USE_GTKSOURCEVIEW_2
+#ifdef NEWER_SOURCEVIEW
 # include <gtksourceview/gtksourcelanguagemanager.h>
 #else
 # include <gtksourceview/gtksourcelanguagesmanager.h>
 #endif
 
+#ifdef USE_GTKSOURCEVIEW_3
+# define GTK_IS_SOURCE_VIEW GTK_SOURCE_IS_VIEW
+#endif
+
 #define GUIDE_PAGE  999
 #define SCRIPT_PAGE 998
 #define GFR_PAGE    997
@@ -99,14 +107,24 @@ static void get_char_width_and_height (GtkWidget *widget,
 				       int *width,
 				       int *height)
 {
+#if GTK_MAJOR_VERSION >= 3
+    GtkStyleContext *style;
+#else
+    GtkRcStyle *style;
+#endif
     PangoLayout *pl;
     PangoContext *pc;
-    GtkRcStyle *style;
     int w = 0, h = 0;
 
     pc = gtk_widget_get_pango_context(widget);
+
+#if GTK_MAJOR_VERSION >= 3
+    style = gtk_widget_get_style_context(widget);
+    pango_context_set_font_description(pc, gtk_style_context_get_font(style, 0));
+#else
     style = gtk_widget_get_modifier_style(widget);
     pango_context_set_font_description(pc, style->font_desc);
+#endif
 
     pl = pango_layout_new(pc);
     pango_layout_set_text(pl, "X", -1);
@@ -407,7 +425,7 @@ static int source_buffer_load_buf (GtkSourceBuffer *sbuf, const char *buf)
     return 0;
 }
 
-#ifdef USE_GTKSOURCEVIEW_2
+#ifdef NEWER_SOURCEVIEW
 
 static void sourceview_apply_language (windata_t *vwin)
 {
@@ -582,7 +600,7 @@ script_key_handler (GtkWidget *w, GdkEventKey *key, windata_t *vwin)
 void create_source (windata_t *vwin, int hsize, int vsize, 
 		    gboolean editable)
 {
-#ifdef USE_GTKSOURCEVIEW_2
+#ifdef NEWER_SOURCEVIEW
     GtkSourceLanguageManager *lm = gtk_source_language_manager_new();
 #else
     GtkSourceLanguagesManager *lm = gtk_source_languages_manager_new();
@@ -597,7 +615,7 @@ void create_source (windata_t *vwin, int hsize, int vsize,
 			   lm, (GDestroyNotify) g_object_unref); 
     g_object_unref(lm); 
 
-#ifdef USE_GTKSOURCEVIEW_2
+#ifdef NEWER_SOURCEVIEW
     gtk_source_buffer_set_highlight_matching_brackets(sbuf, TRUE);
 #else
     gtk_source_buffer_set_check_brackets(sbuf, TRUE);
@@ -646,14 +664,9 @@ void text_zoom (GtkAction *action, gpointer data)
 {
     const gchar *s = gtk_action_get_name(action);
     windata_t *vwin = (windata_t *) data;
-    GtkTextBuffer *tbuf;
-    GtkTextTagTable *table;
     static PangoFontDescription *hpf;
     static gint fsize;
 
-    tbuf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(vwin->text));
-    table = gtk_text_buffer_get_tag_table(tbuf);
-
     if (hpf == NULL) {
 	hpf = pango_font_description_copy(fixed_font);
 	fsize = pango_font_description_get_size(hpf) / PANGO_SCALE;
@@ -1051,7 +1064,6 @@ static void open_script_link (GtkTextTag *tag)
     g_object_get(G_OBJECT(tag), "name", &fname, NULL);
     sprintf(fullname, "%sscripts%cmisc%c%s", gretldir, 
 	    SLASH, SLASH, fname);
-    g_free(fname);
 
     fp = gretl_fopen(fullname, "r");
     if (fp != NULL) {
@@ -1061,6 +1073,8 @@ static void open_script_link (GtkTextTag *tag)
 		SLASH, fname);
     }
 
+    g_free(fname);
+
     view_file(fullname, 0, 0, 78, 370, VIEW_SCRIPT);
 }
 
@@ -1235,13 +1249,10 @@ set_cursor_if_appropriate (GtkTextView *view, gint x, gint y)
 {
     static gboolean hovering_over_link = FALSE;
     GSList *tags = NULL, *tagp = NULL;
-    GtkTextBuffer *tbuf;
     GtkTextIter iter;
     gboolean hovering = FALSE;
 
-    tbuf = gtk_text_view_get_buffer(view);
     gtk_text_view_get_iter_at_location(view, &iter, x, y);
-  
     tags = gtk_text_iter_get_tags(&iter);
 
     for (tagp = tags; tagp != NULL; tagp = tagp->next) {
@@ -1257,7 +1268,6 @@ set_cursor_if_appropriate (GtkTextView *view, gint x, gint y)
 
     if (hovering != hovering_over_link) {
 	hovering_over_link = hovering;
-
 	if (hovering_over_link) {
 	    gdk_window_set_cursor(gtk_text_view_get_window(view, GTK_TEXT_WINDOW_TEXT), 
 				  hand_cursor);
@@ -1906,7 +1916,8 @@ void script_tabs_dialog (GtkWidget *w, windata_t *vwin)
     int smt = smarttab;
     int resp;
 
-    resp = checks_dialog(title, NULL, &opt, 1, &smt,
+    resp = checks_dialog(title, NULL, 
+			 &opt, 1, &smt, 0, 0,
 			 0, NULL, /* no radio buttons */
 			 &tsp, spintxt, 2, 8, 0);
 
