@@ -84,6 +84,7 @@ enum {
     HELP_ITEM,
     CMD_HELP_ITEM,
     GP_HELP_ITEM,
+    X12A_HELP_ITEM,
     SORT_ITEM,
     SORT_BY_ITEM,
     FORMAT_ITEM,
@@ -466,7 +467,7 @@ static void coeffint_set_alpha (GtkWidget *w, windata_t *vwin)
     GtkWidget *dialog, *tmp, *hbox;
     GtkWidget *vbox, *b, *hb2;
     GSList *group = NULL;
-    GtkObject *adj;
+    GtkAdjustment *adj;
     gchar txt[16];
     double x = 1.0 - cf->alpha;
     gboolean defset = FALSE;
@@ -520,7 +521,7 @@ static void coeffint_set_alpha (GtkWidget *w, windata_t *vwin)
     b = gtk_radio_button_new_with_label(group, _("Other"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b), !defset);
     gtk_box_pack_start(GTK_BOX(hb2), b, FALSE, FALSE, 0);
-    adj = gtk_adjustment_new(x, 0.60, 0.99, 0.01, 0, 0);
+    adj = (GtkAdjustment *) gtk_adjustment_new(x, 0.60, 0.99, 0.01, 0, 0);
     tmp = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 0.01, 2);
     g_signal_connect(tmp, "value-changed", 
 		     G_CALLBACK(set_double_from_spinner), &x);
@@ -653,6 +654,7 @@ static GretlToolItem viewbar_items[] = {
     { N_("Help on command"), GTK_STOCK_HELP, G_CALLBACK(activate_script_help), CMD_HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(window_help), HELP_ITEM },
     { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_gnuplot_help), GP_HELP_ITEM },
+    { N_("Help"), GTK_STOCK_HELP, G_CALLBACK(display_x12a_help), X12A_HELP_ITEM },
     { N_("Windows"), GRETL_STOCK_COMPASS, G_CALLBACK(compass_callback), 0 },
     { N_("Close"), GTK_STOCK_CLOSE, G_CALLBACK(delete_file_viewer), 0 }
 };
@@ -752,6 +754,8 @@ static GCallback item_get_callback (GretlToolItem *item, windata_t *vwin,
 	return NULL;
     } else if (r != EDIT_GP && f == GP_HELP_ITEM) {
 	return NULL;
+    } else if (r != EDIT_X12A && f == X12A_HELP_ITEM) {
+	return NULL;
     } else if (f == SAVE_ITEM && !save_ok) {
 	return NULL;
     } else if (f == SAVE_AS_ITEM) {
@@ -1030,7 +1034,7 @@ static void tbar_model (void)
 static void tbar_iconview (void)
 {
     if (data_status) {
-	view_session(NULL);
+	view_session();
     } else {
 	warnbox(_("Please open a data file first"));
     }
