@@ -790,9 +790,9 @@ void varinfo_dialog (int varnum)
 
     /* descriptive string, or genr formula */
     hbox = gtk_hbox_new(FALSE, 5);
-    tmp = gtk_combo_box_new_text();
-    gtk_combo_box_append_text(GTK_COMBO_BOX(tmp), _("Description:"));
-    gtk_combo_box_append_text(GTK_COMBO_BOX(tmp), _("Formula:"));
+    tmp = gtk_combo_box_text_new();
+    combo_box_append_text(tmp, _("Description:"));
+    combo_box_append_text(tmp, _("Formula:"));
     gtk_combo_box_set_active(GTK_COMBO_BOX(tmp), 0);
     g_signal_connect(G_OBJECT(tmp), "changed", 
 		     G_CALLBACK(vset_toggle_formula), vset);
@@ -860,11 +860,11 @@ void varinfo_dialog (int varnum)
 	gtk_box_pack_start(GTK_BOX(hbox), tmp, FALSE, FALSE, 5);
 	gtk_widget_show(tmp);
 
-	vset->compaction_menu = gtk_combo_box_new_text();
+	vset->compaction_menu = gtk_combo_box_text_new();
 
 	for (i=COMPACT_NONE; i<COMPACT_WDAY; i++) {
-	    gtk_combo_box_append_text(GTK_COMBO_BOX(vset->compaction_menu), 
-				      _(comp_int_to_string(i)));
+	    combo_box_append_text(vset->compaction_menu, 
+				  _(comp_int_to_string(i)));
 	}
 	
 	gtk_combo_box_set_active(GTK_COMBO_BOX(vset->compaction_menu), 
