@@ -296,17 +296,32 @@ static void show_lags_test (MODEL *pmod, int order, PRN *prn)
     }
 }
 
-static void DF_header (const char *s, int p, PRN *prn)
+static void DF_header (const char *s, int p, int pmax,
+		       gretlopt opt, PRN *prn)
 {
+    pputc(prn, '\n');
+
     if (p <= 0) {
-	pprintf(prn, _("\nDickey-Fuller test for %s\n"), s);	
+	if (opt & OPT_G) {
+	    pprintf(prn, _("Dickey-Fuller (GLS) test for %s\n"), s);
+	} else {
+	    pprintf(prn, _("Dickey-Fuller test for %s\n"), s);
+	}	
     } else {
-	pprintf(prn, _("\nAugmented Dickey-Fuller test for %s\n"), s);
+	if (opt & OPT_G) {
+	    pprintf(prn, _("Augmented Dickey-Fuller (GLS) test for %s\n"), s);
+	} else {
+	    pprintf(prn, _("Augmented Dickey-Fuller test for %s\n"), s);
+	}
 	if (p == 1) {
 	    pprintf(prn, _("including one lag of (1-L)%s"), s);
 	} else {
 	    pprintf(prn, _("including %d lags of (1-L)%s"), p, s);
 	}
+	if (pmax >= p) {
+	    pputc(prn, ' ');
+	    pprintf(prn, _("(max was %d)"), pmax);
+	}
 	pputc(prn, '\n');
     }
 }
@@ -360,7 +375,7 @@ static const char *DF_test_string (int i)
 }
 
 static void 
-print_adf_results (int order, int auto_order, double DFt, double pv, 
+print_adf_results (int order, int pmax, double DFt, double pv, 
 		   MODEL *dfmod, int dfnum, const char *vname, 
 		   int *blurb_done, unsigned char flags, int i, 
 		   int niv, int nseas, gretlopt opt, PRN *prn)
@@ -387,7 +402,7 @@ print_adf_results (int order, int auto_order, double DFt, double pv,
     } 
 
     if (*blurb_done == 0) {
-	DF_header(vname, order, prn);
+	DF_header(vname, order, pmax, opt, prn);
 	pprintf(prn, _("sample size %d\n"), dfmod->nobs);
 	if (flags & ADF_PANEL) {
 	    pputc(prn, '\n');
@@ -400,9 +415,6 @@ print_adf_results (int order, int auto_order, double DFt, double pv,
 
     if (!(flags & ADF_EG_RESIDS)) {
 	pprintf(prn, "   %s ", _(DF_test_string(i)));
-	if (opt & OPT_G) {
-	    pputs(prn, "(GLS) ");
-	}
 	if (nseas > 0 && i > 0) {
 	    pputs(prn, _("plus seasonal dummies"));
 	}
@@ -445,7 +457,7 @@ print_adf_results (int order, int auto_order, double DFt, double pv,
 }
 
 static int auto_adjust_order (int *list, int order_max,
-			      double ***pZ, DATAINFO *pdinfo,
+			      double **Z, DATAINFO *pdinfo,
 			      PRN *prn)
 {
     MODEL kmod;
@@ -453,9 +465,7 @@ static int auto_adjust_order (int *list, int order_max,
     int k, pos;
 
     for (k=order_max; k>0; k--) {
-
-	kmod = lsq(list, *pZ, pdinfo, OLS, OPT_A);
-
+	kmod = lsq(list, Z, pdinfo, OLS, OPT_A);
 	if (kmod.errcode) {
 	    fprintf(stderr, "auto_adjust_order: k = %d, err = %d\n", k,
 		    kmod.errcode);
@@ -463,11 +473,9 @@ static int auto_adjust_order (int *list, int order_max,
 	    k = -1;
 	    break;
 	}
-
 #if ADF_DEBUG
 	printmodel(&kmod, pdinfo, OPT_NONE, prn);
 #endif
-
 	pos = k + kmod.ifc;
 	tstat = kmod.coeff[pos] / kmod.sderr[pos];
 	clear_model(&kmod);
@@ -669,16 +677,15 @@ static int real_adf_test (int varno, int order, int niv,
     if (opt & OPT_E) {
 	/* testing down */
 	auto_order = 1;
+	order_max = order;
     }
 
     if (order < 0) {
 	/* testing down: backward compatibility */
 	auto_order = 1;
-	order = -order;
+	order = order_max = -order;
     }
 
-    order_max = order;
-
 #if ADF_DEBUG
     fprintf(stderr, "real_adf_test: order = %d, auto_order = %d\n", order, auto_order);
 #endif
@@ -803,7 +810,7 @@ static int real_adf_test (int varno, int order, int niv,
     skipdet:
 
 	if (auto_order) {
-	    order = auto_adjust_order(list, order_max, pZ, pdinfo, prn);
+	    order = auto_adjust_order(list, order_max, *pZ, pdinfo, prn);
 	    if (order < 0) {
 		err = 1;
 		clear_model(&dfmod);
@@ -857,7 +864,7 @@ static int real_adf_test (int varno, int order, int niv,
 	}
 
 	if (!(opt & OPT_Q) && !(flags & ADF_PANEL)) {
-	    print_adf_results(order, auto_order, DFt, pv, &dfmod, dfnum, 
+	    print_adf_results(order, order_max, DFt, pv, &dfmod, dfnum, 
 			      pdinfo->varname[varno], &blurb_done, flags,
 			      itv, niv, nseas, opt, prn);
 	}
@@ -968,9 +975,9 @@ static int do_IPS_test (double tbar, int n, const int *Ti,
 	}
     }
 
-    if (Oi != NULL || order > 0 ) {
+    if (Oi != NULL || order > 0) {
 	/* non-zero lag order: use IPS's W_{tbar} statistic */
-	double E, V, Etbar, Vtbar, Wtbar;
+	double E, V, Wtbar, Etbar = 0, Vtbar = 0;
 	int order_i;
 
 	IPS_tbar_rho_moments = get_plugin_function("IPS_tbar_rho_moments", &handle);
@@ -995,7 +1002,7 @@ static int do_IPS_test (double tbar, int n, const int *Ti,
 	}
     } else if (Tmax > Tmin) {
 	/* sample sizes differ: use IPS's Z_{tbar} */
-	double E, V, Etbar, Vtbar, Ztbar;
+	double E, V, Ztbar, Etbar = 0, Vtbar = 0;
 
 	IPS_tbar_moments = get_plugin_function("IPS_tbar_moments", &handle);
 
@@ -1102,11 +1109,8 @@ static int panel_DF_test (int v, int order,
     if (!quiet) {
 	int j = DF_index(opt);
 
-	DF_header(pdinfo->varname[v], (opt & OPT_E)? 0 : order, prn);
+	DF_header(pdinfo->varname[v], (opt & OPT_E)? 0 : order, 0, opt, prn);
 	pprintf(prn, "   %s ", _(DF_test_string(j)));
-	if (opt & OPT_G) {
-	    pputs(prn, "(GLS) ");
-	}
 	pputc(prn, '\n');
 	pprintf(prn, "   %s: %s\n\n", _("model"), 
 		(order > 0)? ADF_model_string(j) : DF_model_string(j));
@@ -1256,8 +1260,8 @@ int levin_lin_test (int vnum, const int *plist,
  * @opt: option flags.
  * @prn: gretl printing struct.
  *
- * Carries out and prints the results of the Augmented Dickey-Fuller test for 
- * a unit root. 
+ * Carries out and prints the results of the Augmented Dickey-Fuller 
+ * test for a unit root. 
  *
  * By default two tests are performed, one for a model
  * including a constant and one including a linear trend. The 
@@ -1308,6 +1312,15 @@ int adf_test (int order, const int *list, double ***pZ,
 	    vlist[1] = v = list[i];
 	    err = list_adjust_sample(vlist, &pdinfo->t1, &pdinfo->t2, 
 				     (const double **) *pZ);
+	    if (!err && order == -1) {
+		/* default to L_{12}: see G. W. Schwert, "Tests for Unit Roots:
+		   A Monte Carlo Investigation", Journal of Business and
+		   Economic Statistics, 7(2), 1989, pp. 5-17.
+		*/
+		int T = pdinfo->t2 - pdinfo->t1 + 1;
+
+		order = 12.0 * pow(T/100.0, 0.25);
+	    }
 	    if (!err) {
 		err = real_adf_test(v, order, 1, pZ, pdinfo, opt, 
 				    0, NULL, prn);
@@ -1564,7 +1577,7 @@ static int panel_kpss_test (int order, int v,
     double ppv = 0.0, zpv = 0.0, lpv = 0.0;
     int gt_10 = 0, lt_01 = 0;
     double pval;
-    int i, err;
+    int i, err = 0;
 
     /* run a KPSS test for each unit and record the
        results */
