@@ -757,6 +757,19 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	    myopt |= OPT_I;
 	    set_optval_double(QUANTREG, OPT_I, x);
 	}
+    } else if (orig->ci == TOBIT) {
+	double x;
+
+	x = gretl_model_get_double(orig, "llimit");
+	if (!na(x)) {
+	    myopt |= OPT_L;
+	    set_optval_double(TOBIT, OPT_L, x);
+	}	
+	x = gretl_model_get_double(orig, "rlimit");
+	if (!na(x)) {
+	    myopt |= OPT_M;
+	    set_optval_double(TOBIT, OPT_M, x);
+	}
     } else if (orig->ci == IVREG) {
 	transcribe_option_flags(&myopt, orig->opt, OPT_L | OPT_G);
     }
@@ -791,7 +804,7 @@ static MODEL replicate_estimator (const MODEL *orig, int **plist,
 	rep = logit_probit(list, pZ, pdinfo, orig->ci, myopt, NULL);
 	break;
     case TOBIT:
-	rep = tobit_model(list, pZ, pdinfo, OPT_NONE, NULL);
+	rep = tobit_driver(list, pZ, pdinfo, myopt, NULL);
 	break;
     case LAD:
 	if (gretl_model_get_int(orig, "rq")) {
@@ -905,9 +918,16 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 	err = aux.errcode;
 	fprintf(stderr, "auxiliary regression failed\n");
     } else {
-	double trsq = aux.rsq * aux.nobs;
-	int df = list[0] - pmod->list[0];
-	double pval = chisq_cdf_comp(df, trsq);
+	double pval, trsq = aux.rsq * aux.nobs;
+	int df = aux.ncoeff - pmod->ncoeff;
+
+	if (df <= 0) {
+	    /* couldn't add any regressors */
+	    err = E_SINGULAR;
+	    goto bailout;
+	}
+
+	pval = chisq_cdf_comp(df, trsq);
 
 	aux.aux = aux_code;
 
@@ -919,8 +939,8 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 	}
 
 	pprintf(prn, "  %s: TR^2 = %g,\n  ", _("Test statistic"), trsq);
-	pprintf(prn, _("with p-value = prob(Chi-square(%d) > %g) = %g\n\n"), 
-		df, trsq, pval);
+	pprintf(prn, "%s = P(%s(%d) > %g) = %g\n\n", 
+		_("with p-value"), _("Chi-square"), df, trsq, pval);
 
 	if (opt & OPT_S) {
 	    ModelTest *test;
@@ -939,6 +959,8 @@ real_nonlinearity_test (MODEL *pmod, int *list,
 	record_test_result(trsq, pval, _("non-linearity"));
     } 
 
+ bailout:
+
     clear_model(&aux);
 
     return err;
@@ -988,8 +1010,7 @@ int nonlinearity_test (MODEL *pmod, double ***pZ, DATAINFO *pdinfo,
     impose_model_smpl(pmod, pdinfo);
 
     /* add squares or logs */
-    tmplist = augment_regression_list(pmod->list, aux, 
-				      pZ, pdinfo);
+    tmplist = augment_regression_list(pmod->list, aux, pZ, pdinfo);
     if (tmplist == NULL) {
 	return E_ALLOC;
     } else if (tmplist[0] == pmod->list[0]) {
@@ -1571,7 +1592,9 @@ double get_DW_pvalue_for_model (const MODEL *pmod,
     int *list = NULL;
     double pv = NADBL;
 
-    if (pmod == NULL || pmod->list == NULL) {
+    if (pZ == NULL || *pZ == NULL || pdinfo == NULL) {
+	*err = E_NODATA;
+    } else if (pmod == NULL || pmod->list == NULL) {
 	*err = E_DATA;
     } else if ((pmod->ci != OLS && pmod->ci != PANEL) || 
 	       model_has_missing_obs(pmod) || na(pmod->dw)) {
