@@ -1121,7 +1121,7 @@ static int writehdr (const char *hdrfile, const int *list,
 int get_precision (const double *x, int n, int placemax)
 {
     int t, p, pmax = 0;
-    char *s, numstr[48];
+    char *s, numstr[64];
     int n_ok = 0;
     double z;
 
@@ -1408,8 +1408,17 @@ int write_data (const char *fname, int *list,
 		    datestr, pdinfo->pd);
 	}
 
+	if (fmt == GRETL_FMT_CSV) {
+	    /* optional comment */
+	    const char *msg = get_optval_string(STORE, OPT_E);
+
+	    if (msg != NULL && *msg != '\0') {
+		fprintf(fp, "# %s\n", msg);
+	    }
+	}	    
+
 	if (fmt == GRETL_FMT_CSV && (opt & OPT_N)) {
-	    ;/* no header */
+	    ; /* no header */
 	} else {
 	    /* header: variable names */
 	    if (fmt == GRETL_FMT_CSV && print_obs && 
@@ -1455,13 +1464,15 @@ int write_data (const char *fname, int *list,
 	}
     } else if (fmt == GRETL_FMT_OCTAVE) { 
 	/* GNU Octave: write out data as a matrix */
-	fprintf(fp, "# name: X\n# type: matrix\n# rows: %d\n# columns: %d\n", 
-		n, list[0]);
+	const char *matname = (opt & OPT_F)? "gretldata" : "X";
+
+	fprintf(fp, "# name: %s\n# type: matrix\n# rows: %d\n# columns: %d\n", 
+		matname, n, list[0]);
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
 	    for (i=1; i<=list[0]; i++) {
 		v = list[i];
 		if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
-		    fprintf(fp, "%.12g ", Z[v][t]);
+		    fprintf(fp, "%.15g ", Z[v][t]);
 		} else {
 		    fprintf(fp, "%.*f ", pmax[i-1], Z[v][t]); 
 		}
@@ -1786,7 +1797,7 @@ int gretl_get_data (char *fname, double ***pZ, DATAINFO *pdinfo,
 	gzsuff = has_suffix(fname, ".gz");
     }
 
-    if (addpath(fname, 0) == NULL) { 
+    if (gretl_addpath(fname, 0) == NULL) { 
 	/* not found yet */
 	char tryfile[MAXLEN];
 	int found = 0;
@@ -1796,7 +1807,7 @@ int gretl_get_data (char *fname, double ***pZ, DATAINFO *pdinfo,
 	    *tryfile = '\0';
 	    strncat(tryfile, fname, MAXLEN-1);
 	    try_gdt(tryfile); 
-	    found = (addpath(tryfile, 0) != NULL);
+	    found = (gretl_addpath(tryfile, 0) != NULL);
 	    if (found) {
 		gdtsuff = 1;
 	    }
@@ -1807,7 +1818,7 @@ int gretl_get_data (char *fname, double ***pZ, DATAINFO *pdinfo,
 	*/
 	if (!found && !gzsuff) { 
 	    sprintf(tryfile, "%s.gz", fname);
-	    if (addpath(tryfile, 0) != NULL) {
+	    if (gretl_addpath(tryfile, 0) != NULL) {
 		gzsuff = 1;
 		found = 1;
 	    }
@@ -1839,8 +1850,12 @@ int gretl_get_data (char *fname, double ***pZ, DATAINFO *pdinfo,
 	gz_switch_ext(lblfile, fname, "lbl");
     }
 
-    /* read data header file */
+    /* try reading data header file */
     err = readhdr(hdrfile, tmpdinfo, &binary, &old_byvar);
+    if (err) {
+	free(tmpdinfo);
+    }
+
     if (err == E_FOPEN) {
 	/* no header file, so maybe it's just an ascii datafile */
 	return import_csv(fname, pZ, pdinfo, OPT_NONE, prn);
@@ -2728,8 +2743,8 @@ octave_varname (char *name, const char *s, int nnum, int v)
 
 static int get_max_line_length (FILE *fp, PRN *prn)
 {
-    int c, c1, cbak = 0, cc = 0;
-    int comment = 0, maxlen = 0;
+    int c, c1, cc = 0;
+    int maxlen = 0;
 
     while ((c = fgetc(fp)) != EOF) {
 	if (c == 0x0d) {
@@ -2753,16 +2768,12 @@ static int get_max_line_length (FILE *fp, PRN *prn)
 	    cc = 0;
 	    continue;
 	}
-	cbak = c;
 	if (!isspace((unsigned char) c) && !isprint((unsigned char) c) &&
 	    !(c == CTRLZ)) {
 	    pprintf(prn, M_("Binary data (%d) encountered: this is not a valid "
 			   "text file\n"), c);
 	    return -1;
 	}
-	if (cc == 0) {
-	    comment = (c == '#');
-	}
 	cc++;
     }
 
@@ -3179,7 +3190,6 @@ int gretl_is_pkzip_file (const char *fname)
 GretlFileType detect_filetype (char *fname, gretlopt opt)
 {
     int i, c, ftype = GRETL_NATIVE_DATA;
-    char teststr[5];
     FILE *fp;
 
     /* might be a script file? (watch out for DOS-mangled names) */
@@ -3217,13 +3227,15 @@ GretlFileType detect_filetype (char *fname, gretlopt opt)
 	return GRETL_CSV;
     if (has_suffix(fname, ".txt"))
 	return GRETL_CSV;
+    if (has_suffix(fname, ".asc"))
+	return GRETL_CSV;
     if (has_suffix(fname, ".m"))
 	return GRETL_OCTAVE;
     if (has_suffix(fname, ".bn7"))
 	return GRETL_PCGIVE_DB;
 
     if (opt & OPT_P) {
-	addpath(fname, 0); 
+	gretl_addpath(fname, 0); 
     }
 
     if (gretl_is_xml_file(fname)) {
@@ -3250,13 +3262,9 @@ GretlFileType detect_filetype (char *fname, gretlopt opt)
 	    ftype = GRETL_NATIVE_DATA; /* native binary data? */
 	    break;
 	}
-	if (i < 4) {
-	    teststr[i] = c;
-	}
     }
 
     fclose(fp);
-    teststr[4] = 0;
 
     return ftype;
 }
@@ -3346,6 +3354,21 @@ int check_atoi (const char *numstr)
     return 1;
 }
 
+static int transpose_varname_used (const char *vname, 
+				   DATAINFO *dinfo,
+				   int imax)
+{
+    int i;
+
+    for (i=0; i<imax; i++) {
+	if (!strcmp(vname, dinfo->varname[i])) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
 /**
  * transpose_data:
  * @pZ: pointer to data array.
@@ -3379,11 +3402,24 @@ int transpose_data (double ***pZ, DATAINFO *pdinfo)
     }
 
     for (t=0; t<pdinfo->n; t++) {
+	int k = t + 1;
+	char *targ = tinfo->varname[k];
+
 	if (pdinfo->S != NULL && pdinfo->S[t][0] != '\0') {
-	    tinfo->varname[t+1][0] = '\0';
-	    strncat(tinfo->varname[t+1], pdinfo->S[t], VNAMELEN - 1);
+	    int err;
+
+	    *targ = '\0';
+	    strncat(targ, pdinfo->S[t], VNAMELEN - 1);
+	    charsub(targ, ' ', '_');
+	    err = check_varname(targ);
+	    if (err) {
+		sprintf(targ, "v%d", k);
+		gretl_error_clear();
+	    } else if (transpose_varname_used(targ, tinfo, k)) {
+		sprintf(targ, "v%d", k);
+	    }
 	} else {
-	    sprintf(tinfo->varname[t+1], "v%d", t+1);
+	    sprintf(targ, "v%d", k);
 	}
     }
 
