@@ -1697,7 +1697,7 @@ static void ODBC_info_clear_read (void)
     gretl_odinfo.nvars = 0;
 }
 
-void ODBC_info_clear_all (void)
+static void ODBC_info_clear_all (void)
 {
     free(gretl_odinfo.dsn);
     gretl_odinfo.dsn = NULL;
@@ -2794,8 +2794,8 @@ static double *compact_series (const double *src, int i, int n, int oldn,
     double *x;
 
 #if DB_DEBUG
-    printf("compact_series: startskip=%d, min_startskip=%d, compfac=%d "
-	   "lead=%d\n", startskip, min_startskip, compfac, lead);
+    fprintf(stderr, "compact_series: startskip=%d, min_startskip=%d, compfac=%d "
+	    "lead=%d\n", startskip, min_startskip, compfac, lead);
 #endif
 
     x = malloc(n * sizeof *x);
@@ -3017,29 +3017,32 @@ get_startskip_etc (int compfac, int startmin, int endmin,
 		   int oldn, CompactMethod method, 
 		   int *startskip, int *newn) 
 {
-    int ss = compfac - (startmin % compfac) + 1;
-    int es, n;
-
-    ss = ss % compfac;
+    int ss, es, n;
 
     if (method == COMPACT_EOP) {
-	if (ss > 0) {
-	    ss--;
-	} else {
-	    /* move to end of initial period */
-	    ss = compfac - 1;
+	int unused;
+
+	ss = (compfac - (startmin % compfac)) % compfac;
+	n = oldn / compfac;
+	unused = oldn - 1 - ss - (n-1) * compfac;
+	if (unused >= compfac) {
+	    n++;
 	}
-    }
+    } else if (method == COMPACT_SOP) {
+	int unused;
 
-    es = endmin % compfac;
-    if (method == COMPACT_SOP && es > 1) {
-	es--;
+	ss = (compfac - (startmin % compfac) + 1) % compfac;
+	n = oldn / compfac;
+	unused = oldn - 1 - ss - (n-1) * compfac;
+	if (unused >= compfac) {
+	    n++;
+	}
+    } else {
+	ss = (compfac - (startmin % compfac) + 1) % compfac;
+	es = endmin % compfac;
+	n = (oldn - ss - es) / compfac;
     }
 
-    n = (oldn - ss - es) / compfac;
-    if (ss && method == COMPACT_EOP) n++;
-    if (es && method == COMPACT_SOP) n++;
-
     *startskip = ss;
     *newn = n;
 }
@@ -3116,7 +3119,7 @@ get_global_compact_params (int compfac, int startmin, int endmin,
 		} else {
 		    n_not_eop++;
 		}
-	    }
+	    } 
 	}
 	if (startskip < *min_startskip) {
 	    *min_startskip = startskip;
@@ -3359,13 +3362,17 @@ weeks_to_months_check (const DATAINFO *pdinfo, int *startyr, int *endyr,
 	monbak = mon;
     }
 
-    /* flush the last observation */
+    if (err) {
+	mcount = 0;
+    } else {
+	/* flush the last observation */
 #if WEEKLY_DEBUG
-    fprintf(stderr, "month %d ('%d'), weekly obs = %d\n", 
-	    mcount, monbak, wcount);
+	fprintf(stderr, "month %d ('%d'), weekly obs = %d\n", 
+		mcount, monbak, wcount);
 #endif
-    *endyr = yr;
-    *endmon = mon;
+	*endyr = yr;
+	*endmon = mon;
+    }
 
     return mcount;
 }
@@ -3375,8 +3382,8 @@ static int weekly_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
 {
     double **mZ = NULL;
     DATAINFO minfo;
-    int startyr = 1, endyr;
-    int startmon = 1, endmon;
+    int startyr = 1, endyr = 1;
+    int startmon = 1, endmon = 1;
     int err = 0;
 
     minfo.n = weeks_to_months_check(pdinfo, &startyr, &endyr, &startmon, &endmon);
@@ -3416,7 +3423,7 @@ static int weekly_dataset_to_monthly (double ***pZ, DATAINFO *pdinfo,
 
 static int shorten_the_constant (double **Z, int n)
 {
-    double *tmp = realloc(Z[0], n * sizeof **Z);
+    double *tmp = realloc(Z[0], n * sizeof *tmp);
 
     if (tmp == NULL) {
 	return E_ALLOC;
@@ -3587,27 +3594,29 @@ insert_missing_hidden_obs (double ***pZ, DATAINFO *pdinfo,
 			   int nmiss)
 {
     int oldn = pdinfo->n;
-    double *tmp;
+    double *tmp, **Z;
     int i, s, t, skip;
     int err = 0;
 
-    tmp = malloc(oldn * sizeof *tmp);
-    if (tmp == NULL) {
-	return E_ALLOC;
-    }
-
-    err = dataset_add_observations(nmiss, pZ, pdinfo, OPT_A);
+    err = dataset_add_observations(nmiss, pZ, pdinfo, OPT_NONE);
     if (err) {
-	free(tmp);
 	return err;
     }
 
+#if DB_DEBUG
+    fprintf(stderr, "daily data: expanded n from %d to %d\n",
+	    oldn, pdinfo->n);
+#endif
+
+    Z = *pZ;
+    tmp = Z[0];
+
     for (i=1; i<pdinfo->v && !err; i++) {
 	for (s=0; s<oldn; s++) {
-	    tmp[s] = (*pZ)[i][s];
+	    tmp[s] = Z[i][s];
 	}
 
-	(*pZ)[i][0] = tmp[0];
+	Z[i][0] = tmp[0];
 	t = 1;
 	for (s=1; s<oldn; s++) {
 	    skip = get_daily_skip(pdinfo, s);
@@ -3616,20 +3625,31 @@ insert_missing_hidden_obs (double ***pZ, DATAINFO *pdinfo,
 		break;
 	    }
 	    while (skip--) {
-		(*pZ)[i][t++] = NADBL;
+		Z[i][t++] = NADBL;
 	    }
-	    (*pZ)[i][t++] = tmp[s];
+	    Z[i][t++] = tmp[s];
 	}
     }
 
-    free(tmp);
+    for (t=0; t<pdinfo->n; t++) {
+	Z[0][t] = 1.0;
+	if (pdinfo->S != NULL) {
+	    calendar_date_string(pdinfo->S[t], t, pdinfo);
+	}
+    }
 
     if (!err) {
-	dataset_destroy_obs_markers(pdinfo);
 	pdinfo->t2 = pdinfo->n - 1;
 	ntodate(pdinfo->endobs, pdinfo->n - 1, pdinfo);
     }
 
+#if DB_DEBUG > 1
+    fprintf(stderr, "insert_missing_hidden_obs, done, err = %d\n", err);
+    for (t=0; t<pdinfo->n; t++) {
+	fprintf(stderr, "Z[1][%d] = %14g\n", t, Z[1][t]);
+    }    
+#endif
+
     return err;
 }
 
@@ -3693,6 +3713,8 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
 	if (err) {
 	    gretl_errmsg_set("Error expanding daily data with missing observations");
 	    return err;
+	} else {
+	    oldn = pdinfo->n;
 	}
     }
 
@@ -3754,7 +3776,11 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
 	}
 	sprintf(stobs, "%d", startmaj);
     } else if (newpd == 52) {
-	strcpy(stobs, "1");
+	if (oldpd >= 5 && oldpd <= 7 && pdinfo->S != NULL) {
+	    strcpy(stobs, pdinfo->S[min_startskip]);
+	} else {
+	    strcpy(stobs, "1");
+	}
     } else {
 	int m0 = startmin + min_startskip;
 	int minor = m0 / compfac + (m0 % compfac > 0);
@@ -3775,7 +3801,7 @@ int compact_data_set (double ***pZ, DATAINFO *pdinfo, int newpd,
     pdinfo->t2 = pdinfo->n - 1;
     ntodate(pdinfo->endobs, pdinfo->t2, pdinfo);
     
-    if (oldpd >= 5 && oldpd <= 7) {
+    if (oldpd >= 5 && oldpd <= 7 && pdinfo->markers) {
 	/* remove any daily date strings */
 	dataset_destroy_obs_markers(pdinfo);
     }
