@@ -419,6 +419,10 @@ static int opg_from_ascore (op_container *OC, gretl_matrix *GG)
     }
 
     ll = op_loglik(OC->theta, OC);
+    if (na(ll)) {
+	free(g0);
+	return E_DATA;
+    }
 
     for (j=0; j<k; j++) {
 	g0[j] = OC->g[j];
@@ -447,10 +451,11 @@ static int opg_from_ascore (op_container *OC, gretl_matrix *GG)
 
 static int ihess_from_ascore (op_container *OC, gretl_matrix *inH) 
 {
-    int i, j, err, k = OC->k;
     double smal = 1.0e-07;  /* "small" is some sort of macro on win32 */
     double smal2 = 2.0 * smal;
     double ti, x, ll, *g0;
+    int i, j, k = OC->k;
+    int err = 0;
 
     g0 = malloc(k * sizeof *g0);
     if (g0 == NULL) {
@@ -461,11 +466,21 @@ static int ihess_from_ascore (op_container *OC, gretl_matrix *inH)
 	ti = OC->theta[i];
 	OC->theta[i] -= smal;
 	ll = op_loglik(OC->theta, OC);
+	if (na(ll)) {
+	    OC->theta[i] = ti;
+	    err = E_DATA;
+	    break;
+	}
 	for (j=0; j<k; j++) {
 	    g0[j] = OC->g[j];
 	}
 	OC->theta[i] += smal2;
 	ll = op_loglik(OC->theta, OC);
+	if (na(ll)) {
+	    OC->theta[i] = ti; 
+	    err = E_DATA;
+	    break;
+	}	
 	for (j=0; j<k; j++) {
 	    x = (OC->g[j] - g0[j]) / smal2;
 	    gretl_matrix_set(inH, i, j, -x);
@@ -474,17 +489,14 @@ static int ihess_from_ascore (op_container *OC, gretl_matrix *inH)
 	OC->theta[i] = ti;
     }
 
-    gretl_matrix_xtr_symmetric(inH);
-
     free(g0);
 
-#if LPDEBUG
-    gretl_matrix_print(inH, "inverse of Hessian");
-#endif
-
-    err = gretl_invert_symmetric_matrix(inH);
-    if (err) {
-	fprintf(stderr, "ihess_from_ascore: failed to invert numerical Hessian\n");
+    if (!err) {
+	gretl_matrix_xtr_symmetric(inH);
+	err = gretl_invert_symmetric_matrix(inH);
+	if (err) {
+	    fprintf(stderr, "ihess_from_ascore: failed to invert numerical Hessian\n");
+	}	
     }
 
     return err;
@@ -619,10 +631,105 @@ static void op_LR_test (MODEL *pmod, op_container *OC,
     OC->k += nx;
 }
 
+static int oprobit_normtest (MODEL *pmod, op_container *OC)
+{
+    int nobs = OC->nobs;
+    int k = OC->k;
+    int nx = OC->nx;
+    int M = OC->ymax;
+    double *theta = OC->theta;
+    int t, s, i, yt, err = 0;
+    gretl_matrix *CMtestmat;
+    gretl_matrix *y;
+    gretl_matrix *beta;
+    double m0, m1, u, v, a2v, b2u;
+    double a = 0, b = 0;
+    double e3, e4;
+
+    CMtestmat = gretl_matrix_alloc(nobs, k+2);
+    y = gretl_unit_matrix_new(nobs, 1);
+    beta = gretl_matrix_alloc(k+2, 1);
+
+    if (CMtestmat == NULL || y == NULL || beta == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    s = 0;
+    for (t=OC->pmod->t1; t<=OC->pmod->t2; t++) {
+	if (na(OC->pmod->uhat[t])) {
+	    continue;
+	}
+
+	yt = OC->y[s];
+
+	if (yt == 0) {
+	    m0 = theta[nx];
+	    b = OC->ndx[s] + m0;
+	} else {
+	    m0 = theta[nx + yt - 1];
+	    a = OC->ndx[s] + m0;
+	    if (yt < M) {
+		m1 = theta[nx + yt];
+		b = OC->ndx[s] + m1;
+	    }
+	} 
+
+	if (yt == 0) {
+	    u = OC->G[nx][s];
+	    b2u = b*b*u;
+	    v = a2v = 0;
+	} else {
+	    v = -OC->G[nx + yt - 1][s];
+	    a2v = a*a*v;
+	    if (yt < M) {
+		u = OC->G[nx + yt][s];
+		b2u = b*b*u;
+	    } else {
+		b2u = u = 0;
+	    }
+	} 
+
+	for (i=0; i<k; i++) {
+	    gretl_matrix_set(CMtestmat, s, i, OC->G[i][s]);
+	}
+
+	e3 = 2*(v-u) + (a2v - b2u);
+	e4 = 3*(a*v-b*u) + (a*a2v - b*b2u);
+
+	gretl_matrix_set(CMtestmat, s, k, e3);
+	gretl_matrix_set(CMtestmat, s, k+1, e4);
+
+	s++;
+    }
+
+    err = gretl_matrix_ols(y, CMtestmat, beta, NULL, NULL, NULL);
+
+    if (!err) {
+	double X2 = nobs;
+
+	gretl_matrix_multiply(CMtestmat, beta, y);
+	for (t=0; t<y->rows; t++) {
+	    X2 -= (1 - y->val[t]) * (1 - y->val[t]);
+	}
+
+	gretl_model_add_normality_test(pmod, X2);
+    }
+
+
+ bailout:
+
+    gretl_matrix_free(CMtestmat);
+    gretl_matrix_free(y);
+    gretl_matrix_free(beta);
+
+    return err;
+}
+
 static void fill_op_model (MODEL *pmod, const int *list,
 			   const double **Z, const DATAINFO *pdinfo, 
 			   op_container *OC, gretl_matrix *V, 
-			   int fnc, int grc)
+			   int fncount, int grcount)
 {
     int npar = OC->k;
     int nx = OC->nx;
@@ -634,13 +741,18 @@ static void fill_op_model (MODEL *pmod, const int *list,
 
     gretl_model_set_int(pmod, "ordered", 1);
     gretl_model_set_int(pmod, "nx", OC->nx);
-    gretl_model_set_int(pmod, "fncount", fnc);
-    gretl_model_set_int(pmod, "grcount", grc);
+
+    if (grcount > 0) {
+	gretl_model_set_int(pmod, "fncount", fncount);
+	gretl_model_set_int(pmod, "grcount", grcount);
+    } else {
+	gretl_model_set_int(pmod, "iters", fncount);
+    }
 
     pmod->ncoeff = npar;
 
     if (V != NULL) {
-	pmod->errcode = gretl_model_write_vcv(pmod, V);
+	pmod->errcode = gretl_model_write_vcv(pmod, V, -1);
 	if (pmod->errcode) {
 	    return;
 	}
@@ -655,6 +767,10 @@ static void fill_op_model (MODEL *pmod, const int *list,
 	pmod->opt |= OPT_R;
     }
 
+    if (OC->ci == PROBIT) {
+	oprobit_normtest(pmod, OC);
+    }
+
     s = 0;
     for (t=OC->t1; t<=OC->t2; t++) {
 	if (na(OC->pmod->uhat[t])) {
@@ -709,6 +825,7 @@ static void fill_op_model (MODEL *pmod, const int *list,
 	op_LR_test(pmod, OC, Z);
 	gretl_model_set_coeff_separator(pmod, NULL, nx);
     }
+
 }
 
 static gretl_matrix *oprobit_vcv (op_container *OC, int *err)
@@ -779,6 +896,7 @@ static int do_ordered (int ci, int ndum,
     gretl_matrix *V = NULL;
     double *theta = NULL;
     double toler;
+    int use_newton = 0;
     int err;
 
     OC = op_container_new(ci, ndum, Z, pmod, opt);
@@ -814,18 +932,31 @@ static int do_ordered (int ci, int ndum,
 	    op_loglik(theta, OC));
 #endif
 
-    BFGS_defaults(&maxit, &toler, PROBIT);
+    if (libset_get_int(GRETL_OPTIM) == OPTIM_NEWTON) {
+	use_newton = 1;
+    }
 
-    err = BFGS_max(theta, npar, maxit, toler, 
-		   &fncount, &grcount, op_loglik, C_LOGLIK,
-		   op_score, OC, NULL, 
-		   (prn != NULL)? OPT_V : OPT_NONE, prn);
+    if (use_newton) {
+	double crittol = 1.0e-7;
+	double gradtol = 1.0e-7;
+
+	err = newton_raphson_max(theta, npar, maxit, 
+				 crittol, gradtol, &fncount, 
+				 C_LOGLIK, op_loglik, 
+				 op_score, NULL, OC, 
+				 (prn != NULL)? OPT_V : OPT_NONE,
+				 prn);
+    } else {
+	BFGS_defaults(&maxit, &toler, PROBIT);
+	err = BFGS_max(theta, npar, maxit, toler, 
+		       &fncount, &grcount, op_loglik, C_LOGLIK,
+		       op_score, OC, NULL, 
+		       (prn != NULL)? OPT_V : OPT_NONE, prn);
+    }
 
     if (err) {
 	goto bailout;
-    } else {
-	fprintf(stderr, "Number of iterations = %d (%d)\n", fncount, grcount);
-    }
+    } 
 
     /* transform back to 'real' theta */
     op_get_real_theta(OC, theta);
@@ -1377,7 +1508,7 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
     gretl_matrix_qform(H, GRETL_MOD_NONE, S,
 		       V, GRETL_MOD_NONE);
 
-    err = gretl_model_write_vcv(pmod, V);
+    err = gretl_model_write_vcv(pmod, V, -1);
 
     if (!err) {
 	gretl_model_set_vcv_info(pmod, VCV_ML, VCV_QML);
@@ -1394,8 +1525,6 @@ compute_QML_vcv (MODEL *pmod, const double **Z)
     return err;
 }
 
-
-
 /* calculate standard errors etc using the Hessian */
 
 static int logit_probit_vcv (MODEL *dmod, gretlopt opt, const double **Z)
@@ -1836,9 +1965,77 @@ static int add_slopes_to_model (MODEL *pmod, const double **Z)
     return err;
 }
 
-static void binary_model_add_stats (MODEL *pmod, const double *y,
+/* Binary probit normality test as in Jarque, Bera and Lee (1984),
+   also quoted in Verbeek, chapter 7: we regress a column of 1s on the
+   products of the generalized residual with X, (X\beta)^2 and
+   (X\beta)^3.  The test statistic is T times the uncentered
+   R-squared, and is distributed as chi-square(2). It can be shown
+   that this test is numerically identical to the Chesher-Irish (87)
+   test in the probit case (although C&I make no mention of this in
+   their article).  
+*/
+
+static int binary_probit_normtest (MODEL *pmod, double **Z)
+{
+    gretl_matrix *X, *y, *b = NULL;
+    double Xb, et;
+    int i, vi, s, t;
+    int k = pmod->ncoeff;
+    int err = 0;
+
+    X = gretl_matrix_alloc(pmod->nobs, k + 2);
+    y = gretl_unit_matrix_new(pmod->nobs, 1);
+    b = gretl_matrix_alloc(k+2, 1);
+
+    if (X == NULL || y == NULL || b == NULL) {
+	err = E_ALLOC;
+	goto bailout;
+    }
+
+    s = 0;
+    for (t=pmod->t1; t<=pmod->t2; t++) {
+	if (na(pmod->uhat[t])) {
+	    continue;
+	}
+	et = pmod->uhat[t];
+	Xb = 0;
+	for (i=0; i<k; i++) {
+	    vi = pmod->list[i+2];
+	    Xb += pmod->coeff[i] * Z[vi][t];
+	    gretl_matrix_set(X, s, i, et * Z[vi][t]);
+	}
+	gretl_matrix_set(X, s, k, et * Xb * Xb);
+	gretl_matrix_set(X, s, k+1, et * Xb * Xb * Xb);
+	s++;
+    }
+
+    err = gretl_matrix_ols(y, X, b, NULL, NULL, NULL);
+
+    if (!err) {
+	double X2 = pmod->nobs;
+
+	gretl_matrix_multiply(X, b, y);
+	for (t=0; t<y->rows; t++) {
+	    X2 -= (1 - y->val[t]) * (1 - y->val[t]);
+	}
+
+	gretl_model_add_normality_test(pmod, X2);
+    }
+	
+ bailout:
+    
+    gretl_matrix_free(X);
+    gretl_matrix_free(y);
+    gretl_matrix_free(b);    
+
+    return err;
+}
+
+static void binary_model_add_stats (MODEL *pmod, int depvar, 
+				    double **Z, 
 				    const DATAINFO *pdinfo)
 {
+    const double *y = Z[depvar];
     int *act_pred;
     double *llt;
     double F, xx = 0.0;
@@ -1905,6 +2102,10 @@ static void binary_model_add_stats (MODEL *pmod, const double *y,
     }
 
     mle_criteria(pmod, 0);
+
+    if (pmod->ci == PROBIT) {
+	binary_probit_normtest(pmod, Z);
+    }
 }
 
 static MODEL 
@@ -1957,8 +2158,8 @@ binary_logit_probit (const int *inlist, double **Z, DATAINFO *pdinfo,
 
     if (!dmod.errcode) {
 	dmod = lsq(list, Z, pdinfo, OLS, OPT_A);
-	if (dmod.errcode == 0 && dmod.ncoeff != nx) {
-	    dmod.errcode = E_DATA;
+	if (dmod.errcode == 0 && dmod.ncoeff < nx) {
+	    dmod.errcode = E_SINGULAR;
 	}
     }
 
@@ -2007,7 +2208,7 @@ binary_logit_probit (const int *inlist, double **Z, DATAINFO *pdinfo,
     }
 
     if (!dmod.errcode) {
-	binary_model_add_stats(&dmod, Z[depvar], pdinfo);
+	binary_model_add_stats(&dmod, depvar, Z, pdinfo);
 	if (opt & OPT_A) {
 	    dmod.aux = AUX_AUX;
 	} else {
@@ -2132,6 +2333,7 @@ static int mn_logit_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll
     mnl_info *mnl = (mnl_info *) ptr;
     double x, xti, pti, p, g;
     int i, j, k, t, yt;
+    int err = 0;
 
     errno = 0;
 
@@ -2163,16 +2365,20 @@ static int mn_logit_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll
 	}
     }
 
-    return 1;
+    if (errno != 0) {
+	err = E_NAN;
+    }
+
+    return err;
 }
 
-/* multinomial logit; form the negative inverse of the analytical
+/* multinomial logit: form the negative of the analytical
    Hessian */
 
-static gretl_matrix *mnl_hess_inverse (mnl_info *mnl, int *err)
+static int mnl_hessian (double *theta, gretl_matrix *H, void *data)
 {
+    mnl_info *mnl = data;
     gretl_matrix_block *B;
-    gretl_matrix *H;
     gretl_matrix *x;
     gretl_matrix *xx;
     gretl_matrix *hjk;
@@ -2180,20 +2386,12 @@ static gretl_matrix *mnl_hess_inverse (mnl_info *mnl, int *err)
     int r, c;
     int i, j, k, t;
 
-    H = gretl_zero_matrix_new(mnl->npar, mnl->npar);
-    if (H == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
     B = gretl_matrix_block_new(&x, 1, mnl->k,
 			       &xx, mnl->k, mnl->k,
 			       &hjk, mnl->k, mnl->k,
 			       NULL);
     if (B == NULL) {
-	*err = E_ALLOC;
-	gretl_matrix_free(H);
-	return NULL;
+	return E_ALLOC;
     }
 
     r = c = 0;
@@ -2226,11 +2424,22 @@ static gretl_matrix *mnl_hess_inverse (mnl_info *mnl, int *err)
 
     gretl_matrix_block_destroy(B);
 
-    *err = gretl_invert_symmetric_matrix(H);
+    return 0;
+}
 
-    if (*err) {
-	gretl_matrix_free(H);
-	H = NULL;
+static gretl_matrix *mnl_hessian_inverse (mnl_info *mnl, int *err)
+{
+    gretl_matrix *H;
+
+    H = gretl_zero_matrix_new(mnl->npar, mnl->npar);
+    if (H == NULL) {
+	*err = E_ALLOC;
+    } else {
+	*err = mnl_hessian(mnl->theta, H, mnl);
+    }
+
+    if (!*err) {
+	*err = gretl_invert_symmetric_matrix(H);
     }
 
     return H;
@@ -2285,7 +2494,7 @@ static int mnl_add_variance_matrix (MODEL *pmod, mnl_info *mnl,
     gretl_matrix *V = NULL;
     int err = 0;
 
-    H = mnl_hess_inverse(mnl, &err);
+    H = mnl_hessian_inverse(mnl, &err);
     if (err) {
 	return err;
     }
@@ -2305,13 +2514,13 @@ static int mnl_add_variance_matrix (MODEL *pmod, mnl_info *mnl,
 
     if (!err) {
 	if (opt & OPT_R) {
-	    err = gretl_model_write_vcv(pmod, V);
+	    err = gretl_model_write_vcv(pmod, V, -1);
 	    if (!err) {
 		gretl_model_set_vcv_info(pmod, VCV_ML, VCV_QML);
 		pmod->opt |= OPT_R;
 	    }
 	} else {
-	    err = gretl_model_write_vcv(pmod, H);
+	    err = gretl_model_write_vcv(pmod, H, -1);
 	    if (!err) {
 		gretl_model_set_vcv_info(pmod, VCV_ML, VCV_HESSIAN);
 	    }
@@ -2743,6 +2952,7 @@ static MODEL mnl_model (const int *list, double ***pZ, DATAINFO *pdinfo,
     int *valcount = NULL;
     int *yvals = NULL;
     int n, k = list[0] - 1;
+    int use_bfgs = 0;
     int i, vi, t, s;
 
     /* we'll start with OLS to flush out data issues */
@@ -2804,10 +3014,27 @@ static MODEL mnl_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    mod.errcode = BFGS_max(mnl->theta, mnl->npar, maxit, 0.0, 
-			   &fncount, &grcount, mn_logit_loglik, C_LOGLIK,
-			   mn_logit_score, mnl, NULL,
-			   (prn != NULL)? OPT_V : OPT_NONE, prn);
+    if (libset_get_int(GRETL_OPTIM) == OPTIM_BFGS) {
+	use_bfgs = 1;
+    }
+
+    if (use_bfgs) {
+	mod.errcode = BFGS_max(mnl->theta, mnl->npar, maxit, 0.0, 
+			       &fncount, &grcount, mn_logit_loglik, C_LOGLIK,
+			       mn_logit_score, mnl, NULL,
+			       (prn != NULL)? OPT_V : OPT_NONE, prn);
+    } else {	
+	double crittol = 1.0e-8;
+	double gradtol = 1.0e-7;
+	
+	maxit = 100;
+	mod.errcode = newton_raphson_max(mnl->theta, mnl->npar, maxit, 
+					 crittol, gradtol, &fncount, 
+					 C_LOGLIK, mn_logit_loglik, 
+					 mn_logit_score, mnl_hessian, mnl,
+					 (prn != NULL)? OPT_V : OPT_NONE,
+					 prn);
+    } 
 
     if (!mod.errcode) {
 	mnl_finish(mnl, &mod, valcount, yvals, pdinfo, opt);
@@ -3433,10 +3660,15 @@ MODEL interval_model (int *list, double ***pZ, DATAINFO *pdinfo,
 /**
  * tobit_model:
  * @list: dependent variable plus list of regressors.
+ * @llim: left bound on dependent variable; use #NADBL for
+ * no left-censoring.
+ * @rlim: right bound on dependent variable; use #NADBL for
+ * no right-censoring.
  * @pZ: pointer to data array.
  * @pdinfo: dataset information.
- * @opt: may include OPT_V for verbose operation.
- * @prn: printing struct for iteration info (or NULL is this is not
+ * @opt: may include OPT_V for verbose operation, OPT_R for
+ * robust (QML) standard errors.
+ * @prn: printing struct for iteration info (or NULL if this is not
  * wanted).
  *
  * Produce Tobit estimates of the model given in @list.
@@ -3444,24 +3676,26 @@ MODEL interval_model (int *list, double ***pZ, DATAINFO *pdinfo,
  * Returns: a #MODEL struct, containing the estimates.
  */
 
-MODEL tobit_model (const int *list, double ***pZ, DATAINFO *pdinfo, 
+MODEL tobit_model (const int *list, double llim, double rlim,
+		   double ***pZ, DATAINFO *pdinfo, 
 		   gretlopt opt, PRN *prn)
 {
-    MODEL tmod;
-    void *handle;
-    MODEL (* tobit_estimate) (const int *, double ***, DATAINFO *, 
+    MODEL (* tobit_estimate) (const int *, double, double,
+			      double ***, DATAINFO *, 
 			      gretlopt, PRN *);
+    void *handle;
+    MODEL tmod;    
 
     gretl_error_clear();
 
-    tobit_estimate = get_plugin_function("tobit_estimate", &handle);
+    tobit_estimate = get_plugin_function("tobit_via_intreg", &handle);
     if (tobit_estimate == NULL) {
 	gretl_model_init(&tmod);
 	tmod.errcode = E_FOPEN;
 	return tmod;
     }
 
-    tmod = (*tobit_estimate) (list, pZ, pdinfo, opt, prn);
+    tmod = (*tobit_estimate) (list, llim, rlim, pZ, pdinfo, opt, prn);
 
     close_plugin(handle);
     set_model_id(&tmod);
