@@ -24,26 +24,24 @@
 
 #include "gretl_matrix.h"
 
-MODEL lsq (const int *list, double ***pZ, DATAINFO *pdinfo, 
+MODEL lsq (const int *list, double **Z, DATAINFO *pdinfo, 
 	   GretlCmdIndex ci, gretlopt opt);
 
-MODEL ar1_lsq (const int *list, double ***pZ, DATAINFO *pdinfo, 
-	    GretlCmdIndex ci, gretlopt opt, double rho);
-
 MODEL ar_model (const int *list,  
 		double ***pZ, DATAINFO *pdinfo, 
 		gretlopt opt, PRN *prn);
 
-double estimate_rho (const int *list, double ***pZ, DATAINFO *pdinfo,
-		     gretlopt opt, PRN *prn, int *err);
+MODEL ar1_model (const int *list,  
+		 double ***pZ, DATAINFO *pdinfo, 
+		 gretlopt opt, PRN *prn);
 
-MODEL lad (const int *list, double ***pZ, DATAINFO *pdinfo); 
+MODEL lad (const int *list, double **Z, DATAINFO *pdinfo); 
 
 MODEL quantreg (const gretl_matrix *tau, const int *list, 
-		double ***pZ, DATAINFO *pdinfo,
+		double **Z, DATAINFO *pdinfo,
 		gretlopt opt, PRN *prn);
 
-MODEL arma (const int *list, const char *pqspec,
+MODEL arma (const int *list, const int *pqlags,
 	    const double **Z, const DATAINFO *pdinfo, 
 	    gretlopt opt, PRN *prn);
 
@@ -58,12 +56,13 @@ MODEL panel_model (const int *list, double ***pZ, DATAINFO *pdinfo,
 MODEL ivreg (const int *list, double ***pZ, DATAINFO *pdinfo,
 	     gretlopt opt);
 
-MODEL arbond_model (const int *list, const char *istr, const double **Z, 
-		    const DATAINFO *pdinfo, gretlopt opt, 
-		    PRN *prn);
+MODEL arbond_model (const int *list, const char *ispec, const double **Z, 
+		    const DATAINFO *pdinfo, gretlopt opt, PRN *prn);
 
-MODEL dpd_model (const int *list, const double **Z, const DATAINFO *pdinfo, 
-		 gretlopt opt, PRN *prn);
+MODEL dpd_model (const int *list, const int *laglist,
+		 const char *ispec, const double **Z, 
+		 const DATAINFO *pdinfo, gretlopt opt, 
+		 PRN *prn);
 
 MODEL hsk_model (const int *list, double ***pZ, DATAINFO *pdinfo);
 
