@@ -545,14 +545,18 @@ static double fcast_get_ldv (Forecast *fc, int i, int t, int lag,
 
 /* Get forecasts, plus standard errors for same, for models without
    autoregressive errors and without "special requirements"
-   (e.g. nonlinearity).  The forecast standard errors include both
-   uncertainty over the error process and parameter uncertainty
-   (Davidson and MacKinnon method).
+   (e.g. nonlinearity).  
+
+   The forecast standard errors include both uncertainty over the
+   error process and parameter uncertainty (Davidson and MacKinnon
+   method) -- unless @opt contains OPT_M, in which case the standard
+   errors pertain to prediction of mean Y.
 */
 
 static int
 static_fcast_with_errs (Forecast *fc, MODEL *pmod, 
-			const double **Z, const DATAINFO *pdinfo) 
+			const double **Z, const DATAINFO *pdinfo,
+			gretlopt opt) 
 {
     gretl_matrix *V = NULL;
     gretl_vector *Xt = NULL;
@@ -612,7 +616,9 @@ static_fcast_with_errs (Forecast *fc, MODEL *pmod,
 	if (na(vyh)) {
 	    err = 1;
 	} else {
-	    vyh += s2;
+	    if (!(opt & OPT_M)) {
+		vyh += s2;
+	    }
 	    if (vyh >= 0.0) {
 		fc->sderr[t] = sqrt(vyh);
 	    } else {
@@ -1346,6 +1352,14 @@ static int arma_fcast (Forecast *fc, MODEL *pmod,
 	goto bailout;
     }
 
+#if 0
+    if (theta != NULL) {
+	for (i=0; i<=q; i++) {
+	    fprintf(stderr, "integrated theta[%d] = %g\n", i, theta[i]);
+	}
+    }
+#endif
+
     beta = arma_model_get_x_coeffs(pmod);
 
     if (want_x_beta_prep(pmod, xlist)) {
@@ -2245,7 +2259,7 @@ static int real_get_fcast (FITRESID *fr, MODEL *pmod,
     if (pmod->ci == PANEL && (pmod->opt & OPT_F)) {
 	err = fixed_effects_fcast(&fc, pmod, Z, pdinfo);
     } else if (DM_errs) {
-	err = static_fcast_with_errs(&fc, pmod, Z, pdinfo);
+	err = static_fcast_with_errs(&fc, pmod, Z, pdinfo, opt);
     } else if (pmod->ci == NLS) {
 	err = nls_fcast(&fc, pmod, pZ, pdinfo);
     } else if (SIMPLE_AR_MODEL(pmod->ci) || dummy_AR) {
@@ -3137,9 +3151,9 @@ int do_forecast (const char *str, double ***pZ, DATAINFO *pdinfo,
 	return E_BADSTAT;
     }
 
-    if ((opt & (OPT_R | OPT_I)) && type != GRETL_OBJ_EQN) {
-	/* "rolling" or "integrate": single equations only */
-	err = E_NOTIMP;
+    if ((opt & (OPT_R | OPT_I | OPT_U)) && type != GRETL_OBJ_EQN) {
+	/* "rolling", "integrate", plot option: single equations only */
+	err = E_BADOPT;
     } else if (type == GRETL_OBJ_EQN) {
 	err = model_do_forecast(str, ptr, pZ, pdinfo, opt, prn);
     } else if (type == GRETL_OBJ_SYS || type == GRETL_OBJ_VAR) {
@@ -3315,11 +3329,13 @@ void forecast_options_for_model (MODEL *pmod, const double **Z,
 
     dv = gretl_model_get_depvar(pmod);
 
-    if (pmod->ci == OLS && is_standard_diff(dv, pdinfo, NULL)) {
-	*flags |= FC_INTEGRATE_OK;
-    }
-
-    if (pmod->ci == NLS) {
+    if (pmod->ci == OLS) {
+	if (is_standard_diff(dv, pdinfo, NULL)) {
+	    *flags |= FC_INTEGRATE_OK;
+	} else {
+	    *flags |= FC_MEAN_OK;
+	}
+    } else if (pmod->ci == NLS) {
 	/* we'll try winging it! */
 	if (gretl_model_get_int(pmod, "dynamic") && pmod->t2 < pdinfo->n - 1) {
 	    *flags |= FC_AUTO_OK;
