@@ -223,37 +223,31 @@ int rank_series (const double *x, double *y, int f,
 int diff_series (const double *x, double *y, int f, 
 		 const DATAINFO *pdinfo)
 {
-    int t1 = pdinfo->t1;
     int k = (f == F_SDIFF)? pdinfo->pd : 1;
-    int t, err = 0;
+    int t, t1 = pdinfo->t1;
 
     if (t1 < k) {
 	t1 = k;
     }
 
     for (t=t1; t<=pdinfo->t2; t++) {
-
 	if (dataset_is_panel(pdinfo) && t % pdinfo->pd == 0) {
 	    /* skip first observation in panel unit */
 	    continue;
 	}
-
 	if (na(x[t]) || na(x[t-k])) {
 	    continue;
 	}
-
 	if (f == F_DIFF || f == F_SDIFF) {
 	    y[t] = x[t] - x[t-k];
 	} else if (f == F_LDIFF) {
-	    if (x[t] <= 0.0 || x[t-k] <= 0.0) {
-		err = E_LOGS; /* FIXME: should be warning? */
-	    } else {
+	    if (x[t] > 0.0 && x[t-k] > 0.0) {
 		y[t] = log(x[t]) - log(x[t-k]);
-	    }
+	    } 
 	}
     }
 
-    return 0; /* see FIXME above */
+    return 0;
 }
 
 /**
@@ -2165,11 +2159,19 @@ int dummy (double ***pZ, DATAINFO *pdinfo, int center)
     char vname[VNAMELEN];
     char vlabel[MAXLABEL];
     int vi, t, pp;
-    int ndums = pdinfo->pd, nnew = 0;
-    int di, di0 = pdinfo->v;
+    int ndums, nnew = 0;
+    int di, di0;
     double xx, dx;
 
-    if (ndums == 1 || ndums > 99999) {
+    if (pdinfo == NULL || pdinfo->n == 0) {
+	gretl_errmsg_set(_("No dataset is in place"));
+	return 0;
+    }
+
+    ndums = pdinfo->pd;
+    di0 = pdinfo->v;
+
+    if (ndums < 2 || ndums > 99999) {
 	gretl_errmsg_set(_("This command won't work with the current periodicity"));
 	return 0;
     }
@@ -3886,7 +3888,11 @@ static int theil_decomp (double *m, double MSE,
     return err;
 }
 
-/* cf. http://www.economicsnetwork.ac.uk/showcase/cook_forecast
+/* 
+  Forecast evaluation statistics: @y is the data series, @f the
+  forecast.
+
+   cf. http://www.economicsnetwork.ac.uk/showcase/cook_forecast
    by Steven Cook of Swansea University <s.cook@Swansea.ac.uk>
 
    OPT_D indicates that we should include the Theil decomposition.
@@ -3899,6 +3905,7 @@ gretl_matrix *forecast_stats (const double *y, const double *f,
     gretl_matrix *m = NULL;
     double ME, MSE, MAE, MPE, MAPE, U;
     double x, u[2];
+    int nstats = (opt & OPT_D)? 9 : 6;
     int t, T = t2 - t1 + 1;
 
     for (t=t1; t<=t2; t++) {
@@ -3908,6 +3915,12 @@ gretl_matrix *forecast_stats (const double *y, const double *f,
 	}
     }
 
+    m = gretl_column_vector_alloc(nstats);
+    if (m == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
     ME = MSE = MAE = MPE = MAPE = U = 0.0;
     u[0] = u[1] = 0.0;
 
@@ -3946,17 +3959,6 @@ gretl_matrix *forecast_stats (const double *y, const double *f,
 	U = sqrt(u[0] / T) / sqrt(u[1] / T);
     }
 
-    if (opt & OPT_D) {
-	m = gretl_column_vector_alloc(9);
-    } else {
-	m = gretl_column_vector_alloc(6);
-    }
-    
-    if (m == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
     gretl_vector_set(m, 0, ME);
     gretl_vector_set(m, 1, MSE);
     gretl_vector_set(m, 2, MAE);
@@ -4249,3 +4251,95 @@ gretl_matrix *matrix_chowlin (const gretl_matrix *Y,
     
     return ret;
 }
+
+int list_ok_dollar_vars (double ***pZ, DATAINFO *pdinfo, PRN *prn)
+{
+    int nm = 0;
+    int i;
+
+    pprintf(prn, "\n%s\n", _("model-related"));
+
+    for (i=R_MAX+1; i<M_MAX; i++) {
+	GretlType type = GRETL_TYPE_NONE;
+	double x = NADBL;
+	double *px = NULL;
+	gretl_matrix *m = NULL;
+	int *list = NULL;
+	int err = 0;
+
+	if (i < M_SCALAR_MAX) {
+	    x = saved_object_get_scalar(NULL, i, pZ, pdinfo, &err);
+	    if (!na(x)) {
+		type = GRETL_TYPE_DOUBLE;
+	    }
+	} else if (i > M_SCALAR_MAX && i < M_SERIES_MAX) {
+	    type = GRETL_TYPE_SERIES;
+	    px = saved_object_get_series(NULL, i, pdinfo, &err);
+	    if (err) {
+		if (i == M_UHAT || i == M_YHAT || i == M_SIGMA) {
+		    /* maybe the result is a matrix? */
+		    int ci = 0;
+		    GretlObjType otype = gretl_model_get_type_and_ci(NULL, &ci);
+
+		    if (otype != GRETL_OBJ_EQN) {
+			type = GRETL_TYPE_MATRIX;
+		    } else if ((i == M_UHAT || i == M_YHAT) && ci == BIPROBIT) {
+			type = GRETL_TYPE_MATRIX;
+		    }
+		    if (type == GRETL_TYPE_MATRIX) {
+			m = saved_object_get_matrix(NULL, i, &err);
+		    }
+		}
+	    }
+	} else if (i > M_SERIES_MAX && i < M_MATRIX_MAX) {
+	    type = GRETL_TYPE_MATRIX;
+	    m = saved_object_get_matrix(NULL, i, &err);
+	} else if (i > M_MATRIX_MAX && i < M_MBUILD_MAX) {
+	    type = GRETL_TYPE_MATRIX;
+	    m = saved_object_build_matrix(NULL, i,
+					  (const double **) *pZ,
+					  pdinfo, &err);
+	} else {
+	    type = GRETL_TYPE_LIST;
+	    list = saved_object_get_list(NULL, i, &err);
+	}
+
+	if (!err && type != GRETL_TYPE_NONE) {
+	    const char *typestr = gretl_arg_type_name(type);
+
+	    if (!na(x)) {
+		pprintf(prn, " %s (%s: %g)\n", mvarname(i), typestr, x);
+	    } else {
+		pprintf(prn, " %s (%s)\n", mvarname(i), typestr);
+	    }
+	    free(px);
+	    gretl_matrix_free(m);
+	    free(list);
+	    nm++;
+	}
+    }
+
+    if (nm == 0) {
+	pprintf(prn, " %s\n", _("none"));
+    }
+
+    pprintf(prn, "\n%s\n", _("other"));
+
+    for (i=1; i<R_SCALAR_MAX; i++) {
+	double x;
+
+	if (i == R_NSCAN) {
+	    /* this will be trashed before long */
+	    continue;
+	}
+
+	x = dvar_get_scalar(i, pdinfo, NULL);
+	if (!na(x)) {
+	    pprintf(prn, " %s (scalar: %g)\n", dvarname(i), x);
+	}
+    }
+
+    pputc(prn, '\n');
+
+    return 0;
+}
