@@ -87,7 +87,7 @@ struct str_table dvars[] = {
     { R_WINDOWS,   "$windows" },
     { R_VERSION,   "$version" },
     { R_ERRNO,     "$error" },
-    { R_SWITCH,    "$switch" },
+    { R_SEED,      "$seed" },
     { 0,           NULL },
 };
 
@@ -119,6 +119,10 @@ struct str_table mvars[] = {
     { M_RHO,     "$rho" },
     { M_COMPAN,  "$compan" },
     { M_XTXINV,  "$xtxinv" },
+    { M_VECG,    "$vecGamma" },
+    { M_VMA,     "$vma" },
+    { M_FEVD,    "$fevd" },
+    { M_EVALS,   "$evals" },
     { M_JALPHA,  "$jalpha" }, 
     { M_JBETA,   "$jbeta" },
     { M_JVBETA,  "$jvbeta" },
@@ -139,6 +143,8 @@ struct str_table mvars[] = {
     { M_EHAT,    "$ehat" },
     { M_MNLPROBS, "$mnlprobs" },
     { M_XLIST,   "$xlist" },
+    { M_COMMAND, "$command" },
+    { M_DEPVAR,  "$depvar" },
     { 0,         NULL }
 };
 
@@ -270,11 +276,13 @@ struct str_table funcs[] = {
     { F_QR,       "qrdecomp" },
     { F_EIGSYM,   "eigensym" },
     { F_EIGGEN,   "eigengen" },
+    { F_EIGSOLVE, "eigsolve" },
     { F_NULLSPC,  "nullspace" },
     { F_PRINCOMP, "princomp" },
     { F_MEXP,     "mexp" },
     { F_FDJAC,    "fdjac" },
     { F_BFGSMAX,  "BFGSmax" },
+    { F_NRMAX,    "NRmax" },
     { F_OBSNUM,   "obsnum" },
     { F_ISSERIES, "isseries" },
     { F_ISSCALAR, "isscalar" },
@@ -289,12 +297,12 @@ struct str_table funcs[] = {
     { F_CRIT,     "critical" },
     { F_RANDGEN,  "randgen" },
     { F_URCPVAL,  "urcpval" },
-    { F_MAKEMASK, "makemask" },
     { F_VALUES,   "values" },
     { F_MSHAPE,   "mshape" },
     { F_SVD,      "svd" },
     { F_MOLS,     "mols" },
     { F_MPOLS,    "mpols" },
+    { F_MRLS,     "mrls" },
     { F_MREAD,    "mread" },
     { F_MWRITE,   "mwrite" },
     { F_MCSEL,    "selifc" },
@@ -311,6 +319,7 @@ struct str_table funcs[] = {
     { F_KSIMUL,   "ksimul" },
     { F_TRIMR,    "trimr" },
     { F_GETENV,   "getenv" },
+    { F_NGETENV,  "ngetenv" },
     { F_ARGNAME,  "argname" },
     { F_OBSLABEL, "obslabel" },
     { F_READFILE, "readfile" },
@@ -323,6 +332,7 @@ struct str_table funcs[] = {
     { F_VARNUM,   "varnum" },
     { F_TOLOWER,  "tolower" },
     { F_COLNAMES, "colnames" },
+    { F_ROWNAMES, "rownames" },
     { F_LJUNGBOX, "ljungbox" },
     { F_MSORTBY,  "msortby" },
     { F_LINCOMB,  "lincomb" },
@@ -352,6 +362,13 @@ struct str_table funcs[] = {
     { F_CHOWLIN,  "chowlin" },
     { F_VARSIMUL, "varsimul" },
     { F_STRSPLIT, "strsplit" },
+    { F_INLIST,   "inlist" },
+    { F_ERRMSG,   "errmsg" },
+    { F_ISCONST,  "isconst" },
+    { F_IRF,      "irf" },
+    { F_INBUNDLE, "inbundle" },
+    { F_STRSUB,   "strsub" },
+    { F_COLNAME,  "colname" },
     { 0,          NULL }
 };
 
@@ -415,8 +432,7 @@ static int real_function_lookup (const char *s, int a)
 {
     static GHashTable *fht;
     gpointer p;
-    int ret = 0;
-
+ 
     if (s == NULL) {
 	/* cleanup signal */
 	if (fht != NULL) {
@@ -432,10 +448,10 @@ static int real_function_lookup (const char *s, int a)
     
     p = g_hash_table_lookup(fht, s);
     if (p != NULL) {
-	ret = GPOINTER_TO_INT(p);
+	return GPOINTER_TO_INT(p);
     }
 
-    if (ret == 0 && a == ALLOW_ALIAS) {
+    if (a == ALLOW_ALIAS) {
 	int i;
 
 	for (i=0; func_alias[i].id != 0; i++) {
@@ -445,7 +461,7 @@ static int real_function_lookup (const char *s, int a)
 	} 
     }   
 
-    return ret;
+    return 0;
 }
 
 void gretl_function_hash_cleanup (void)
@@ -636,14 +652,14 @@ int genr_function_word (const char *s)
     return ret;
 }
 
-static void undefined_symbol_error (const char *s, parser *p)
+void undefined_symbol_error (const char *s, parser *p)
 {
     parser_print_input(p);
 
     if (p->ch == '.') {
-	gretl_errmsg_sprintf(_("%s: no such object\n"), s);
+	gretl_errmsg_sprintf(_("%s: no such object"), s);
     } else {
-	gretl_errmsg_sprintf(_("The symbol '%s' is undefined\n"), s);
+	gretl_errmsg_sprintf(_("The symbol '%s' is undefined"), s);
     }
 
     p->err = E_UNKVAR;
@@ -653,8 +669,9 @@ static void function_noargs_error (const char *s, parser *p)
 {
     parser_print_input(p);
 
-    pprintf(p->prn, _("'%s': no argument was given\n"), s);
-    gretl_errmsg_sprintf(_("'%s': no argument was given\n"), s);
+    pprintf(p->prn, _("'%s': no argument was given"), s);
+    pputc(p->prn, '\n');
+    gretl_errmsg_sprintf(_("'%s': no argument was given"), s);
 
     p->err = E_ARGS;
 }
@@ -921,7 +938,8 @@ static void look_up_word (const char *s, parser *p)
 
 #define could_be_matrix(t) (model_data_matrix(t) || \
 			    model_data_matrix_builder(t) || \
-			    t == M_UHAT || t == M_YHAT)
+			    t == M_UHAT || t == M_YHAT || \
+			    t == R_TEST_STAT || R_TEST_PVAL)
 
 static void word_check_next_char (const char *s, parser *p)
 {
@@ -952,7 +970,8 @@ static void word_check_next_char (const char *s, parser *p)
 	if (p->sym == UMAT) {
 	    /* slice of user matrix */
 	    p->sym = MSL;
-	} else if (p->sym == MVAR && could_be_matrix(p->idnum)) {
+	} else if ((p->sym == MVAR || p->sym == DVAR) && 
+		   could_be_matrix(p->idnum)) {
 	    /* slice of $ matrix */
 	    p->sym = DMSL;
 	} else if (p->sym == USERIES) {
@@ -1243,6 +1262,10 @@ void lex (parser *p)
 	    p->sym = B_DIV;
 	    parser_getc(p);
 	    return;
+        case '\\': 
+	    p->sym = B_LDIV;
+	    parser_getc(p);
+	    return;
         case '%': 
 	    p->sym = B_MOD;
 	    parser_getc(p);
@@ -1401,8 +1424,11 @@ void lex (parser *p)
 		parser_getc(p);
 		return;
 	    }
-	    if (bare_data_type(p->sym) || closing_sym(p->sym) ||
-		(p->targ == LIST && p->sym == LAG)) {
+	    if (p->targ == LIST && *(p->point - 2) == ' ' && 
+		(bare_data_type(p->sym) || closing_sym(p->sym) ||
+		 (p->sym == LAG))) {
+		/* may be forming a list, but only if there are 
+		   spaces between the terms */
 		p->sym = B_LCAT;
 		return;
 	    }
@@ -1513,6 +1539,8 @@ const char *getsymb (int t, const parser *p)
 	return "'";
     case B_DIV: 
 	return "/";
+    case B_LDIV:
+	return "\\";
     case B_MOD: 
 	return "%";
     case B_POW: 
