@@ -224,27 +224,6 @@ int function_from_string (const char *s)
     return 0;
 }
 
-/**
- * is_gretl_function_call:
- * @s: the string to check.
- *
- * Returns: 1 if @s starts with the name of a built-in 
- * gretl function, otherwise 0.
- */
-
-int is_gretl_function_call (const char *s)
-{
-    char c, word[12];
-
-    sscanf(s, "%10[^ (]", word);
-    c = s[strlen(word)];
-    if (c == ' ' || c == '(') {
-	return function_lookup(word);
-    } else {
-	return 0;
-    }
-}
-
 static const char *reswords[] = {
     /* constants */
     "const",
@@ -362,53 +341,57 @@ static int try_for_listvar (const DATAINFO *pdinfo, const char *s)
 int series_index (const DATAINFO *pdinfo, const char *varname)
 {
     const char *s = varname;
-    int i, fd, ret;
+    int fd = 0, ret = -1;
 
-    if (pdinfo == NULL) {
-	return -1;
-    }
+    if (pdinfo != NULL) {
+	int i;
+	
+	ret = pdinfo->v;
 
-    ret = pdinfo->v;
-
-    if (s == NULL || *s == 0 || isdigit(*s)) {
-	return ret;
-    }
-
-    if (!strcmp(s, "const")) {
-	return 0;
-    }
-
-    if (strchr(s, '.')) {
-	return try_for_listvar(pdinfo, s);
-    }
-
-    fd = gretl_function_depth();
-
-    if (fd == 0) {
-	/* not inside a user function: easy */
-	for (i=1; i<pdinfo->v; i++) { 
-	    if (strcmp(pdinfo->varname[i], s) == 0) {
-		ret = i;
-		break;
-	    }
+	if (s == NULL || *s == '\0' || isdigit(*s)) {
+	    goto bailout;
 	}
-    } else {
-	/* The condition for recognizing a series by name, if we're
-	   inside a user function: it must exist at the current level
-	   of function execution, and its tenure at that level must
-	   not just be the result of its being a member of a list
-	   that was passed as an argument.
-	*/
-	for (i=1; i<pdinfo->v; i++) { 
-	    if (fd == STACK_LEVEL(pdinfo, i) &&
-		!var_is_listarg(pdinfo, i) && 
-		strcmp(pdinfo->varname[i], s) == 0) {
-		ret = i;
-		break;
+
+	if (strcmp(s, "const") == 0) {
+	    ret = 0;
+	    goto bailout;
+	}
+
+	if (strchr(s, '.') != NULL) {
+	    ret = try_for_listvar(pdinfo, s);
+	    goto bailout;
+	}
+
+	fd = gretl_function_depth();
+
+	if (fd == 0) {
+	    /* not inside a user function: easy */
+	    for (i=1; i<pdinfo->v; i++) { 
+		if (strcmp(pdinfo->varname[i], s) == 0) {
+		    ret = i;
+		    break;
+		}
+	    }
+	} else {
+	    /* The condition for recognizing a series by name, if we're
+	       inside a user function: it must exist at the current level
+	       of function execution, and its tenure at that level must
+	       not just be the result of its being a member of a list
+	       that was passed as an argument.
+	    */
+	    for (i=1; i<pdinfo->v; i++) { 
+		if (fd == STACK_LEVEL(pdinfo, i) &&
+		    !var_is_listarg(pdinfo, i) && 
+		    strcmp(pdinfo->varname[i], s) == 0) {
+		    ret = i;
+		    break;
+		}
 	    }
 	}
     }
 
+ bailout:
+
 #if GEN_LEVEL_DEBUG
     fprintf(stderr, "series_index for '%s', fd = %d: got %d (pdinfo->v = %d)\n", 
 	    s, fd, ret, pdinfo->v);
@@ -466,6 +449,10 @@ static int gen_special (const char *s, const char *line,
     int write_label = 0;
     int err = 0;
 
+    if (pdinfo == NULL || pdinfo->n == 0) {
+	return E_NODATA;
+    }
+
     if (!strcmp(s, "markers")) {
 	return generate_obs_markers(line, pZ, pdinfo);
     } else if (!strcmp(s, "dummy")) {
@@ -518,7 +505,11 @@ static int gen_special (const char *s, const char *line,
 	p->err = 0;
 	p->prn = prn;
 	gen_write_message(p, orig_v, prn);
-    }	    
+    }
+
+    if (pdinfo->v > orig_v) {
+	set_dataset_is_changed();
+    }
 
     return err;
 }
@@ -893,7 +884,7 @@ parser *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo,
 /* run a previously compiled generator */
 
 int execute_genr (parser *p, double ***pZ, DATAINFO *pdinfo,
-		  gretlopt opt, PRN *prn)
+		  PRN *prn)
 {
 #if GDEBUG
     fprintf(stderr, "\n*** execute_genr: p=%p, LHS='%s', Z=%p\n", 
