@@ -32,10 +32,8 @@ typedef enum {
     R_WINDOWS,    /* running on MS Windows (1) or not (0) */
     R_VERSION,    /* gretl version number */
     R_ERRNO,      /* internal gretl error code */
-    R_SWITCH,     /* integer switch set via "--switch=" on command line */
+    R_SEED,       /* RNG seed */
     R_DSET_MAX,   /* separator */
-    R_TEST_STAT,  /* test statistic from last explicit test performed */
-    R_TEST_PVAL,  /* p-value from last explicit test performed */
     R_TEST_LNL,   /* log-likelihood from last test (if applicable) */
     R_KLNL,       /* log-likelihood from Kalman filter (if applicable) */
     R_KS2,        /* variance estimate from Kalman filter (if applicable) */
@@ -45,6 +43,9 @@ typedef enum {
     R_SCALAR_MAX, /* separator: scalars vs series */
     R_INDEX,      /* consecutive observations index */
     R_PUNIT,      /* 1-based panel unit index */
+    R_SERIES_MAX, /* separator: series vs matrices */
+    R_TEST_STAT,  /* last test statistic(s) (scalar or matrix) */
+    R_TEST_PVAL,  /* last test p-value(s) (scalar or matrix) */
     R_MAX
 } RetrievalIndex;
 
@@ -86,13 +87,14 @@ typedef enum {
     M_RHO,        /* autoregressive coefficients */
     M_COMPAN,     /* VAR companion matrix */
     M_XTXINV,     /* VARs, VECMs: X'X^{-1} */
+    M_VECG,       /* VECMs: the Gamma matrices */
+    M_EVALS,      /* VECMs: eigenvalues */
     M_JALPHA,     /* Johansen's alpha */
     M_JBETA,      /* Johansen's beta */
     M_JVBETA,     /* Covariance matrix for Johansen's normalized beta */
     M_JS00,       /* VECM residual covariance matrix (1st differences) */
     M_JS11,       /* VECM residual covariance matrix (levels) */
     M_JS01,       /* VECM residual cross-product matrix */
-    M_EC,         /* VECM error-correction terms */
     M_HAUSMAN,    /* Hausman test after tsls or fixed effects */
     M_SARGAN,     /* Sargan over-identification test after tsls */
     M_SYSGAM,     /* Parameter matrix Gamma (simultaneous systems) */
@@ -104,10 +106,16 @@ typedef enum {
     M_KLLT,       /* Kalman log-likelihood, per time-step */
     M_KUHAT,      /* Kalman: current prediction error */
     M_EHAT,       /* ARMA: vector of estimated innovations */
-    M_MATRIX_MAX, /* -- SEPARATOR, matrices/matrix-builders -- */
+    M_MATRIX_MAX, /* -- SEPARATOR, end of matrices -- */
+    M_EC,         /* VECM error-correction terms */
+    M_VMA,        /* VARs, VECMs: vector moving average representation */
+    M_FEVD,       /* VAR variance decomposition */
     M_MNLPROBS,   /* case probabilities for multinomial logit */
-    M_MBUILD_MAX, /* -- SEPARATOR, matrix-builders/lists -- */
+    M_MBUILD_MAX, /* -- SEPARATOR, end of matrix-builders -- */
     M_XLIST,      /* list of regressors */
+    M_LIST_MAX,   /* -- SEPARATOR, end of lists -- */
+    M_COMMAND,    /* model command word */
+    M_DEPVAR,     /* name of dependent variable */
     M_MAX         /* sentinel */
 } ModelDataIndex;
 
@@ -116,7 +124,8 @@ typedef enum {
 #define model_data_matrix(i) (i > M_SERIES_MAX && i < M_MATRIX_MAX)
 #define model_data_matrix_builder(i) (i > M_MATRIX_MAX && \
 				      i < M_MBUILD_MAX)
-#define model_data_list(i)   (i > M_MBUILD_MAX && i < M_MAX)
+#define model_data_list(i)   (i > M_MBUILD_MAX && i < M_LIST_MAX)
+#define model_data_string(i) (i > M_LIST_MAX && i < M_MAX)
 
 typedef struct parser_ GENERATOR;
 
@@ -127,7 +136,7 @@ GENERATOR *genr_compile (const char *s, double ***pZ, DATAINFO *pdinfo,
 			 gretlopt opt, int *err);
 
 int execute_genr (GENERATOR *genr, double ***pZ, DATAINFO *pdinfo,
-		  gretlopt opt, PRN *prn);
+		  PRN *prn);
 
 void destroy_genr (GENERATOR *genr);
 
@@ -194,8 +203,6 @@ int genr_get_series_max (GENERATOR *genr);
 
 int function_from_string (const char *s);
 
-int is_gretl_function_call (const char *s);
-
 int function_lookup (const char *s);
 
 int const_lookup (const char *s);
