@@ -30,8 +30,6 @@
 # define MDEBUG 0
 #endif
 
-#define set_transpose(n) (n->flags |= TRANSP_NODE)
-
 static NODE *powterm (parser *p);
 
 #if SDEBUG
@@ -42,7 +40,7 @@ static void notify (const char *s, NODE *t, parser *p)
 }
 #endif
 
-static NODE *newempty (int t)
+static NODE *newempty (void)
 {  
     NODE *n = malloc(sizeof *n);
 
@@ -51,7 +49,7 @@ static NODE *newempty (int t)
 #endif
 
     if (n != NULL) {
-	n->t = t;
+	n->t = EMPTY;
 	n->v.idnum = 0;
 	n->flags = 0;
 	n->vnum = NO_VNUM;
@@ -280,29 +278,6 @@ static void unmatched_symbol_error (int c, parser *p)
 #define matrix_ref_node(p) (p->sym == UMAT || (p->sym == MVAR && \
                             model_data_matrix(p->idnum)))
 
-/* try to recognize a unary ', indicating that we're taking
-   the transpose of the preceding matrix */
-
-static int unary_apost (parser *p) 
-{
-    if (p->ch == '\'') {
-	/* peek ahead */
-	int c = parser_next_nonspace_char(p);
-
-	if (isalpha(c) || c == '$' || c == '(') {
-	    /* next symbol is variable or expression: the
-	       apostrophe must be the binary operator, 
-	       B_TRMUL
-	    */
-	    return 0;
-	} else {
-	    return 1;
-	}
-    } 
-
-    return 0;
-}
-
 static NODE *base (parser *p, NODE *up)
 { 
     NODE *t = NULL;
@@ -332,22 +307,16 @@ static NODE *base (parser *p, NODE *up)
     case LIST:
     case BUNDLE:
 	t = newref(p, p->sym);
-	if (matrix_ref_node(p) && unary_apost(p)) {
-	    set_transpose(t);
-	    parser_getc(p);
-	}
 	lex(p);
 	break;
     case DUM:
 	if (p->idnum == DUM_NULL) {
-	    t = newempty(EMPTY);
+	    t = newempty();
 	} else {
 	    t = newref(p, p->sym);
 	}
 	lex(p);
 	break;
-    case B_SUB:
-    case B_ADD:
     case B_RANGE:
     case P_DOT:
 	lex(p);
@@ -357,10 +326,6 @@ static NODE *base (parser *p, NODE *up)
 	lex(p);
 	t = expr(p);
 	if (p->sym == G_RPR) {
-	    if (up != NULL && up->t != LAG && unary_apost(p)) {
-		set_transpose(up);
-		parser_getc(p);
-	    } 
 	    lex(p);
 	} else if (p->err == 0) {
 	    expected_symbol_error(')', p);
@@ -374,12 +339,6 @@ static NODE *base (parser *p, NODE *up)
 	    t = obs_node(p);
 	}
 	if (p->sym == G_RBR) {
-	    if (up->t == MSL || up->t == DMSL) {
-		if (p->ch == '\'') {
-		    set_transpose(up);
-		    parser_getc(p);
-		}
-	    }
 	    lex(p);
 	} else if (p->err == 0) {
 	    expected_symbol_error(']', p);
@@ -391,8 +350,19 @@ static NODE *base (parser *p, NODE *up)
 	break;
     }
 
+    if (p->err == 0 && p->sym != EOT && p->sym != QUERY &&
+	!(p->sym > U_MAX && p->sym < PUNCT_MAX)) {
+	/* catch high-level syntax errors: a "base" is followed
+	   by something that cannot really follow, e.g. a series
+	   is followed by another series
+	*/
+	fprintf(stderr, "gensytax: base: bad exit sym = %d\n", p->sym);
+	p->err = E_PARSE;
+    }
+
 #if SDEBUG
     notify("base", t, p);
+    fprintf(stderr, "on exit from base, p->sym = %d\n", p->sym);
 #endif    
 
     return t;
@@ -634,7 +604,7 @@ static NODE *get_final_string_arg (parser *p, int sym, int eat_last)
 
     if (wrapped) {
 	unwrap_string_arg(p);
-    } else if (sym != F_ISSTRING) {
+    } else if (sym != F_ISSTRING && sym != F_ISNULL) {
 	/* not quoted: give priority to string variables */
 	const char *s = get_string_by_name(p->idstr);
 
@@ -766,14 +736,11 @@ static void get_matrix_def (NODE *t, parser *p, int *sub)
 	    if (p->sym == P_COM) {
 		lex(p);
 	    } else if (p->sym == P_SEMI) {
-		n = newempty(EMPTY);
+		n = newempty();
 		p->err = push_bn_node(t, n);
 		lex(p);
 	    } else if (p->sym == G_RCB) {
-		if (p->ch == '\'') {
-		    set_transpose(t);
-		    parser_getc(p);
-		} else if (p->ch == '[') {
+		if (p->ch == '[') {
 		    parser_ungetc(p);
 		    *sub = 1;
 		}
@@ -811,7 +778,7 @@ static void get_slice_parts (NODE *t, parser *p)
 	lex(p);
 	if (p->sym == P_COM) {
 	    /* empty row spec, OK */
-	    t->v.b2.l = newempty(EMPTY);
+	    t->v.b2.l = newempty();
 	} else {
 	    t->v.b2.l = expr(p);
 	}
@@ -821,14 +788,15 @@ static void get_slice_parts (NODE *t, parser *p)
 	    lex(p);
 	    if (p->sym == P_COM || p->sym == G_RBR) {
 		/* reached end: second part implicitly empty */
-		t->v.b2.l->v.b2.r = newempty(EMPTY);
+		t->v.b2.l->v.b2.r = newempty();
 	    } else {
 		t->v.b2.l->v.b2.r = expr(p);
 	    }
 	}
 	if (p->sym == G_RBR) {
 	    /* no comma, no second arg string: may be OK */
-	    t->v.b2.r = newempty(ABSENT);
+	    t->v.b2.r = newempty();
+	    t->v.b2.r->t = ABSENT;
 	    lex(p);
 	    set_matrix_slice_off();
 	    return;
@@ -837,7 +805,7 @@ static void get_slice_parts (NODE *t, parser *p)
 	    lex(p);
 	    if (p->sym == G_RBR) {
 		/* empty column spec, OK */
-		t->v.b2.r = newempty(EMPTY);
+		t->v.b2.r = newempty();
 	    } else {
 		t->v.b2.r = expr(p);
 	    }
@@ -847,16 +815,12 @@ static void get_slice_parts (NODE *t, parser *p)
 		lex(p);
 		if (p->sym == G_RBR) {
 		    /* reached end: second part implicitly empty */
-		    t->v.b2.r->v.b2.r = newempty(EMPTY);
+		    t->v.b2.r->v.b2.r = newempty();
 		} else {
 		    t->v.b2.r->v.b2.r = expr(p);
 		}
 	    }
 	    if (p->sym == G_RBR) {
-		if (p->ch == '\'') {
-		    set_transpose(t); /* ?? */
-		    parser_getc(p);
-		}
 		lex(p);
 	    } else {
 		cexp = ']';
@@ -914,7 +878,7 @@ static void pad_parent (NODE *parent, int k, int i, parser *p)
     int j;
 
     for (j=i; j<k; j++) {
-	n = newempty(EMPTY);
+	n = newempty();
 	attach_child(parent, n, k, j, p);
     }
 }
@@ -934,7 +898,7 @@ static void get_args (NODE *t, parser *p, int f, int k, int opt, int *next)
     int i = 0;
 
 #if SDEBUG
-    fprintf(stderr, "get_args: k = %d...\n", k);
+    fprintf(stderr, "get_args: f = %d, k = %d...\n", f, k);
 #endif    
 
     if (p->sym != G_LPR) {
@@ -990,14 +954,9 @@ static void get_args (NODE *t, parser *p, int f, int k, int opt, int *next)
 	    if (i < k) {
 		pad_parent(t, k, i, p);
 	    } 
-	    /* matrix functions: handle trailing transpose */
-	    if (p->ch == '\'') {
-		*next = p->ch;
-		parser_getc(p);
-	    }
 	    lex(p);
 	    if (p->sym == G_LBR) {
-		*next = G_LBR;
+		*next = '[';
 	    }
 	}
     }
@@ -1228,7 +1187,9 @@ static NODE *powterm (parser *p)
 	    lex(p);
 	    t->v.b1.b = newbn(FARGS);
 	    if (t != NULL) {
-		get_args(t->v.b1.b, p, sym, -1, opt, &next);
+		int k = (sym == F_NRMAX)? 4 : -1;
+
+		get_args(t->v.b1.b, p, sym, k, opt, &next);
 	    }
 	}
     } else if (sym == STR || sym == VSTR) {
@@ -1238,7 +1199,7 @@ static NODE *powterm (parser *p)
 	t = base(p, NULL);
     }
 
-    if (next == G_LBR) {
+    if (next == '[') {
 	/* support func(args)[slice] */
 	t = newb2(MSL, t, NULL);
 	if (t != NULL) {
@@ -1247,9 +1208,6 @@ static NODE *powterm (parser *p)
 		get_slice_parts(t->v.b2.r, p);
 	    }
 	}	
-    } else if (next == '\'') {
-	/* support func(args)' */
-	set_transpose(t);
     }
 
 #if SDEBUG
@@ -1286,12 +1244,28 @@ static void convert_pow_term (NODE *n)
 #define pow_sym(t)     (t == B_POW || t == B_DOTPOW)
 #define pow_pow_node(n) (pow_sym(n->t) && pow_sym(n->v.b2.l->t))
 
+/* Test for whether the ' symbol must represent the unary
+   transposition operator rather than binary transpose-multiply,
+   based on the following symbol.
+*/
+
+#define must_be_unary(t) (t == EOT || \
+                          t < OP_MAX || \
+			  t == P_COM || \
+                          t == G_RPR || \
+	                  t == G_RBR ||	\
+                          t == G_RCB)
+
 static NODE *factor (parser *p)
 {  
     int sym = p->sym == B_SUB ? U_NEG : 
 	p->sym == B_ADD ? U_POS : p->sym;
     NODE *t;
 
+#if SDEBUG
+    fprintf(stderr, "factor: starting...\n");
+#endif
+
     if (p->err) {
 	return NULL;
     }
@@ -1309,22 +1283,24 @@ static NODE *factor (parser *p)
     } else {
 	t = powterm(p);
 	if (t != NULL) {
-	    if (p->sym == B_TRMUL && p->ch == 0) {
-		/* can't really be TRMUL at end of input */
-		set_transpose(t);
-	    } else {
-		while (!p->err && (p->sym == B_POW || 
-				   p->sym == B_DOTPOW ||
-				   p->sym == B_TRMUL)) {
-		    t = newb2(p->sym, t, NULL);
-		    if (t != NULL) {
-			lex(p);
+	    int upsym = p->sym;
+
+	    while (!p->err && (p->sym == B_POW || 
+			       p->sym == B_DOTPOW ||
+			       p->sym == B_TRMUL)) {
+		t = newb2(p->sym, t, NULL);
+		if (t != NULL) {
+		    lex(p);
+		    if (upsym == B_TRMUL && must_be_unary(p->sym)) {
+			/* dummy RHS for unary transpose */
+			t->v.b2.r = newempty();
+		    } else {
 			t->v.b2.r = powterm(p);
 		    }
-		    if (!p->err && pow_pow_node(t)) {
-			/* make exponentiation associate rightward */
-			convert_pow_term(t);
-		    }
+		}
+		if (!p->err && pow_pow_node(t)) {
+		    /* make exponentiation associate rightward */
+		    convert_pow_term(t);
 		}
 	    }
 	}
@@ -1341,13 +1317,18 @@ static NODE *term (parser *p)
 {  
     NODE *t;
 
+#if SDEBUG
+    fprintf(stderr, "term: starting...\n");
+#endif
+
     if (p->err || (t = factor(p)) == NULL) {
 	return NULL;
     }
 
     while (!p->err && (p->sym == B_MUL || p->sym == B_DIV || 
-		       p->sym == B_MOD || p->sym == B_DOTMULT || 
-		       p->sym == B_DOTDIV || p->sym == B_KRON)) {
+		       p->sym == B_LDIV || p->sym == B_MOD || 
+		       p->sym == B_DOTMULT || p->sym == B_DOTDIV || 
+		       p->sym == B_KRON)) {
 	t = newb2(p->sym, t, NULL);
 	if (t != NULL) {
 	    lex(p);
@@ -1366,6 +1347,10 @@ static NODE *expr4 (parser *p)
 {  
     NODE *t;
 
+#if SDEBUG
+    fprintf(stderr, "expr4: starting...\n");
+#endif
+
     if (p->err || (t = term(p)) == NULL) {
 	return NULL;
     }
@@ -1392,6 +1377,10 @@ static NODE *expr3 (parser *p)
 {  
     NODE *t;
 
+#if SDEBUG
+    fprintf(stderr, "expr3: starting...\n");
+#endif
+
     if (p->err || (t = expr4(p)) == NULL) {
 	return NULL;
     }
@@ -1417,6 +1406,10 @@ static NODE *expr2 (parser *p)
 {  
     NODE *t;
 
+#if SDEBUG
+    fprintf(stderr, "expr2: starting...\n");
+#endif
+
     if (p->err || (t = expr3(p)) == NULL) {
 	return NULL;
     }
@@ -1441,6 +1434,10 @@ static NODE *expr1 (parser *p)
 {  
     NODE *t;
 
+#if SDEBUG
+    fprintf(stderr, "expr1: starting...\n");
+#endif
+
     if (p->err || (t = expr2(p)) == NULL) {
 	return NULL;
     }
@@ -1460,10 +1457,14 @@ static NODE *expr1 (parser *p)
     return t;
 }
 
-NODE *expr0 (parser *p)
+static NODE *expr0 (parser *p)
 {  
     NODE *t;
 
+#if SDEBUG
+    fprintf(stderr, "expr0: starting...\n");
+#endif
+
     if (p->err || (t = expr1(p)) == NULL) {
 	return NULL;
     }
@@ -1487,6 +1488,10 @@ NODE *expr (parser *p)
 {  
     NODE *t;
 
+#if SDEBUG
+    fprintf(stderr, "expr: starting...\n");
+#endif
+
     if (p->err || (t = expr0(p)) == NULL) {
 	return NULL;
     }
