@@ -96,6 +96,7 @@ typedef enum {
     PLOT_BI_GRAPH,
     PLOT_MANY_TS,
     PLOT_RQ_TAU,
+    PLOT_FACTORIZED,
     PLOT_BOXPLOTS,
     PLOT_CURVE,
     PLOT_QQ,
@@ -107,6 +108,7 @@ typedef enum {
     PLOT_FIT_NONE,
     PLOT_FIT_OLS,
     PLOT_FIT_QUADRATIC,
+    PLOT_FIT_CUBIC,
     PLOT_FIT_INVERSE,
     PLOT_FIT_LOESS,
     PLOT_FIT_NA       /* fit option not applicable */
@@ -217,6 +219,12 @@ int gretl_panel_ts_plot (int vnum, const double **Z, DATAINFO *pdinfo,
 int plot_fcast_errs (const FITRESID *fr, const double *maxerr,
 		     const DATAINFO *pdinfo, gretlopt opt);
 
+int plot_simple_fcast_bands (const MODEL *pmod, 
+			     const FITRESID *fr, 
+			     const double **Z,
+			     const DATAINFO *pdinfo, 
+			     gretlopt opt);
+
 int plot_tau_sequence (const MODEL *pmod, const DATAINFO *pdinfo,
 		       int k);
 
@@ -228,6 +236,9 @@ gretl_VAR_plot_impulse_response (GRETL_VAR *var,
 				 const DATAINFO *pdinfo,
 				 gretlopt opt);
 
+int gretl_VAR_plot_FEVD (GRETL_VAR *var, int targ, int periods, 
+			 const DATAINFO *pdinfo);
+
 int 
 gretl_VAR_plot_multiple_irf (GRETL_VAR *var, 
 			     int periods, double alpha,
@@ -235,6 +246,10 @@ gretl_VAR_plot_multiple_irf (GRETL_VAR *var,
 			     const DATAINFO *pdinfo,
 			     gretlopt opt);
 
+int gretl_VECM_combined_EC_plot (GRETL_VAR *var, 
+				 const double **Z, 
+				 const DATAINFO *pdinfo);
+
 int gretl_system_residual_plot (void *p, int ci, const DATAINFO *pdinfo);
 
 int gretl_system_residual_mplot (void *p, int ci, const DATAINFO *pdinfo); 
