@@ -17,7 +17,7 @@
  * 
  */
 
-/* The BFGS optimizer and the fdjac Jacobian function */
+/* The BFGS and Newton optimizers plus the fdjac Jacobian function */
 
 #include "libgretl.h"
 #include "gretl_bfgs.h"
@@ -102,40 +102,46 @@ static double **triangular_array_new (int n)
     return m;
 }
 
-/* In case an analytical score function (@gradfun) is available,
-   construct a numerical approximation to the Hessian using that
-   function.  This is intended for building a covariance matrix at
-   convergence; note that it will be unlikely to work at an arbitrary
-   point in the parameter space.  
+/**
+ * hessian_from_score:
+ * @b: array of k parameter estimates.
+ * @H: k x k matrix to receive the (negative) Hessian.
+ * @gradfunc: function to compute gradient.
+ * @cfunc: function to compute criterion (or NULL, see below).
+ * @data: data to be passed to the @gradfunc callback.
+ *
+ * Uses the score function (@gradfunc) is to construct a 
+ * numerical approximation to the Hessian. This is primarily 
+ * intended for building a covariance matrix at convergence; 
+ * note that it may not work well at an arbitrary point in 
+ * the parameter space. 
+ *
+ * Note that the only use of @cfunc within this function is
+ * as an argument to be passed to @gradfunc. It is therefore
+ * OK to pass NULL for @cfunc provided that @gradfunc does not 
+ * use its 4th argument, which corresponds to the BFGS_CRIT_FUNC 
+ * parameter.
+ * 
+ * Returns: 0 on successful completion, non-zero error code
+ * on error.
+ */
 
-   Also note that the NULL pointer passed to gradfun in the 4th 
-   argument corresponds to the BFGS_CRIT_FUNC parameter, so this 
-   will fail horribly unless @gradfun calculates the score independently 
-   of the criterion function (and so does not try to access the 4th 
-   argument).
-*/
-
-gretl_matrix *hessian_from_score (double *b, int n, 
-				  BFGS_GRAD_FUNC gradfun, 
-				  void *data, int *err)
+int hessian_from_score (double *b, gretl_matrix *H,
+			BFGS_GRAD_FUNC gradfunc,
+			BFGS_CRIT_FUNC cfunc,
+			void *data)
 {
-    gretl_matrix *H = NULL;
     double *g, *splus, *sminus;
     double x, eps = 1.0e-05;
-    int i, j;
+    int n = gretl_matrix_rows(H);
+    int i, j, err = 0;
     
     splus  = malloc(n * sizeof *splus);
     sminus = malloc(n * sizeof *sminus);
     g      = malloc(n * sizeof *g);
 
     if (splus == NULL || sminus == NULL || g == NULL) {
-	*err = E_ALLOC;
-	goto bailout;
-    }
-
-    H = gretl_matrix_alloc(n, n);
-    if (H == NULL) {
-	*err = E_ALLOC;
+	err = E_ALLOC;
 	goto bailout;
     }
 
@@ -143,15 +149,15 @@ gretl_matrix *hessian_from_score (double *b, int n,
 	double b0 = b[i];
 
 	b[i] = b0 + eps;
-	*err = gradfun(b, g, n, NULL, data);
-	if (*err) break;
+	err = gradfunc(b, g, n, cfunc, data);
+	if (err) break;
 	for (j=0; j<n; j++) {
 	    splus[j] = g[j];
 	}
 
 	b[i] = b0 - eps;
-	*err = gradfun(b, g, n, NULL, data);
-	if (*err) break;
+	err = gradfunc(b, g, n, cfunc, data);
+	if (err) break;
 	for (j=0; j<n; j++) {
 	    sminus[j] = g[j];
 	}
@@ -163,22 +169,57 @@ gretl_matrix *hessian_from_score (double *b, int n,
 	}
     }
 
-    if (!*err) {
+    if (!err) {
 	gretl_matrix_xtr_symmetric(H);
-	*err = gretl_invert_symmetric_matrix(H);
     }
 
  bailout:
 
-    if (*err) {
-	gretl_matrix_free(H);
-	H = NULL;
-    }
-
     free(splus);
     free(sminus);
     free(g);
 
+    return err;
+}
+
+/**
+ * hessian_inverse_from_score:
+ * @b: array of parameter estimates.
+ * @n: the number of elements in @b.
+ * @gradfunc: function to compute gradient.
+ * @cfunc: function to compute criterion.
+ * @data: data to be passed to the @gradfunc callback.
+ * @err: location to receive error code.
+ *
+ * A wrapper for hessian_from_score() which takes care of 
+ * (a) allocation of the Hessian and (b) inversion.
+ * 
+ * Returns: the inverse of the (negative) Hessian on successful 
+ * completion, NULL on error.
+ */
+
+gretl_matrix *hessian_inverse_from_score (double *b, int n,
+					  BFGS_GRAD_FUNC gradfunc,
+					  BFGS_CRIT_FUNC cfunc,
+					  void *data, int *err)
+{
+    gretl_matrix *H = gretl_zero_matrix_new(n, n);
+
+    if (H == NULL) {
+	*err = E_ALLOC;
+    } else {
+	*err = hessian_from_score(b, H, gradfunc, cfunc, data);
+    }
+
+    if (!*err) {
+	*err = gretl_invert_symmetric_matrix(H);
+	if (*err) {
+	    fprintf(stderr, "hessian_inverse_from_score: failed\n");
+	    gretl_matrix_free(H);
+	    H = NULL;
+	}
+    }
+
     return H;
 }
 
@@ -224,11 +265,9 @@ static void hess_b_adjust_ij (double *c, const double *b, double *h, int n,
    the negative inverse of the Hessian.
 */
 
-gretl_matrix *numerical_hessian (const double *b, int n, 
-				 BFGS_CRIT_FUNC func, 
-				 void *data, int *err)
+static int numerical_hessian (const double *b, gretl_matrix *H,
+			      BFGS_CRIT_FUNC func, void *data)
 {
-    gretl_matrix *H = NULL;
     double Dx[RSTEPS];
     double Hx[RSTEPS];
     double *wspace;
@@ -240,14 +279,15 @@ gretl_matrix *numerical_hessian (const double *b, int n,
     double v = 2.0;      /* reduction factor for h */
     double f0, f1, f2;
     double p4m, hij;
+    int n = gretl_matrix_rows(H);
     int vn = (n * (n + 1)) / 2;
     int dn = vn + n;
     int i, j, k, m, u;
+    int err = 0;
 
     wspace = malloc((4 * n + dn) * sizeof *wspace);
     if (wspace == NULL) {
-	*err = E_ALLOC;
-	return NULL;
+	return E_ALLOC;
     }
 
     c = wspace;
@@ -256,12 +296,6 @@ gretl_matrix *numerical_hessian (const double *b, int n,
     Hd = h + n;
     D = Hd + n; /* D is of length dn */
 
-    H = gretl_matrix_alloc(n, n);
-    if (H == NULL) {
-	*err = E_ALLOC;
-	goto bailout;
-    }	
-
     for (i=0; i<n; i++) {
 	h0[i] = (fabs(b[i]) < 0.01)? eps : d * b[i];
     }
@@ -278,7 +312,7 @@ gretl_matrix *numerical_hessian (const double *b, int n,
 	    if (na(f1)) {
 		fprintf(stderr, "numerical_hessian: 1st derivative: "
 			"criterion = NA for theta[%d] = %g\n", i, c[i]);
-		*err = E_NAN;
+		err = E_NAN;
 		goto bailout;
 	    }
 	    hess_b_adjust_i(c, b, h, n, i, -1);
@@ -286,7 +320,7 @@ gretl_matrix *numerical_hessian (const double *b, int n,
 	    if (na(f2)) {
 		fprintf(stderr, "numerical_hessian: 1st derivative: "
 			"criterion = NA for theta[%d] = %g\n", i, c[i]);
-		*err = E_NAN;
+		err = E_NAN;
 		goto bailout;
 	    }
 	    /* F'(i) */
@@ -322,7 +356,7 @@ gretl_matrix *numerical_hessian (const double *b, int n,
 		    if (na(f1)) {
 			fprintf(stderr, "numerical_hessian: 2nd derivatives (%d,%d): "
 				"objective function gave NA\n", i, j);
-			*err = E_NAN;
+			err = E_NAN;
 			goto bailout;
 		    }
 		    hess_b_adjust_ij(c, b, h, n, i, j, -1);
@@ -330,7 +364,7 @@ gretl_matrix *numerical_hessian (const double *b, int n,
 		    if (na(f2)) {
 			fprintf(stderr, "numerical_hessian: 2nd derivatives (%d,%d): "
 				"objective function gave NA\n", i, j);
-			*err = E_NAN;
+			err = E_NAN;
 			goto bailout;
 		    }
 		    /* cross-partial */
@@ -361,19 +395,69 @@ gretl_matrix *numerical_hessian (const double *b, int n,
 	}
     }
 
-    *err = gretl_invert_symmetric_matrix(H);
-
  bailout:
 
-    if (*err != 0 && *err != E_ALLOC) {
+    if (err && err != E_ALLOC) {
 	gretl_errmsg_set(_("Failed to compute numerical Hessian"));
     }
 
     free(wspace);
 
+    return err;
+}
+
+/**
+ * numerical_hessian_inverse:
+ * @b: array of parameter estimates.
+ * @n: the number of elements in @b.
+ * @func: function to compute criterion.
+ * @data: data to be passed to the @gradfunc callback.
+ * @err: location to receive error code.
+ *
+ * A wrapper for numerical_hessian() which takes care of 
+ * (a) allocation of the Hessian and (b) inversion.
+ * 
+ * Returns: the inverse of the (negative) Hessian on successful 
+ * completion, NULL on error.
+ */
+
+gretl_matrix *numerical_hessian_inverse (const double *b, int n, 
+					 BFGS_CRIT_FUNC func, 
+					 void *data, int *err)
+{
+    gretl_matrix *H = gretl_zero_matrix_new(n, n);
+
+    if (H == NULL) {
+	*err = E_ALLOC;
+    } else {
+	*err = numerical_hessian(b, H, func, data);
+    }
+
+    if (!*err) {
+	*err = gretl_invert_symmetric_matrix(H);
+	if (*err) {
+	    fprintf(stderr, "numerical_hessian_inverse: failed\n");
+	    gretl_errmsg_set(_("Failed to compute numerical Hessian"));
+	    gretl_matrix_free(H);
+	    H = NULL;
+	}
+    }
+
     return H;
 }
 
+static int NR_fallback_hessian (double *b, gretl_matrix *H,
+				BFGS_GRAD_FUNC gradfunc,
+				BFGS_CRIT_FUNC cfunc,
+				void *data)
+{
+    if (gradfunc != NULL) {
+	return hessian_from_score(b, H, gradfunc, cfunc, data);
+    } else {
+	return numerical_hessian(b, H, cfunc, data);
+    }
+}
+
 #define ALT_OPG 0
 
 /* build the G matrix, given a set of coefficient estimates, b, and a
@@ -381,9 +465,9 @@ gretl_matrix *numerical_hessian (const double *b, int n,
    to the loglikelihood, lltfun 
 */
 
-gretl_matrix *build_score_matrix (double *b, int k, int T,
-				  BFGS_LLT_FUNC lltfun,
-				  void *data, int *err)
+gretl_matrix *numerical_score_matrix (double *b, int k, int T,
+				      BFGS_LLT_FUNC lltfun,
+				      void *data, int *err)
 {
     double h = 1e-8;
 #if ALT_OPG
@@ -524,8 +608,8 @@ static int simple_gradient (double *b, double *g, int n,
 
 /* default numerical calculation of gradient in context of BFGS */
 
-int BFGS_numeric_gradient (double *b, double *g, int n,
-			   BFGS_CRIT_FUNC func, void *data)
+int numeric_gradient (double *b, double *g, int n,
+		      BFGS_CRIT_FUNC func, void *data)
 {
     int err = 0;
 
@@ -541,7 +625,7 @@ int BFGS_numeric_gradient (double *b, double *g, int n,
     }
 
 #if BFGS_DEBUG
-    fprintf(stderr, "BFGS_numeric_gradient returning, err = %d\n", err);
+    fprintf(stderr, "numeric_gradient returning, err = %d\n", err);
 #endif
 
     return err;
@@ -573,9 +657,9 @@ static int broken_gradient (double *g, int n)
    maximum number of iterations and the convergence tolerance.
 */
 
-static void BFGS_get_user_values (double *b, int n, int *maxit,
-				  double *reltol, double *gradmax,
-				  gretlopt opt, PRN *prn)
+static void optim_get_user_values (double *b, int n, int *maxit,
+				   double *reltol, double *gradmax,
+				   gretlopt opt, PRN *prn)
 {
     const gretl_matrix *uinit;
     int uilen, umaxit;
@@ -611,6 +695,11 @@ static void BFGS_get_user_values (double *b, int n, int *maxit,
 	}
     }
 
+    if (reltol == NULL || gradmax == NULL) {
+	/* Newton */
+	return;
+    }
+
     /* then check for a setting of the maximum number
        of iterations */
 
@@ -692,10 +781,10 @@ static int BFGS_orig (double *b, int n, int maxit, double reltol,
     int i, j, ilast, iter;
     int err = 0;
 
-    BFGS_get_user_values(b, n, &maxit, &reltol, &gradmax, opt, prn);
+    optim_get_user_values(b, n, &maxit, &reltol, &gradmax, opt, prn);
 
     if (gradfunc == NULL) {
-	gradfunc = BFGS_numeric_gradient;
+	gradfunc = numeric_gradient;
     }
 
     wspace = malloc(4 * n * sizeof *wspace);
@@ -981,13 +1070,10 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 	       int crittype, BFGS_GRAD_FUNC gradfunc, void *data, 
 	       gretlopt opt, PRN *prn)
 {
-    double *g = NULL;
-    double *l = NULL;
-    double *u = NULL;
-    double *wa = NULL;
-    int *nbd = NULL;
-    int *iwa = NULL;
-    int i, m, wadim;
+    double *wspace = NULL;
+    double *g, *l, *u, *wa;
+    int *iwa, *nbd = NULL;
+    int i, m, dim;
     char task[60];
     char csave[60];
     double f, pgtol;
@@ -1002,7 +1088,7 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 
     *fncount = *grcount = 0;    
 
-    BFGS_get_user_values(b, n, &maxit, &reltol, &gradmax, opt, prn);
+    optim_get_user_values(b, n, &maxit, &reltol, &gradmax, opt, prn);
 
     /*
       m: the number of corrections used in the limited memory matrix.
@@ -1014,26 +1100,28 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
     */
     m = libset_get_int(LBFGS_MEM); 
 
-    wadim = (2*m+4)*n + 12*m*m + 12*m;
+    dim = (2*m+4)*n + 12*m*m + 12*m; /* for wa */
+    dim += 3*n; /* for g, l and u */
 
-    g = malloc(n * sizeof *g);
-    l = malloc(n * sizeof *l);
-    u = malloc(n * sizeof *u);
-    wa = malloc(wadim * sizeof *wa);
-    nbd = malloc(n * sizeof *nbd);
-    iwa = malloc(3*n * sizeof *iwa);
+    wspace = malloc(dim * sizeof *wspace);
+    nbd = malloc(4*n * sizeof *nbd);
 
-    if (g == NULL || l == NULL || u == NULL ||
-	wa == NULL || nbd == NULL || iwa == NULL) {
+    if (wspace == NULL || nbd == NULL) {
 	err = E_ALLOC;
 	goto bailout;
     }
 
+    g = wspace;
+    l = g + n;
+    u = l + n;
+    wa = u + n;
+    iwa = nbd + n;
+
     verbskip = libset_get_int("bfgs_verbskip");
     show_activity = show_activity_func_installed();
 
     if (gradfunc == NULL) {
-	gradfunc = BFGS_numeric_gradient;
+	gradfunc = numeric_gradient;
     }
 
     /* Gradient convergence criterion (not used -- we use reltol instead) */
@@ -1050,7 +1138,6 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
     strcpy(task, "START");
 
     while (1) {
-
 	/* Call the L-BFGS-B code */
 	setulb_(&n, &m, b, l, u, nbd, &f, g, &reltol, &pgtol, wa, iwa, 
 		task, csave, lsave, isave, dsave);
@@ -1058,7 +1145,6 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 	iter = isave[29] + 1;
 
 	if (!strncmp(task, "FG", 2)) {
-
 	    /* Compute function value, f */
 	    f = cfunc(b, data);
 	    if (!na(f)) {
@@ -1069,12 +1155,10 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 		break;
 	    }
 	    *fncount += 1;
-
 	    /* Compute gradient, g */
 	    gradfunc(b, g, n, cfunc, data);
 	    reverse_gradient(g, n);
 	    *grcount += 1;
-	    
 	} else if (!strncmp(task, "NEW_X", 5)) {
 	    /* The optimizer has produced a new set of parameter values */
 	    if (isave[33] >= maxit) {
@@ -1116,12 +1200,8 @@ int LBFGS_max (double *b, int n, int maxit, double reltol,
 
  bailout:
 
-    free(g);
-    free(l);
-    free(u);
-    free(wa);
+    free(wspace);
     free(nbd);
-    free(iwa);
 
     return err;
 }
@@ -1188,7 +1268,7 @@ int BFGS_max (double *b, int n, int maxit, double reltol,
     return ret;
 }
 
-/* user-level access to BFGS and fdjac */
+/* user-level access to BFGS, NR and fdjac */
 
 typedef struct umax_ umax;
 
@@ -1196,12 +1276,15 @@ struct umax_ {
     GretlType gentype;    /* GRETL_TYPE_DOUBLE or GRETL_TYPE_MATRIX */
     gretl_matrix *b;      /* parameter vector */
     gretl_matrix *g;      /* gradient vector */
+    gretl_matrix *h;      /* hessian matrix */
     int ncoeff;           /* number of coefficients */
     GENERATOR *gf;        /* for generating scalar or matrix result */
     GENERATOR *gg;        /* for generating gradient */
+    GENERATOR *gh;        /* for generating Hessian */
     double fx_out;        /* function double value */
     gretl_matrix *fm_out; /* function matrix value */
     char gmname[VNAMELEN]; /* name of user-defined gradient vector */
+    char hmname[VNAMELEN]; /* name of user-defined Hessian matrix */
     double ***Z;          /* pointer to data array */
     DATAINFO *dinfo;      /* dataset info */
     PRN *prn;             /* optional printing struct */
@@ -1215,12 +1298,15 @@ static umax *umax_new (GretlType t)
 	u->gentype = t;
 	u->b = NULL;
 	u->g = NULL;
+	u->h = NULL;
 	u->ncoeff = 0;
 	u->gf = NULL;
 	u->gg = NULL;
+	u->gh = NULL;
 	u->fx_out = NADBL;
 	u->fm_out = NULL;
 	u->gmname[0] = '\0';
+	u->hmname[0] = '\0';
 	u->Z = NULL;
 	u->dinfo = NULL;
 	u->prn = NULL;
@@ -1241,11 +1327,12 @@ static void umax_destroy (umax *u)
 
     destroy_genr(u->gf);
     destroy_genr(u->gg);
+    destroy_genr(u->gh);
 
     free(u);
 }
 
-/* user-defined BFGS: get the criterion value */
+/* user-defined optimizer: get the criterion value */
 
 static double user_get_criterion (const double *b, void *p)
 {
@@ -1257,7 +1344,7 @@ static double user_get_criterion (const double *b, void *p)
 	u->b->val[i] = b[i];
     }
 
-    err = execute_genr(u->gf, u->Z, u->dinfo, OPT_S, u->prn); 
+    err = execute_genr(u->gf, u->Z, u->dinfo, u->prn); 
 
     if (err) {
 	return NADBL;
@@ -1284,7 +1371,7 @@ static double user_get_criterion (const double *b, void *p)
     return x;
 }
 
-/* user-defined BFGS: get the gradient, if specified */
+/* user-defined optimizer: get the gradient, if specified */
 
 static int user_get_gradient (double *b, double *g, int k,
 			      BFGS_CRIT_FUNC func, void *p)
@@ -1297,7 +1384,7 @@ static int user_get_gradient (double *b, double *g, int k,
 	u->b->val[i] = b[i];
     }
 
-    err = execute_genr(u->gg, u->Z, u->dinfo, OPT_S, u->prn); 
+    err = execute_genr(u->gg, u->Z, u->dinfo, u->prn); 
 
     if (err) {
 	return err;
@@ -1318,14 +1405,47 @@ static int user_get_gradient (double *b, double *g, int k,
     return err;
 }
 
-/* parse the name of the user gradient matrix (vector) out of
-   the gradient function call, where it must be the first
-   argument, given in pointer form
+/* user-defined optimizer: get the hessian, if specified */
+
+static int user_get_hessian (double *b, gretl_matrix *H,
+			     void *p)
+{
+    umax *u = (umax *) p;
+    gretl_matrix *uH;
+    int k = H->rows;
+    int i, err;
+
+    for (i=0; i<k; i++) {
+	u->b->val[i] = b[i];
+    }
+
+    err = execute_genr(u->gh, u->Z, u->dinfo, u->prn); 
+
+    if (err) {
+	return err;
+    }
+
+    uH = get_matrix_by_name(u->hmname);
+
+    if (uH == NULL) {
+	err = E_UNKVAR;
+    } else if (uH->rows != k || uH->cols != k) {
+	err = E_NONCONF;
+    } else {
+	gretl_matrix_copy_values(H, uH);
+    } 
+
+    return err;
+}
+
+/* parse the name of the user gradient matrix (vector) or
+   Hessian out of the associated function call, where it 
+   must be the first argument, given in pointer form
 */
 
-static int get_grad_vector_name (umax *u, const char *gradcall)
+int optimizer_get_matrix_name (const char *fncall, char *name)
 {
-    const char *s = strchr(gradcall, '(');
+    const char *s = strchr(fncall, '(');
     int n, err = 0;
 
     if (s == NULL) {
@@ -1341,7 +1461,7 @@ static int get_grad_vector_name (umax *u, const char *gradcall)
 	    if (n >= VNAMELEN) {
 		err = E_DATA;
 	    } else {
-		strncat(u->gmname, s, n);
+		strncat(name, s, n);
 	    }
 	}
     }
@@ -1352,6 +1472,7 @@ static int get_grad_vector_name (umax *u, const char *gradcall)
 static int user_gen_setup (umax *u,
 			   const char *fncall,
 			   const char *gradcall,
+			   const char *hesscall,
 			   double ***pZ, 
 			   DATAINFO *pdinfo)
 {
@@ -1368,16 +1489,27 @@ static int user_gen_setup (umax *u,
 
     if (!err) {
 	/* see if the formula actually works */
-	err = execute_genr(u->gf, pZ, pdinfo, OPT_S, u->prn);
+	err = execute_genr(u->gf, pZ, pdinfo, u->prn);
     }
 
     if (!err && gradcall != NULL) {
 	/* process gradient formula */
-	err = get_grad_vector_name(u, gradcall);
+	err = optimizer_get_matrix_name(gradcall, u->gmname);
 	if (!err) {
 	    u->gg = genr_compile(gradcall, pZ, pdinfo, OPT_P | OPT_U, &err);
 	    if (!err) {
-		err = execute_genr(u->gg, pZ, pdinfo, OPT_S, u->prn);
+		err = execute_genr(u->gg, pZ, pdinfo, u->prn);
+	    } 
+	}
+    }
+
+    if (!err && hesscall != NULL) {
+	/* process Hessian formula */
+	err = optimizer_get_matrix_name(hesscall, u->hmname);
+	if (!err) {
+	    u->gh = genr_compile(hesscall, pZ, pdinfo, OPT_P | OPT_U, &err);
+	    if (!err) {
+		err = execute_genr(u->gg, pZ, pdinfo, u->prn);
 	    } 
 	}
     }
@@ -1423,7 +1555,7 @@ double user_BFGS (gretl_matrix *b,
 
     u->b = b;
 
-    *err = user_gen_setup(u, fncall, gradcall, pZ, pdinfo);
+    *err = user_gen_setup(u, fncall, gradcall, NULL, pZ, pdinfo);
     if (*err) {
 	return NADBL;
     }
@@ -1457,6 +1589,65 @@ double user_BFGS (gretl_matrix *b,
     return ret;
 }
 
+double user_NR (gretl_matrix *b, 
+		const char *fncall,
+		const char *gradcall, 
+		const char *hesscall,
+		double ***pZ, DATAINFO *pdinfo,
+		PRN *prn, int *err)
+{
+    umax *u;
+    double ret = NADBL;
+    double crittol = 1.0e-7;
+    double gradtol = 1.0e-7;
+    gretlopt opt = OPT_NONE;
+    int verbose = libset_get_bool(MAX_VERBOSE);
+    int maxit = 100;
+    int iters = 0;
+
+    u = umax_new(GRETL_TYPE_DOUBLE);
+    if (u == NULL) {
+	*err = E_ALLOC;
+	return ret;
+    }
+
+    u->ncoeff = gretl_vector_get_length(b);
+    if (u->ncoeff == 0) {
+	*err = E_DATA;
+	goto bailout;
+    }
+
+    u->b = b;
+
+    *err = user_gen_setup(u, fncall, gradcall, hesscall, pZ, pdinfo);
+    if (*err) {
+	return NADBL;
+    }
+
+    if (verbose) {
+	opt = OPT_V;
+	u->prn = prn;
+    }
+
+    *err = newton_raphson_max(b->val, u->ncoeff, maxit, 
+			      crittol, gradtol, 
+			      &iters, C_OTHER, 
+			      user_get_criterion,
+			      (u->gg == NULL)? NULL : user_get_gradient,
+			      (u->gh == NULL)? NULL : user_get_hessian,
+			      u, opt, prn);
+
+    if (!*err) {
+	ret = u->fx_out;
+    }
+
+ bailout:
+
+    umax_destroy(u);
+
+    return ret;
+}
+
 #define JAC_DEBUG 0
 
 static int user_calc_fvec (integer *m, integer *n, double *x, double *fvec,
@@ -1474,7 +1665,7 @@ static int user_calc_fvec (integer *m, integer *n, double *x, double *fvec,
     gretl_matrix_print(u->b, "user_calc_fvec: u->b");
 #endif
 
-    err = execute_genr(u->gf, u->Z, u->dinfo, OPT_S, u->prn); 
+    err = execute_genr(u->gf, u->Z, u->dinfo, u->prn); 
     if (err) {
 	fprintf(stderr, "execute_genr: err = %d\n", err); 
     }
@@ -1553,7 +1744,7 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
     u->b = theta;
     u->ncoeff = n;
 
-    *err = user_gen_setup(u, fncall, NULL, pZ, pdinfo);
+    *err = user_gen_setup(u, fncall, NULL, NULL, pZ, pdinfo);
     if (*err) {
 	fprintf(stderr, "fdjac: error %d from user_gen_setup\n", *err);
 	goto bailout;
@@ -1598,7 +1789,7 @@ gretl_matrix *fdjac (gretl_matrix *theta, const char *fncall,
     return J;
 }
 
-/* Below: experimental Newton-Raphson code, starting with a few
+/* Below: Newton-Raphson code, starting with a few
    auxiliary functions */
 
 static int broken_matrix (const gretl_matrix *m)
@@ -1646,39 +1837,116 @@ static double scalar_xpx (const gretl_matrix *m)
 }
 
 enum {
-    GRADTOL_MET = 1 << 0,
-    CRITTOL_MET = 1 << 1,
-    STEPMIN_MET = 1 << 2
+    GRADTOL_MET = 1,
+    CRITTOL_MET,
+    STEPMIN_MET
 };
 
-/* Newton-Raphson maximizer, loosely based on R's maxNR(). 
+static void print_NR_status (int status, double crittol, double gradtol, 
+			     double sumgrad, PRN *prn)
+{
+    if (status == GRADTOL_MET) {
+	pprintf(prn, "Gradient within tolerance (%g)\n", gradtol);
+    } else if (status == CRITTOL_MET) {
+	pprintf(prn, "Successive criterion values within tolerance (%g)\n",
+		crittol);
+    } else if (status == STEPMIN_MET) {
+	if (sumgrad > 0) {
+	    pprintf(prn, "Warning: couldn't improve criterion (gradient = %g)\n",
+		    sumgrad);
+	} else {
+	    pprintf(prn, "Warning: couldn't improve criterion\n");
+	}
+    }
+}
 
-   The functions cfunc (computes the criterion, usually a
-   loglikelihood) and gradfunc (score, provides an updated
-   estimate of the gradient in its second argument) are just as in
-   BFGS_max above.
+/* 
+   The strategy here may be simplistic, but appears to be effective in
+   quite a few cases: If the (negative) Hessian is not pd, we try to
+   nudge it towards positive definiteness without losing too much
+   information on curvature by downsizing the off-diagonal elements of
+   H. In practice, we use
 
-   The precise working of the function hessfunc is debatable.  As
-   things stand below, this function should write into its second
-   argument the negative inverse of the Hessian (or some variant
-   thereof).  This pushes onto hessfunc the problem of what to do if
-   the observed Hessian is not negative definite.  
+   C = \lambda H + (1-lambda) diag(H)
 
-   The alternative would be to get hessfunc to provide the Hessian
-   itself, negative definite or not, and for newton_raphson_max to
-   take on the task of fixing it up if need be (which is what R's
-   maxNR does). But it seemed to me that the best fix might depend on
-   the particular application.  For example if the full gradient
-   matrix, G, is available one might substitute (GG')^{-1}, hence
-   falling back on BHHH temporarily.
-
-   Note that unlike maxNR, there is no provision here for automatic
-   substitution of numerical routines for gradfunc or hessfunc if
-   these functions are not provided. My guess is that that without
-   analytical functions Newton-Raphson is bound to perform worse than
-   BFGS.
+   where lambda is reasonably close to 1. Note that, if lambda was 0,
+   a NR iteration would just amount to an iteration of the simple
+   gradient method (on a scaled version of the coefficients).
+   Hopefully, a few of those should be able to "tow us away" from the
+   non-pd region.
 */
 
+static int NR_invert_hessian (gretl_matrix *H, const gretl_matrix *Hcpy)
+{
+    int err = gretl_invert_symmetric_matrix(H);
+
+    if (err == E_NOTPD) {
+	double x, lambda = 1.0;
+	int i, j, s;
+
+	for (s=0; lambda > 0.1 && err; s++) {
+	    lambda *= 0.9;
+	    fprintf(stderr, "newton hessian fixup: round %d, lambda=%g\n",
+		    s, lambda);
+
+	    gretl_matrix_copy_values(H, Hcpy);
+	    for (i=0; i<H->rows; i++) {
+		for (j=0; j<i; j++) {
+		    x = lambda * gretl_matrix_get(H, i, j);
+		    gretl_matrix_set(H, i, j, x);
+		    gretl_matrix_set(H, j, i, x);
+		}
+	    }
+#if 0
+	    gretl_matrix_print(H, "after shrinkage");
+#endif
+	    err = gretl_invert_symmetric_matrix(H);
+	}
+    }
+
+    return err;
+}
+
+/**
+ * newton_raphson_max:
+ * @b: array of adjustable coefficients.
+ * @n: number elements in array @b.
+ * @maxit: the maximum number of iterations to allow.
+ * @crittol: tolerance for terminating iteration, in terms of
+ * the change in the criterion.
+ * @gradtol: tolerance for terminating iteration, in terms of
+ * the gradient.
+ * @itercount: location to receive count of iterations.
+ * @crittype: code for type of the maximand/minimand: should
+ * be %C_LOGLIK, %C_GMM or %C_OTHER.  Used only in printing
+ * iteration info.
+ * @cfunc: pointer to function used to calculate maximand.
+ * @gradfunc: pointer to function used to calculate the 
+ * gradient, or %NULL for default numerical calculation.
+ * @hessfunc: pointer to function used to calculate the
+ * Hessian.
+ * @data: pointer that will be passed as the last
+ * parameter to the callback functions @cfunc, @gradfunc
+ * and @hessfunc.
+ * @opt: may contain %OPT_V for verbose operation.
+ * @prn: printing struct (or %NULL).  Only used if @opt
+ * includes %OPT_V.
+ *
+ * The functions @cfunc (computes the criterion, usually a
+ * loglikelihood) and @gradfunc (score, provides an updated
+ * estimate of the gradient in its second argument) are just as in
+ * BFGS_max above.
+ *
+ * The @hessfunc callback should compute the negative Hessian,
+ * _not_ inverted; newton_raphson_max takes care of the inversion,
+ * with a routine for fixing up the matrix if it's not positive
+ * definite. If @hessfunc is NULL we fall back on a numerical
+ * approximation to the Hessian.
+ * 
+ * Returns: 0 on successful completion, non-zero error code
+ * on error.
+ */
+
 int newton_raphson_max (double *b, int n, int maxit, 
 			double crittol, double gradtol, 
 			int *itercount, int crittype, 
@@ -1692,10 +1960,9 @@ int newton_raphson_max (double *b, int n, int maxit,
     double stepmin = 1.0e-6;
     gretl_matrix_block *B;
     gretl_matrix *H0, *H1;
-    gretl_matrix *g0, *g1;
-    gretl_matrix *a;
+    gretl_matrix *g, *a;
     double *b0, *b1;
-    double f0, f1;
+    double f0, f1, sumgrad = 0;
     double steplen = 1.0;
     int status = 0;
     int iter = 0;
@@ -1708,8 +1975,7 @@ int newton_raphson_max (double *b, int n, int maxit,
     
     B = gretl_matrix_block_new(&H0, n, n,
 			       &H1, n, n,
-			       &g0, n, 1,
-			       &g1, n, 1,
+			       &g, n, 1,
 			       &a, n, 1,
 			       NULL);
     if (B == NULL) {
@@ -1717,6 +1983,9 @@ int newton_raphson_max (double *b, int n, int maxit,
 	return E_ALLOC;
     }
 
+    /* needs some work */
+    optim_get_user_values(b, n, NULL, NULL, NULL, opt, prn);
+
     b1 = b0 + n;
     copy_to(b1, b, n);
 
@@ -1727,11 +1996,23 @@ int newton_raphson_max (double *b, int n, int maxit,
     }
 
     if (!err) {
-	err = gradfunc(b, g1->val, n, cfunc, data);
+	if (gradfunc != NULL) {
+	    err = gradfunc(b, g->val, n, cfunc, data);
+	} else {
+	    err = numeric_gradient(b, g->val, n, cfunc, data);
+	}
     }
 
     if (!err) {
-	err = hessfunc(b, H1, data);
+	if (hessfunc != NULL) {
+	    err = hessfunc(b, H1, data);
+	} else {
+	    err = NR_fallback_hessian(b, H1, gradfunc, cfunc, data);
+	}
+	if (!err) {
+	    gretl_matrix_copy_values(H0, H1);
+	    err = NR_invert_hessian(H1, H0);
+	}
     }
 
     while (status == 0 && !err) {
@@ -1741,8 +2022,7 @@ int newton_raphson_max (double *b, int n, int maxit,
 
 	copy_to(b0, b1, n);
 
-	gretl_matrix_copy_values(g0, g1);
-	if (broken_matrix(g0)) {
+	if (broken_matrix(g)) {
 	    fprintf(stderr, "NA in gradient\n");
 	    err = E_NAN;
 	    break;
@@ -1756,7 +2036,7 @@ int newton_raphson_max (double *b, int n, int maxit,
 	}
 
 	/* apply quadratic approximation */
-	gretl_matrix_multiply(H0, g0, a);
+	gretl_matrix_multiply(H0, g, a);
 	copy_plus(b1, b0, steplen, a->val, n);
 	f1 = cfunc(b1, data);
 
@@ -1768,27 +2048,47 @@ int newton_raphson_max (double *b, int n, int maxit,
 	}
 
 	if (verbose) {
-	    print_iter_info(iter, f1, crittype, n, b1, g1->val, 
+	    print_iter_info(iter, f1, crittype, n, b1, g->val, 
 			    steplen, prn);
 	}
 
-	err = gradfunc(b1, g1->val, n, cfunc, data);
-	if (err || broken_matrix(g1)) {
+	if (gradfunc != NULL) {
+	    err = gradfunc(b1, g->val, n, cfunc, data);
+	} else {
+	    err = numeric_gradient(b1, g->val, n, cfunc, data);
+	}
+
+	if (err || broken_matrix(g)) {
 	    err = (err == 0)? E_NAN : err;
 	    break;
 	}	
 
-	err = hessfunc(b1, H1, data);
+	if (hessfunc != NULL) {
+	    err = hessfunc(b1, H1, data);
+	} else {
+	    err = NR_fallback_hessian(b1, H1, gradfunc, cfunc, data);
+	}
+
 	if (err || broken_matrix(H1)) {
 	    err = (err == 0)? E_NAN : err;
 	    break;
-	}	
+	}
+
+	if (!err) {
+	    gretl_matrix_copy_values(H0, H1);
+	    err = NR_invert_hessian(H1, H0);
+	    if (err) {
+		break;
+	    }
+	}
+
+	sumgrad = sqrt(scalar_xpx(g));
 
 	if (steplen < stepmin) {
 	    status = STEPMIN_MET;
 	} else if (iter > maxit) {
 	    err = E_NOCONV;
-	} else if (sqrt(scalar_xpx(g1)) < gradtol) {
+	} else if (sumgrad < gradtol) {
 	    status = GRADTOL_MET;
 	} else if (f1 - f0 < crittol) {
 	    status = CRITTOL_MET;
@@ -1796,7 +2096,7 @@ int newton_raphson_max (double *b, int n, int maxit,
     }
 
     if (verbose) {
-	print_iter_info(-1, f1, crittype, n, b1, g1->val, 
+	print_iter_info(-1, f1, crittype, n, b1, g->val, 
 			steplen, prn);
 	pputc(prn, '\n');
     }
@@ -1805,14 +2105,8 @@ int newton_raphson_max (double *b, int n, int maxit,
 
     if (!err) {
 	copy_to(b, b1, n);
-	if (status == GRADTOL_MET) {
-	    fprintf(stderr, "Gradient close to zero; may be a solution.\n");
-	} else if (status == CRITTOL_MET) {
-	    fprintf(stderr, "Successive criterion values within tolerance; "
-		    "may be a solution.\n");
-	} else if (status == STEPMIN_MET) {
-	    fprintf(stderr, "Couldn't increase criterion; "
-		    "may be near a solution?\n");
+	if (prn != NULL) {
+	    print_NR_status(status, crittol, gradtol, sumgrad, prn);
 	}
     }
 
