@@ -17,11 +17,14 @@
  * 
  */
 
+#define FULL_XML_HEADERS 1
+
 #include "libgretl.h"
 #include "gretl_func.h"
 #include "libset.h"
 #include "gretl_scalar.h"
 #include "gretl_string_table.h"
+#include "gretl_xml.h"
 #include "gretl_bundle.h"
 
 #include <glib.h>
@@ -35,7 +38,8 @@
  */
 
 struct gretl_bundle_ {
-    char name[VNAMELEN]; 
+    char name[VNAMELEN];
+    char *creator;
     GHashTable *ht;      /* holds key/value pairs */
     int level;           /* level of function execution */
 };
@@ -57,6 +61,8 @@ struct bundled_item_ {
 static gretl_bundle **bundles;
 static int n_saved_bundles;
 
+static void bundle_count (gpointer key, gpointer value, gpointer p);
+
 /* given a bundle pointer, return its 0-based stack position
    or -1 if it's not on the stack */
 
@@ -94,6 +100,60 @@ static gretl_bundle *get_bundle_pointer (const char *name, int level)
     return NULL;
 }
 
+const char *gretl_bundle_get_name (gretl_bundle *b)
+{
+    int i;
+
+    for (i=0; i<n_saved_bundles; i++) {
+	if (b == bundles[i]) {
+	    return b->name;
+	}
+    }
+
+    return NULL;
+}
+
+int gretl_bundle_set_name (gretl_bundle *b, const char *name)
+{
+    if (b == NULL) {
+	return E_UNKVAR;
+    }
+
+    *b->name = '\0';
+    strncat(b->name, name, VNAMELEN - 1);
+
+    return 0;
+}
+
+int gretl_bundle_is_stacked (gretl_bundle *b)
+{
+    int i;
+
+    for (i=0; i<n_saved_bundles; i++) {
+	if (b == bundles[i]) {
+	    return 1;
+	}
+    }
+
+    return 0;
+}
+
+int gretl_bundle_get_n_keys (gretl_bundle *b)
+{
+    int n_items = 0;
+
+    if (b != NULL) {
+	g_hash_table_foreach(b->ht, bundle_count, &n_items);
+    }
+
+    return n_items;
+}
+
+int n_user_bundles (void)
+{
+    return n_saved_bundles;
+}
+
 static int bundle_index_by_name (const char *name, int level)
 {
     return get_bundle_index(get_bundle_pointer(name, level));
@@ -216,6 +276,7 @@ void gretl_bundle_destroy (gretl_bundle *bundle)
 	if (bundle->ht != NULL) {
 	    g_hash_table_destroy(bundle->ht);
 	}
+	free(bundle->creator);
 	free(bundle);
     }
 }
@@ -232,6 +293,7 @@ gretl_bundle *gretl_bundle_new (void)
 
     if (b != NULL) {
 	*b->name = '\0';
+	b->creator = NULL;
 	b->level = 0;
 	b->ht = g_hash_table_new_full(g_str_hash, g_str_equal, 
 				      g_free, bundled_item_destroy);
@@ -264,15 +326,18 @@ static int resize_bundle_stack (int n)
     }
 }
 
-/* push a bundle onto the saved stack: note that if this
-   fails we destroy the bundle and return non-zero */
+/* Push a bundle onto the saved stack. Note that if this
+   fails we destroy the bundle, unless @preserve is 
+   non-zero */
 
-static int gretl_bundle_push (gretl_bundle *b)
+static int gretl_bundle_push (gretl_bundle *b, int preserve)
 {
     int n = n_saved_bundles + 1;
 
     if (resize_bundle_stack(n)) {
-	gretl_bundle_destroy(b);
+	if (!preserve) {
+	    gretl_bundle_destroy(b);
+	}
 	return E_ALLOC;
     }
 
@@ -327,6 +392,73 @@ static int real_unstack_bundle (int i, int mode)
     return err;
 }
 
+char *get_bundle_temp_name (void)
+{
+    char tmpname[VNAMELEN];
+
+    sprintf(tmpname, "btmp___%d", n_saved_bundles);
+    return gretl_strdup(tmpname);
+}
+
+static int bundle_name_is_temp (const char *name)
+{
+    return !strncmp(name, "btmp___", 7) && isdigit(name[7]);
+}
+
+char *get_bundle_default_name (void)
+{
+    char name[VNAMELEN];
+    int i, n = 0;
+
+    for (i=0; i<n_saved_bundles; i++) {
+	if (!bundle_name_is_temp(bundles[i]->name)) {
+	    n++;
+	}
+    } 
+
+    sprintf(name, "bundle%d", n + 1);
+    return gretl_strdup(name);
+}
+
+int gretl_bundle_is_temp (gretl_bundle *bundle)
+{
+    int ret = 0;
+
+    if (bundle != NULL) {
+	if (get_bundle_index(bundle) < 0) {
+	    ret = 1;
+	} else if (bundle_name_is_temp(bundle->name)) {
+	    ret = 1;
+	}
+    }  
+
+    return ret;
+}
+
+/**
+ * gretl_bundle_destroy_if_temp:
+ * @bundle: bundle to destroy, conditionally.
+ *
+ * Frees all contents of @bundle as well as the pointer itself,
+ * if the bundle in question has not been saved on the stack of
+ * named bundles.
+ */
+
+void gretl_bundle_destroy_if_temp (gretl_bundle *bundle)
+{
+    if (bundle != NULL) {
+	int idx = get_bundle_index(bundle);
+
+	if (idx < 0) {
+	    /* not on named stack */
+	    gretl_bundle_destroy(bundle);
+	} else if (bundle_name_is_temp(bundle->name)) {
+	    /* on stack, but with stylized temp name */
+	    real_unstack_bundle(idx, UNSTACK_AND_FREE);
+	}
+    }    
+}
+
 /* Determine whether @name is the name of a saved bundle. */
 
 int gretl_is_bundle (const char *name)
@@ -354,6 +486,24 @@ gretl_bundle *get_gretl_bundle_by_name (const char *name)
     }
 }
 
+/**
+ * get_gretl_bundle_by_index:
+ * @idx: index number to look up.
+ *
+ * Designed for use in GUI session management.
+ *
+ * Returns: pointer to a saved gretl bundle, if found, else NULL.
+ */
+
+gretl_bundle *get_gretl_bundle_by_index (int idx)
+{
+    if (idx >= 0 && idx < n_saved_bundles) {
+	return bundles[idx];
+    } else {
+	return NULL;
+    }
+}
+
 static int gretl_bundle_has_data (gretl_bundle *b, const char *key)
 {
     gpointer p = g_hash_table_lookup(b->ht, key);
@@ -365,13 +515,18 @@ static int gretl_bundle_has_data (gretl_bundle *b, const char *key)
  * gretl_bundle_get_data:
  * @bundle: bundle to access.
  * @key: name of key to access.
- * @type: location to receive data type.
+ * @type: location to receive data type, or NULL (see below).
  * @size: location to receive size of data (= series
  * length for GRETL_TYPE_SERIES, otherwise 0), or NULL.
  * @err:location to receive error code.
  *
  * Returns: the data pointer associated with @key in the
- * specified @bundle, or NULL on failure.
+ * specified @bundle, or NULL on failure. If @type is non-NULL
+ * (the usual case) the content of @err is set to a non-zero
+ * value if @bundle contains no data-item with key @key. But
+ * if @type is NULL the call is understood as a query as to
+ * whether @bundle contains the specified data-item, and 
+ * @err is not set if @key is not present.
  */
 
 void *gretl_bundle_get_data (gretl_bundle *bundle, const char *key,
@@ -386,13 +541,15 @@ void *gretl_bundle_get_data (gretl_bundle *bundle, const char *key,
 
 	if (p != NULL) {
 	    bundled_item *item = p;
-	    
-	    *type = item->type;
+
 	    ret = item->data;
+	    if (type != NULL) {
+		*type = item->type;
+	    }
 	    if (size != NULL) {
 		*size = item->size;
 	    }
-	} else {
+	} else if (type != NULL) {
 	    gretl_errmsg_sprintf("\"%s\": %s", key, _("no such item"));
 	    *err = E_DATA;
 	}
@@ -526,6 +683,19 @@ const char *gretl_bundle_get_note (gretl_bundle *bundle,
     return ret;
 }
 
+/**
+ * gretl_bundle_get_creator:
+ * @bundle: bundle to access.
+ *
+ * Returns: the name of the package that created @bundle, if any, 
+ * otherwise NULL.
+ */
+
+const char *gretl_bundle_get_creator (gretl_bundle *bundle)
+{
+    return (bundle != NULL)? bundle->creator : NULL;
+}
+
 /**
  * bundled_item_get_data:
  * @item: bundled item to access.
@@ -685,6 +855,39 @@ int gretl_bundle_set_note (gretl_bundle *bundle, const char *key,
     return err;
 }
 
+/**
+ * gretl_bundle_set_creator:
+ * @bundle: target bundle.
+ * @name: name of function package that built the bundle.
+ * 
+ * Sets the "creator" attribute of @bundle. This is called 
+ * automatically when a bundle is returned to top-lavel 
+ * userspace by a packaged function.
+ *
+ * Returns: 0 on success, error code on error.
+ */
+
+int gretl_bundle_set_creator (gretl_bundle *bundle, const char *name)
+{
+    int err = 0;
+
+    if (bundle == NULL) {
+	err = E_DATA;
+    } else {
+	free(bundle->creator);
+	if (name == NULL) {
+	    bundle->creator = NULL;
+	} else {
+	    bundle->creator = gretl_strdup(name);
+	    if (bundle->creator == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+    }
+
+    return err;
+}
+
 /**
  * save_named_bundle:
  * @name: name to give the bundle.
@@ -711,13 +914,33 @@ int save_named_bundle (const char *name)
 	} else {
 	    strcpy(b->name, name);
 	    b->level = level;
-	    err = gretl_bundle_push(b);
+	    err = gretl_bundle_push(b, 0);
 	}
     }
 
     return err;
 }
 
+/* callback for adding icons representing bundles to the GUI
+   session window */
+
+static void (*bundle_add_callback)(void);
+
+/**
+ * set_bundle_add_callback:
+ * @callback: function function to out in place.
+ *
+ * Sets the callback function to be invoked when a
+ * bundle is added to the stack of saved objects.  Intended
+ * for synchronizing the GUI program with the saved object
+ * state.
+ */
+
+void set_bundle_add_callback (void (*callback))
+{
+    bundle_add_callback = callback; 
+}
+
 /* For use by geneval: take @bundle, created on the fly, and stack
    it under @name.  If there's already a bundle with the given name
    destroy and replace it.
@@ -737,7 +960,13 @@ int gretl_bundle_add_or_replace (gretl_bundle *bundle, const char *name)
 	gretl_bundle_destroy(bundles[b0idx]);
 	bundles[b0idx] = bundle;
     } else {
-	err = gretl_bundle_push(bundle);
+	err = gretl_bundle_push(bundle, 0);
+
+	if (bundle_add_callback != NULL && 
+	    !bundle_name_is_temp(bundle->name) &&
+	    gretl_function_depth() == 0) {
+	    (*bundle_add_callback)();
+	}
     }
 
     return err;
@@ -825,22 +1054,22 @@ int gretl_bundle_copy_as (const char *name, const char *copyname)
 
     if (b1 != NULL) {
 	g_hash_table_destroy(b1->ht);
-	b1->ht = NULL;
+	b1->ht = g_hash_table_new_full(g_str_hash, 
+				       g_str_equal, 
+				       g_free, 
+				       bundled_item_destroy);
     } else {
-	b1 = malloc(sizeof *b1);
+	b1 = gretl_bundle_new();
 	if (b1 == NULL) {
 	    err = E_ALLOC;
 	} else {
 	    strcpy(b1->name, copyname);
-	    b1->ht = NULL;
 	    b1->level = level;
-	    err = gretl_bundle_push(b1);
+	    err = gretl_bundle_push(b1, 0);
 	}
     }
 
     if (!err) {
-	b1->ht = g_hash_table_new_full(g_str_hash, g_str_equal, 
-				       g_free, bundled_item_destroy);
 	g_hash_table_foreach(b0->ht, copy_bundled_item, b1);
     }
 
@@ -856,7 +1085,7 @@ int gretl_bundle_copy_as (const char *name, const char *copyname)
  * are themselves copied), or NULL on failure.
  */
 
-gretl_bundle *gretl_bundle_copy (gretl_bundle *bundle, int *err)
+gretl_bundle *gretl_bundle_copy (const gretl_bundle *bundle, int *err)
 {
     gretl_bundle *bcpy = NULL;
 
@@ -884,10 +1113,52 @@ static void bundle_count (gpointer key, gpointer value, gpointer p)
 static void print_bundled_item (gpointer key, gpointer value, gpointer p)
 {
     bundled_item *item = value;
+    const gchar *kstr = key;
+    gretl_matrix *m;
+    double x;
+    char *s;
     PRN *prn = p;
 
-    pprintf(prn, " %s (%s)", (const char *) key, 
-	    gretl_arg_type_name(item->type));
+    switch (item->type) {
+    case GRETL_TYPE_DOUBLE:
+	x = *(double *) item->data;
+	if (na(x)) {
+	    pprintf(prn, " %s = NA", kstr);
+	} else {
+	    pprintf(prn, " %s = %g", kstr, x);
+	}	
+	break;
+    case GRETL_TYPE_STRING:
+	s = (char *) item->data;
+	if (strlen(s) < 64) {
+	    pprintf(prn, " %s = %s", kstr, s);
+	} else {
+	    pprintf(prn, " %s (%s)", kstr, 
+		    gretl_arg_type_name(item->type));
+	}	
+	break;
+    case GRETL_TYPE_BUNDLE:
+	pprintf(prn, " %s (%s)", kstr, 
+		gretl_arg_type_name(item->type));
+	break;
+    case GRETL_TYPE_MATRIX:
+    case GRETL_TYPE_MATRIX_REF:
+	m = item->data;
+	if (m->rows == 1 && m->cols == 1) {
+	    pprintf(prn, " %s = %g", kstr, m->val[0]);
+	} else {
+	    pprintf(prn, " %s (%s: %d x %d)", kstr, 
+		    gretl_arg_type_name(item->type), 
+		    m->rows, m->cols);
+	}
+	break;
+    case GRETL_TYPE_SERIES:
+	pprintf(prn, " %s (%s: length %d)", kstr, 
+		gretl_arg_type_name(item->type), item->size);
+	break;
+    default:
+	break;
+    }
 
     if (item->note != NULL) {
 	pprintf(prn, " %s\n", item->note);
@@ -918,7 +1189,12 @@ int gretl_bundle_print (gretl_bundle *bundle, PRN *prn)
 	if (n_items == 0) {
 	    pprintf(prn, "bundle %s: empty\n", bundle->name);
 	} else {
-	    pprintf(prn, "bundle %s:\n", bundle->name);
+	    if (bundle->creator != NULL) {
+		pprintf(prn, "bundle %s, created by %s:\n", 
+			bundle->name, bundle->creator);
+	    } else {
+		pprintf(prn, "bundle %s:\n", bundle->name);
+	    }
 	    g_hash_table_foreach(bundle->ht, print_bundled_item, prn);
 	    pputc(prn, '\n');
 	}
@@ -989,6 +1265,35 @@ gretl_bundle *gretl_bundle_pull_from_stack (const char *name,
     return b;
 }
 
+/**
+ * copy_bundle_arg_as:
+ * @b: the original bundle.
+ * @newname: the name to be given to the copy.
+ *
+ * A copy of bundle @b is added to the stack of saved bundles
+ * under the name @newname.  This is intended for use when a bundle
+ * is given as the argument to a user-defined function: it is copied
+ * under the name assigned by the function's parameter list.
+ *
+ * Returns: 0 on success, non-zero on error.
+ */
+
+int copy_bundle_arg_as (const gretl_bundle *b, const char *newname)
+{
+    gretl_bundle *b2;
+    int err = 0;
+
+    b2 = gretl_bundle_copy(b, &err);
+
+    if (!err) {
+	strcpy(b2->name, newname);
+	b2->level += 1;
+	err = gretl_bundle_push(b2, 0);
+    }
+
+    return err;
+}
+
 /**
  * gretl_bundle_localize:
  * @origname: name of bundle at caller level.
@@ -1016,7 +1321,7 @@ int gretl_bundle_localize (const char *origname,
 	b->level += 1;
     }
 
-    return err;
+     return err;
 }
 
 /**
@@ -1141,3 +1446,203 @@ void destroy_user_bundles (void)
     set_n_saved_bundles(0);
 }
 
+/* serialize a gretl bundled item as XML */
+
+static void xml_put_bundled_item (gpointer keyp, gpointer value, gpointer p)
+{
+    const char *key = keyp;
+    bundled_item *item = value;
+    FILE *fp = p;
+    int j;
+
+    if (item->type == GRETL_TYPE_STRING) {
+	char *s = item->data;
+
+	if (s == NULL || *s == '\0') {
+	    fprintf(stderr, "bundle -> XML: skipping empty string %s\n", key);
+	    return;
+	} 
+    }
+
+    fprintf(fp, "<bundled-item key=\"%s\" type=\"%s\"", key,
+	    gretl_arg_type_name(item->type));
+
+    if (item->note != NULL) {
+	fprintf(fp, " note=\"%s\"", item->note);
+    } 
+
+    if (item->size > 0) {
+	fprintf(fp, " size=\"%d\">\n", item->size);
+    } else if (item->type == GRETL_TYPE_STRING) {
+	fputc('>', fp);
+    } else {
+	fputs(">\n", fp);
+    }
+
+    if (item->type == GRETL_TYPE_DOUBLE) {
+	double x = *(double *) item->data;
+	
+	if (na(x)) {
+	    fputs("NA", fp);
+	} else {
+	    fprintf(fp, "%.15g", x);
+	}
+    } else if (item->type == GRETL_TYPE_SERIES) {
+	double *vals = (double *) item->data;
+
+	for (j=0; j<item->size; j++) {
+	    if (na(vals[j])) {
+		fputs("NA ", fp);
+	    } else {
+		fprintf(fp, "%.15g ", vals[j]);
+	    }
+	}	    
+    } else if (item->type == GRETL_TYPE_STRING) {
+	fputs((char *) item->data, fp);
+    } else if (item->type == GRETL_TYPE_MATRIX) {
+	gretl_matrix *m = (gretl_matrix *) item->data;
+
+	gretl_xml_put_matrix(m, NULL, fp);
+    } else {
+	fprintf(stderr, "bundle -> XML: skipping %s\n", key);
+    }
+
+    fputs("</bundled-item>\n", fp);    
+}
+
+static void xml_put_bundle (gretl_bundle *b, FILE *fp)
+{
+    fprintf(fp, "<gretl-bundle name=\"%s\">\n", b->name);
+    g_hash_table_foreach(b->ht, xml_put_bundled_item, fp);
+    fputs("</gretl-bundle>\n", fp); 
+}
+
+/**
+ * write_bundles_to_file:
+ *
+ * Serializes all saved bundles as XML, writing to @fp.
+ */
+
+void write_bundles_to_file (FILE *fp)
+{
+    int i;
+
+    gretl_xml_header(fp);
+    fprintf(fp, "<gretl-bundles count=\"%d\">\n", n_saved_bundles);
+
+    gretl_push_c_numeric_locale();
+
+    for (i=0; i<n_saved_bundles; i++) {
+	if (bundles[i]->ht != NULL) {
+	    xml_put_bundle(bundles[i], fp);
+	}
+    }
+
+    gretl_pop_c_numeric_locale();
+
+    fputs("</gretl-bundles>\n", fp);
+}
+
+/* For internal use only, called from gretl_xml.c: @p1 should be of
+   type xmlNodePtr and @p2 should be an xmlDocPtr. We suppress the
+   actual pointer types in the prototype so that it's possible for a
+   module to include gretl_bundle.h without including the full libxml
+   headers.  
+*/
+
+int load_bundle_from_xml (void *p1, void *p2, const char *name)
+{
+    xmlNodePtr node = (xmlNodePtr) p1;
+    xmlDocPtr doc = (xmlDocPtr) p2;
+    xmlNodePtr cur = node->xmlChildrenNode;
+    gretl_bundle *b;
+    char *key, *typename;
+    int err = 0;
+
+    b = gretl_bundle_new();
+    if (b == NULL) {
+	return E_ALLOC;
+    }
+
+    *b->name = '\0';
+    strncat(b->name, name, VNAMELEN -1);
+
+    fprintf(stderr, "load_bundle_from_xml: '%s'\n", b->name);
+
+    while (cur != NULL && !err) {
+        if (!xmlStrcmp(cur->name, (XUC) "bundled-item")) {
+	    key = (char *) xmlGetProp(cur, (XUC) "key");
+	    typename = (char *) xmlGetProp(cur, (XUC) "type");
+	    if (key == NULL || typename == NULL) {
+		err = E_DATA;
+	    } else {
+		int type = gretl_type_from_string(typename);
+		int size = 0;
+
+		if (type == GRETL_TYPE_DOUBLE) {
+		    double x;
+
+		    if (!gretl_xml_node_get_double(cur, doc, &x)) {
+			err = E_DATA;
+		    } else {
+			err = gretl_bundle_set_data(b, key, &x, type, size);
+		    }
+		} else if (type == GRETL_TYPE_STRING) {
+		    char *s;
+
+		    if (!gretl_xml_node_get_trimmed_string(cur, doc, &s)) {
+			err = E_DATA;
+		    } else {
+			err = gretl_bundle_set_data(b, key, s, type, size);
+			free(s);
+		    }
+		} else if (type == GRETL_TYPE_SERIES) {
+		    double *xvec = gretl_xml_get_double_array(cur, doc, &size, &err);
+
+		    if (!err) {
+			err = gretl_bundle_set_data(b, key, xvec, type, size);
+			free(xvec);
+		    }
+		} else if (type == GRETL_TYPE_MATRIX) {
+		    xmlNodePtr child = cur->xmlChildrenNode;
+		    gretl_matrix *m;
+
+		    if (child == NULL) {
+			err = E_DATA;
+		    } else {
+			m = gretl_xml_get_matrix(child, doc, &err);
+			if (!err) {
+			    err = gretl_bundle_set_data(b, key, m, type, size);
+			    gretl_matrix_free(m);
+			}
+		    }
+		} else {
+		    fprintf(stderr, "bundle: ignoring unhandled type %d\n", type);
+		}
+
+		if (!err) {
+		    char *note = (char *) xmlGetProp(cur, (XUC) "note");
+
+		    if (note != NULL) {
+			gretl_bundle_set_note(b, key, note);
+			xmlFree(note);
+		    }
+		}
+
+		xmlFree(key);
+		xmlFree(typename);
+	    }
+	}
+	cur = cur->next;
+    }
+
+    if (!err) {
+	err = gretl_bundle_push(b, 0);
+	fprintf(stderr, "gretl_bundle_push: err = %d\n", err);
+    } else {
+	gretl_bundle_destroy(b);
+	fprintf(stderr, "bundle is broken (err = %d)\n", err);
+    }
+
+    return err;
+}
