@@ -50,6 +50,7 @@ static gchar *gretl_dot_dir;
 static gchar *gretl_Rprofile;
 static gchar *gretl_Rsrc;
 static gchar *gretl_Rout;
+static gchar *gretl_Rmsg;
 static gchar *gretl_Oxprog;
 static gchar *gretl_octave_prog;
 
@@ -120,6 +121,7 @@ static void make_gretl_R_names (void)
 	gretl_Rprofile = g_strdup_printf("%sgretl.Rprofile", gretl_dot_dir);
 	gretl_Rsrc = g_strdup_printf("%sRsrc", gretl_dot_dir);
 	gretl_Rout = g_strdup_printf("%sR.out", gretl_dot_dir);
+	gretl_Rmsg = g_strdup_printf("%sR.msg", gretl_dot_dir);
 	done = 1;
     }
 }
@@ -134,23 +136,34 @@ static int lib_run_R_sync (gretlopt opt, PRN *prn)
     cmd = g_strdup_printf("\"%s\" --no-save --no-init-file --no-restore-data "
 			  "--slave", gretl_rbin_path());
 
+#if FDEBUG
+    fprintf(stderr, "Running R binary, '%s'\n", gretl_rbin_path());
+#endif
+
     err = win_run_sync(cmd, NULL);
 
-    if (!err && !(opt & OPT_Q)) {
-	FILE *fp = gretl_fopen(gretl_Rout, "r");
+    if (!(opt & OPT_Q)) {
+	const gchar *outname;
+	FILE *fp;
 
-	if (fp == NULL) {
-	    err = E_FOPEN;
-	} else {
+	outname = (err)? gretl_Rmsg : gretl_Rout;
+	fp = gretl_fopen(outname, "r");
+	
+	if (fp != NULL) {
 	    char line[1024];
 
 	    while (fgets(line, sizeof line, fp)) {
 		pputs(prn, line);
 	    }
 	    fclose(fp);
+	    gretl_remove(outname);
 	}
     }
 
+#if FDEBUG
+    fprintf(stderr, "win_run_sync: err = %d\n", err);
+#endif
+
     g_free(cmd);
 
     return err;
@@ -463,7 +476,35 @@ int write_gretl_ox_file (const char *buf, gretlopt opt, const char **pfname)
     return 0;
 }
 
-int write_gretl_octave_file (const char *buf, gretlopt opt, const char **pfname)
+/* write out current dataset as an octave matrix, and, if this succeeds,
+   write appropriate octave commands to @fp to source the data
+*/
+
+static int write_data_for_octave (const double **Z, 
+				  const DATAINFO *pdinfo,
+				  FILE *fp)
+{
+    gchar *mdata;
+    int err;
+
+    mdata = g_strdup_printf("%smdata.tmp", gretl_dotdir());
+    err = write_data(mdata, NULL, Z, pdinfo, OPT_M | OPT_F, 0);
+ 
+    if (err) {
+	gretl_errmsg_sprintf("write_data_for_octave: failed with err = %d\n", err);
+    } else {
+	fputs("% load data from gretl\n", fp);
+	fprintf(fp, "load '%s'\n", mdata);
+    }
+
+    g_free(mdata);
+
+    return err;
+}
+
+int write_gretl_octave_file (const char *buf, gretlopt opt, 
+			     const double **Z, const DATAINFO *pdinfo,
+			     const char **pfname)
 {
     const gchar *fname = gretl_octave_filename();
     FILE *fp = gretl_fopen(fname, "w");
@@ -476,6 +517,17 @@ int write_gretl_octave_file (const char *buf, gretlopt opt, const char **pfname)
 	if (!err) {
 	    add_gretl_include(fp);
 	}
+
+	if (opt & OPT_D) {
+	    /* --send-data */
+	    fprintf(stderr, "*** writing data for octave\n");
+	    err = write_data_for_octave(Z, pdinfo, fp);
+	    if (err) {
+		fclose(fp);
+		return err;
+	    }
+	}
+
 	if (buf != NULL) {
 	    /* pass on the material supplied in the 'buf' argument */
 	    char line[1024];
@@ -519,13 +571,12 @@ static int write_data_for_R (const double **Z,
 
     err = write_data(Rdata, NULL, Z, pdinfo, OPT_R, 0);
     if (err) {
+	gretl_errmsg_sprintf("write_data_for_R: failed with err = %d\n", err);
 	g_free(Rdata);
 	return err;
     }
 
     fputs("# load data from gretl\n", fp);
-    fprintf(fp, "gretldata <- read.table(\"%s\", header=TRUE)\n", Rdata);
-    g_free(Rdata);
 
     if (ts) {
 	char *p, datestr[OBSLEN];
@@ -536,14 +587,18 @@ static int write_data_for_R (const double **Z,
 	if (p != NULL) {
 	    subper = atoi(p + 1);
 	}
-	    
+
+	fprintf(fp, "gretldata <- read.table(\"%s\", header=TRUE)\n", Rdata);
 	fprintf(fp, "gretldata <- ts(gretldata, start=c(%d, %d), frequency = %d)\n", 
 		atoi(datestr), subper, pdinfo->pd);
-    } else {
+    } else {	
+	fprintf(fp, "gretldata <- read.table(\"%s\", header=TRUE)\n", Rdata);
 	fputs("attach(gretldata)\n", fp);
     }
 
-    if (!err && (opt & OPT_I)) {
+    g_free(Rdata);
+
+    if (opt & OPT_I) {
 	/* let the (interactive) user see that this worked */
 	if (ts) {
 	    fputs("gretlmsg <- \"current data loaded as ts object \\\"gretldata\\\"\\n\"\n", fp);
@@ -665,10 +720,15 @@ static int write_R_source_file (const char *buf,
     if (fp == NULL) {
 	err = E_FOPEN;
     } else {
+	int sunk = 0;
+
 #ifdef G_OS_WIN32
 	if (!(opt & (OPT_I | OPT_L))) {
 	    /* non-interactive, but not using Rlib */
-	    fprintf(fp, "sink(\"%s\")\n", gretl_Rout);
+	    fprintf(fp, "sink(\"%s\", type=\"output\")\n", gretl_Rout);
+	    fprintf(fp, "errout <- file(\"%s\", open=\"wt\")\n", gretl_Rmsg);
+	    fputs("sink(errout, type=\"message\")\n", fp);
+	    sunk = 1;
 	}
 #endif
 
@@ -683,7 +743,12 @@ static int write_R_source_file (const char *buf,
 		put_R_startup_content(fp);
 		startup_done = 1;
 	    }
-	    fprintf(fp, "sink(\"%s\")\n", gretl_Rout);
+	    fprintf(fp, "sink(\"%s\", type=\"output\")\n", gretl_Rout);
+	    if (!(opt & OPT_I)) {
+		fprintf(fp, "errout <- file(\"%s\", open=\"wt\")\n", gretl_Rmsg);
+		fputs("sink(errout, type=\"message\")\n", fp);
+	    }
+	    sunk = 1;
 #ifdef G_OS_WIN32
 	    /* can this go in the "startup content"? */
 	    maybe_print_R_path_addition(fp);
@@ -691,8 +756,12 @@ static int write_R_source_file (const char *buf,
 	}
 
 	if (opt & OPT_D) {
-	    /* send data */
+	    /* --send-data */
 	    err = write_data_for_R(Z, pdinfo, opt, fp);
+	    if (err) {
+		fclose(fp);
+		return err;
+	    }
 	}
 
 	if (buf != NULL) {
@@ -706,7 +775,7 @@ static int write_R_source_file (const char *buf,
 	    }
 	}
 
-	if (opt & OPT_L) {
+	if (sunk) {
 	    fputs("sink()\n", fp);
 	}
 
@@ -884,7 +953,7 @@ static int load_R_symbols (void)
     int err = 0;
 
 #if FDEBUG
-    printf("Loading libR symbols\n");
+    printf("Loading libR symbols from '%s'\n", libpath);
 #endif
 
     Rhandle = gretl_dlopen(libpath, 1);
@@ -1078,9 +1147,13 @@ static int lib_run_Rlib_sync (gretlopt opt, PRN *prn)
 	R_unprotect(2);
     }
 
-    if (!err && prn != NULL) {
-	FILE *fp = gretl_fopen(gretl_Rout, "r");
+    if (prn != NULL) {
+	const gchar *outname;
+	FILE *fp;
 
+	outname = (err)? gretl_Rmsg : gretl_Rout;
+	fp = gretl_fopen(outname, "r");
+	
 	if (fp != NULL) {
 	    char line[512];
 
@@ -1088,7 +1161,7 @@ static int lib_run_Rlib_sync (gretlopt opt, PRN *prn)
 		pputs(prn, line);
 	    }
 	    fclose(fp);
-	    gretl_remove(gretl_Rout);
+	    gretl_remove(outname);
 	}
     }
 
@@ -1324,6 +1397,24 @@ int gretl_R_function_exec (const char *name, int *rtype, void **ret)
     return err;
 }
 
+static int run_R_lib (const double **Z, const DATAINFO *pdinfo, 
+		      gretlopt opt, PRN *prn)
+{
+    int err;
+
+    /* we don't want gretl.Rprofile in the way */
+    gretl_remove(gretl_Rprofile);
+
+    /* by passing OPT_L below we indicate that we're
+       using the library */
+    err = write_R_source_file(NULL, Z, pdinfo, opt | OPT_L);
+    if (!err) {
+	err = lib_run_Rlib_sync(opt, prn);
+    }
+
+    return err;
+}
+
 #endif /* USE_RLIB */
 
 static int foreign_block_init (const char *line, gretlopt opt, PRN *prn)
@@ -1415,24 +1506,6 @@ static int run_R_binary (const double **Z, const DATAINFO *pdinfo,
     return err;
 }
 
-static int run_R_lib (const double **Z, const DATAINFO *pdinfo, 
-		      gretlopt opt, PRN *prn)
-{
-    int err;
-
-    /* we don't want gretl.Rprofile in the way */
-    gretl_remove(gretl_Rprofile);
-
-    /* by passing OPT_L below we indicate that we're
-       using the library */
-    err = write_R_source_file(NULL, Z, pdinfo, opt | OPT_L);
-    if (!err) {
-	err = lib_run_Rlib_sync(opt, prn);
-    }
-
-    return err;
-}
-
 /**
  * foreign_execute:
  * @Z: data array.
@@ -1465,15 +1538,15 @@ int foreign_execute (const double **Z, const DATAINFO *pdinfo,
     foreign_opt |= opt;
 
     if (foreign_lang == LANG_R) {
-#ifndef USE_RLIB
-	err = run_R_binary(Z, pdinfo, foreign_opt, prn);	
-#else
+#ifdef USE_RLIB
 	if (gretl_use_Rlib()) {
 	    err = run_R_lib(Z, pdinfo, foreign_opt, prn);
 	} else {
 	    err = run_R_binary(Z, pdinfo, foreign_opt, prn);
 	}
-#endif	    
+#else
+	err = run_R_binary(Z, pdinfo, foreign_opt, prn);
+#endif
     } else if (foreign_lang == LANG_OX) {
 	err = write_gretl_ox_file(NULL, foreign_opt, NULL);
 	if (err) {
@@ -1482,7 +1555,8 @@ int foreign_execute (const double **Z, const DATAINFO *pdinfo,
 	    err = lib_run_other_sync(foreign_opt, prn);
 	}
     } else if (foreign_lang == LANG_OCTAVE) {
-	err = write_gretl_octave_file(NULL, foreign_opt, NULL);
+	err = write_gretl_octave_file(NULL, foreign_opt, 
+				      Z, pdinfo, NULL);
 	if (err) {
 	    delete_gretl_octave_file();
 	} else {
