@@ -414,6 +414,7 @@ static struct localeinfo locales[] = {
     { LANG_DE,    "german" },
     { LANG_ES,    "spanish" },
     { LANG_FR,    "french" },
+    { LANG_EL,    "greek" },
     { LANG_IT,    "italian" },
     { LANG_PL,    "polish" },
     { LANG_TR,    "turkish" },
@@ -455,6 +456,7 @@ static struct langinfo langs[] = {
     { LANG_CS,    "Czech",                "cs_CZ" },
     { LANG_FR,    "French",               "fr_FR" },
     { LANG_DE,    "German",               "de_DE" },
+    { LANG_EL,    "Greek",                "el_GR" },
     { LANG_IT,    "Italian",              "it_IT" },
     { LANG_PL,    "Polish",               "pl_PL" },
     { LANG_PT,    "Portuguese",           "pt_PT" },
