@@ -22,7 +22,11 @@
 
 #include <stdarg.h>
 
-#define M_NA (0.0/0.0)
+#ifdef NAN
+# define M_NA NAN
+#else
+# define M_NA (0.0/0.0)
+#endif
 
 /* minimum value of diagonal element of R (as in X = QR) that counts
    as non-zero for the purpose of determining the rank of X */
@@ -251,7 +255,7 @@ gretl_matrix *gretl_matrix_get_diagonal (const gretl_matrix *m, int *err);
 
 int gretl_matrix_get_row (const gretl_matrix *m, int i, gretl_vector *v);
 
-double gretl_matrix_trace (const gretl_matrix *m, int *err);
+double gretl_matrix_trace (const gretl_matrix *m);
 
 int gretl_matrix_random_fill (gretl_matrix *m, int dist);
 
@@ -291,6 +295,11 @@ gretl_matrix *gretl_matrix_complex_multiply (const gretl_matrix *a,
 					     const gretl_matrix *b,
 					     int *err);
 
+gretl_matrix *gretl_matrix_divide (const gretl_matrix *a, 
+				   const gretl_matrix *b,
+				   GretlMatrixMod mod,
+				   int *err);
+
 gretl_matrix *gretl_matrix_complex_divide (const gretl_matrix *a, 
 					   const gretl_matrix *b,
 					   int *err);
@@ -412,6 +421,9 @@ gretl_matrix *gretl_matrix_column_mean (const gretl_matrix *m, int *err);
 
 gretl_matrix *gretl_matrix_column_sd (const gretl_matrix *m, int *err);
 
+gretl_matrix *gretl_matrix_column_sd2 (const gretl_matrix *m, 
+				       int df, int *err);
+
 double gretl_matrix_row_i_mean (const gretl_matrix *m, int row);
 
 double gretl_matrix_column_j_mean (const gretl_matrix *m, int col);
@@ -423,7 +435,8 @@ void gretl_matrix_demean_by_column (gretl_matrix *m);
 gretl_matrix *gretl_matrix_vcv (gretl_matrix *m);
 
 gretl_matrix *gretl_matrix_quantiles (const gretl_matrix *m,
-				      double p, int *err);
+				      const gretl_matrix *p,
+				      int *err);
 
 double gretl_matrix_determinant (gretl_matrix *a, int *err);
 
@@ -597,6 +610,15 @@ gretl_matrix_restricted_ols (const gretl_vector *y, const gretl_matrix *X,
 			     gretl_vector *b, gretl_matrix *vcv,
 			     gretl_vector *uhat, double *s2);
 
+int 
+gretl_matrix_restricted_multi_ols (const gretl_matrix *Y, 
+				   const gretl_matrix *X,
+				   const gretl_matrix *R, 
+				   const gretl_matrix *q,
+				   gretl_matrix *B, 
+				   gretl_matrix *U,
+				   gretl_matrix **W);
+
 int gretl_matrix_SVD_ols (const gretl_vector *y, const gretl_matrix *X,
 			  gretl_vector *b, gretl_matrix *vcv,
 			  gretl_vector *uhat, double *s2);
