@@ -140,7 +140,8 @@ struct VCVInfo_ {
 
 #define LIMDEP(c) (c == LOGIT || \
                    c == PROBIT || \
-                   c == TOBIT)
+                   c == TOBIT || \
+                   c == INTREG)
 
 #define COUNT_MODEL(c) (c == POISSON || c == NEGBIN)
 
@@ -338,7 +339,7 @@ int gretl_model_get_coeff_separator (const MODEL *pmod, const char **ps, int *pp
 
 int gretl_model_new_vcv (MODEL *pmod, int *nelem);
 
-int gretl_model_write_vcv (MODEL *pmod, const gretl_matrix *V);
+int gretl_model_write_vcv (MODEL *pmod, const gretl_matrix *V, int k);
 
 VMatrix *gretl_model_get_vcv (MODEL *pmod, const DATAINFO *pdinfo);
 
@@ -355,6 +356,8 @@ int command_ok_for_model (int test_ci, gretlopt opt, int mci);
 int model_test_ok (int ci, gretlopt opt, const MODEL *pmod, 
 		   const DATAINFO *pdinfo);
 
+int gretl_is_simple_OLS (const MODEL *pmod);
+
 int gretl_is_arima_model (const MODEL *pmod);
 
 int get_first_model_stat (const char **word, const char **desc);
@@ -363,7 +366,7 @@ int get_next_model_stat (const char **word, const char **desc);
 
 int get_model_count (void);
 
-void reset_model_count (void);
+void set_model_count (int c);
 
 int model_count_plus (void);
 
@@ -422,6 +425,10 @@ void gretl_model_add_allocated_varnames (MODEL *pmod, char **vnames);
 
 int gretl_model_add_y_median (MODEL *pmod, const double *y);
 
+int gretl_model_add_normality_test (MODEL *pmod, double X2);
+
+int gretl_model_get_normality_test (const MODEL *pmod, PRN *prn);
+
 char *gretl_model_get_fitted_formula (const MODEL *pmod, int xvar,
 				      const double **Z,
 				      const DATAINFO *pdinfo);
