@@ -42,7 +42,15 @@
 #include <glib.h>
 
 #if (GLIB_MAJOR_VERSION > 2 || GLIB_MINOR_VERSION >= 6)
-#include <glib/gstdio.h>
+# include <glib/gstdio.h>
+#endif
+
+#ifndef WIN32
+# ifdef USE_GTK3
+#  define PLUGIN_SFX "gretl-gtk3/"
+# else
+#  define PLUGIN_SFX "gretl-gtk2/"
+# endif
 #endif
 
 struct INTERNAL_PATHS {
@@ -245,6 +253,39 @@ FILE *gretl_fopen (const char *fname, const char *mode)
     return fp;
 }
 
+/**
+ * gretl_try_fopen:
+ * @fname: name of file to be opened.
+ * @mode: mode in which to open the file.
+ *
+ * basically the same as gretl_fopen(), except that no
+ * error message is logged if @fname cannot be opened.
+ *
+ * Returns: file pointer, or %NULL on failure.
+ */
+
+FILE *gretl_try_fopen (const char *fname, const char *mode)
+{
+    gchar *fconv = NULL;
+    FILE *fp = NULL;
+    int err;
+
+    gretl_error_clear();
+
+    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+
+    if (!err) {
+	if (fconv != NULL) {
+	    fp = fopen(fconv, mode);
+	    g_free(fconv);
+	} else {
+	    fp = fopen(fname, mode);
+	}
+    }
+
+    return fp;
+}
+
 /**
  * gretl_open:
  * @pathname: name of file to be opened.
@@ -453,6 +494,28 @@ gzFile gretl_gzopen (const char *fname, const char *mode)
     return fz;
 }
 
+static gzFile gretl_try_gzopen (const char *fname, const char *mode)
+{
+    gchar *fconv = NULL;
+    gzFile fz = NULL;
+    int err;
+
+    gretl_error_clear();
+
+    err = maybe_recode_path(fname, stdio_use_utf8, &fconv);
+
+    if (!err) {
+	if (fconv != NULL) {
+	    fz = gzopen(fconv, mode);
+	    g_free(fconv);
+	} else {
+	    fz = gzopen(fname, mode);
+	}
+    }
+
+    return fz;
+}
+
 /**
  * gretl_chdir:
  * @path: name of directory.
@@ -596,16 +659,29 @@ int gretl_mkstemp (char *tmpl)
 
 int gretl_mkdir (const char *path)
 {
-    int err = 0;
+    int err;
 
     errno = 0;
 
-    if (mkdir(path, 0755)) {
-	if (errno != EEXIST) { 
-	    fprintf(stderr, "%s: %s\n", path, strerror(errno));
-	    err = 1;
+#if (GLIB_MAJOR_VERSION > 2 || GLIB_MINOR_VERSION >= 8)
+    err = g_mkdir_with_parents(path, 0755);
+#else
+    err = mkdir(path, 0755);
+    if (err && errno == EEXIST) {
+	/* OK if we have an existing directory */
+	DIR *dir = opendir(path);
+
+	if (dir != NULL) {
+	    closedir(dir);
+	    err = 0;
 	}
     }
+#endif
+
+    if (err) {
+	fprintf(stderr, "%s: %s\n", path, strerror(errno));
+	err = 1;
+    }
 
     return err;
 }
@@ -760,7 +836,7 @@ int gretl_is_xml_file (const char *fname)
     char test[6];
     int ret = 0;
 
-    fz = gretl_gzopen(fname, "rb");
+    fz = gretl_try_gzopen(fname, "rb");
     if (fz != Z_NULL) {
 	if (gzread(fz, test, 5)) {
 	    test[5] = '\0';
@@ -824,10 +900,10 @@ static int try_open_file (char *targ, const char *finddir,
     strcat(tmp, "\\");
     strcat(tmp, targ);
 
-    fp = gretl_fopen(tmp, "r");
+    fp = gretl_try_fopen(tmp, "r");
     if (fp == NULL && code == DATA_SEARCH) {
 	if (add_suffix(tmp, ".gdt")) {
-	    fp = gretl_fopen(tmp, "r");
+	    fp = gretl_try_fopen(tmp, "r");
 	}
     }
 
@@ -916,10 +992,10 @@ static int try_open_file (char *targ, const char *finddir,
     strcat(tmp, "/");
     strcat(tmp, targ);
 
-    fp = gretl_fopen(tmp, "r");
+    fp = gretl_try_fopen(tmp, "r");
     if (fp == NULL && code == DATA_SEARCH) {
 	if (add_suffix(tmp, ".gdt")) {
-	    fp = gretl_fopen(tmp, "r");
+	    fp = gretl_try_fopen(tmp, "r");
 	}
     }
 
@@ -996,19 +1072,19 @@ static char *search_dir (char *fname, const char *topdir, int code)
     strcpy(orig, fname);
 
     if (gretl_path_prepend(fname, topdir) == 0) {
-	test = gretl_fopen(fname, "r");
+	test = gretl_try_fopen(fname, "r");
 	if (test != NULL) {
 	    fclose(test);
 	    return fname;
 	}
 	if (code == DATA_SEARCH && add_suffix(fname, ".gdt")) {
-	    test = gretl_fopen(fname, "r");
+	    test = gretl_try_fopen(fname, "r");
 	    if (test != NULL) {
 		fclose(test);
 		return fname;
 	    }
 	} else if (code == FUNCS_SEARCH && add_suffix(fname, ".gfn")) {
-	    test = gretl_fopen(fname, "r");
+	    test = gretl_try_fopen(fname, "r");
 	    if (test != NULL) {
 		fclose(test);
 		return fname;
@@ -1114,7 +1190,145 @@ static int shelldir_open_dotfile (char *fname, char *orig)
 }
 
 /**
- * addpath:
+ * get_plausible_functions_dir:
+ * @fndir: location to receive directory name; should be of
+ * length FILENAME_MAX or more.
+ * @i: call index; should be zero on the first call.
+ * 
+ * This function can be called repeatedly, with the call index
+ * incremented each time -- so long as the return value is non-zero.
+ * The @fndir argument will be filled out with the name of the "next"
+ * directory that may plausibly contain gretl gfn files. If the call
+ * index is out of bounds 0 is returned and @fndir is set to an
+ * empty string.
+ *
+ * Returns: 1 if there are any more directories available, otherwise
+ * zero.
+ */
+
+int get_plausible_functions_dir (char *fndir, int i)
+{
+    int ret = 1;
+
+    *fndir = '\0';
+    
+    if (i == 0) {
+	/* pick up any function files in system dir */
+	build_path(fndir, gretl_home(), "functions", NULL);
+    } else if (i == 1) {
+	/* or any function files in the user's working dir... */
+	strcpy(fndir, gretl_workdir());
+    } else if (i == 2) {
+	/* or in any "functions" subdir thereof */
+	build_path(fndir, gretl_workdir(), "functions", NULL);
+    } else if (i == 3) {
+	/* or any in the user's dotdir */
+	build_path(fndir, gretl_dotdir(), "functions", NULL);
+    } else if (i == 4) {
+	/* or any in the default working dir, if not already searched */
+	const char *wdir = gretl_default_workdir();
+
+	if (wdir != NULL) {
+	    build_path(fndir, wdir, "functions", NULL);
+	} 
+    } else {
+	ret = 0;
+    }
+
+    return ret;
+}
+
+/* it's a dirent thing */
+#ifndef NAME_MAX
+# define NAME_MAX 255
+#endif
+
+/**
+ * gretl_function_package_get_path:
+ * @name: the name of the package to find, without the .gfn extension.
+ * @type: %PKG_REGULAR for a standard gfn package or %PKG_ADDON for
+ * a reognized gretl "addon".
+ * 
+ * Searches a list of directories in which we might expect to find
+ * function packages, and, if the package in question is found,
+ * returns a newly allocated string holding the full path to
+ * the package's gfn file. Public (system) directories are
+ * searched first, then directories in the user's filespace.
+ *
+ * Returns: allocated path on success, otherwise NULL.
+ */
+
+char *gretl_function_package_get_path (const char *name,
+				       PkgType type)
+{
+    char *ret = NULL;
+    char fndir[FILENAME_MAX];
+    char path[FILENAME_MAX];
+    int found = 0;
+    int i = 0;
+
+    *path = '\0';
+
+    while (get_plausible_functions_dir(fndir, i++) && !found) {
+	struct dirent *dirent;
+	const char *dname;
+	char *p, test[NAME_MAX+1];
+	DIR *dir;
+	
+	if (*fndir == '\0' || (dir = opendir(fndir)) == NULL) {
+	    continue;
+	}
+
+	/* look preferentially for .gfn files in their own
+	   subdirectories */
+
+	while ((dirent = readdir(dir)) != NULL && !found) {
+	    dname = dirent->d_name;
+	    if (!strcmp(dname, name)) {
+		FILE *fp;
+
+		sprintf(path, "%s%c%s%c%s.gfn", fndir, SLASH, 
+			dname, SLASH, dname);
+		fp = gretl_try_fopen(path, "r");
+		if (fp != NULL) {
+		    fclose(fp);
+		    found = 1;
+		} else {
+		    *path = '\0';
+		}
+	    }
+	}
+
+	if (!found && type != PKG_ADDON) {
+	    /* look for .gfn files in the top-level functions
+	       directory */
+	    rewinddir(dir);
+	    while ((dirent = readdir(dir)) != NULL && !found) {
+		dname = dirent->d_name;
+		if (has_suffix(dname, ".gfn")) {
+		    strcpy(test, dname);
+		    p = strrchr(test, '.');
+		    *p = '\0';
+		    if (!strcmp(test, name)) {
+			sprintf(path, "%s%c%s", fndir, SLASH, dname);
+			found = 1;
+		    }
+		}
+	    }
+	}
+
+	closedir(dir);
+    }
+
+    if (*path != '\0') {
+	ret = gretl_strdup(path);
+    }
+
+    return ret;
+}
+
+/**
+ * gretl_addpath:
  * @fname: initially given file name.
  * @script: if non-zero, suppose the file is a command script.
  * 
@@ -1125,7 +1339,7 @@ static int shelldir_open_dotfile (char *fname, char *orig)
  * file could be found.
  */
 
-char *addpath (char *fname, int script)
+char *gretl_addpath (char *fname, int script)
 {
     char orig[MAXLEN];
     char *tmp = fname;
@@ -1137,18 +1351,8 @@ char *addpath (char *fname, int script)
 	return fname;
     }  
 
-    if (!g_path_is_absolute(orig) && has_suffix(orig, ".gfn")) {
-	/* new as of 2009-08-27, AC */
-	const char *gfnpath = get_include_path();
-
-	if (gfnpath != NULL) {
-	    sprintf(fname, "%s%s", gfnpath, orig);
-	    return fname;
-	}
-    }
-
     /* try opening filename as given */
-    test = gretl_fopen(fname, "r");
+    test = gretl_try_fopen(fname, "r");
 
     if (test != NULL) { 
 	/* fine, got it */
@@ -1299,6 +1503,30 @@ static int substitute_homedir (char *fname)
     return err;
 }
 
+static int get_gfn_special (char *fname)
+{
+    int ok = 0;
+
+    if (!strchr(fname, '/') && !strchr(fname, '\\')) {
+	char *p, pkgname[64];
+	char *pkgpath;
+
+	*pkgname = '\0';
+	strncat(pkgname, fname, 63);
+	p = strstr(pkgname, ".gfn");
+	*p = '\0';
+	pkgpath = gretl_function_package_get_path(pkgname, PKG_ALL);
+
+	if (pkgpath != NULL) {
+	    strcpy(fname, pkgpath);
+	    free(pkgpath);
+	    ok = 1;
+	}
+    }
+
+    return ok;
+}
+
 /**
  * getopenfile:
  * @line: command line (e.g. "open foo").
@@ -1344,8 +1572,12 @@ int getopenfile (const char *line, char *fname, gretlopt opt)
 	return 0;
     }
 
+    if (has_suffix(fname, ".gfn") && get_gfn_special(fname)) {
+	return 0;
+    }
+
     /* try a basic path search on this filename */
-    fullname = addpath(fname, script);
+    fullname = gretl_addpath(fname, script);
 
     if (fullname != NULL && (opt & OPT_S)) {
 	int spos = slashpos(fname);
@@ -1393,19 +1625,25 @@ static void set_gretl_libpath (const char *path)
 #ifdef WIN32
     strcpy(paths.libpath, path);
 #else
-    const char *sfx = "-gtk2/";
+# ifdef LIBDIR
+    /* respect the libdir set at compile time, e.g. /usr/lib or
+       /usr/lib64 
+    */
+    build_path(paths.libpath, LIBDIR, PLUGIN_SFX, NULL);
+# else
     char *p = strstr(path, "/share");
-
+    
     if (p) {
 	size_t len = p - path;
 
 	*paths.libpath = '\0';
 	strncat(paths.libpath, path, len);
-	strcat(paths.libpath, "/lib/gretl");
-	strcat(paths.libpath, sfx);
+	strcat(paths.libpath, "/lib/");
+	strcat(paths.libpath, PLUGIN_SFX);
     } else {
-	sprintf(paths.libpath, "%s/lib/gretl%s", path, sfx);
+	sprintf(paths.libpath, "%s/lib/%s", path, PLUGIN_SFX);
     }
+# endif /* !LIBDIR */
 #endif /* !WIN32 */
 }
 
@@ -1640,26 +1878,23 @@ static int validate_writedir (const char *dirname)
     err = gretl_mkdir(dirname);
     if (err) {
 	gretl_errmsg_sprintf( _("Couldn't create directory '%s'"), dirname);
-    }
+    } 
 
     if (!err) {
 	/* ensure the directory is writable */
-	char *testname;
+	char testname[FILENAME_MAX];
 	FILE *fp;
 
-	testname = g_strdup_printf("%s%cwrite.chk", dirname, SLASH);
-	if (testname != NULL) {
-	    fp = gretl_fopen(testname, "w");
-	    if (fp == NULL) {
-		gretl_errmsg_sprintf(_("Couldn't write to '%s': "
-				       "gretl will not work properly!"), 
-				     dirname);
-		err = E_FOPEN;
-	    } else {
-		fclose(fp);
-		gretl_remove(testname);
-	    }
-	    g_free(testname);
+	build_path(testname, dirname, "write.chk", NULL);
+	fp = gretl_fopen(testname, "w");
+	if (fp == NULL) {
+	    gretl_errmsg_sprintf(_("Couldn't write to '%s': "
+				   "gretl will not work properly!"), 
+				 dirname);
+	    err = E_FOPEN;
+	} else {
+	    fclose(fp);
+	    gretl_remove(testname);
 	}
     }
 
@@ -2316,7 +2551,11 @@ static void load_default_path (char *targ)
 #ifdef OSX_BUILD
 	strcpy(targ, "Sans 9");
 #else
-	strcpy(targ, "Vera 9");
+	if (chinese_locale()) {
+	    strcpy(targ, "NSimSun 10");
+	} else {
+	    strcpy(targ, "Vera 9");
+	}
 #endif /* OSX_BUILD */	
     }
 }
@@ -2428,7 +2667,8 @@ int gretl_set_paths (ConfigPaths *cpaths, gretlopt opt)
 	    /* try falling back on the default working dir */
 	    const char *defpath = gretl_default_workdir();
 
-	    if (*defpath != '\0' && strcmp(defpath, paths.workdir)) {
+	    if (defpath != NULL && *defpath != '\0' && 
+		strcmp(defpath, paths.workdir)) {
 		err1 = validate_writedir(defpath);
 		if (err1 == 0) {
 		    strcpy(paths.workdir, defpath);
