@@ -36,6 +36,11 @@ typedef enum {
     GRETL_CLI_HELPFILE
 } HelpPaths;
 
+typedef enum {
+    PKG_ALL,
+    PKG_ADDON
+} PkgType;
+
 typedef struct ConfigPaths_ ConfigPaths;
 
 /* these are all the gretl paths which are recorded in
@@ -77,6 +82,8 @@ int string_is_utf8 (const unsigned char *s);
 
 FILE *gretl_fopen (const char *fname, const char *mode);
 
+FILE *gretl_try_fopen (const char *fname, const char *mode);
+
 FILE *gretl_read_user_file (const char *fname);
 
 int gretl_open (const char *pathname, int flags);
@@ -103,7 +110,7 @@ int gretl_is_xml_file (const char *fname);
 
 int gretl_isdir (const char *path);
 
-char *addpath (char *fname, int script);
+char *gretl_addpath (char *fname, int script);
 
 int getopenfile (const char *line, char *fname, gretlopt opt);
 
@@ -117,6 +124,11 @@ int gretl_set_paths (ConfigPaths *paths, gretlopt opt);
 
 int gretl_update_paths (ConfigPaths *cpaths, gretlopt opt);
 
+int get_plausible_functions_dir (char *fndir, int i);
+
+char *gretl_function_package_get_path (const char *name,
+				       PkgType type);
+
 const char *helpfile_path (int id);
 
 const char *gretl_home (void);
