@@ -1081,7 +1081,9 @@ int print_start_redirection (PRN *prn, FILE *fp)
 	    /* disable printing */
 	    prn->fixed = 1;
 	} else {
-	    /* hook output to specified file */
+	    /* record current stream in fpaux, and
+	       hook output to specified stream
+	    */
 	    prn->fpaux = prn->fp;
 	    prn->fp = fp;
 	}
@@ -1110,7 +1112,9 @@ int print_end_redirection (PRN *prn)
 	if (prn->fixed) {
 	    prn->fixed = 0;
 	} else if (prn->fp != NULL) {
-	    fclose(prn->fp);
+	    if (prn->fp != stdout && prn->fp != stderr) {
+		fclose(prn->fp);
+	    }
 	    prn->fp = prn->fpaux;
 	    prn->fpaux = NULL;
 	}
