@@ -32,7 +32,7 @@
 
 #define RDEBUG 0
 
-#define EQN_UNSPEC -9
+#define R_UNSPEC -9
 
 enum {
     VECM_NONE = 0,
@@ -77,9 +77,9 @@ struct gretl_restriction_ {
     int code;
 };
 
-#define eqn_specified(r,i,j,c) (((c=='b' && r->bmulti) || (c=='a' && r->amulti)) \
-                                && r->rows[i]->eq != NULL \
-                                && r->rows[i]->eq[j] != EQN_UNSPEC)
+#define targ_specified(r,i,j,c) (((c=='b' && r->bmulti) || (c=='a' && r->amulti)) \
+				 && r->rows[i]->eq != NULL		\
+				 && r->rows[i]->eq[j] != R_UNSPEC)
 
 /* @R is putatively the matrix in R \beta -q = 0.  Check that it makes
    sense in that role, i.e., that RR' is non-singular. */
@@ -87,27 +87,27 @@ struct gretl_restriction_ {
 static int check_R_matrix (const gretl_matrix *R)
 {
     gretl_matrix *m;
-    int g = gretl_matrix_rows(R);
     int err = 0;
 
-    m = gretl_matrix_alloc(g, g);
+    m = gretl_matrix_alloc(R->rows, R->rows);
+
     if (m == NULL) {
-	return E_ALLOC;
+	err = E_ALLOC;
+    } else {
+	gretl_matrix_multiply_mod(R, GRETL_MOD_NONE,
+				  R, GRETL_MOD_TRANSPOSE,
+				  m, GRETL_MOD_NONE);
+
+	err = gretl_invert_general_matrix(m);
+
+	if (err == E_SINGULAR) {
+	    gretl_errmsg_set(_("Matrix inversion failed: restrictions may be "
+			       "inconsistent or redundant"));
+	}
+
+	gretl_matrix_free(m);
     }
 
-    gretl_matrix_multiply_mod(R, GRETL_MOD_NONE,
-			      R, GRETL_MOD_TRANSPOSE,
-			      m, GRETL_MOD_NONE);
-
-    err = gretl_invert_general_matrix(m);
-
-    if (err == E_SINGULAR) {
-	gretl_errmsg_set(_("Matrix inversion failed: restrictions may be "
-			   "inconsistent or redundant"));
-    }
-    
-    gretl_matrix_free(m);
-
     return err;
 }
 
@@ -121,7 +121,7 @@ static int augment_vecm_restriction (gretl_restriction *rset,
 {
     GRETL_VAR *vecm = rset->obj;
     const gretl_matrix *R0, *q0;
-    gretl_matrix *R2, *q2;
+    gretl_matrix *R2 = NULL, *q2 = NULL;
     int err = 0;
 
     if (letter == 'b') {
@@ -179,31 +179,31 @@ static int add_old_vecm_restriction (gretl_restriction *rset,
 				     char letter)
 {
     GRETL_VAR *vecm = rset->obj; 
-    const gretl_matrix *R;
-    const gretl_matrix *q;
     int err = 0;
 
     if (letter == 'b') {
-	R = gretl_VECM_R_matrix(vecm);
-	q = gretl_VECM_q_matrix(vecm);
+	const gretl_matrix *R = gretl_VECM_R_matrix(vecm);
+	const gretl_matrix *q = gretl_VECM_q_matrix(vecm);
+
 	rset->R = gretl_matrix_copy(R);
 	if (rset->R == NULL) {
 	    err = E_ALLOC;
-	} else {
+	} else if (q != NULL) {	
 	    rset->q = gretl_matrix_copy(q);
-	    if (q != NULL && rset->q == NULL) {
+	    if (rset->q == NULL) {
 		err = E_ALLOC;
 	    }
 	}
     } else {
-	R = gretl_VECM_Ra_matrix(vecm);
-	q = gretl_VECM_qa_matrix(vecm);
-	rset->Ra = gretl_matrix_copy(R);
+	const gretl_matrix *Ra = gretl_VECM_Ra_matrix(vecm);
+	const gretl_matrix *qa = gretl_VECM_qa_matrix(vecm);
+
+	rset->Ra = gretl_matrix_copy(Ra);
 	if (rset->Ra == NULL) {
 	    err = E_ALLOC;
-	} else {
-	    rset->qa = gretl_matrix_copy(q);
-	    if (q != NULL && rset->qa == NULL) {
+	} else if (qa != NULL) {	
+	    rset->qa = gretl_matrix_copy(qa);
+	    if (rset->qa == NULL) {
 		err = E_ALLOC;
 	    }
 	}
@@ -258,17 +258,15 @@ static int get_R_sys_column (const gretl_restriction *rset,
 
 static double get_restriction_param (const rrow *r, int k)
 {
-    double x = 0.0;
     int i;    
 
     for (i=0; i<r->nterms; i++) {
 	if (r->bnum[i] == k) {
-	    x = r->mult[i];
-	    break;
+	    return r->mult[i];
 	}
     }
 
-    return x;
+    return 0.0;
 }
 
 static void vecm_cross_error (void)
@@ -282,14 +280,13 @@ static void vecm_cross_error (void)
 
 static int vecm_ab_check (gretl_restriction *rset)
 {
-    rrow *r;
     int atotal = 0, btotal = 0;
     int i, j, err = 0;
 
     for (i=0; i<rset->g && !err; i++) {
+	rrow *r = rset->rows[i];
 	int a = 0, b = 0;
 
-	r = rset->rows[i];
 	for (j=0; j<r->nterms && !err; j++) {
 	    if (r->letter[j] == 'a') {
 		a++;
@@ -322,21 +319,23 @@ static int vecm_ab_check (gretl_restriction *rset)
 
 static int vecm_x_check (gretl_restriction *rset, char letter)
 {
-    rrow *r;
     int *multi = (letter == 'a')? &rset->amulti : &rset->bmulti;
     int *ki = (letter == 'a')? &rset->ga : &rset->gb;
     int unspec = 0, anyspec = 0;
     int i, j, err = 0;
 
     for (i=0; i<rset->g && !err; i++) {
-	r = rset->rows[i];
+	rrow *r = rset->rows[i];
+
 	if (r->letter[0] != letter) {
 	    continue;
 	}
+
 	*ki += 1;
+
 	for (j=0; j<r->nterms && !err; j++) {
 	    if (r->eq != NULL) {
-		if (r->eq[j] == EQN_UNSPEC) {
+		if (r->eq[j] == R_UNSPEC) {
 		    unspec = 1;
 		} else if (r->eq[j] >= 0) {
 		    anyspec = 1;
@@ -436,8 +435,6 @@ static int rset_allocate_R_q (gretl_restriction *rset, int nc)
 static int equation_form_matrices (gretl_restriction *rset)
 {
     MODEL *pmod = rset->obj;
-    rrow *r;
-    double x;
     int nc = 0;
     int col, i, j;
 
@@ -456,7 +453,9 @@ static int equation_form_matrices (gretl_restriction *rset)
     }
 
     for (i=0; i<rset->g; i++) { 
-	r = rset->rows[i];
+	rrow *r = rset->rows[i];
+	double x;
+
 	col = 0;
 	for (j=0; j<pmod->ncoeff; j++) {
 	    if (rset->mask[j]) {
@@ -475,23 +474,20 @@ static int equation_form_matrices (gretl_restriction *rset)
 
 static int sys_form_matrices (gretl_restriction *rset)
 {
-    rrow *r;
-    double x;
-    int nc, col, i, j;
+    int nc = system_n_indep_vars(rset->obj);
+    int col, i, j;
     int err = 0;
 
-    nc = system_n_indep_vars(rset->obj);
-
     if (rset_allocate_R_q(rset, nc)) {
 	return E_ALLOC;
     }
 
     for (i=0; i<rset->g; i++) { 
-	r = rset->rows[i];
+	rrow *r = rset->rows[i];
+
 	for (j=0; j<r->nterms; j++) {
 	    col = get_R_sys_column(rset, i, j);
-	    x = r->mult[j];
-	    gretl_matrix_set(rset->R, i, col, x);
+	    gretl_matrix_set(rset->R, i, col, r->mult[j]);
 	}
 	gretl_vector_set(rset->q, i, r->rhs);
     } 
@@ -533,8 +529,6 @@ static int vecm_form_matrices (gretl_restriction *rset)
        (if both are given) */
 
     for (m=0; m<2; m++) {
-	rrow *r;
-	double x;
 	gretl_matrix *R = (m == 0)? rset->R : rset->Ra;
 	gretl_matrix *q = (m == 0)? rset->q : rset->qa;
 	char letter = (m == 0)? 'b' : 'a';
@@ -546,18 +540,24 @@ static int vecm_form_matrices (gretl_restriction *rset)
 	}
 
 	for (i=0; i<rset->g; i++) { 
-	    r = rset->rows[i];
-	    if (r->letter[0] != letter) {
-		continue;
+	    rrow *r = rset->rows[i];
+
+	    if (r->letter[0] == letter) {
+		for (j=0; j<r->nterms; j++) {
+		    col = get_R_vecm_column(rset, i, j, letter);
+		    gretl_matrix_set(R, row, col, r->mult[j]);
+		}
+		gretl_vector_set(q, row, r->rhs);
+		row++;
 	    }
-	    for (j=0; j<r->nterms; j++) {
-		col = get_R_vecm_column(rset, i, j, letter);
-		x = r->mult[j];
-		gretl_matrix_set(R, row, col, x);
-	    }
-	    gretl_vector_set(q, row, r->rhs);
-	    row++;
 	}
+
+#if RDEBUG
+	if (letter == 'a') {
+	    gretl_matrix_print(R, "Ra, constructed");
+	    gretl_matrix_print(q, "qa, constructed");
+	}
+#endif
     }
 
     /* cumulate prior restrictions if needed */
@@ -817,7 +817,7 @@ static int pick_apart (gretl_restriction *r, const char *s,
 	}
     } else {
 	/* only one field: [bnum] */
-	*eq = EQN_UNSPEC;
+	*eq = R_UNSPEC;
 	if (isdigit(*s1)) {
 	    *bnum = positive_int_from_string(s1);
 	} else {
@@ -863,7 +863,7 @@ static int parse_b_bit (gretl_restriction *r, const char *s,
 	    return err;
 	}
 	if (r->otype == GRETL_OBJ_VAR) {
-	    *eq = EQN_UNSPEC;
+	    *eq = R_UNSPEC;
 	}
 	err = 0;
     } else if (*s == '[') {
@@ -880,14 +880,14 @@ static int parse_b_bit (gretl_restriction *r, const char *s,
 	*bnum -= 1; /* convert to zero base */
     }
 
-    if (*eq == EQN_UNSPEC) {
-	/* didn't get an equation number */
+    if (*eq == R_UNSPEC) {
+	/* didn't get a specific equation number */
 	if (r->otype == GRETL_OBJ_EQN) {
 	    *eq = 0;
 	} else if (r->otype != GRETL_OBJ_VAR) {
 	    err = E_PARSE;
 	}
-    } else if (*eq < 1 && *eq >= 0) {
+    } else if (*eq == 0) {
 	gretl_errmsg_sprintf(_("Equation number (%d) is out of range"), 
 			     *eq);
 	err = 1;
@@ -1046,6 +1046,8 @@ void destroy_restriction_set (gretl_restriction *rset)
 #if RDEBUG
     fprintf(stderr, "destroy_restriction_set: R at %p, q at %p\n",
 	    (void *) rset->R, (void *) rset->q);
+    fprintf(stderr, " Ra at %p, qa at %p\n",
+	    (void *) rset->Ra, (void *) rset->qa);
 #endif
     
     gretl_matrix_free(rset->R);
@@ -1159,7 +1161,7 @@ static void print_restriction (const gretl_restriction *rset,
 	letter = (r->letter != NULL)? r->letter[j] : 'b';
 	k = r->bnum[j];
 	print_mult(r->mult[j], j == 0, prn);
-	if (eqn_specified(rset, i, j, letter)) {
+	if (targ_specified(rset, i, j, letter)) {
 	    pprintf(prn, "%c[%d,%d]", letter, r->eq[j] + 1, k + 1);
 	} else if (rset->otype == GRETL_OBJ_VAR) {
 	    pprintf(prn, "%c[%d]", letter, k + 1);
@@ -1184,7 +1186,7 @@ static void print_restriction (const gretl_restriction *rset,
 
 static void print_restriction_set (const gretl_restriction *rset, 
 				   const DATAINFO *pdinfo, 
-				   PRN *prn)
+				   gretlopt opt, PRN *prn)
 {
     int i;
 
@@ -1203,6 +1205,25 @@ static void print_restriction_set (const gretl_restriction *rset,
 	}
 	print_restriction(rset, i, pdinfo, prn);
     }
+
+    if (opt & OPT_V) {
+	if (rset->R != NULL || rset->q != NULL ||
+	    rset->Ra != NULL || rset->qa != NULL) {
+	    pputc(prn, '\n');
+	}
+	if (rset->R != NULL) {
+	    gretl_matrix_print_to_prn(rset->R, "R", prn);
+	}
+	if (rset->q != NULL) {
+	    gretl_matrix_print_to_prn(rset->q, "q", prn);
+	}
+	if (rset->Ra != NULL) {
+	    gretl_matrix_print_to_prn(rset->Ra, "R(alpha)", prn);
+	}
+	if (rset->qa != NULL) {
+	    gretl_matrix_print_to_prn(rset->qa, "q(alpha)", prn);
+	}
+    }	
 }
 
 void print_restriction_from_matrices (const gretl_matrix *R,
@@ -1358,6 +1379,37 @@ static gretl_restriction *restriction_set_new (void *ptr,
     return rset;
 }
 
+gretl_restriction *rset_from_VECM (GRETL_VAR *var, int *err)
+{
+    const gretl_matrix *R = gretl_VECM_R_matrix(var);
+    const gretl_matrix *q = gretl_VECM_q_matrix(var);
+    const gretl_matrix *Ra = gretl_VECM_Ra_matrix(var);
+    const gretl_matrix *qa = gretl_VECM_qa_matrix(var);
+    gretl_restriction *rset;
+
+    if (R == NULL && q == NULL && Ra == NULL && qa == NULL) {
+	/* no restriction in place */
+	return NULL;
+    }
+
+    rset = calloc(1, sizeof *rset);
+    if (rset == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    rset->obj = var;
+    rset->otype = GRETL_OBJ_VAR;
+    rset->opt = OPT_B;
+
+    rset->R = (gretl_matrix *) R;
+    rset->q = (gretl_matrix *) q;
+    rset->Ra = (gretl_matrix *) Ra;
+    rset->qa = (gretl_matrix *) qa;
+
+    return rset;
+}
+
 /* check that the coefficients referenced in a restriction are
    within bounds, relative to the equation or system that is
    to be restricted */
@@ -1371,11 +1423,11 @@ static int bnum_out_of_bounds (const gretl_restriction *rset,
 	GRETL_VAR *var = rset->obj;
 
 	if (i >= gretl_VECM_rank(var)) {
-	    gretl_errmsg_sprintf(_("Equation number (%d) is out of range"), 
+	    gretl_errmsg_sprintf(_("Coefficient number (%d) is out of range"), 
 				 i + 1);
 	} else if ((letter == 'b' && j >= gretl_VECM_n_beta(var)) ||
 		   (letter == 'a' && j >= gretl_VECM_n_alpha(var))) {
-	    gretl_errmsg_sprintf(_("Coefficient number (%d) is out of range"), 
+	    gretl_errmsg_sprintf(_("Equation number (%d) is out of range"), 
 				 j + 1);
 	} else {
 	    ret = 0;
@@ -1958,7 +2010,7 @@ static int save_restricted_model (ExecState *state,
 	for (i=0; i<rmod->ncoeff; i++) {
 	    rmod->coeff[i] = b->val[i];
 	}
-	err = gretl_model_write_vcv(rmod, S);
+	err = gretl_model_write_vcv(rmod, S, -1);
     }
 
     if (!err) {
@@ -2408,7 +2460,7 @@ GRETL_VAR *gretl_restricted_vecm (gretl_restriction *rset,
     *err = restriction_set_form_matrices(rset);
 
     if (rset->rows != NULL) {
-	print_restriction_set(rset, pdinfo, prn);
+	print_restriction_set(rset, pdinfo, opt, prn);
     } 
 
     if (!*err) {
@@ -2548,8 +2600,7 @@ gretl_restriction_finalize_full (ExecState *state,
 				 gretlopt opt,
 				 PRN *prn)
 {
-    int silent, t = rset->otype;
-    int err = 0;
+    int silent, err = 0;
 
 #if RDEBUG
     fprintf(stderr, "gretl_restriction_finalize: starting\n");
@@ -2562,7 +2613,7 @@ gretl_restriction_finalize_full (ExecState *state,
     rset->opt |= opt;
     silent = (rset->opt & OPT_S);
 
-    if (t == GRETL_OBJ_EQN && (opt & OPT_F)) {
+    if (rset->otype == GRETL_OBJ_EQN && (opt & OPT_F)) {
 	/* --full option is for OLS only */
 	MODEL *pmod = rset->obj;
 
@@ -2577,7 +2628,7 @@ gretl_restriction_finalize_full (ExecState *state,
 	return err;
     }
 
-    if (t != GRETL_OBJ_EQN) {
+    if (rset->otype != GRETL_OBJ_EQN) {
 	err = restriction_set_form_matrices(rset);
 	if (err) {
 	    destroy_restriction_set(rset);
@@ -2585,16 +2636,20 @@ gretl_restriction_finalize_full (ExecState *state,
 	}
     }
 
-    if ((rset->rows != NULL) && !silent) {
-	print_restriction_set(rset, pdinfo, prn);
+    if (!silent && rset->rows != NULL) {
+	print_restriction_set(rset, pdinfo, opt, prn);
     }
 
-    if (t == GRETL_OBJ_VAR) {
+    if (rset->otype == GRETL_OBJ_VAR) {
 	err = gretl_VECM_test(rset->obj, rset, pdinfo, rset->opt, prn);
-    } else if (t == GRETL_OBJ_SYS) {
-	system_set_restriction_matrices(rset->obj, rset->R, rset->q);
-	rset->R = NULL;
-	rset->q = NULL;
+    } else if (rset->otype == GRETL_OBJ_SYS) {
+	if (rset->opt & OPT_W) {
+	    err = system_wald_test(rset->obj, rset->R, rset->q, opt, prn);
+	} else {
+	    system_set_restriction_matrices(rset->obj, rset->R, rset->q);
+	    rset->R = NULL;
+	    rset->q = NULL;
+	}
     } else {
 	/* single-equation model */
 	err = do_single_equation_test(state, rset, Z, pdinfo, prn);
