@@ -416,6 +416,7 @@ static saved_string built_ins[] = {
     { "shelldir", 0, NULL },
     { "Rbin",     0, NULL },
     { "Rlib",     0, NULL },
+    { "pkgdir",   0, NULL }
 };
 
 #ifdef WIN32
@@ -441,12 +442,16 @@ void gretl_insert_builtin_string (const char *name, const char *s)
     for (i=0; i<n; i++) {
 	if (!strcmp(name, built_ins[i].name)) {
 	    free(built_ins[i].s);
-	    m = strlen(s);
-	    if (s[m-1] == SLASH) {
-		/* drop trailing dir separator for paths */
-		built_ins[i].s = gretl_strndup(s, m - 1);
+	    if (s == NULL) {
+		built_ins[i].s = NULL;
 	    } else {
-		built_ins[i].s = gretl_strdup(s);
+		m = strlen(s);
+		if (s[m-1] == SLASH) {
+		    /* drop trailing dir separator for paths */
+		    built_ins[i].s = gretl_strndup(s, m - 1);
+		} else {
+		    built_ins[i].s = gretl_strdup(s);
+		}
 	    }
 	    return;
 	}
@@ -673,6 +678,21 @@ void saved_strings_cleanup (void)
     clean_up_codevars();
 }
 
+void destroy_user_strings (void)
+{
+    int i;
+
+    for (i=0; i<n_saved_strings; i++) {
+	free(saved_strings[i].s);
+    }
+
+    free(saved_strings);
+    saved_strings = NULL;
+    n_saved_strings = 0;
+
+    clean_up_codevars();
+}
+
 /* called on exiting a user-defined function to clean
    up any strings defined therein */
 
@@ -790,14 +810,16 @@ char *gretl_backtick (const char *arg, int *err)
     return val;
 }
 
-char *gretl_getenv (const char *key, int *err)
+char *gretl_getenv (const char *key, int *defined, int *err)
 {
     char *test = getenv(key);
     char *val = NULL;
 
     if (test == NULL) {
+	*defined = 0;
 	val = gretl_strdup("");
     } else {
+	*defined = 1;
 	val = gretl_strdup(test);
     }
 
@@ -860,7 +882,7 @@ char *retrieve_file_content (const char *fname, int *err)
 
 	*fullname = '\0';
 	strncat(fullname, fname, FILENAME_MAX - 1);
-	addpath(fullname, 0);
+	gretl_addpath(fullname, 0);
 
 	g_file_get_contents(fullname, &ret, &len, &gerr);
 
