@@ -42,6 +42,8 @@
 # include <glib.h>
 #endif
 
+static void gretl_tests_cleanup (void);
+
 /**
  * date:
  * @nt: observation number (zero-based).
@@ -409,10 +411,11 @@ int gretl_isdiscrete (int t1, int t2, const double *x)
 	n = few_vals(t1, t2, x, &r);
 	if (n > FEWVALS) {
 	    d = 0;
-	} else if (r > 0.9) {
+	} else if (r > 0.9 && n > 30) {
 	    /* somewhat arbitrary: but if r (= ratio of distinct
-	       values to number of cases) is "too high", perhaps we
-	       should not take the var as discrete
+	       values to number of cases) is "too high", and the
+	       number of observations is not tiny, perhaps we should
+	       not take the var as discrete
 	    */
 	    d = 0;
 	} else if (n < 5) {
@@ -911,7 +914,7 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
 	stobs[8] = '\0';
     }
 
-    /* does frequency make sense? */
+    /* does the supplied frequency make sense? */
     if (pd < 1 || (pdinfo->n > 0 && pd > pdinfo->n && opt != OPT_T)) {
 	gretl_errmsg_sprintf(_("frequency (%d) does not make seem to make sense"), pd);
 	return 1;
@@ -965,9 +968,9 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
 	    return 1;
 	}
 
-	/* catch undated daily or weekly data */
 	if ((pd == 5 || pd == 6 || pd == 7 || pd == 52)  
 	    && min == 0 && opt != OPT_X && opt != OPT_S && opt != OPT_C) {
+	    /* catch undated daily or weekly data */
 	    structure = TIME_SERIES;
 	} else {
 	    if (catch_setobs_errors(stobs, pd, pdinfo->n, min, opt)) {
@@ -983,6 +986,10 @@ int set_obs (const char *line, double ***pZ, DATAINFO *pdinfo,
 		    }
 		}
 	    } else {
+		if (structure == TIME_SERIES && min > 0 &&
+		    !recognized_ts_frequency(pd)) {
+		    structure = SPECIAL_TIME_SERIES;
+		}
 		real_format_obs(stobs, maj, min, pd, '.');
 		if (structure == STRUCTURE_UNKNOWN && 
 		    recognized_ts_frequency(pd)) {
@@ -1572,6 +1579,15 @@ int gretl_delete_var_by_name (const char *s, PRN *prn)
 {
     int err = 0;
 
+    if (s == NULL || *s == '\0') {
+	return E_PARSE;
+    }
+
+    if (object_is_function_arg(s)) {
+	gretl_errmsg_sprintf("The variable %s is read-only", s);
+	return E_DATA;
+    }
+
     if (!strcmp(s, "kalman")) {
 	err = delete_kalman(prn);	
     } else if (gretl_is_scalar(s)) {
@@ -1594,7 +1610,7 @@ int gretl_delete_var_by_name (const char *s, PRN *prn)
    otherwise fall back on clock()
 */ 
 
-#if defined(_OPENMP)
+#if defined(_OPENMP) && defined(USE_OPENMP)
 
 #include <omp.h>
 
@@ -1688,27 +1704,42 @@ double gretl_stopwatch (void)
 void libgretl_init (void)
 {
     libset_init();
+
+#ifdef WIN32
+    fprintf(stderr, "libgretl_init: initializing RNG\n");
+#endif
     gretl_rand_init();
+#ifdef WIN32
+    fprintf(stderr, " RNG ready\n");
+#endif
+
     gretl_stopwatch_init();
 }
 
 void libgretl_session_cleanup (int mode)
 {
     gretl_saved_objects_cleanup();
-    gretl_transforms_cleanup();
-    gretl_lists_cleanup();
-    gretl_plotx(NULL, NULL);
 
-    if (mode != SESSION_PRESERVE_MATRICES) {
+    if (mode != SESSION_CLEAR_OTHER) {
+	/* trash dataset-related items */
+	gretl_transforms_cleanup();
+	gretl_lists_cleanup();
+	gretl_tests_cleanup();
+	gretl_plotx(NULL, NULL);
+    }
+
+    if (mode != SESSION_CLEAR_DATASET) {
+	/* trash non-dataset items */
 	destroy_user_scalars();
 	destroy_user_matrices();
 	destroy_user_bundles();
+	destroy_user_strings();
     }
 }
 
 void libgretl_cleanup (void)
 {
-    libgretl_session_cleanup(SESSION_CLEAR_FULL);
+    libgretl_session_cleanup(SESSION_CLEAR_ALL);
 
     gretl_rand_free();
     gretl_functions_cleanup();
@@ -1733,12 +1764,15 @@ enum {
     SET_TEST_STAT,
     GET_TEST_STAT,
     GET_TEST_PVAL,
-    GET_TEST_LNL
+    GET_TEST_LNL,
+    TESTS_CLEANUP
 };
 
 #define getcode(c) (c == GET_TEST_STAT || c == GET_TEST_PVAL || \
                     c == GET_TEST_LNL)
 
+static GretlType last_test_type;
+
 static double
 record_or_get_test_result (double teststat, double pval, double lnl,
 			   char *instr, int code)
@@ -1747,10 +1781,10 @@ record_or_get_test_result (double teststat, double pval, double lnl,
     static double val = NADBL;
     static double pv = NADBL;
     static double ll = NADBL;
-
     double ret = NADBL;
 
     if (code == SET_TEST_STAT) {
+	last_test_type = GRETL_TYPE_DOUBLE;
 	val = teststat;
 	pv = pval;
 	ll = lnl;
@@ -1758,7 +1792,7 @@ record_or_get_test_result (double teststat, double pval, double lnl,
 	if (instr != NULL) {
 	    strncat(savestr, instr, MAXLABEL - 1);
 	} 
-    } else if (getcode(code)) {
+    } else if (getcode(code) && last_test_type == GRETL_TYPE_DOUBLE) {
 	if (instr != NULL) {
 	    if (code == GET_TEST_STAT) {
 		sprintf(instr, _("%s test"), savestr);
@@ -1780,6 +1814,59 @@ record_or_get_test_result (double teststat, double pval, double lnl,
     return ret;
 }
 
+static gretl_matrix *
+record_or_get_test_matrix (gretl_matrix *tests, 
+			   gretl_matrix *pvals, 
+			   int code, int *err)
+{
+    static gretl_matrix *vals = NULL;
+    static gretl_matrix *pvs = NULL;
+    gretl_matrix *ret = NULL;
+
+    if (code == TESTS_CLEANUP) {
+	gretl_matrix_free(vals);
+	gretl_matrix_free(pvs);
+	vals = pvs = NULL;
+	last_test_type = GRETL_TYPE_NONE;
+	return NULL;
+    }
+
+    if (code == SET_TEST_STAT) {
+	last_test_type = GRETL_TYPE_MATRIX;
+	gretl_matrix_free(vals);
+	vals = tests;
+	gretl_matrix_free(pvs);
+	pvs = pvals;
+    } else if (getcode(code)) {
+	gretl_matrix *src = (code == GET_TEST_STAT)? vals : pvs;
+
+	if (src != NULL) {
+	    ret = gretl_matrix_copy(src);
+	    if (ret == NULL) {
+		*err = E_ALLOC;
+	    }
+	} else {
+	    *err = E_BADSTAT;
+	}
+    } 
+	
+    return ret;
+}
+
+static void gretl_tests_cleanup (void)
+{
+    record_or_get_test_matrix(NULL, NULL, TESTS_CLEANUP, NULL);
+}
+
+/* Returns the type (scalar, matrix or none) of the last recorded
+   test statistic/p-value pair.
+*/
+
+int get_last_test_type (void)
+{
+    return last_test_type;
+}
+
 void record_test_result (double teststat, double pval, char *blurb)
 {
     record_or_get_test_result(teststat, pval, NADBL, blurb, SET_TEST_STAT);
@@ -1791,6 +1878,17 @@ void record_LR_test_result (double teststat, double pval, double lnl,
     record_or_get_test_result(teststat, pval, lnl, blurb, SET_TEST_STAT);
 }
 
+/* Note: the hypothesis-test recorder "takes ownership" of the
+   two input matrices, @tests and @pvals, which should therefore 
+   NOT be freed by the caller.
+*/
+
+void record_matrix_test_result (gretl_matrix *tests, 
+				gretl_matrix *pvals)
+{
+    record_or_get_test_matrix(tests, pvals, SET_TEST_STAT, NULL);
+}
+
 double get_last_test_statistic (char *blurb)
 {
     return record_or_get_test_result(0, 0, 0, blurb, GET_TEST_STAT);
@@ -1806,6 +1904,16 @@ double get_last_lnl (char *blurb)
     return record_or_get_test_result(0, 0, 0, blurb, GET_TEST_LNL);
 }
 
+gretl_matrix *get_last_test_matrix (int *err)
+{
+    return record_or_get_test_matrix(NULL, NULL, GET_TEST_STAT, err);
+}
+
+gretl_matrix *get_last_pvals_matrix (int *err)
+{
+    return record_or_get_test_matrix(NULL, NULL, GET_TEST_PVAL, err);
+}
+
 
 
 
