@@ -386,6 +386,21 @@ int gretl_spawn (char *cmdline)
     return real_win_run_sync(cmdline, NULL, 0);
 }
 
+int gretl_spawn_with_fixup (char *prog, char *arg)
+{
+    char shortpath[FILENAME_MAX];
+    gchar *cmdline;
+    int err;
+
+    *shortpath = '\0';
+    GetShortPathName(arg, shortpath, FILENAME_MAX);
+    cmdline = g_strdup_printf("\"%s\" \"%s\"", prog, shortpath);
+    err = real_win_run_sync(cmdline, NULL, 0);
+    g_free(cmdline);
+
+    return err;
+}
+
 /* Retrieve various special paths from the bowels of MS
    Windows.  Note that these paths will be in the locale
    encoding, not UTF-8 */
@@ -821,6 +836,17 @@ static int try_for_R_path (HKEY tree, char *s)
     return err;
 }
 
+static void append_R_filename (char *s, int which)
+{
+    if (which == RGUI) {
+	strcat(s, "Rgui.exe");
+    } else if (which == RTERM) {
+	strcat(s, "Rterm.exe");
+    } else if (which == RLIB) {
+	strcat(s, "R.dll");
+    }
+}
+
 /* See if we can get the R installation path from the Windows
    registry. This is not a sure thing, since at least as of R
    2.11.1 recording the path in the registry on installation
@@ -844,16 +870,27 @@ int R_path_from_registry (char *s, int which)
 	err = try_for_R_path(HKEY_CURRENT_USER, s);
     }
 
-    if (!err) {
-	if (which != RBASE) {
-	    strcat(s, "\\bin\\");
-	    if (which == RGUI) {
-		strcat(s, "Rgui.exe");
-	    } else if (which == RTERM) {
-		strcat(s, "Rterm.exe");
-	    } else if (which == RLIB) {
-		strcat(s, "R.dll");
-	    } 
+    if (!err && which != RBASE) {
+	FILE *fp;
+
+	strcat(s, "\\bin\\");
+	append_R_filename(s, which);
+
+	fp = fopen(s, "r");
+	if (fp != NULL) {
+	    fclose(fp);
+	} else {
+	    char *p = strrchr(s, 'R');
+
+	    *p = '\0';
+	    strcat(s, "i386\\");
+	    append_R_filename(s, which);
+	    fp = fopen(s, "r");
+	    if (fp != NULL) {
+		fclose(fp);
+	    } else {
+		err = E_FOPEN;
+	    }
 	}
     }
 
@@ -955,3 +992,79 @@ int maybe_print_R_path_addition (FILE *fp)
     return err;
 }
 
+/*
+  The original notice for the following functions
+  (for which see http://ab-initio.mit.edu/~stevenj/align.c):
+
+  _align_malloc and friends, implemented using Microsoft's public
+  interfaces and with the help of the algorithm description provided
+  by Wu Yongwei: http://sourceforge.net/mailarchive/message.php?msg_id=3847075
+
+  I hereby place this implementation in the public domain.
+               -- Steven G. Johnson (stevenj@alum.mit.edu)
+*/
+
+#include <errno.h>
+#include <stddef.h> /* ptrdiff_t */
+
+#ifdef HAVE_STDINT_H
+# include <stdint.h> /* uintptr_t */
+#else
+# define uintptr_t size_t
+#endif
+
+#define NOT_POWER_OF_TWO(n) (((n) & ((n) - 1)))
+#define UI(p) ((uintptr_t) (p))
+#define CP(p) ((char *) p)
+
+#define PTR_ALIGN(p0, alignment, offset)				\
+            ((void *) (((UI(p0) + (alignment + sizeof(void*)) + offset)	\
+			& (~UI(alignment - 1)))				\
+		       - offset))
+
+/* pointer must sometimes be aligned; assume sizeof(void*) is a power of two */
+#define ORIG_PTR(p) (*(((void **) (UI(p) & (~UI(sizeof(void*) - 1)))) - 1))
+
+static void *aligned_offset_malloc (size_t size, size_t alignment, 
+				    size_t offset)
+{
+    void *p0, *p;
+
+    if (NOT_POWER_OF_TWO(alignment)) {
+	errno = EINVAL;
+	return NULL;
+    }
+
+    if (size == 0)
+	return NULL;
+
+    if (alignment < sizeof(void *))
+	alignment = sizeof(void *);
+
+    /* including the extra sizeof(void*) is overkill on a 32-bit
+       machine, since malloc is already 8-byte aligned, as long
+       as we enforce alignment >= 8 ...but oh well */
+
+    p0 = malloc(size + (alignment + sizeof(void *)));
+    if (!p0) {
+	return NULL;
+    }
+
+    p = PTR_ALIGN(p0, alignment, offset);
+    ORIG_PTR(p) = p0;
+
+    return p;
+}
+
+void *win32_memalign (size_t size, size_t alignment)
+{
+    return aligned_offset_malloc(size, alignment, 0);
+}
+
+void win32_aligned_free (void *mem)
+{
+    if (mem) {
+	free(ORIG_PTR(mem));
+    }
+}
+
