@@ -55,6 +55,8 @@ int win_run_sync (char *cmdline, const char *currdir);
 
 int gretl_spawn (char *cmdline);
 
+int gretl_spawn_with_fixup (char *prog, char *arg);
+
 int gretl_shell (const char *arg, PRN *prn);
 
 int gretl_win32_grab_output (const char *cmdline, char **sout);
@@ -77,6 +79,10 @@ int R_path_from_registry (char *s, int which);
 
 int maybe_print_R_path_addition (FILE *fp);
 
+void *win32_memalign (size_t size, size_t alignment);
+
+void win32_aligned_free (void *mem);
+
 #endif /* WIN32 */
 
 #endif /* GRETL_WIN32_H */
