@@ -25,6 +25,7 @@
 #include "gretl_func.h"
 #include "usermat.h"
 #include "gretl_scalar.h"
+#include "gretl_bundle.h"
 #include "dbread.h"
 
 #include <sys/types.h>
@@ -271,6 +272,28 @@ void gretl_xml_put_strings_array (const char *tag, const char **strs, int n,
     fprintf(fp, "</%s>\n", tag); 
 }
 
+/**
+ * gretl_xml_put_strings_array_quoted:
+ * @tag: name to give array.
+ * @strs: array of strings to put.
+ * @n: number of strings in @strs.
+ * @fp: file to which to write.
+ * 
+ */
+
+void gretl_xml_put_strings_array_quoted (const char *tag, 
+					 const char **strs, int n,
+					 FILE *fp)
+{
+    int i;
+
+    fprintf(fp, "<%s count=\"%d\">\n", tag, n);
+    for (i=0; i<n; i++) {
+	fprintf(fp, "\"%s\" ", strs[i]);
+    }
+    fprintf(fp, "</%s>\n", tag); 
+}
+
 /**
  * gretl_xml_put_tagged_string:
  * @tag: name to give string.
@@ -706,17 +729,12 @@ int gretl_xml_node_get_trimmed_string (xmlNodePtr node, xmlDocPtr doc,
 		break;
 	    }
 	}
-	if (len == strlen(tmp)) {
-	    *pstr = tmp;
+	*pstr = gretl_strndup(s, len);
+	if (*pstr != NULL) {
 	    ret = 1;
-	} else if (len > 0) {
-	    *pstr = gretl_strndup(s, len);
-	    if (*pstr != NULL) {
-		ret = 1;
-	    }
-	    free(tmp);
 	}
-    }
+	free(tmp);	
+    } 
 
     return ret;
 }
@@ -822,6 +840,7 @@ int gretl_xml_child_get_string (xmlNodePtr node, xmlDocPtr doc,
     return ret;
 }
 
+
 #define SMALLVAL(x) (x > -1e-40 && x < 1e-40)
 
 static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
@@ -839,7 +858,11 @@ static void *gretl_xml_get_array (xmlNodePtr node, xmlDocPtr doc,
     *nelem = 0;
 
     if (tmp == NULL) {
-	fprintf(stderr, "gretl_xml_get_array: failed\n");
+	tmp = xmlGetProp(node, (XUC) "size");
+    }
+
+    if (tmp == NULL) {
+	fprintf(stderr, "gretl_xml_get_array: didn't find count\n");
 	*err = E_DATA;
 	return NULL;
     }
@@ -1038,15 +1061,23 @@ static char *chunk_strdup (const char *src, const char **ptr, int *err)
 	const char *p;
 	int len = 0;
 
-	while (*src && isspace(*src)) {
-	    src++;
-	}
-
+	src += strspn(src, " \n");
 	p = src;
 
-	while (*src && !isspace(*src)) {
-	    len++;
-	    src++;
+	if (*src == '"') {
+	    p = ++src;
+	    while (*src && *src != '"') {
+		len++;
+		src++;
+	    }	
+	    if (*src == '"') {
+		src++;
+	    }
+	} else {
+	    while (*src && !isspace(*src)) {
+		len++;
+		src++;
+	    }
 	}
 
 	if (ptr != NULL) {
@@ -1131,11 +1162,45 @@ char **gretl_xml_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
     return S;
 }
 
+/**
+ * gretl_xml_child_get_strings_array:
+ * @node: XML node pointer.
+ * @doc: XML document pointer.
+ * @name: name of child node.
+ * @pstrs: location to receive strings array.
+ * @nelem: location to receive number of strings.
+ * 
+ * Returns: 1 if an array of strings is found and read successfully,
+ * 0 otherwise.
+ */
+
+int gretl_xml_child_get_strings_array (xmlNodePtr node, xmlDocPtr doc, 
+				       const char *name, char ***pstrs,
+				       int *nstrs)
+{
+    xmlNodePtr cur = node->xmlChildrenNode;
+    int ret = 0;
+
+    while (cur != NULL) {
+	if (!xmlStrcmp(cur->name, (XUC) name)) {
+	    int err = 0;
+
+	    *pstrs = gretl_xml_get_strings_array(cur, doc, nstrs, 0, &err);
+	    ret = !err;
+	    break;
+	}
+	cur = cur->next;
+    }
+
+    return ret;
+}
+
 /**
  * xml_get_user_matrix:
  * @node: XML node pointer.
  * @doc: XML document pointer.
- * @colnames: location to receive column strings, if present.
+ * @colnames: location to receive column labels, if present.
+ * @rownames: location to receive row labels, if present.
  * @err: location to receive error code.
  * 
  * Returns: allocated gretl matrix read from @node, or %NULL 
@@ -1143,7 +1208,8 @@ char **gretl_xml_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
  */
 
 static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc, 
-					  char **colnames, int *err)
+					  char **colnames, char **rownames,
+					  int *err)
 {
     gretl_matrix *m = NULL;
     xmlChar *tmp;
@@ -1181,6 +1247,15 @@ static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc,
 
     free(tmp);
 
+    if (rows == 0 && cols == 0) {
+	/* allow case of empty matrix */
+	m = gretl_null_matrix_new();
+	if (m == NULL) {
+	    *err = E_ALLOC;
+	}
+	return m;
+    }	
+
     if (rows <= 0 || cols <= 0) {
 	*err = E_DATA;
 	return NULL;
@@ -1202,6 +1277,10 @@ static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc,
 	*colnames = (char *) xmlGetProp(node, (XUC) "colnames");
     }
 
+    if (rownames != NULL) {
+	*rownames = (char *) xmlGetProp(node, (XUC) "rownames");
+    }
+
     m = gretl_matrix_alloc(rows, cols);
     if (m == NULL) {
 	*err = E_ALLOC;
@@ -1261,7 +1340,7 @@ static gretl_matrix *xml_get_user_matrix (xmlNodePtr node, xmlDocPtr doc,
 gretl_matrix *gretl_xml_get_matrix (xmlNodePtr node, xmlDocPtr doc, 
 				    int *err)
 {
-    return xml_get_user_matrix(node, doc, NULL, err);
+    return xml_get_user_matrix(node, doc, NULL, NULL, err);
 }
 
 /**
@@ -1437,6 +1516,8 @@ int gretl_write_matrix_as_gdt (const char *fname,
     return err;
 }
 
+#define GDT_DIGITS 17
+
 /**
  * gretl_write_gdt:
  * @fname: name of file to write.
@@ -1518,9 +1599,16 @@ int gretl_write_gdt (const char *fname, const int *list,
 
     if (sz) (*show_progress)(0, sz, SP_SAVE_INIT); 
 
-    for (i=1; i<=nvars; i++) {
-	v = savenum(list, i);
-	pmax[i-1] = get_precision(&Z[v][pdinfo->t1], tsamp, 10);
+    for (i=0; i<nvars; i++) {
+	int prec;
+
+	v = savenum(list, i+1);
+	prec = get_precision(&Z[v][pdinfo->t1], tsamp, GDT_DIGITS);
+	if (prec < GDT_DIGITS) {
+	    pmax[i] = prec;
+	} else {
+	    pmax[i] = PMAX_NOT_AVAILABLE;
+	}
     }
 
     ntodate(startdate, pdinfo->t1, pdinfo);
@@ -1698,7 +1786,7 @@ int gretl_write_gdt (const char *fname, const int *list,
 	    if (na(Z[v][t])) {
 		strcpy(numstr, "NA ");
 	    } else if (pmax[i-1] == PMAX_NOT_AVAILABLE) {
-		sprintf(numstr, "%.12g ", Z[v][t]);
+		sprintf(numstr, "%.*g ", GDT_DIGITS, Z[v][t]);
 	    } else {
 		sprintf(numstr, "%.*f ", pmax[i-1], Z[v][t]);
 	    }
@@ -2557,16 +2645,16 @@ static char *gretl_xml_get_doc_type (const char *fname, int *err)
 
     if (doc == NULL) {
 	gretl_errmsg_sprintf(_("xmlParseFile failed on %s"), fname);
-	*err = 1;
+	*err = E_DATA;
     } else {
 	node = xmlDocGetRootElement(doc);
 	if (node == NULL) {
 	    gretl_errmsg_sprintf(_("%s: empty document"), fname);
-	    *err = 1;
+	    *err = E_DATA;
 	} else {
 	    ret = gretl_strdup((char *) node->name);
 	    if (ret == NULL) {
-		*err = 1;
+		*err = E_ALLOC;
 	    }
 	}
     }
@@ -2585,6 +2673,7 @@ int load_user_matrix_file (const char *fname)
     xmlNodePtr cur = NULL;
     gretl_matrix *m;
     char *colnames;
+    char *rownames;
     char *name;
     int err = 0;
 
@@ -2602,15 +2691,20 @@ int load_user_matrix_file (const char *fname)
 	    if (name == NULL) {
 		err = 1;
 	    } else {
-		colnames = NULL;
-		m = xml_get_user_matrix(cur, doc, &colnames, &err);
+		colnames = rownames = NULL;
+		m = xml_get_user_matrix(cur, doc, &colnames, 
+					&rownames, &err);
 		if (m != NULL) {
 		    err = user_matrix_add(m, name);
 		    if (!err && colnames != NULL) {
-			umatrix_set_colnames_from_string(m, colnames);
+			umatrix_set_names_from_string(m, colnames, 0);
 		    }
+		    if (!err && rownames != NULL) {
+			umatrix_set_names_from_string(m, rownames, 1);
+		    }		    
 		}
 		free(colnames);
+		free(rownames);
 		free(name);
 	    }
 	}
@@ -2625,6 +2719,46 @@ int load_user_matrix_file (const char *fname)
     return err;
 }
 
+int load_user_bundle_file (const char *fname)
+{
+    xmlDocPtr doc = NULL;
+    xmlNodePtr cur = NULL;
+    char *name;
+    int err = 0;
+
+    xmlKeepBlanksDefault(0);
+
+    err = gretl_xml_open_doc_root(fname, "gretl-bundles", &doc, &cur);
+    if (err) {
+	return err;
+    }
+
+    gretl_push_c_numeric_locale();
+
+    cur = cur->xmlChildrenNode;
+    while (cur != NULL && !err) {
+        if (!xmlStrcmp(cur->name, (XUC) "gretl-bundle")) {
+	    name = (char *) xmlGetProp(cur, (XUC) "name");
+	    if (name == NULL) {
+		err = 1;
+	    } else {
+		err = load_bundle_from_xml(cur, doc, name);
+		free(name);
+	    }
+	}
+	cur = cur->next;
+    }
+
+    gretl_pop_c_numeric_locale();
+
+    if (doc != NULL) {
+	xmlFreeDoc(doc);
+	xmlCleanupParser();
+    }
+
+    return err;
+}
+
 int load_user_scalars_file (const char *fname) 
 {
     xmlDocPtr doc = NULL;
@@ -2674,9 +2808,12 @@ int load_user_XML_file (const char *fname)
     int err = 0;
 
     rootname = gretl_xml_get_doc_type(fname, &err);
+    if (err) {
+	return err;
+    }
 
     if (!strcmp(rootname, "gretl-functions")) {
-	err = load_function_package_from_file(fname);
+	err = load_function_package_by_filename(fname);
     } else if (!strcmp(rootname, "gretl-matrices")) {
 	err = load_user_matrix_file(fname);
     } else if (!strcmp(rootname, "gretl-scalars")) {
