@@ -41,6 +41,10 @@ void gretl_xml_put_double_array (const char *tag, double *x, int n,
 void gretl_xml_put_strings_array (const char *tag, const char **strs, int n,
 				  FILE *fp);
 
+void gretl_xml_put_strings_array_quoted (const char *tag, 
+					 const char **strs, int n,
+					 FILE *fp);
+
 void gretl_xml_put_named_list (const char *name, const int *list, FILE *fp);
 
 void gretl_xml_put_tagged_list (const char *tag, const int *list, FILE *fp);
@@ -98,6 +102,10 @@ cmplx *gretl_xml_get_cmplx_array (xmlNodePtr node, xmlDocPtr doc,
 char **gretl_xml_get_strings_array (xmlNodePtr node, xmlDocPtr doc,
 				    int *nelem, int slop, int *err);
 
+int gretl_xml_child_get_strings_array (xmlNodePtr node, xmlDocPtr doc, 
+				       const char *name, char ***pstrs,
+				       int *nstrs);
+
 gretl_matrix *gretl_xml_get_matrix (xmlNodePtr node, xmlDocPtr doc, 
 				    int *err);
 
@@ -121,6 +129,8 @@ char *gretl_get_gdt_description (const char *fname);
 
 int load_user_matrix_file (const char *fname);
 
+int load_user_bundle_file (const char *fname);
+
 int load_user_scalars_file (const char *fname);
 
 int load_user_XML_file (const char *fname);
