@@ -97,6 +97,7 @@ typedef enum {
     GRETL_TYPE_NONE,
     GRETL_TYPE_BOOL,
     GRETL_TYPE_INT,
+    GRETL_TYPE_OBS,
     GRETL_TYPE_LIST,
     GRETL_TYPE_DOUBLE,
     GRETL_TYPE_INT_ARRAY,
@@ -120,6 +121,7 @@ typedef enum {
 
 #define gretl_scalar_type(t) (t == GRETL_TYPE_BOOL || \
                               t == GRETL_TYPE_INT || \
+			      t == GRETL_TYPE_OBS || \
 			      t == GRETL_TYPE_DOUBLE)
 
 #define gretl_ref_type(t) (t == GRETL_TYPE_SCALAR_REF || \
