@@ -30,6 +30,8 @@
 #include <unistd.h>
 #include <errno.h>
 
+#include <glib.h>
+
 #define PDEBUG 0
 
 enum {
@@ -53,14 +55,13 @@ enum {
     STATE_MAX_VERBOSE     = 1 << 11, /* verbose output from maximizer? */
     STATE_USE_FCP         = 1 << 12, /* use FCP garch code */
     STATE_WARN_ON         = 1 << 13, /* print numerical warning messages */
-    STATE_VERBOSE_INCLUDE = 1 << 14, /* verbose include */
-    STATE_SKIP_MISSING    = 1 << 15, /* skip NAs when building matrix from series */
-    STATE_LOOPING         = 1 << 16, /* loop is in progress at this level */
-    STATE_LOOP_QUIET      = 1 << 17, /* loop commands should be quiet */
-    STATE_LOOP_PROG       = 1 << 18, /* progressive loop is in progress */
-    STATE_BFGS_RSTEP      = 1 << 19, /* use Richardson method in BFGS numerical
+    STATE_SKIP_MISSING    = 1 << 14, /* skip NAs when building matrix from series */
+    STATE_LOOPING         = 1 << 15, /* loop is in progress at this level */
+    STATE_LOOP_QUIET      = 1 << 16, /* loop commands should be quiet */
+    STATE_LOOP_PROG       = 1 << 17, /* progressive loop is in progress */
+    STATE_BFGS_RSTEP      = 1 << 18, /* use Richardson method in BFGS numerical
 					gradient */
-    STATE_DPDSTYLE_ON     = 1 << 20, /* emulate dpd in dynamic panel data models */
+    STATE_DPDSTYLE_ON     = 1 << 19  /* emulate dpd in dynamic panel data models */
 };    
 
 /* for values that really want a non-negative integer */
@@ -86,6 +87,7 @@ struct set_vars_ {
     int loop_maxiter;           /* max no. of iterations in non-for loops */
     char delim;                 /* delimiter for CSV data export */
     int vecm_norm;              /* VECM beta normalization */
+    int optim;                  /* code for preferred optimizer */
     int bfgs_maxiter;           /* max iterations, BFGS */         
     double bfgs_toler;          /* convergence tolerance, BFGS */
     double bfgs_maxgrad;        /* max acceptable gradient norm, BFGS */
@@ -99,6 +101,7 @@ struct set_vars_ {
     int rq_maxiter;             /* max iterations for quantreg, simplex */
     int gmm_maxiter;            /* max iterations for iterated GMM */
     gretl_matrix *initvals;     /* parameter initializer */
+    gretl_matrix *matmask;      /* mask for series -> matrix conversion */
     struct robust_opts ropts;   /* robust standard error options */
     char shelldir[MAXLEN];      /* working dir for shell commands */
     char csv_na[8];             /* representation of NA in CSV output */
@@ -124,7 +127,6 @@ struct set_vars_ {
 			   !strcmp(s, SHELL_OK) || \
 			   !strcmp(s, USE_CWD) || \
 			   !strcmp(s, USE_FCP) || \
-                           !strcmp(s, VERBOSE_INCLUDE) || \
                            !strcmp(s, SKIP_MISSING) || \
 			   !strcmp(s, R_FUNCTIONS) || \
 			   !strcmp(s, R_LIB) || \
@@ -149,6 +151,7 @@ struct set_vars_ {
 		       !strcmp(s, LOOP_MAXITER) || \
                        !strcmp(s, RQ_MAXITER) || \
 		       !strcmp(s, VECM_NORM) || \
+		       !strcmp(s, GRETL_OPTIM) || \
 		       !strcmp(s, GRETL_DEBUG) || \
 		       !strcmp(s, BLAS_NMK_MIN))
 
@@ -158,7 +161,6 @@ static int gretl_debug;
 static int user_mp_bits;
 static int R_functions;
 static int R_lib = 1;
-static char include_path[MAXLEN];
 
 static int boolvar_get_flag (const char *s);
 static const char *hac_lag_string (void);
@@ -226,12 +228,25 @@ static const char *vecm_norm_strs[] = {
     NULL
 };
 
+static const char *optim_strs[] = {
+    "auto",
+    "BFGS",
+    "newton",
+    NULL
+};
+
 static const char *normal_rand_strs[] = {
     "ziggurat",
     "box-muller",
     NULL
 };
 
+static const char *RNG_strs[] = {
+    "MT",
+    "SFMT",
+    NULL
+};
+
 static const char **libset_option_strings (const char *s)
 {
     if (!strcmp(s, GARCH_VCV)) {
@@ -244,8 +259,12 @@ static const char **libset_option_strings (const char *s)
 	return hc_version_strs;
     } else if (!strcmp(s, VECM_NORM)) {
 	return vecm_norm_strs;
+    } else if (!strcmp(s, GRETL_OPTIM)) {
+	return optim_strs;
     } else if (!strcmp(s, NORMAL_RAND)) {
 	return normal_rand_strs;
+    } else if (!strcmp(s, RNG)) {
+	return RNG_strs;
     } else if (!strcmp(s, "csv_delim")) {
 	return csv_delim_args;
     } else {
@@ -292,8 +311,12 @@ static const char *libset_option_string (const char *s)
 	return hc_version_strs[state->ropts.hc_version];
     } else if (!strcmp(s, VECM_NORM)) {
 	return vecm_norm_strs[state->vecm_norm];
+    } else if (!strcmp(s, GRETL_OPTIM)) {
+	return optim_strs[state->optim];
     } else if (!strcmp(s, NORMAL_RAND)) {
 	return normal_rand_strs[gretl_rand_get_box_muller()];
+    } else if (!strcmp(s, RNG)) {
+	return RNG_strs[gretl_rand_get_sfmt()];
     } else {
 	return "?";
     }
@@ -355,6 +378,7 @@ static void state_vars_copy (set_vars *sv)
     sv->nls_toler = state->nls_toler;
     sv->delim = state->delim; 
     sv->vecm_norm = state->vecm_norm;
+    sv->optim = state->optim;
     sv->bfgs_maxiter = state->bfgs_maxiter;
     sv->bfgs_toler = state->bfgs_toler;
     sv->bfgs_maxgrad = state->bfgs_maxgrad;
@@ -366,6 +390,7 @@ static void state_vars_copy (set_vars *sv)
     sv->garch_robust_vcv = state->garch_robust_vcv;
 
     sv->initvals = gretl_matrix_copy(state->initvals);
+    sv->matmask = gretl_matrix_copy(state->matmask);
     strcpy(sv->shelldir, state->shelldir);
     strcpy(sv->csv_na, state->csv_na);
 
@@ -388,7 +413,9 @@ static void state_vars_init (set_vars *sv)
     sv->gmm_maxiter = 250;
     sv->delim = UNSET_INT;
     sv->vecm_norm = NORM_PHILLIPS;
+    sv->optim = OPTIM_AUTO;
     sv->initvals = NULL;
+    sv->matmask = NULL;
 
     sv->bfgs_maxiter = -1;
     sv->bfgs_toler = NADBL;
@@ -536,6 +563,31 @@ int n_init_vals (void)
     }
 }
 
+const gretl_matrix *get_matrix_mask (void)
+{
+    check_for_state();
+    return state->matmask;
+}
+
+int get_matrix_mask_nobs (void)
+{
+    int n = 0;
+
+    check_for_state();
+
+    if (state->matmask != NULL) {
+	int i;
+
+	for (i=0; i<state->matmask->rows; i++) {
+	    if (state->matmask->val[i] != 0.0) {
+		n++;
+	    }
+	}
+    }
+
+    return n;
+}
+
 char *get_shelldir (void)
 {
     check_for_state();
@@ -742,50 +794,67 @@ static int parse_libset_int_code (const char *key,
 {
     int i, err = E_DATA;
 
-    if (!strcmp(key, HC_VERSION)) {
+    if (!g_ascii_strcasecmp(key, HC_VERSION)) {
 	err = parse_hc_variant(val);
-    } else if (!strcmp(key, HAC_LAG)) {
+    } else if (!g_ascii_strcasecmp(key, HAC_LAG)) {
 	err = parse_hac_lag_variant(val);
-    } else if (!strcmp(key, GARCH_VCV)) {
+    } else if (!g_ascii_strcasecmp(key, GARCH_VCV)) {
 	for (i=0; i<VCV_MAX; i++) {
-	    if (!strcmp(val, garch_vcv_strs[i])) {
+	    if (!g_ascii_strcasecmp(val, garch_vcv_strs[i])) {
 		state->garch_vcv = i;
 		err = 0;
 		break;
 	    }
 	}
-    } else if (!strcmp(key, ARMA_VCV)) {
-	if (!strcmp(val, "op")) {
+    } else if (!g_ascii_strcasecmp(key, ARMA_VCV)) {
+	if (!g_ascii_strcasecmp(val, "op")) {
 	    state->arma_vcv = VCV_OP;
 	    err = 0;
-	} else if (!strcmp(val, "hessian")) {
+	} else if (!g_ascii_strcasecmp(val, "hessian")) {
 	    state->arma_vcv = VCV_HESSIAN;
 	    err = 0;
 	}
-    } else if (!strcmp(key, HAC_KERNEL)) {
+    } else if (!g_ascii_strcasecmp(key, HAC_KERNEL)) {
 	for (i=0; i<KERNEL_MAX; i++) {
-	    if (!strcmp(val, hac_kernel_strs[i])) {
+	    if (!g_ascii_strcasecmp(val, hac_kernel_strs[i])) {
 		state->ropts.hkern = i;
 		err = 0;
 		break;
 	    }
 	}
-    } else if (!strcmp(key, VECM_NORM)) {
+    } else if (!g_ascii_strcasecmp(key, VECM_NORM)) {
 	for (i=0; i<NORM_MAX; i++) {
-	    if (!strcmp(val, vecm_norm_strs[i])) {
+	    if (!g_ascii_strcasecmp(val, vecm_norm_strs[i])) {
 		state->vecm_norm = i;
 		err = 0;
 		break;
 	    }
 	}
-    } else if (!strcmp(key, NORMAL_RAND)) {
+    } else if (!g_ascii_strcasecmp(key, GRETL_OPTIM)) {
+	for (i=0; i<OPTIM_MAX; i++) {
+	    if (!g_ascii_strcasecmp(val, optim_strs[i])) {
+		state->optim = i;
+		err = 0;
+		break;
+	    }
+	}	
+    } else if (!g_ascii_strcasecmp(key, NORMAL_RAND)) {
 	for (i=0; normal_rand_strs[i] != NULL; i++) {
-	    if (!strcmp(val, normal_rand_strs[i])) {
+	    if (!g_ascii_strcasecmp(val, normal_rand_strs[i])) {
 		gretl_rand_set_box_muller(i);
 		err = 0;
+		break;
 	    }
 	}
-    }
+    } else if (!g_ascii_strcasecmp(key, RNG)) {
+	for (i=0; RNG_strs[i] != NULL; i++) {
+	    if (!g_ascii_strcasecmp(val, RNG_strs[i])) {
+		gretl_rand_set_sfmt(i);
+		err = 0;
+		break;
+	    }
+	}
+    }	
 
     if (err) {
 	gretl_errmsg_sprintf("%s: invalid value '%s'\n", key, val);
@@ -914,6 +983,9 @@ static int set_initvals (const char *s, PRN *prn)
 	    pputc(prn, '\n');
 	    err = E_DATA;
 	} else {
+	    if (state->initvals != NULL) {
+		gretl_matrix_free(state->initvals);
+	    }
 	    state->initvals = gretl_matrix_copy(m);
 	    if (state->initvals == NULL) {
 		err = E_ALLOC;
@@ -924,6 +996,49 @@ static int set_initvals (const char *s, PRN *prn)
     return err;
 }
 
+static int set_matmask (const char *s, const double **Z,
+			const DATAINFO *pdinfo,
+			PRN *prn)
+{
+    char vname[VNAMELEN];
+    int err = 0;
+
+    /* skip past "set matrix_mask" */
+    s += 15;
+
+    if (sscanf(s, "%15s", vname) != 1 || !strcmp(vname, "null")) {
+	gretl_matrix_free(state->matmask);
+	state->matmask = NULL;
+    } else {
+	int t, v = current_series_index(pdinfo, vname);
+
+	if (v < 0) {
+	    err = E_UNKVAR;
+	} else {
+	    if (state->matmask != NULL) {
+		gretl_matrix_free(state->matmask);
+	    }
+	    state->matmask = gretl_column_vector_alloc(pdinfo->n);
+	    if (state->matmask == NULL) {
+		err = E_ALLOC;
+	    } else {
+		for (t=0; t<pdinfo->n; t++) {
+		    state->matmask->val[t] = Z[v][t];
+		}
+	    }
+	}
+    }
+
+    return err;
+}
+
+void destroy_matrix_mask (void)
+{
+    check_for_state();
+    gretl_matrix_free(state->matmask);
+    state->matmask = NULL;
+}
+
 void shelldir_init (const char *s)
 {
     if (s != NULL) {
@@ -1032,12 +1147,11 @@ static int set_workdir (const char *s)
 	err = E_DATA;
     } else {
 	char workdir[MAXLEN];
-	int n = 0;
 
 	if (*s == '"') {
-	    n = sscanf(s+1, "%511[^\"]", workdir);
+	    sscanf(s+1, "%511[^\"]", workdir);
 	} else {
-	    n = sscanf(s, "%511s", workdir);
+	    sscanf(s, "%511s", workdir);
 	}
 	if (workdir_callback != NULL) {
 	    err = (*workdir_callback)(workdir);
@@ -1095,7 +1209,9 @@ static void libset_print_bool (const char *s, PRN *prn,
 			 !strcmp(s, HAC_KERNEL) || \
                          !strcmp(s, HC_VERSION) || \
 			 !strcmp(s, VECM_NORM) || \
-			 !strcmp(s, NORMAL_RAND))
+			 !strcmp(s, GRETL_OPTIM) || \
+			 !strcmp(s, NORMAL_RAND) || \
+                         !strcmp(s, RNG))
 
 const char *intvar_code_string (const char *s)
 {
@@ -1204,24 +1320,14 @@ static int print_settings (PRN *prn, gretlopt opt)
     } 
 
     libset_print_bool(USE_CWD, prn, opt);
-    libset_print_bool(VERBOSE_INCLUDE, prn, opt);
     libset_print_bool(SKIP_MISSING, prn, opt);
 
     libset_print_bool(R_LIB, prn, opt);
     libset_print_bool(R_FUNCTIONS, prn, opt);
 
-    if (opt & OPT_D) {
-	if (*include_path) {
-	    pprintf(prn, " include_path = '%s'\n", include_path);
-	} else {
-	    pputs(prn, " include_path = unset\n");
-	}
-    } else if (*include_path) {
-	pprintf(prn, "set include_path \"%s\"\n", include_path);
-    }
-
     libset_header(N_("Numerical methods"), prn, opt);
 
+    libset_print_int(GRETL_OPTIM, prn, opt);
     libset_print_int(BFGS_MAXITER, prn, opt);
     libset_print_double(BFGS_TOLER, prn, opt);
     libset_print_double(BFGS_MAXGRAD, prn, opt);
@@ -1243,9 +1349,11 @@ static int print_settings (PRN *prn, gretlopt opt)
     if (opt & OPT_D) {
 	pprintf(prn, " seed = %u\n", gretl_rand_get_seed());
 	pprintf(prn, " normal_rand = %s\n", libset_option_string(NORMAL_RAND));
+	pprintf(prn, " RNG = %s\n", libset_option_string(RNG));
     } else {
 	pprintf(prn, "set seed %u\n", gretl_rand_get_seed());
 	pprintf(prn, "set normal_rand %s\n", libset_option_string(NORMAL_RAND));
+	pprintf(prn, "set RNG %s\n", libset_option_string(RNG));
     }
 
     libset_header(N_("Robust estimation"), prn, opt);
@@ -1306,6 +1414,13 @@ static int libset_query_settings (const char *s, PRN *prn)
 	} else {
 	    pprintf(prn, "%s: matrix, currently null\n", s);
 	}
+    } else if (!strcmp(s, "matrix_mask")) {
+	if (state->matmask != NULL) {
+	    pprintf(prn, "%s: matrix, currently\n", s);
+	    gretl_matrix_print_to_prn(state->matmask, NULL, prn);
+	} else {
+	    pprintf(prn, "%s: matrix, currently null\n", s);
+	}
     } else if (!strcmp(s, "seed")) {
 	pprintf(prn, "%s: unsigned int, currently %u\n",
 		s, state->seed ? state->seed : gretl_rand_get_seed());
@@ -1319,9 +1434,6 @@ static int libset_query_settings (const char *s, PRN *prn)
     } else if (!strcmp(s, "workdir")) {
 	pprintf(prn, "%s: string, currently \"%s\"\n", s,
 		gretl_workdir());
-    } else if (!strcmp(s, "include_path")) {
-	pprintf(prn, "%s: string, currently \"%s\"\n", s,
-		include_path);
     } else if (!strcmp(s, "csv_na")) {
 	pprintf(prn, "%s: string, currently \"%s\"\n", s,
 		state->csv_na);
@@ -1371,8 +1483,9 @@ static int write_or_read_settings (gretlopt opt, PRN *prn)
     return err;
 }
 
-int execute_set_line (const char *line, DATAINFO *pdinfo, 
-		      gretlopt opt, PRN *prn)
+int execute_set_line (const char *line, const double **Z,
+		      DATAINFO *pdinfo, gretlopt opt, 
+		      PRN *prn)
 {
     char setobj[32], setarg[32], setarg2[32];
     int k, nw, err = E_PARSE;
@@ -1396,16 +1509,15 @@ int execute_set_line (const char *line, DATAINFO *pdinfo,
     if (nw > 1) {
 	if (!strcmp(setobj, "initvals")) {
 	    return set_initvals(line, prn);
+	} else if (!strcmp(setobj, "matrix_mask")) {
+	    return set_matmask(line, Z, pdinfo, prn);
 	} else if (!strcmp(setobj, "shelldir")) {
 	    return set_shelldir(line);
 	} else if (!strcmp(setobj, "codevars")) {
 	    return set_codevars(line);
 	} else if (!strcmp(setobj, "workdir")) {
 	    return set_workdir(line);
-	} else if (!strcmp(setobj, "include_path")) {
-	    set_include_path(line);
-	    return 0;
-	}
+	} 
     }
 
     if (nw == 1) {
@@ -1617,6 +1729,8 @@ int libset_get_int (const char *key)
 	return state->loop_maxiter;
     } else if (!strcmp(key, VECM_NORM)) {
 	return state->vecm_norm;
+    } else if (!strcmp(key, GRETL_OPTIM)) {
+	return state->optim;
     } else if (!strcmp(key, GRETL_DEBUG)) {
 	return gretl_debug;
     } else if (!strcmp(key, BLAS_NMK_MIN)) {
@@ -1674,6 +1788,10 @@ static int intvar_min_max (const char *s, int *min, int *max,
 	*min = 0;
 	*max = NORM_MAX;
 	*var = &state->vecm_norm;
+    } else if (!strcmp(s, GRETL_OPTIM)) {
+	*min = 0;
+	*max = OPTIM_MAX;
+	*var = &state->optim;
     } else if (!strcmp(s, GRETL_DEBUG)) {
 	*min = 0;
 	*max = 4;
@@ -1745,8 +1863,6 @@ static int boolvar_get_flag (const char *s)
 	return STATE_PREWHITEN;
     } else if (!strcmp(s, PCSE)) {
 	return STATE_USE_PCSE;
-    } else if (!strcmp(s, VERBOSE_INCLUDE)) {
-	return STATE_VERBOSE_INCLUDE;
     } else if (!strcmp(s, SKIP_MISSING)) {
 	return STATE_SKIP_MISSING;
     } else if (!strcmp(s, BFGS_RSTEP)) {
@@ -1953,9 +2069,8 @@ int push_program_state (void)
 
 static void free_state (set_vars *sv)
 {
-    if (sv->initvals != NULL) {
-	gretl_matrix_free(sv->initvals);
-    }
+    gretl_matrix_free(sv->initvals);
+    gretl_matrix_free(sv->matmask);
 
     free(sv);
 }
@@ -2216,22 +2331,6 @@ DEBUG_OUTPUT get_debug_output_func (void)
     return dbg_output;
 }
 
-/* mechanism for passing an integer parameter to
-   a gretl script, accessible via $switch
-*/
-
-static int script_switch;
-
-void set_script_switch (int s)
-{
-    script_switch = s;
-}
-
-int get_script_switch (void)
-{
-    return script_switch;
-}
-
 /* for setting what we print for NAs on CSV output */
 
 const char *get_csv_na_string (void)
@@ -2253,40 +2352,6 @@ void set_csv_na_string (const char *s)
     strncat(state->csv_na, s, 7);
 }
 
-void set_include_path (const char *s)
-{
-    if (gretl_function_depth() > 0) {
-	gretl_errmsg_set("set include_path: cannot be done inside a function");
-	return;
-    }
-
-    /* skip past "include_path" and space */
-    s += 16;
-    s += strspn(s, " ");
-
-    if (*s != '\0') {
-	char tmp[MAXLEN];
-	int n = 0;
-
-	if (*s == '"') {
-	    n = sscanf(s+1, "%511[^\"]", tmp);
-	} else {
-	    n = sscanf(s, "%511s", tmp);
-	}
-
-	if (n > 0) {
-	    *include_path = '\0';
-	    strncat(include_path, tmp, MAXLEN - 2);
-	    slash_terminate(include_path);
-	}
-    }
-}
-
-const char *get_include_path (void)
-{
-    return (*include_path == '\0')? NULL : include_path;
-}
-
 int libset_write_script (const char *fname)
 {
     PRN *prn;
@@ -2320,7 +2385,7 @@ static int real_libset_read_script (const char *fname,
 	char fullname[FILENAME_MAX];
 
 	strcpy(fullname, fname);
-	addpath(fullname, 0);
+	gretl_addpath(fullname, 0);
 	fp = gretl_fopen(fname, "r");
 	if (fp == NULL) {
 	    err = E_FOPEN;
@@ -2335,7 +2400,7 @@ static int real_libset_read_script (const char *fname,
 		continue;
 	    }
 	    tailstrip(line);
-	    err = execute_set_line(line, NULL, OPT_NONE, prn);
+	    err = execute_set_line(line, NULL, NULL, OPT_NONE, prn);
 	    if (err && prn != NULL) {
 		break;
 	    }
