@@ -615,6 +615,98 @@ gretl_matrix_data_subset_masked (const int *list, const double **Z,
 					 M_MISSING_ERROR, err);
 }
 
+/* count the number of selected rows in the current
+   sample range */
+
+static int mmask_row_count (const gretl_matrix *mask, 
+			    const DATAINFO *pdinfo)
+{
+    int t, m = 0;
+
+    for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+	if (mask->val[t] != 0) {
+	    m++;
+	}
+    }
+
+    return m;
+}
+
+/**
+ * gretl_matrix_data_subset_special:
+ * @list: list of variables to process.
+ * @Z: data array.
+ * @pdinfo: dataset information.
+ * @mmask: matrix holding desired observations mask.
+ * @err: location to receive error code.
+ *
+ * Creates a gretl matrix holding the subset of variables from
+ * @Z specified by @list, using the observations (data rows)
+ * selected by non-zero elements in @mmask. This is designed
+ * to support the gretl "libset" variable %matrix_mask.
+ *
+ * The length of the vector @mmask must equal the number of
+ * observations in the dataset, the member %n of @pdinfo.
+ *
+ * Returns: allocated matrix or NULL on failure. 
+ */
+
+gretl_matrix *
+gretl_matrix_data_subset_special (const int *list, const double **Z,
+				  const DATAINFO *pdinfo,
+				  const gretl_matrix *mmask,
+				  int *err)
+{
+    int n = gretl_vector_get_length(mmask);
+    gretl_matrix *X = NULL;
+
+    if (list == NULL || n != pdinfo->n) {
+	*err = E_DATA;
+    } else if (list[0] == 0) {
+	X = gretl_null_matrix_new();
+    } else {
+	int T = mmask_row_count(mmask, pdinfo);
+	int k = list[0];
+
+	if (T == 0) {
+	    X = gretl_null_matrix_new();
+	} else {
+	    X = gretl_matrix_alloc(T, k);
+	}
+	
+	if (X != NULL && T > 0) {
+	    const double *xi;
+	    double xti;
+	    int i, s, t;
+
+	    for (i=0; i<k; i++) {
+		xi = Z[list[i+1]];
+		s = 0;
+		for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
+		    if (mmask->val[t] != 0) {
+			xti = xi[t];
+			if (na(xti)) {
+			    xti = M_NA;
+			}
+			if (s == 0) {
+			    X->t1 = t;
+			} else if (s == T - 1) {
+			    X->t2 = t;
+			}
+			gretl_matrix_set(X, s++, i, xti);
+		    }
+		}
+	    }
+	}
+    }
+
+    if (X == NULL && *err == 0) {
+	*err = E_ALLOC;
+    }
+
+    return X;
+}
+
 /**
  * gretl_dataset_from_matrix:
  * @m: source matrix.
@@ -630,11 +722,13 @@ gretl_matrix_data_subset_masked (const int *list, const double **Z,
  * or NULL on failure.
  */
 
-DATAINFO *gretl_dataset_from_matrix (gretl_matrix *m, const int *list,
+DATAINFO *gretl_dataset_from_matrix (const gretl_matrix *m, 
+				     const int *list,
 				     double ***pZ, int *err)
 {
     DATAINFO *dinfo = NULL;
     const char **names;
+    double x;
     int i, t, col, nv, T;
 
     if (gretl_is_null_matrix(m)) {
@@ -668,12 +762,13 @@ DATAINFO *gretl_dataset_from_matrix (gretl_matrix *m, const int *list,
 	return NULL;
     }
 
-    names = user_matrix_get_column_names(m);
+    names = user_matrix_get_names(m, 0);
 
     for (i=1; i<=nv; i++) {
 	col = (list != NULL)? list[i] - 1 : i - 1;
 	for (t=0; t<T; t++) {
-	    (*pZ)[i][t] = gretl_matrix_get(m, t, col);
+	    x = gretl_matrix_get(m, t, col);
+	    (*pZ)[i][t] = x;
 	}
 	if (names != NULL) {
 	    strcpy(dinfo->varname[i], names[col]);
@@ -739,7 +834,9 @@ int gretl_plotfit_matrices (const double *yvar, const double *xvar,
 	return E_MISSDATA;
     }
 
-    if (fit == PLOT_FIT_QUADRATIC) {
+    if (fit == PLOT_FIT_CUBIC) {
+	k = 4;
+    } else if (fit == PLOT_FIT_QUADRATIC) {
 	k = 3;
     } else if (fit == PLOT_FIT_LOESS) {
 	k = 1;
@@ -769,8 +866,11 @@ int gretl_plotfit_matrices (const double *yvar, const double *xvar,
 	    } else {
 		gretl_matrix_set(X, i, j++, xt);
 	    }
-	    if (fit == PLOT_FIT_QUADRATIC) {
-		gretl_matrix_set(X, i, j, xt * xt);
+	    if (fit == PLOT_FIT_QUADRATIC || fit == PLOT_FIT_CUBIC) {
+		gretl_matrix_set(X, i, j++, xt * xt);
+	    }
+	    if (fit == PLOT_FIT_CUBIC) {
+		gretl_matrix_set(X, i, j, xt * xt * xt);
 	    }
 	    i++;
 	}
@@ -953,14 +1053,31 @@ static int max_numchars (const gretl_matrix *m)
     return cmax;
 }
 
+static int max_label_length (const char **names, int n)
+{
+    int i, len, maxlen = 0;
+
+    for (i=0; i<n; i++) {
+	if (names[i] != NULL) {
+	    len = strlen(names[i]);
+	    if (len > maxlen) {
+		maxlen = len;
+	    }
+	}
+    }
+
+    return maxlen;
+}
+
 static void 
 real_matrix_print_to_prn (const gretl_matrix *m, const char *msg, 
 			  int packed, int errout, int plain,
 			  const char **heads, PRN *prn)
 {
+    const char **rownames = NULL;
     char numstr[32];
     double x;
-    int i, j;
+    int i, j, llen = 0;
 
     if (prn == NULL) {
 	return;
@@ -987,10 +1104,20 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
     }
 
     if (heads == NULL) {
-	heads = user_matrix_get_column_names(m);
+	heads = user_matrix_get_names(m, 0);
+    }
+
+    if (!packed) {
+	rownames = user_matrix_get_names(m, 1);
+	if (rownames != NULL) {
+	    llen = max_label_length(rownames, m->rows);
+	}
     }
 
     if (heads != NULL) {
+	if (rownames != NULL) {
+	    bufspace(llen + 1, prn);
+	}
 	for (j=0; j<m->cols; j++) {
 	    pprintf(prn, "%12.12s ", heads[j]);
 	}
@@ -1027,6 +1154,9 @@ real_matrix_print_to_prn (const gretl_matrix *m, const char *msg,
 	} 
 
 	for (i=0; i<m->rows; i++) {
+	    if (rownames != NULL) {
+		pprintf(prn, "%*s ", llen, rownames[i]);
+	    }
 	    for (j=0; j<m->cols; j++) {
 		x = gretl_matrix_get(m, i, j);
 		if (cmax) {
@@ -1081,11 +1211,12 @@ void gretl_matrix_print_with_col_heads (const gretl_matrix *m,
 static void maybe_print_col_heads (const gretl_matrix *m, 
 				   const char *fmt,
 				   int wid, int prec,
-				   int icast, PRN *prn)
+				   int icast, int llen,
+				   PRN *prn)
 {
     const char **heads;
 
-    heads = user_matrix_get_column_names(m);
+    heads = user_matrix_get_names(m, 0);
     
     if (heads != NULL) {
 	char wtest[32];
@@ -1114,6 +1245,10 @@ static void maybe_print_col_heads (const gretl_matrix *m,
 	    }
 	}
 
+	if (llen > 0) {
+	    bufspace(llen + 1, prn);
+	}
+
 	n = strlen(wtest);
 	for (j=0; j<m->cols; j++) {
 	    pprintf(prn, "%*s", n, heads[j]);
@@ -1152,7 +1287,8 @@ void gretl_matrix_print_with_format (const gretl_matrix *m,
     if (gretl_is_null_matrix(m) || fmt == NULL || *fmt == '\0') {
 	real_matrix_print_to_prn(m, NULL, 0, 0, 1, NULL, prn);
     } else {
-	int intcast = 0;
+	const char **rownames = NULL;
+	int llen = 0, intcast = 0;
 	double x;
 	int i, j, c;
 
@@ -1161,9 +1297,17 @@ void gretl_matrix_print_with_format (const gretl_matrix *m,
 	    intcast = 1;
 	}
 
-	maybe_print_col_heads(m, fmt, wid, prec, intcast, prn);
+	rownames = user_matrix_get_names(m, 1);
+	if (rownames != NULL) {
+	    llen = max_label_length(rownames, m->rows);
+	}
+
+	maybe_print_col_heads(m, fmt, wid, prec, intcast, llen, prn);
 
 	for (i=0; i<m->rows; i++) {
+	    if (rownames != NULL) {
+		pprintf(prn, "%*s ", llen, rownames[i]);
+	    }
 	    for (j=0; j<m->cols; j++) {
 		x = gretl_matrix_get(m, i, j);
 		if (intcast) {
