@@ -63,7 +63,14 @@ gretl_matrix *gretl_matrix_data_subset (const int *list, const double **Z,
 					int t1, int t2, int missop, 
 					int *err);
 
-DATAINFO *gretl_dataset_from_matrix (gretl_matrix *m, const int *list,
+gretl_matrix *
+gretl_matrix_data_subset_special (const int *list, const double **Z,
+				  const DATAINFO *pdinfo,
+				  const gretl_matrix *mmask,
+				  int *err);
+
+DATAINFO *gretl_dataset_from_matrix (const gretl_matrix *m, 
+				     const int *list,
 				     double ***pZ, int *err);
 
 int gretl_plotfit_matrices (const double *yvar, const double *xvar,
