@@ -17,7 +17,7 @@
  * 
  */
 
-/*  monte_carlo.c - loop procedures */
+/* monte_carlo.c - loop procedures */
 
 #include "libgretl.h" 
 #include "monte_carlo.h"
@@ -56,7 +56,7 @@ enum loop_types {
 			 l->type == EACH_LOOP)
 
 typedef struct {
-    int linenum;   /* location: line number in loop */
+    int lineno;    /* location: line number in loop */
     int n;         /* number of repetitions */
     int *list;     /* list of vars to print */
     bigval *sum;   /* running sum of values */
@@ -69,7 +69,7 @@ typedef struct {
 /* below: used only in "progressive" loops */ 
 
 typedef struct {
-    int linenum;            /* location: line number in loop */
+    int lineno;             /* location: line number in loop */
     int n;                  /* number of repetitions */
     int nc;                 /* number of coefficients */
     MODEL *model0;          /* copy of initial model */
@@ -85,7 +85,7 @@ typedef struct {
 } LOOP_MODEL;
 
 typedef struct {
-    int linenum;      /* location: line number in loop */ 
+    int lineno;       /* location: line number in loop */ 
     int n;            /* number of observations */
     char *fname;      /* filename for output */
     gretlopt opt;     /* formatting option */
@@ -109,6 +109,23 @@ struct controller_ {
 
 typedef struct controller_ controller;
 
+enum loop_command_codes {
+    LOOP_CMD_GEN     = 1 << 0, /* compiled genr */
+    LOOP_CMD_LIT     = 1 << 1, /* literal printing */
+    LOOP_CMD_NODOL   = 1 << 2, /* no $-substitution this line */
+    LOOP_CMD_NOSUB   = 1 << 3, /* no @-substitution this line */
+    LOOP_CMD_NOOPT   = 1 << 4, /* no option flags in this line */
+    LOOP_CMD_NOSAVE  = 1 << 5  /* no "foo <- cmd" in this line */
+};
+
+struct loop_command_ {
+    char *line;
+    int ci;
+    char flags;
+};
+
+typedef struct loop_command_ loop_command;
+
 typedef struct LOOPSET_ LOOPSET;
 
 struct LOOPSET_ {
@@ -146,12 +163,11 @@ struct LOOPSET_ {
     int n_children;
 
     /* subsidiary objects */
-    char **lines;
-    int *ci;
-    char **eachstrs;
-    GENERATOR **genrs;
-    MODEL **models;
-    int *model_lines;
+    loop_command *cmds;   /* saved command info */
+    char **eachstrs;      /* for use with "foreach" loop */
+    GENERATOR **genrs;    /* saved genr trees */
+    MODEL **models;       /* regular model pointers */
+    int *model_lines;     
     LOOP_MODEL *lmodels;
     LOOP_PRINT *prns;
     LOOP_STORE store;
@@ -160,12 +176,12 @@ struct LOOPSET_ {
     int parent_line;
 };
 
-#define loop_is_progressive(l) (l->flags & LOOP_PROGRESSIVE)
+#define loop_is_progressive(l)  (l->flags & LOOP_PROGRESSIVE)
 #define loop_set_progressive(l) (l->flags |= LOOP_PROGRESSIVE)
-#define loop_is_verbose(l) (l->flags & LOOP_VERBOSE)
-#define loop_set_verbose(l) (l->flags |= LOOP_VERBOSE)
-#define loop_is_quiet(l) (l->flags & LOOP_QUIET)
-#define loop_set_quiet(l) (l->flags |= LOOP_QUIET)
+#define loop_is_verbose(l)      (l->flags & LOOP_VERBOSE)
+#define loop_set_verbose(l)     (l->flags |= LOOP_VERBOSE)
+#define loop_is_quiet(l)        (l->flags & LOOP_QUIET)
+#define loop_set_quiet(l)       (l->flags |= LOOP_QUIET)
 
 #define is_list_loop(l) (l->listname[0] != '\0')
 
@@ -328,12 +344,12 @@ static void set_loop_opts (LOOPSET *loop, gretlopt opt)
 int ok_in_loop (int c)
 {
     /* here are the commands we _don't_ currently allow */
+
     if (c == CORRGM ||
 	c == CUSUM ||
 	c == DATA ||
 	c == DELEET ||
 	c == EQNPRINT ||
-	/* c == FOREIGN || */
 	c == FUNC ||
 	c == HURST ||
 	c == INCLUDE ||
@@ -367,7 +383,7 @@ static int loop_attach_child (LOOPSET *loop, LOOPSET *child)
     loop->children = children;
     loop->children[nc] = child;
     child->parent = loop;
-    child->parent_line = loop->n_cmds + 1;
+    child->parent_line = loop->n_cmds;
     child->level = loop->level + 1;
 
 #if LOOP_DEBUG
@@ -382,7 +398,7 @@ static int loop_attach_child (LOOPSET *loop, LOOPSET *child)
 
 static void loop_store_init (LOOP_STORE *lstore)
 {
-    lstore->linenum = -1;
+    lstore->lineno = -1;
     lstore->n = 0;
     lstore->fname = NULL;
     lstore->opt = OPT_NONE;
@@ -420,8 +436,7 @@ static void gretl_loop_init (LOOPSET *loop)
     loop->n_genrs = 0;
     loop->genrs = NULL;
 
-    loop->lines = NULL;
-    loop->ci = NULL;
+    loop->cmds = NULL;
     loop->model_lines = NULL;
 
     loop->eachstrs = NULL;
@@ -478,29 +493,20 @@ static void gretl_loop_destroy (LOOPSET *loop)
     controller_free(&loop->delta);
     controller_free(&loop->final);
 
-    if (loop->lines != NULL) {
+    if (loop->cmds != NULL) {
 	for (i=0; i<loop->n_cmds; i++) {
-	    free(loop->lines[i]);
+	    free(loop->cmds[i].line);
 	}
-	free(loop->lines);
+	free(loop->cmds);
     }
 
-    if (loop->ci != NULL) { 
-	free(loop->ci);
-    }
-
-    if (loop->model_lines != NULL) { 
-	free(loop->model_lines);
-    }    
+    free(loop->model_lines);
+    free(loop->models);
 
     if (loop->eachstrs != NULL) {
 	free_strings_array(loop->eachstrs, loop->itermax);
     }
 
-    if (loop->models != NULL) {
-	free(loop->models);
-    } 
-
     if (loop->lmodels != NULL) {
 	for (i=0; i<loop->n_loop_models; i++) {
 #if LOOP_DEBUG
@@ -808,7 +814,7 @@ static int find_list_in_parentage (LOOPSET *loop, const char *s)
 
     while ((loop = loop->parent) != NULL) {
 	for (i=0; i<loop->n_cmds; i++) {
-	    if (sscanf(loop->lines[i], "list %15[^ =]", lname)) {
+	    if (sscanf(loop->cmds[i].line, "list %15[^ =]", lname)) {
 		if (!strcmp(lname, s)) {
 		    return 1;
 		}
@@ -1061,7 +1067,8 @@ parse_as_for_loop (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo, char *s)
     }  
 
     if (!err && (sc != 2 || s != q)) {
-	/* must have two semi-colons; must have reached rightmost ')' */
+	/* we've reached the reached rightmost ')' but have not
+	   found two semi-colons */
 	err = E_PARSE;
     }
 
@@ -1171,8 +1178,7 @@ start_new_loop (char *s, LOOPSET *inloop,
     }
 
     if (*err) {
-	free(loop->lines);
-	free(loop->ci);
+	free(loop->cmds);
 	free(loop);
 	loop = NULL;
     } 
@@ -1234,12 +1240,9 @@ loop_condition (LOOPSET *loop, double ***pZ, DATAINFO *pdinfo, int *err)
 	if (loop->iter < loop->itermax) {
 	    ok = 1;
 	}
-    } else {
+    } else if (!loop_count_too_high(loop)) {
 	/* more complex forms of control (for, while) */
-	if (loop_count_too_high(loop)) {
-	    /* safeguard against infinite loops */
-	    ok = 0;
-	} else if (loop->type == FOR_LOOP) {
+	if (loop->type == FOR_LOOP) {
 	    if (loop->iter > 0) {
 		loop_delta(loop, pZ, pdinfo, err);
 	    }
@@ -1268,17 +1271,29 @@ static void controller_free (controller *clr)
     }
 }
 
+static void loop_cmds_init (LOOPSET *loop, int i1, int i2)
+{
+    int i;
+
+    for (i=i1; i<i2; i++) {
+	loop->cmds[i].line = NULL;
+	loop->cmds[i].ci = 0;
+	loop->cmds[i].flags = 0;
+    }
+}
+
 static int gretl_loop_prepare (LOOPSET *loop)
 {
     mpf_set_default_prec(256);
 
     /* allocate some initial lines/commands for loop */
-    loop->lines = malloc(LOOP_BLOCK * sizeof *loop->lines); 
-    loop->ci = malloc(LOOP_BLOCK * sizeof *loop->ci);
+    loop->cmds = malloc(LOOP_BLOCK * sizeof *loop->cmds); 
     
-    if (loop->lines == NULL || loop->ci == NULL) {
-	return 1;
-    }
+    if (loop->cmds == NULL) {
+	return E_ALLOC;
+    } else {
+	loop_cmds_init(loop, 0, LOOP_BLOCK);
+    } 
 
     return 0;
 }
@@ -1307,27 +1322,35 @@ static void loop_model_free (LOOP_MODEL *lmod)
 
 /* Reset the loop model */
 
-static void loop_model_zero (LOOP_MODEL *lmod)
+static void loop_model_zero (LOOP_MODEL *lmod, int started)
 {
-    int i;
+    int i, bnc = 4 * lmod->nc;
 
-    lmod->n = 0;
+#if LOOP_DEBUG
+    fprintf(stderr, "loop_model_zero: %p\n", (void *) lmod);
+#endif
+
+    for (i=0; i<bnc; i++) {
+	if (started) {
+	    mpf_set_d(lmod->bigarray[i], 0.0);
+	} else {
+	    mpf_init(lmod->bigarray[i]);
+	}
+    }
 
     for (i=0; i<lmod->nc; i++) {
-	mpf_init(lmod->sum_coeff[i]);
-	mpf_init(lmod->ssq_coeff[i]);
-	mpf_init(lmod->sum_sderr[i]);
-	mpf_init(lmod->ssq_sderr[i]);
 	lmod->cbak[i] = lmod->sbak[i] = NADBL;
 	lmod->cdiff[i] = lmod->sdiff[i] = 0;
     }
+
+    lmod->n = 0;
 }
 
 /* Set everything in lmod to 0/null in case of failure */
 
 static void loop_model_init (LOOP_MODEL *lmod, int lno)
 {
-    lmod->linenum = lno;
+    lmod->lineno = lno;
     lmod->nc = 0;
     lmod->model0 = NULL;
     lmod->bigarray = NULL;
@@ -1381,7 +1404,7 @@ static int loop_model_start (LOOP_MODEL *lmod, const MODEL *pmod)
     }
 
     if (!err) {
-	loop_model_zero(lmod);
+	loop_model_zero(lmod, 0);
 #if LOOP_DEBUG
 	fprintf(stderr, " model copied to %p, returning 0\n", 
 		(void *) lmod->model0);
@@ -1414,15 +1437,20 @@ static void loop_print_free (LOOP_PRINT *lprn)
     free(lprn->na);
 }
 
-static void loop_print_zero (LOOP_PRINT *lprn)
+static void loop_print_zero (LOOP_PRINT *lprn, int started)
 {
     int i;
 
     lprn->n = 0;
 
     for (i=0; i<lprn->list[0]; i++) { 
-	mpf_init(lprn->sum[i]);
-	mpf_init(lprn->ssq[i]);
+	if (started) {
+	    mpf_set_d(lprn->sum[i], 0.0);
+	    mpf_set_d(lprn->ssq[i], 0.0);
+	} else {
+	    mpf_init(lprn->sum[i]);
+	    mpf_init(lprn->ssq[i]);
+	}
 	lprn->xbak[i] = NADBL;
 	lprn->diff[i] = 0;
 	lprn->na[i] = 0;
@@ -1431,7 +1459,7 @@ static void loop_print_zero (LOOP_PRINT *lprn)
 
 static void loop_print_init (LOOP_PRINT *lprn, int lno)
 {
-    lprn->linenum = lno;
+    lprn->lineno = lno;
     lprn->list = NULL;
     lprn->sum = NULL;
     lprn->ssq = NULL;
@@ -1465,7 +1493,7 @@ static int loop_print_start (LOOP_PRINT *lprn, const int *list)
     lprn->na = malloc(k);
     if (lprn->na == NULL) goto cleanup;
 
-    loop_print_zero(lprn);
+    loop_print_zero(lprn, 0);
 
     return 0;
 
@@ -1487,7 +1515,7 @@ static LOOP_PRINT *get_loop_print_by_line (LOOPSET *loop, int lno, int *err)
     int i, np = loop->n_prints;
 
     for (i=0; i<np; i++) {
-	if (loop->prns[i].linenum == lno) {
+	if (loop->prns[i].lineno == lno) {
 	    return &loop->prns[i];
 	}
     }
@@ -1514,7 +1542,7 @@ static void loop_store_free (LOOP_STORE *lstore)
     free(lstore->fname);
     lstore->fname = NULL;
 
-    lstore->linenum = -1;
+    lstore->lineno = -1;
     lstore->n = 0;
     lstore->opt = OPT_NONE;
 }
@@ -1590,7 +1618,7 @@ static int loop_store_update (LOOPSET *loop, int lno,
 {
     int i, t, err = 0;
 
-    if (loop->store.linenum >= 0 && loop->store.linenum != lno) {
+    if (loop->store.lineno >= 0 && loop->store.lineno != lno) {
 	gretl_errmsg_set("Only one 'store' command is allowed in a "
 			 "progressive loop");
 	return E_DATA;
@@ -1604,7 +1632,7 @@ static int loop_store_update (LOOPSET *loop, int lno,
 	}
     }
 
-    loop->store.linenum = lno;
+    loop->store.lineno = lno;
     t = loop->store.n;
 
     if (t >= loop->store.dinfo->n) {
@@ -1631,16 +1659,16 @@ static MODEL *get_model_record_by_line (LOOPSET *loop, int lno, int *err)
 {
     MODEL **models, *pmod;
     int *modlines;
-    int nm = loop->n_models;
+    int n = loop->n_models;
     int i;
 
-    for (i=0; i<nm; i++) {
+    for (i=0; i<n; i++) {
 	if (lno == loop->model_lines[i]) {
 	    return loop->models[i];
 	}
     }
 
-    modlines = realloc(loop->model_lines, (nm + 1) * sizeof *modlines);
+    modlines = realloc(loop->model_lines, (n + 1) * sizeof *modlines);
     if (modlines == NULL) {
 	*err = E_ALLOC;
 	return NULL;
@@ -1648,7 +1676,7 @@ static MODEL *get_model_record_by_line (LOOPSET *loop, int lno, int *err)
 	loop->model_lines = modlines;
     }
 
-    models = realloc(loop->models, (nm + 1) * sizeof *models);
+    models = realloc(loop->models, (n + 1) * sizeof *models);
     if (models == NULL) {
 	*err = E_ALLOC;
 	return NULL;
@@ -1662,9 +1690,9 @@ static MODEL *get_model_record_by_line (LOOPSET *loop, int lno, int *err)
 	return NULL;
     }
 
-    loop->model_lines[nm] = lno;
-    pmod->ID = nm + 1;
-    loop->models[nm] = pmod;
+    loop->model_lines[n] = lno;
+    pmod->ID = n + 1;
+    loop->models[n] = pmod;
     loop->n_models += 1;
 
     return pmod;
@@ -1696,7 +1724,7 @@ static LOOP_MODEL *
 get_loop_model_by_line (LOOPSET *loop, int lno, int *err)
 {
     LOOP_MODEL *lmods;
-    int nlm = loop->n_loop_models;
+    int n = loop->n_loop_models;
     int i;
 
 #if LOOP_DEBUG
@@ -1704,23 +1732,23 @@ get_loop_model_by_line (LOOPSET *loop, int lno, int *err)
 	    loop->n_loop_models);
 #endif
 
-    for (i=0; i<nlm; i++) {
-	if (loop->lmodels[i].linenum == lno) {
+    for (i=0; i<n; i++) {
+	if (loop->lmodels[i].lineno == lno) {
 	    return &loop->lmodels[i];
 	}
     }
 
-    lmods = realloc(loop->lmodels, (nlm + 1) * sizeof *loop->lmodels);
+    lmods = realloc(loop->lmodels, (n + 1) * sizeof *loop->lmodels);
     if (lmods == NULL) {
 	*err = E_ALLOC;
 	return NULL;
     }
 
     loop->lmodels = lmods;
-    loop_model_init(&loop->lmodels[nlm], lno);
+    loop_model_init(&loop->lmodels[n], lno);
     loop->n_loop_models += 1;
 
-    return &loop->lmodels[nlm];
+    return &loop->lmodels[n];
 }
 
 #define realdiff(x,y) (fabs((x)-(y)) > 2.0e-13)
@@ -1836,21 +1864,21 @@ static int loop_print_update (LOOP_PRINT *lprn,
     return err;
 }
 
-static int add_more_loop_lines (LOOPSET *loop)
+static int add_more_loop_commands (LOOPSET *loop)
 {
     int nb = 1 + (loop->n_cmds + 1) / LOOP_BLOCK;
-    char **lines;
-    int *ci;
+    int totcmds = nb * LOOP_BLOCK;
+    loop_command *cmds;
+
+    /* in case we ran out of space */
+    cmds = realloc(loop->cmds, totcmds * sizeof *cmds); 
     
-    lines = realloc(loop->lines, (nb * LOOP_BLOCK) * sizeof *lines); 
-    ci = realloc(loop->ci, (nb * LOOP_BLOCK) * sizeof *ci);
-    
-    if (lines == NULL || ci == NULL) {
-	return 1;
+    if (cmds == NULL) {
+	return E_ALLOC;
     }
 
-    loop->lines = lines;
-    loop->ci = ci;
+    loop->cmds = cmds;
+    loop_cmds_init(loop, loop->n_cmds, totcmds);
 
     return 0;
 }  
@@ -1858,15 +1886,15 @@ static int add_more_loop_lines (LOOPSET *loop)
 static int real_append_line (ExecState *s, LOOPSET *loop)
 {
     const char *flagstr = NULL;
-    int nc = loop->n_cmds;
+    int n = loop->n_cmds;
     int len, err = 0;
 
 #if LOOP_DEBUG
     fprintf(stderr, "real_append_line: s->line = '%s'\n", s->line);
 #endif
 
-    if ((nc + 1) % LOOP_BLOCK == 0) {
-	if (add_more_loop_lines(loop)) {
+    if ((n + 1) % LOOP_BLOCK == 0) {
+	if (add_more_loop_commands(loop)) {
 	    return E_ALLOC;
 	}
     }
@@ -1884,30 +1912,33 @@ static int real_append_line (ExecState *s, LOOPSET *loop)
 
     if (!err) {
 	if (flagstr != NULL) {
-	    loop->lines[nc] = malloc(len + 1);
+	    loop->cmds[n].line = malloc(len + 1);
 	} else {
-	    loop->lines[nc] = gretl_strdup(s->line);
+	    loop->cmds[n].line = gretl_strdup(s->line);
 	}
-	if (loop->lines[nc] == NULL) {
+	if (loop->cmds[n].line == NULL) {
 	    err = E_ALLOC;
-	} else if (flagstr != NULL) {
-	    sprintf(loop->lines[nc], "%s%s", s->line, flagstr);
+	} else {
+	    if (flagstr != NULL) {
+		sprintf(loop->cmds[n].line, "%s%s", s->line, flagstr);
+	    }
+	    compress_spaces(loop->cmds[n].line);
 	}
     }
 
     if (!err) {
 	if (s->cmd->ci == PRINT && (!loop_is_progressive(loop) || strchr(s->line, '"'))) {
-	    loop->ci[nc] = 0;
-	} else {
-	    loop->ci[nc] = s->cmd->ci;
-	}
+	    /* printing a literal string, not a variable's value */
+	    loop->cmds[n].flags |= LOOP_CMD_LIT;
+	} 
+	loop->cmds[n].ci = s->cmd->ci;
 	loop->n_cmds += 1;
     }
 
 #if LOOP_DEBUG
     fprintf(stderr, "loop %p: n_cmds=%d, line[%d]='%s', ci=%d\n",
-	    (void *) loop, loop->n_cmds, nc, loop->lines[nc],
-	    loop->ci[nc]);
+	    (void *) loop, loop->n_cmds, n, loop->cmds[n].line,
+	    loop->cmds[n].ci);
 #endif
 
     return err;
@@ -2190,6 +2221,8 @@ static int loop_store_save (LOOP_STORE *lstore, PRN *prn)
     return err;
 }
 
+#define loop_literal(l,i) (l->cmds[i].flags & LOOP_CMD_LIT)
+
 /**
  * print_loop_results:
  * @loop: pointer to loop struct.
@@ -2202,31 +2235,30 @@ static int loop_store_save (LOOP_STORE *lstore, PRN *prn)
 static void print_loop_results (LOOPSET *loop, const DATAINFO *pdinfo, 
 				PRN *prn)
 {
-    char linecpy[MAXLINE];
     int iters = loop->iter;
-    int i, j, k;
+    int i, j = 0, k = 0;
 
     if (loop->type != COUNT_LOOP && !(loop_is_quiet(loop))) {
 	pprintf(prn, _("\nNumber of iterations: %d\n\n"), iters);
     }
 
-    j = 0;
-    k = 0;
-
     for (i=0; i<loop->n_cmds; i++) {
 	gretlopt opt = OPT_NONE;
+	int ci = loop->cmds[i].ci;
 
 #if LOOP_DEBUG
-	fprintf(stderr, "print_loop_results: loop command %d (i=%d): %s\n", 
-		i+1, i, loop->lines[i]);
+	fprintf(stderr, "print_loop_results: loop command %d: %s\n", 
+		i, loop->cmds[i].line);
 #endif
 
-	if (plain_model_ci(loop->ci[i])) {
-	    strcpy(linecpy, loop->lines[i]);
+	if (plain_model_ci(ci)) {
+	    char linecpy[MAXLINE];
+
+	    strcpy(linecpy, loop->cmds[i].line);
 	    opt = get_gretl_options(linecpy, NULL);
 	}	    
 
-	if (!loop_is_progressive(loop) && loop->ci[i] == OLS) {
+	if (!loop_is_progressive(loop) && ci == OLS) {
 	    if (model_print_deferred(opt)) {
 		MODEL *pmod = loop->models[j++];
 
@@ -2236,15 +2268,15 @@ static void print_loop_results (LOOPSET *loop, const DATAINFO *pdinfo,
 	}
 
 	if (loop_is_progressive(loop)) {
-	    if (plain_model_ci(loop->ci[i]) && !(opt & OPT_Q)) {
+	    if (plain_model_ci(ci) && !(opt & OPT_Q)) {
 		loop_model_print(&loop->lmodels[j], pdinfo, prn);
-		loop_model_zero(&loop->lmodels[j]);
+		loop_model_zero(&loop->lmodels[j], 1);
 		j++;
-	    } else if (loop->ci[i] == PRINT) {
+	    } else if (ci == PRINT && !loop_literal(loop, i)) {
 		loop_print_print(&loop->prns[k], pdinfo, prn);
-		loop_print_zero(&loop->prns[k]);
+		loop_print_zero(&loop->prns[k], 1);
 		k++;
-	    } else if (loop->ci[i] == STORE) {
+	    } else if (ci == STORE) {
 		loop_store_save(&loop->store, prn);
 	    }
 	}
@@ -2306,7 +2338,8 @@ substitute_dollar_targ (char *str, const LOOPSET *loop,
     } else if (loop->type == INDEX_LOOP) {
 	sprintf(ins, "%d", idx);
     } else if (loop->type == DATED_LOOP) {
-	ntodate(ins, idx, pdinfo);
+	/* note: ntodate is 0-based */
+	ntodate(ins, idx - 1, pdinfo);
     } else if (loop->type == EACH_LOOP) {
 	pins = loop->eachstrs[idx - 1];
     }  
@@ -2473,38 +2506,31 @@ make_dollar_substitutions (char *str, const LOOPSET *loop,
     return err;
 }
 
-/* Try to determine if a "list" command within a loop modifies the
-   list that is controlling the loop: if so, we'll have to arrange to
-   have the list of variable names refreshed at the top of the loop.
+/* Check for the name of the list that's defining the loop
+   appearing on the left-hand side of a genr-type expression:
+   if so, we'd better refresh the loop-controller.
 */
 
-static int modifies_loop_list (const LOOPSET *loop, const char *s)
+static inline int maybe_refresh_list (CMD *cmd, const LOOPSET *loop,
+				      const char *line)
 {
-    int ret = 0;
-
-    if (!strncmp(s, "list", 4)) {
-	char lname[VNAMELEN];
-
-	s += 4;
-	s += strspn(s, " ");
-	*lname = '\0';
-	sscanf(s, "%15[^ +=-]", lname);
-	ret = !strcmp(lname, loop->listname);
-    }
-
-    return ret;
-}
-
-static int maybe_refresh_list (CMD *cmd, const LOOPSET *loop,
-			       const char *line)
-{
-    int ret = 0;
-
     if (cmd->ci == GENR) {
-	ret = modifies_loop_list(loop, line);
+	char *p = strstr(line, loop->listname);
+
+	if (p != NULL) {
+	    int n = strlen(loop->listname);
+
+	    if (n == gretl_namechar_spn(p)) {
+		char *q = strchr(line, '=');
+		
+		if (q != NULL && q - p > 0) {
+		    return 1;
+		}
+	    }
+	}
     }
 
-    return ret;
+    return 0;
 }
 
 static LOOPSET *get_child_loop_by_line (LOOPSET *loop, int lno)
@@ -2534,6 +2560,7 @@ static int add_loop_genr (LOOPSET *loop, GENERATOR *genr, int lno)
     loop->genrs[n] = genr;
     loop->n_genrs = n + 1;
     genr_set_loopline(genr, lno);
+    loop->cmds[lno].flags |= LOOP_CMD_GEN;
 
     return 0;
 }
@@ -2558,11 +2585,11 @@ static GENERATOR *get_loop_genr_by_line (LOOPSET *loop, int lno,
 
     genr = genr_compile(line, pZ, pdinfo, OPT_NONE, err);
 
-#if LOOP_DEBUG > 1
-    fprintf(stderr, "compiled loop genr, err = %d\n", *err);
-#endif
-
     if (!*err) {
+#if LOOP_DEBUG > 1
+	fprintf(stderr, "compiled genr on line %d of loop %p:\n"
+		" '%s'\n", lno, (void *) loop, line);
+#endif
 	*err = add_loop_genr(loop, genr, lno);
     }
 
@@ -2594,12 +2621,12 @@ static int loop_print_save_model (MODEL *pmod, DATAINFO *pdinfo,
    stack of loop commands.
 */
 
-static int next_command (char *targ, LOOPSET *loop, int *pj)
+static int loop_next_command (char *targ, LOOPSET *loop, int *pj)
 {
-    int ret = 1, j = *pj;
+    int ret = 1, j = *pj + 1;
 
     if (j < loop->n_cmds) {
-	strcpy(targ, loop->lines[j++]);
+	strcpy(targ, loop->cmds[j].line);
 	*pj = j;
     } else {
 	ret = 0;
@@ -2608,6 +2635,119 @@ static int next_command (char *targ, LOOPSET *loop, int *pj)
     return ret;
 }
 
+static int loop_process_error (int err, PRN *prn)
+{
+    if (libset_get_bool(HALT_ON_ERR) == 0) {
+	errmsg(err, prn);
+	err = 0;
+    }
+
+    return err;
+}
+
+#define genr_compiled(l,j) (l->cmds[j].flags & LOOP_CMD_GEN)
+#define loop_cmd_nodol(l,j) (l->cmds[j].flags & LOOP_CMD_NODOL)
+#define loop_cmd_nosub(l,j) (l->cmds[j].flags & LOOP_CMD_NOSUB)
+#define loop_cmd_noopt(l,j) (l->cmds[j].flags & LOOP_CMD_NOOPT)
+#define loop_cmd_nosave(l,j) (l->cmds[j].flags & LOOP_CMD_NOSAVE)
+
+/* based on the stored flags in the loop-line record, set
+   or unset some flags for the command parser: this can 
+   reduce the amount of work the parser has to do on each
+   iteration of a loop
+*/
+
+static inline void loop_info_to_cmd (LOOPSET *loop, int j,
+				     CMD *cmd)
+{
+    if (loop_is_progressive(loop)) {
+	cmd->flags |= CMD_PROG;
+    } else {
+	cmd->flags &= ~CMD_PROG;
+    }
+
+    if (loop_cmd_nosub(loop, j)) {
+	/* tell parser not to try for @-substitution */
+	cmd->flags |= CMD_NOSUB;
+    } else {
+	cmd->flags &= ~CMD_NOSUB;
+    }
+    
+    if (loop_cmd_noopt(loop, j)) {
+	/* tell parser not to try for option flags */
+	cmd->flags |= CMD_NOOPT;
+	/* since the cmd->opt is (or can be) persistent, we
+	   should zero it */
+	cmd->opt = OPT_NONE;
+    } else {
+	cmd->flags &= ~CMD_NOOPT;
+    }
+
+    if (loop_cmd_nosave(loop, j)) {
+	/* tell parser not to try for a "savename" */
+	cmd->flags |= CMD_NOSAVE;
+	*cmd->savename = '\0';
+    } else {
+	cmd->flags &= ~CMD_NOSAVE;
+    }
+}
+
+/* based on the parsed info in @cmd, maybe write some flags to
+   the current loop-line record
+*/
+
+static inline void cmd_info_to_loop (LOOPSET *loop, int j,
+				     CMD *cmd, int *subst)
+{
+    if (!loop_cmd_nosub(loop, j)) {
+	if (cmd_subst(cmd)) {
+	    *subst = 1;
+	} else {
+	    /* record: no @-substitution in this line */
+	    loop->cmds[j].flags |= LOOP_CMD_NOSUB;
+	}
+    }
+
+    if (loop_cmd_nosub(loop, j)) {
+	if (cmd->opt == OPT_NONE) {
+	    /* record: no options are present on this line */
+	    loop->cmds[j].flags |= LOOP_CMD_NOOPT;
+	} 
+	if (*cmd->savename == '\0') {
+	    /* record: no savename on this line */
+	    loop->cmds[j].flags |= LOOP_CMD_NOSAVE;
+	}
+    }
+}
+
+static int loop_report_error (LOOPSET *loop, int err, 
+			      const char *errline,
+			      ExecState *state,
+			      PRN *prn)
+{
+    int fd = gretl_function_depth();
+
+    if (err) {
+	if (fd == 0) {
+	    errmsg(err, prn);
+	    if (*errline != '\0') {
+		pprintf(prn, ">> %s\n", errline);
+	    }
+	}
+    } else if (loop->err) {
+	if (fd == 0) {
+	    errmsg(loop->err, prn);
+	}
+	err = loop->err;
+    }
+
+    if (fd > 0 && err && *errline != '\0') {
+	strcpy(state->line, errline);
+    }
+
+    return err;
+}	
+
 static int block_model (CMD *cmd)
 {
     return cmd->ci == END && 
@@ -2632,7 +2772,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
     LOOP_MODEL *lmod;
     LOOP_PRINT *lprn;
     char errline[MAXLINE];
-    int indent0, subst, lrefresh;
+    int indent0, lrefresh;
     int show_activity = 0;
     int j, err = 0;
 
@@ -2665,7 +2805,8 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 #if LOOP_DEBUG
 	fprintf(stderr, "top of loop: iter = %d\n", loop->iter);
 #endif
-	j = lrefresh = subst = 0;
+	lrefresh = 0;
+	j = -1;
 
 	pmod = NULL;
 	lmod = NULL;
@@ -2675,49 +2816,75 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	    print_loop_progress(loop, pdinfo, prn);
 	}
 
-	while (!err && next_command(line, loop, &j)) {
+	while (!err && loop_next_command(line, loop, &j)) {
+	    int subst = 0;
+
 #if LOOP_DEBUG
 	    fprintf(stderr, " j=%d, line='%s'\n", j, line);
 #endif
 	    strcpy(errline, line);
 
-	    if (strchr(line, '$')) {
+	    if (!gretl_if_state_false() && genr_compiled(loop, j)) {
+		/* if the current line already has "compiled genr" status,
+		   we should be able to skip several steps that are
+		   potentially quite time-consuming 
+		*/
+		if (gretl_echo_on() && !loop_is_quiet(loop)) {
+		    pprintf(prn, "? %s\n", line);
+		}
+		genr = get_loop_genr_by_line(loop, j, line, pZ, pdinfo, &err);
+		if (!err) {
+		    err = execute_genr(genr, pZ, pdinfo, prn);
+		}
+		if (err) {
+		    err = loop_process_error(err, prn);
+		} else if (is_list_loop(loop) && 
+			   maybe_refresh_list(cmd, loop, line)) {
+		    loop_list_refresh(loop, pdinfo);
+		}
+		if (err) {
+		    break;
+		} else {
+		    continue;
+		}
+	    }
+
+	    if (!loop_cmd_nodol(loop, j) && strchr(line, '$')) {
 		err = make_dollar_substitutions(line, loop, 
 						(const double **) *pZ,
 						pdinfo, &subst);
 		if (err) {
 		    break;
+		} else if (!subst) {
+		    loop->cmds[j].flags |= LOOP_CMD_NODOL;
 		}
 	    }
 
-	    if (loop_is_progressive(loop)) {
-		cmd->flags |= CMD_PROG;
-	    } else {
-		cmd->flags &= ~CMD_PROG;
-	    }
+	    /* transcribe loop -> cmd */
+	    loop_info_to_cmd(loop, j, cmd);
 
 	    /* We already have the "ci" index recorded, but here
 	       we do some further parsing. 
 	    */
 	    err = parse_command_line(line, cmd, pZ, pdinfo);
 
+#if LOOP_DEBUG
+	    fprintf(stderr, "    after: '%s'\n", line);
+	    fprintf(stderr, "    cmd->savename = '%s'\n", cmd->savename);
+#endif
+
 	    if (cmd->ci < 0) {
 		continue;
 	    } else if (err) {
-		if (libset_get_bool(HALT_ON_ERR)) {
+		err = loop_process_error(err, prn);
+		if (err) {
 		    break;
 		} else {
-		    /* try soldiering on */
-		    errmsg(err, prn);
-		    err = 0;
 		    continue;
 		}
 	    } else {
 		gretl_exec_state_transcribe_flags(s, cmd);
-	    }
-
-	    if (!subst && cmd_subst(cmd)) {
-		subst = 1;
+		cmd_info_to_loop(loop, j, cmd, &subst);
 	    }
 
 	    if (is_list_loop(loop) && maybe_refresh_list(cmd, loop, line)) {
@@ -2780,7 +2947,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 			set_as_last_model(s->models[0], GRETL_OBJ_EQN);
 		    } else if (model_print_deferred(cmd->opt)) {
 			swap_models(s->models[0], pmod);
-			pmod->ID = j;
+			pmod->ID = j + 1;
 			set_as_last_model(pmod, GRETL_OBJ_EQN);
 			model_count_minus();
 		    } else {
@@ -2803,18 +2970,18 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 				     cmd->word);
 		err = 1;
 	    } else if (cmd->ci == GENR) {
-		if (!loop_is_verbose(loop)) {
-		    cmd->opt |= OPT_Q;
-		}
 		if (subst || (cmd->opt & OPT_U)) {
 		    /* can't use a "compiled" genr if string substitution
 		       has been done, since the genr expression will not
 		       be constant */
+		    if (!loop_is_verbose(loop)) {
+			cmd->opt |= OPT_Q;
+		    }
 		    err = generate(line, pZ, pdinfo, cmd->opt, prn);
 		} else {
 		    genr = get_loop_genr_by_line(loop, j, line, pZ, pdinfo, &err);
 		    if (!err) {
-			err = execute_genr(genr, pZ, pdinfo, OPT_L, prn);
+			err = execute_genr(genr, pZ, pdinfo, prn);
 		    }
 		}
 	    } else {
@@ -2824,6 +2991,7 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 		    loop_print_save_model(s->models[0], pdinfo, prn, s);
 		}
 	    }
+
 	} /* end execution of commands within loop */
 
 	if (err) {
@@ -2851,7 +3019,6 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 		loop->idxval += 1;
 		gretl_scalar_set_value(loop->idxname, loop->idxval);
 	    } else if (lrefresh) {
-		/* added 2008-01-11, AC */
 		loop_list_refresh(loop, pdinfo);
 	    }
 	    if (show_activity && (loop->iter % 10 == 0)) {
@@ -2861,21 +3028,17 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     } /* end iterations of loop */
 
+    cmd->flags &= ~CMD_NOSUB;
+    cmd->flags &= ~CMD_NOOPT;
+    cmd->flags &= ~CMD_NOSAVE;
+
     if (loop->brk) {
 	/* turn off break flag */
 	loop->brk = 0;
     }
 
-    if (err) {
-	if (!s->funcerr) {
-	    errmsg(err, prn);
-	    if (*errline != '\0') {
-		pprintf(prn, ">> %s\n", errline);
-	    }
-	}
-    } else if (loop->err) {
-	errmsg(loop->err, prn);
-	err = loop->err;
+    if (err || loop->err) {
+	err = loop_report_error(loop, err, errline, s, prn);
     }
 
     if (!err && loop->iter > 0) {
@@ -2897,10 +3060,13 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 	}
     }
 
-    if (line != NULL) {
+    if (err && gretl_function_depth() > 0) {
+	; /* leave 'line' alone */
+    } else if (line != NULL) {
 	*line = '\0';
     } 
 
+    /* be sure to clear some loop-special parser flags */
     cmd->flags &= ~CMD_PROG;
 
     if (loop->parent == NULL) {
@@ -2912,6 +3078,3 @@ int gretl_loop_exec (ExecState *s, double ***pZ, DATAINFO *pdinfo)
 
     return process_command_error(cmd, err);
 }
-
-
-
