@@ -31,6 +31,7 @@
 #include "cmd_private.h"
 #include "estim_private.h"
 #include "gretl_scalar.h"
+#include "gretl_string_table.h"
 #include "gretl_bfgs.h"
 #include "tsls.h"
 
@@ -51,9 +52,11 @@
 #define ML_DEBUG 0
 
 enum {
-    ANALYTIC_DERIVS = 1 << 0,
-    NLS_AUTOREG     = 1 << 1
-} nls_flags;
+    NL_ANALYTICAL = 1 << 0,
+    NL_AUTOREG    = 1 << 1,
+    NL_AHESS      = 1 << 2,
+    NL_NEWTON     = 1 << 3
+} nl_flags;
 
 struct parm_ {
     char name[VNAMELEN];  /* name of parameter */
@@ -72,8 +75,8 @@ struct parm_ {
 #define matrix_deriv(s,i) (s->params[i].dmat != NULL)
 #define scalar_deriv(s,i) (gretl_is_scalar(s->params[i].dname))
 
-#define numeric_mode(s) (!(s->flags & ANALYTIC_DERIVS))
-#define analytic_mode(s) (s->flags & ANALYTIC_DERIVS)
+#define numeric_mode(s) (!(s->flags & NL_ANALYTICAL))
+#define analytic_mode(s) (s->flags & NL_ANALYTICAL)
 
 /* file-scope global variables */
 
@@ -84,12 +87,12 @@ static char *adjust_saved_nlfunc (char *s);
 
 static void set_numeric_mode (nlspec *s)
 {
-    s->flags &= ~ANALYTIC_DERIVS;
+    s->flags &= ~NL_ANALYTICAL;
 }
 
 static void set_analytic_mode (nlspec *s)
 {
-    s->flags |= ANALYTIC_DERIVS;
+    s->flags |= NL_ANALYTICAL;
 }
 
 static void destroy_genrs_array (GENERATOR **genrs, int n)
@@ -161,7 +164,7 @@ static int nls_dynamic_check (nlspec *s, char *formula)
     genr = genr_compile(formula, s->Z, s->dinfo, OPT_P, &err);
 
     if (!err && genr_is_autoregressive(genr)) {
-	s->flags |= NLS_AUTOREG;
+	s->flags |= NL_AUTOREG;
     }
 
     /* restore the dependent variable */
@@ -271,7 +274,7 @@ static int nls_genr_setup (nlspec *s)
 	   while we're at it
 	*/
 	genr_set_na_check(genrs[i]);
-	err = execute_genr(genrs[i], s->Z, s->dinfo, OPT_S, s->prn);
+	err = execute_genr(genrs[i], s->Z, s->dinfo, s->prn);
 	genr_unset_na_check(genrs[i]);
 
 	if (err) {
@@ -335,6 +338,14 @@ static int nls_genr_setup (nlspec *s)
 #endif
     }
 
+    if (!err && s->hesscall != NULL) {
+	s->hgen = genr_compile(s->hesscall, s->Z, s->dinfo, 
+			       OPT_P | OPT_U, &err);
+	if (!err) {
+	    err = execute_genr(s->hgen, s->Z, s->dinfo, s->prn);
+	} 
+    }	
+
     if (err) {
 	destroy_genrs_array(genrs, ngen);
     } else {
@@ -372,7 +383,7 @@ static int nls_auto_genr (nlspec *s, int i)
 #if NLS_DEBUG
 	fprintf(stderr, " generating aux var %d:\n %s\n", j, s->aux[j]);
 #endif
-	s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, OPT_S, s->prn);
+	s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, s->prn);
 	if (s->generr) {
 	    return s->generr;
 	} 
@@ -389,7 +400,7 @@ static int nls_auto_genr (nlspec *s, int i)
 	    s->naux, i, j, j, (void *) s->genrs[j]);
     fprintf(stderr, " Z[%d] = %p\n", s->dinfo->v-1, (void *) (*s->Z)[s->dinfo->v-1]);
 #endif
-    s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, OPT_S, s->prn);
+    s->generr = execute_genr(s->genrs[j], s->Z, s->dinfo, s->prn);
 
     /* make sure we have a correct pointer to matrix deriv */
     if (!s->generr && i > 0 && matrix_deriv(s, i-1)) {
@@ -551,17 +562,53 @@ static int nlspec_push_param (nlspec *s, const char *name, char *deriv)
 
 static int check_param_name (const char *name)
 {
+    int err = 0;
+
     if (gretl_is_scalar(name)) {
-	return 0;
+	; /* OK */
     } else {
 	gretl_matrix *m = get_matrix_by_name(name);
 
-	if (gretl_vector_get_length(m) == 0) {
-	    return E_DATATYPE;
-	} else {
-	    return 0;
-	}
+	if (m == NULL || gretl_vector_get_length(m) == 0) {
+	    gretl_errmsg_sprintf(_("'%s': expected a scalar or vector"), name);
+	    err = E_DATATYPE;
+	} 
     }
+
+    return err;
+}
+
+static int nlspec_add_param_names (nlspec *spec, const char *s)
+{
+    char sname[VNAMELEN];
+    int n, err = 0;
+
+    s += strspn(s, " ");
+    n = gretl_namechar_spn(s);
+
+    if (n == 0 || n >= VNAMELEN) {
+	err = E_PARSE;
+    } else {
+	*sname = '\0';
+	strncat(sname, s, n);
+	if (!is_user_string(sname)) {
+	    err = E_UNKVAR;
+	} else {
+	    const char *names = get_string_by_name(sname);
+
+	    if (names == NULL || *names == '\0') {
+		err = E_DATA;
+	    } else {
+		free(spec->parnames);
+		spec->parnames = gretl_strdup(names);
+		if (spec->parnames == NULL) {
+		    err = E_ALLOC;
+		}
+	    }
+	} 
+    } 
+
+    return err;
 }
 
 /* For case where analytical derivatives are not given, the user
@@ -1070,7 +1117,7 @@ static int get_nls_derivs (int k, int T, int offset, double *g, double **G,
     return err;
 }
 
-/* analytical derivatives, used in the context of BFGS */
+/* for use with analytical derivatives */
 
 static int get_mle_gradient (double *b, double *g, int n, 
 			     BFGS_CRIT_FUNC llfunc,
@@ -1161,6 +1208,56 @@ static int get_mle_gradient (double *b, double *g, int n,
     return err;
 }
 
+static int get_mle_hessian (double *b, gretl_matrix *H, void *p)
+{
+    nlspec *spec = (nlspec *) p;
+    int k = H->rows;
+    int err;
+
+    if (b != NULL) {
+	update_coeff_values(b, spec);
+    }
+
+    err = execute_genr(spec->hgen, spec->Z, spec->dinfo, spec->prn); 
+
+    if (!err) {
+	gretl_matrix *uH = get_matrix_by_name(spec->hname);
+
+	if (uH == NULL) {
+	    err = E_UNKVAR;
+	} else if (uH->rows != k || uH->cols != k) {
+	    err = E_NONCONF;
+	} else {
+	    gretl_matrix_copy_values(H, uH);
+	}
+    }
+
+    return err;
+}
+
+static gretl_matrix *mle_hessian_inverse (nlspec *spec, int *err)
+{
+    int k = spec->ncoeff;
+    gretl_matrix *H = gretl_matrix_alloc(k, k);
+
+    if (H == NULL) {
+	*err = E_ALLOC;
+    } else {
+	*err = get_mle_hessian(NULL, H, spec);
+    }
+
+    if (!*err) {
+	*err = gretl_invert_symmetric_matrix(H);
+	if (*err) {
+	    fprintf(stderr, "mle_hessian_inverse: failed\n");
+	    gretl_matrix_free(H);
+	    H = NULL;
+	}
+    }
+
+    return H;
+}
+
 /* Compute auxiliary statistics and add them to the NLS 
    model struct */
 
@@ -1261,8 +1358,8 @@ static int mle_build_vcv (MODEL *pmod, nlspec *spec, int *vcvopt)
     }
 
     if (numeric_mode(spec)) {
-	G = build_score_matrix(spec->coeff, k, T, mle_llt_callback, 
-			       (void *) spec, &err);
+	G = numerical_score_matrix(spec->coeff, k, T, mle_llt_callback, 
+				   (void *) spec, &err);
 	if (err) {
 	    gretl_matrix_free(V);
 	    return err;
@@ -1317,12 +1414,21 @@ static int mle_build_vcv (MODEL *pmod, nlspec *spec, int *vcvopt)
 	err = gretl_invert_symmetric_matrix(V);
 	if (err) {
 	    gretl_errmsg_set("failed to invert OPG matrix GG'");
+#if 0 /* experiment: show estimates even tho' VCV has failed */
+	    *vcvopt = VCV_MAX;
+	    k = k * k;
+	    for (i=0; i<k; i++) {
+		V->val[i] = 0.0/0.0;
+	    }
+	    err = 0;
+#endif
+	} else {
+	    *vcvopt = VCV_OP;
 	}
-	*vcvopt = VCV_OP;
     }
 
     if (!err) {
-	err = gretl_model_write_vcv(pmod, V);
+	err = gretl_model_write_vcv(pmod, V, -1);
     }
 
     gretl_matrix_free(G);
@@ -1344,7 +1450,7 @@ static int mle_add_vcv (MODEL *pmod, nlspec *spec)
 	}
     } else if (spec->opt & OPT_H) {
 	vcvopt = VCV_HESSIAN;
-	err = gretl_model_write_vcv(pmod, spec->Hinv);
+	err = gretl_model_write_vcv(pmod, spec->Hinv, -1);
     } else {
 	/* either OPG or QML */
 	err = mle_build_vcv(pmod, spec, &vcvopt);
@@ -1416,7 +1522,7 @@ static char *adjust_saved_nlfunc (char *s)
     return s;
 }
 
- /* Attach additional spcification info to make it possible to
+ /* Attach additional specification info to make it possible to
     reconstruct the model from the saved state.  We need this
     for the "Modify model" option in the gretl GUI, and may
     also make use of it for bootstrapping NLS models.
@@ -1464,6 +1570,10 @@ static int nl_model_add_spec_info (MODEL *pmod, nlspec *spec)
 	}
     }
 
+    if (spec->hesscall != NULL) {
+	pprintf(prn, "hessian %s\n", spec->hesscall);
+    }
+
     pprintf(prn, "end %s\n", cmd);
 
     buf = gretl_print_steal_buffer(prn);
@@ -1473,6 +1583,43 @@ static int nl_model_add_spec_info (MODEL *pmod, nlspec *spec)
     return 0;
 }
 
+static int copy_user_parnames (MODEL *pmod, nlspec *spec)
+{
+    const char *sep = ",";
+    const char *pname;
+    char *tmp;
+    int i, err = 0;
+
+    /* copy the user-defined string before applying strtok */
+    tmp = gretl_strdup(spec->parnames);
+    if (tmp == NULL) {
+	return E_ALLOC;
+    }
+
+    if (strchr(tmp, ',') == NULL) {
+	sep = " ";
+    }
+
+    for (i=0; i<pmod->ncoeff && !err; i++) {
+	pname = strtok((i == 0)? tmp : NULL, sep);
+	if (pname == NULL) {
+	    fprintf(stderr, "copy_user_parnames: too few strings\n");
+	    gretl_errmsg_sprintf(_("modprint: expected %d names"), 
+				 pmod->ncoeff);
+	    err = E_DATA;
+	} else {
+	    pmod->params[i] = gretl_strdup(pname);
+	    if (pmod->params[i] == NULL) {
+		err = E_ALLOC;
+	    }
+	}
+    }
+
+    free(tmp);
+
+    return err;
+}
+
 /* Called for all of NLS, MLE, GMM: attach to the model struct
    the names of the parameters and some other string info
 */
@@ -1512,27 +1659,46 @@ add_param_names_to_model (MODEL *pmod, nlspec *spec, const DATAINFO *pdinfo)
     if (err) {
 	free(pmod->params);
 	return err;
-    }    
+    }  
 
-    i = 0;
-    for (j=0; j<spec->nparam; j++) {
-	if (scalar_param(spec, j)) {
-	    pmod->params[i++] = gretl_strdup(spec->params[j].name);
-	} else {
-	    m = spec->params[j].nc;
-	    sprintf(pname, "%d", m + 1);
-	    n = VNAMELEN - strlen(pname) - 3;
-	    for (k=0; k<m; k++) {
-		sprintf(pname, "%.*s[%d]", n, spec->params[j].name, k + 1);
-		pmod->params[i++] = gretl_strdup(pname);
+    if (spec->parnames != NULL) {
+	/* handle the case where the user has given a "parnames"
+	   string */
+	err = copy_user_parnames(pmod, spec);
+    } else {
+	/* compose automatic parameter names */
+	i = 0;
+	for (j=0; j<spec->nparam && !err; j++) {
+	    if (scalar_param(spec, j)) {
+		pmod->params[i] = gretl_strdup(spec->params[j].name);
+		if (pmod->params[i] == NULL) {
+		    err = E_ALLOC;
+		}
+		i++;
+	    } else {
+		m = spec->params[j].nc;
+		sprintf(pname, "%d", m + 1);
+		n = VNAMELEN - strlen(pname) - 3;
+		for (k=0; k<m && !err; k++) {
+		    sprintf(pname, "%.*s[%d]", n, spec->params[j].name, k + 1);
+		    pmod->params[i] = gretl_strdup(pname);
+		    if (pmod->params[i] == NULL) {
+			err = E_ALLOC;
+		    }
+		    i++;
+		}
 	    }
 	}
     }
 
-    if (spec->ci == NLS || gretl_in_gui_mode()) {
+    if (!err && (spec->ci == NLS || gretl_in_gui_mode())) {
 	err = nl_model_add_spec_info(pmod, spec);
     }
 
+    if (err && !pmod->errcode) {
+	pmod->errcode = err;
+    }
+
     return err;
 }
 
@@ -1556,7 +1722,7 @@ add_fit_resid_to_model (MODEL *pmod, nlspec *spec, double *uhat,
 	}
     }
 
-    if (perfect || (spec->flags & NLS_AUTOREG)) {
+    if (perfect || (spec->flags & NL_AUTOREG)) {
 	pmod->rho = pmod->dw = NADBL;
     }
 }
@@ -1618,7 +1784,6 @@ static MODEL GNR (nlspec *spec, const double **Z, DATAINFO *pdinfo,
     int T = spec->nobs;
     int iters = spec->iters;
     int perfect = 0;
-    int err = 0;
 
 #if NLS_DEBUG
     fprintf(stderr, "GNR: T = %d\n", T);
@@ -1732,7 +1897,6 @@ static MODEL GNR (nlspec *spec, const double **Z, DATAINFO *pdinfo,
     if (gnr.errcode) {
 	pputs(prn, _("In Gauss-Newton Regression:\n"));
 	errmsg(gnr.errcode, prn);
-	err = 1;
     } 
 
     if (gnr.list[0] != glist[0]) {
@@ -1759,7 +1923,7 @@ static MODEL GNR (nlspec *spec, const double **Z, DATAINFO *pdinfo,
 	gretl_model_set_int(&gnr, "iters", iters);
 	gretl_model_set_double(&gnr, "tol", spec->tol);
 	transcribe_nls_function(&gnr, spec->nlfunc);
-	if (spec->flags & NLS_AUTOREG) {
+	if (spec->flags & NL_AUTOREG) {
 	    gretl_model_set_int(&gnr, "dynamic", 1);
 	}
     }
@@ -1856,9 +2020,13 @@ static int make_nl_model (MODEL *pmod, nlspec *spec,
     }
 
     if (!pmod->errcode) {
-	gretl_model_set_int(pmod, "fncount", spec->fncount);
-	gretl_model_set_int(pmod, "grcount", spec->grcount);
-	gretl_model_set_double(pmod, "tol", spec->tol);
+	if (spec->flags & NL_NEWTON) {
+	    gretl_model_set_int(pmod, "iters", spec->fncount);
+	} else {
+	    gretl_model_set_int(pmod, "fncount", spec->fncount);
+	    gretl_model_set_int(pmod, "grcount", spec->grcount);
+	    gretl_model_set_double(pmod, "tol", spec->tol);
+	}
     }
 
     /* mask invalid stats */
@@ -1894,6 +2062,9 @@ static void clear_nlspec (nlspec *spec)
 	return;
     }
 
+    free(spec->parnames);
+    spec->parnames = NULL;
+
     if (spec->params != NULL) {
 	for (i=0; i<spec->nparam; i++) {
 	    free(spec->params[i].deriv);
@@ -1935,6 +2106,11 @@ static void clear_nlspec (nlspec *spec)
     spec->ngenrs = 0;
     spec->generr = 0;
 
+    if (spec->hgen != NULL) {
+	destroy_genr(spec->hgen);
+    }
+    free(spec->hesscall);
+
     free(spec->nlfunc);
     spec->nlfunc = NULL;
 
@@ -2131,8 +2307,15 @@ static int check_derivatives (nlspec *spec, PRN *prn)
 
 static int mle_calculate (nlspec *s, PRN *prn)
 {
+    BFGS_GRAD_FUNC gradfunc = NULL;
+    HESS_FUNC hessfunc = NULL;
+    int maxit, use_newton = 0;
     int err = 0;
 
+    if (libset_get_int(GRETL_OPTIM) == OPTIM_NEWTON) {
+	use_newton = 1;
+    }
+
     if (analytic_mode(s)) {
 	if (!(s->opt & OPT_G)) {
 	    err = check_derivatives(s, prn);
@@ -2140,21 +2323,55 @@ static int mle_calculate (nlspec *s, PRN *prn)
     }
 
     if (!err) {
-	BFGS_GRAD_FUNC gradfun = (analytic_mode(s))?
-	    get_mle_gradient : NULL;
-	int maxit = 500;
-
-	err = BFGS_max(s->coeff, s->ncoeff, maxit, s->tol, 
-		       &s->fncount, &s->grcount, 
-		       get_mle_ll, C_LOGLIK, gradfun, s,
-		       NULL, s->opt, s->prn);
-	if (!err && (s->opt & (OPT_H | OPT_R))) {
-	    /* doing Hessian or QML covariance matrix */
-	    s->Hinv = numerical_hessian(s->coeff, s->ncoeff, 
-					get_mle_ll, s, &err);
+	if (analytic_mode(s)) {
+	    gradfunc = get_mle_gradient;
+	}
+	if (s->hesscall != NULL) {
+	    hessfunc = get_mle_hessian;
+	    s->flags |= NL_AHESS;
 	}
     }
 
+    if (!err && use_newton) {
+	double crittol = 1.0e-7;
+	double gradtol = 1.0e-7;
+
+	maxit = 100;
+	s->flags |= NL_NEWTON;
+	err = newton_raphson_max(s->coeff, s->ncoeff, maxit, 
+				 crittol, gradtol, &s->fncount, 
+				 C_LOGLIK, get_mle_ll, 
+				 gradfunc, hessfunc, s,
+				 s->opt, s->prn);
+    } else if (!err) {
+	maxit = 500;
+	err = BFGS_max(s->coeff, s->ncoeff, maxit, s->tol, 
+		       &s->fncount, &s->grcount, 
+		       get_mle_ll, C_LOGLIK, gradfunc, s,
+		       NULL, s->opt, s->prn);
+    }
+
+    if (!err && (s->opt & (OPT_H | OPT_R))) {
+	/* doing Hessian or QML covariance matrix */
+	if (hessfunc != NULL) {
+	    s->Hinv = mle_hessian_inverse(s, &err);
+	} else if (analytic_mode(s)) {
+	    s->Hinv = hessian_inverse_from_score(s->coeff, s->ncoeff, 
+						 gradfunc, get_mle_ll,
+						 s, &err);
+	} else {
+	    s->Hinv = numerical_hessian_inverse(s->coeff, s->ncoeff, 
+						get_mle_ll, s, &err);
+	}
+
+	if (err) {
+	    /* try dropping back to OPG */
+	    s->opt &= ~OPT_H;
+	    s->opt &= ~OPT_R;
+	    err = 0;
+	}
+    }	
+
     return err;    
 }
 
@@ -2345,10 +2562,8 @@ static int lm_approximate (nlspec *spec, PRN *prn)
 /**
  * nlspec_add_param_with_deriv:
  * @spec: pointer to nls specification.
- * @dstr: string specifying a derivative with respect to a
+ * @s: string specifying a derivative with respect to a
  *   parameter of the regression function.
- * @Z: data array.
- * @pdinfo: information on dataset.
  *
  * Adds an analytical derivative to @spec.  This pointer must
  * have previously been obtained by a call to nlspec_new().
@@ -2363,11 +2578,9 @@ static int lm_approximate (nlspec *spec, PRN *prn)
  * Returns: 0 on success, non-zero error code on error.
  */
 
-int 
-nlspec_add_param_with_deriv (nlspec *spec, const char *dstr,
-			     const double **Z, const DATAINFO *pdinfo)
+int nlspec_add_param_with_deriv (nlspec *spec, const char *s)
 {
-    const char *p = dstr;
+    const char *p = s;
     char *name = NULL;
     char *deriv = NULL;
     int err = 0;
@@ -2384,7 +2597,7 @@ nlspec_add_param_with_deriv (nlspec *spec, const char *dstr,
 
     err = equation_get_lhs_and_rhs(p, &name, &deriv);
     if (err) {
-	fprintf(stderr, "parse error in deriv string: '%s'\n", dstr);
+	fprintf(stderr, "parse error in deriv string: '%s'\n", s);
 	return E_PARSE;
     }
 
@@ -2407,6 +2620,32 @@ nlspec_add_param_with_deriv (nlspec *spec, const char *dstr,
     return err;
 }
 
+static int nlspec_add_hessian (nlspec *spec, const char *hesscall)
+{
+    int err;
+
+    if (spec->ci != MLE) {
+	/* we only do this for MLE */
+	return E_TYPES;
+    }
+
+    if (*spec->hname != '\0') {
+	/* Hessian already added */
+	return E_DATA;
+    }
+
+    err = optimizer_get_matrix_name(hesscall, spec->hname);
+
+    if (!err) {
+	spec->hesscall = gretl_strdup(hesscall);
+	if (spec->hesscall == NULL) {
+	    err = E_ALLOC;
+	}
+    }
+
+    return err;
+}
+
 static int screen_bad_aux (const char *line, const DATAINFO *pdinfo)
 {
     int n = gretl_namechar_spn(line);
@@ -2571,10 +2810,13 @@ void nlspec_set_t1_t2 (nlspec *spec, int t1, int t2)
     }
 }
 
+#define parnames_line(s) (!strncmp(s, "param_names ", 12))
+
 #define param_line(s) (!strncmp(s, "deriv ", 6) || \
                        !strncmp(s, "params ", 7) || \
                        !strncmp(s, "orthog ", 7) || \
-                       !strncmp(s, "weights ", 8))
+                       !strncmp(s, "weights ", 8) || \
+                       !strncmp(s, "hessian ", 8))
 
 #define cmd_start(s) (!strncmp(s, "nls ", 4) || \
                       !strncmp(s, "mle ", 4) || \
@@ -2630,7 +2872,7 @@ int nl_parse_line (int ci, const char *line, const double **Z,
 				       "line and analytical derivatives"));
 		    err = E_PARSE;
 		} else {
-		    err = nlspec_add_param_with_deriv(s, line, Z, pdinfo);
+		    err = nlspec_add_param_with_deriv(s, line);
 		}
 	    } else if (!strncmp(line, "params", 6)) {
 		if (numeric_mode(s)) {
@@ -2644,8 +2886,12 @@ int nl_parse_line (int ci, const char *line, const double **Z,
 		err = nlspec_add_orthcond(s, line + 6, Z, pdinfo);
 	    } else if (!strncmp(line, "weights", 7)) {
 		err = nlspec_add_weights(s, line + 7);
+	    } else if (!strncmp(line, "hessian", 7)) {
+		err = nlspec_add_hessian(s, line + 8);
 	    }
 	}
+    } else if (parnames_line(line)) {
+	err = nlspec_add_param_names(s, line + 12);
     } else {
 	err = nlspec_add_aux(s, line, pdinfo);
     }
@@ -2905,7 +3151,7 @@ static MODEL real_nl_model (nlspec *spec, double ***pZ, DATAINFO *pdinfo,
 	}
     }
 
-    if (!(spec->opt & OPT_Q)) {
+    if (!(spec->opt & OPT_Q) && !(spec->flags & NL_NEWTON)) {
 	pprintf(prn, _("Tolerance = %g\n"), spec->tol);
     }
 
@@ -3045,6 +3291,9 @@ nlspec *nlspec_new (int ci, const DATAINFO *pdinfo)
     spec->ngenrs = 0;
     spec->generr = 0;
 
+    spec->hgen = NULL;
+    spec->hesscall = NULL;
+
     spec->fvec = NULL;
     spec->jac = NULL;
     
@@ -3061,6 +3310,8 @@ nlspec *nlspec_new (int ci, const DATAINFO *pdinfo)
     spec->dv = 0;
     spec->lhtype = GRETL_TYPE_NONE;
     *spec->lhname = '\0';
+    *spec->hname = '\0';
+    spec->parnames = NULL;
     spec->lhv = 0;
     spec->lvec = NULL;
 
