@@ -1058,12 +1058,39 @@ real_get_obj_list (void *p, GretlObjType type, int idx, int *err)
     return list;
 }
 
+static char *
+real_get_obj_string (void *p, GretlObjType type, int idx, 
+		     const DATAINFO *pdinfo, int *err)
+{
+    char *str = NULL;
+
+    if (idx <= 0) {
+	*err = 1;
+	return str;
+    }
+
+    if (type == GRETL_OBJ_EQN && idx == M_COMMAND) {
+	MODEL *pmod = (MODEL *) p;
+
+	str = gretl_strdup(gretl_command_word(pmod->ci));
+    } else if (type == GRETL_OBJ_EQN && idx == M_DEPVAR) {
+	const char *s = gretl_model_get_depvar_name((MODEL *) p, 
+						    pdinfo);
+
+	str = gretl_strdup(s);
+    } else {
+	*err = E_BADSTAT;
+    }
+
+    return str;
+}
+
 static stacker genr_model;
 
-void set_genr_model (MODEL *pmod)
+void set_genr_model (void *ptr, GretlObjType type)
 {
-    genr_model.type = GRETL_OBJ_EQN;
-    genr_model.ptr = pmod;
+    genr_model.ptr = ptr;
+    genr_model.type = type;
 }
 
 void unset_genr_model (void)
@@ -1148,6 +1175,22 @@ int *saved_object_get_list (const char *oname, int idx, int *err)
     return ret;
 }
 
+char *saved_object_get_string (const char *oname, int idx, 
+			       const DATAINFO *pdinfo, int *err)
+{
+    char *ret = NULL;
+    stacker *smatch;
+
+    smatch = find_smatch(oname);
+
+    if (smatch != NULL) {
+	ret = real_get_obj_string(smatch->ptr, smatch->type, idx, 
+				  pdinfo, err);
+    }
+
+    return ret;
+}
+
 double saved_object_get_scalar (const char *oname, int idx, 
 				double ***pZ, DATAINFO *pdinfo,
 				int *err)
@@ -1237,6 +1280,12 @@ saved_object_build_matrix (const char *oname, int idx,
 	*err = E_DATA;
     } else if (idx == M_MNLPROBS && smatch->type == GRETL_OBJ_EQN) {
 	M = mn_logit_probabilities(smatch->ptr, Z, pdinfo, err);
+    } else if (idx == M_EC && smatch->type == GRETL_OBJ_VAR) {
+	M = VECM_get_EC_matrix(smatch->ptr, Z, pdinfo, err);
+    } else if (idx == M_VMA && smatch->type == GRETL_OBJ_VAR) {
+	M = gretl_VAR_get_vma_matrix(smatch->ptr, pdinfo, err);
+    } else if (idx == M_FEVD && smatch->type == GRETL_OBJ_VAR) {
+	M = gretl_VAR_get_full_FEVD_matrix(smatch->ptr, pdinfo, err);
     } else {
 	*err = E_BADSTAT;
     }
@@ -1244,6 +1293,24 @@ saved_object_build_matrix (const char *oname, int idx,
     return M;
 }
 
+gretl_matrix *
+last_model_get_irf_matrix (int targ, int shock, double alpha, 
+			   const double **Z, const DATAINFO *pdinfo,
+			   int *err)
+{
+    stacker *smatch = find_smatch(NULL);
+    gretl_matrix *M = NULL;
+
+    if (smatch == NULL || smatch->type != GRETL_OBJ_VAR) {
+	*err = E_BADSTAT;
+    } else {
+	M = gretl_VAR_get_impulse_response(smatch->ptr, targ, shock, 0,
+					   alpha, Z, pdinfo, err);
+    }
+
+    return M;
+}
+
 static int namechar_spn_with_space (const char *s)
 {
     const char *ok = "abcdefghijklmnopqrstuvwxyz"
@@ -1443,14 +1510,7 @@ int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo,
     }
 
     if (type == GRETL_OBJ_EQN) {
-	MODEL *pmod = ptr;
-
-	if ((pmod->ci == LOGIT || pmod->ci == PROBIT) &&
-	    gretl_model_get_int(pmod, "ordered")) {
-	    err = E_NOTIMP;
-	} else if (!exact_fit_check(pmod, prn)) {
-	    err = model_error_dist(ptr, pZ, pdinfo, opt, prn);
-	}
+	err = model_error_dist(ptr, pZ, pdinfo, opt, prn);
     } else if (type == GRETL_OBJ_SYS) {
 	err = system_normality_test(ptr, prn);
     } else if (type == GRETL_OBJ_VAR) {
