@@ -43,6 +43,7 @@ typedef enum {
     GRETL_OBJ_TEXT,
     GRETL_OBJ_MATRIX,
     GRETL_OBJ_SCALARS,
+    GRETL_OBJ_BUNDLE,
     GRETL_OBJ_ANY,
     GRETL_OBJ_MAX
 } GretlObjType;
@@ -132,8 +133,16 @@ saved_object_build_matrix (const char *oname, int idx,
 			   const double **Z, const DATAINFO *pdinfo,
 			   int *err);
 
+gretl_matrix *
+last_model_get_irf_matrix (int targ, int shock, double alpha, 
+			   const double **Z, const DATAINFO *pdinfo,
+			   int *err);
+
 int *saved_object_get_list (const char *oname, int idx, int *err);
 
+char *saved_object_get_string (const char *oname, int idx, 
+			       const DATAINFO *pdinfo, int *err);
+
 int gretl_object_rename (void *p, GretlObjType type, const char *oname);
 
 int gretl_object_compose_name (void *p, GretlObjType type);
@@ -152,7 +161,7 @@ int last_model_test_ok (int ci, gretlopt opt, const DATAINFO *pdinfo,
 int last_model_test_uhat (double ***pZ, DATAINFO *pdinfo, 
 			  gretlopt opt, PRN *prn);
 
-void set_genr_model (MODEL *pmod);
+void set_genr_model (void *ptr, GretlObjType type);
 
 void unset_genr_model (void);
 
