@@ -52,6 +52,7 @@
 			 c == RESET ||		\
 			 c == RESTRICT ||	\
 			 c == RMPLOT ||		\
+			 c == SYSTEM ||		\
 			 c == VAR ||		\
 			 c == VECM ||		\
 			 c == XCORRGM)
@@ -116,6 +117,8 @@ struct gretl_option gretl_opts[] = {
     { BXPLOT,   OPT_O, "notches", 0 },
     { BXPLOT,   OPT_U, "output", 2 },
     { CHOW,     OPT_D, "dummy", 0 },
+    { CLEAR,    OPT_D, "dataset", 0 },
+    { CLEAR,    OPT_O, "other", 0 },
     { COINT,    OPT_E, "test-down", 0 },
     { COINT,    OPT_N, "nc", 0 },
     { COINT,    OPT_R, "ctt", 0 },     
@@ -125,8 +128,11 @@ struct gretl_option gretl_opts[] = {
     { COINT2,   OPT_D, "seasonals", 0 },
     { COINT2,   OPT_N, "nc", 0 },
     { COINT2,   OPT_R, "rc", 0 },
+    { COINT2,   OPT_C, "uc", 0 },
     { COINT2,   OPT_T, "ct", 0 },
+    { COINT2,   OPT_S, "silent", 0 },
     { COINT2,   OPT_V, "verbose", 0 },
+    { COINT2,   OPT_Y, "asy", 0 },
     { CORR,     OPT_K, "kendall", 0 },
     { CORR,     OPT_S, "spearman", 0 },
     { CORR,     OPT_U, "uniform", 0 },
@@ -167,11 +173,13 @@ struct gretl_option gretl_opts[] = {
 #if 0 /* not yet */
     { TABPRINT, OPT_F, "filename", 0 }, /* backward compatibility */
 #endif
+    { EQUATION, OPT_M, "multi", 0 },
     { ESTIMATE, OPT_I, "iterate", 0 },
     { ESTIMATE, OPT_M, "geomean", 0 },
     { ESTIMATE, OPT_N, "no-df-corr", 0 },
     { ESTIMATE, OPT_V, "verbose", 0 },
     { FCAST,    OPT_D, "dynamic", 0 },
+    { FCAST,    OPT_M, "mean-y", 0 },
     { FCAST,    OPT_N, "no-stats", 0 },
     { FCAST,    OPT_S, "static", 0 },
     { FCAST,    OPT_R, "rolling", 0 },
@@ -205,6 +213,7 @@ struct gretl_option gretl_opts[] = {
     { GNUPLOT,  OPT_L, "loess-fit", 0 },
     { GNUPLOT,  OPT_Q, "quadratic-fit", 0 },
     { GNUPLOT,  OPT_N, "linear-fit", 0 },
+    { GNUPLOT,  OPT_B, "cubic-fit", 0 },
     { GNUPLOT,  OPT_M, "with-impulses", 0 },
     { GNUPLOT,  OPT_S, "suppress-fitted", 0 },
     { GNUPLOT,  OPT_T, "time-series", 0 },
@@ -354,6 +363,7 @@ struct gretl_option gretl_opts[] = {
     { RESTRICT, OPT_L, "lbfgs", 0 },
     { RESTRICT, OPT_N, "no-scaling", 0 },
     { RESTRICT, OPT_S, "silent", 0 },
+    { RESTRICT, OPT_W, "wald", 0 },
     { RUNS,     OPT_D, "difference", 0 },
     { RUNS,     OPT_E, "equal", 0 },
     { SCATTERS, OPT_L, "with-lines", 0 },
@@ -369,6 +379,7 @@ struct gretl_option gretl_opts[] = {
     { SETOBS,   OPT_T, "time-series", 0 },
     { SETOBS,   OPT_X, "cross-section", 0 },
     { SETOBS,   OPT_N, "special-time-series", 0 },
+    { SETOBS,   OPT_M, "labels", 2 },
     { SMPL,     OPT_B, "balanced", 0 },
     { SMPL,     OPT_C, "contiguous", 0 },
     { SMPL,     OPT_F, "full", 0 },
@@ -390,13 +401,18 @@ struct gretl_option gretl_opts[] = {
     { STORE,    OPT_T, "traditional", 0 },
     { STORE,    OPT_Z, "gzipped", 0 },
     { STORE,    OPT_X, "omit-obs", 0 },
+    { STORE,    OPT_E, "comment", 2 },
     { SUMMARY,  OPT_B, "by", 2 },
+    { SUMMARY,  OPT_M, "matrix", 2 },
     { SUMMARY,  OPT_S, "simple", 0 },
     { SYSTEM,   OPT_I, "iterate", 0 },
     { SYSTEM,   OPT_V, "verbose", 0 },
 #if 0
     { TABPRINT, OPT_F, "filename", 0 }, /* backward compatibility */
 #endif
+    { TOBIT,    OPT_L, "llimit", 2 },
+    { TOBIT,    OPT_M, "rlimit", 2 },
+    { TOBIT,    OPT_R, "robust", 0 },
     { TOBIT,    OPT_V, "verbose", 0 },
     { VAR,      OPT_D, "seasonals", 0 },
     { VAR,      OPT_F, "variance-decomp", 0 },
@@ -407,7 +423,8 @@ struct gretl_option gretl_opts[] = {
     { VAR,      OPT_T, "trend", 0 }, 
     { VAR,      OPT_S, "lags", 2 },
     { VAR,      OPT_X, "silent", 0 },
-    { VARLIST,  OPT_S, "scalars", 0 },
+    { VARLIST,  OPT_A, "accessors", 0 },
+    { VARLIST,  OPT_S, "scalars", 0 },    
     { VECM,     OPT_A, "crt", 0 },
     { VECM,     OPT_D, "seasonals", 0 },
     { VECM,     OPT_F, "variance-decomp", 0 },
@@ -425,21 +442,13 @@ struct gretl_option gretl_opts[] = {
     { 0,        0L,    NULL, 0 }
 };
 
-static int compare_strings (const void *a, const void *b)
-{
-    const char **sa = (const char **) a;
-    const char **sb = (const char **) b;
-     
-    return strcmp(*sa, *sb);
-}
-
 /* this function is used in compiling the gretl reference
    manual */
 
 char **get_all_option_strings (int *pn)
 {
     char **optstrs;
-    int i, j, m, n = 0;
+    int i, n = 0;
 
     for (i=0; gretl_opts[i].ci != 0; i++) {
 	n++;
@@ -459,23 +468,8 @@ char **get_all_option_strings (int *pn)
     }
 
     if (optstrs != NULL) {
-	qsort(optstrs, n, sizeof *optstrs, compare_strings);
-	m = n;
-	for (i=0; i<m-1; i++) {
-	    if (!strcmp(optstrs[i], optstrs[i+1])) {
-		free(optstrs[i+1]);
-		for (j=i+1; j<m-1; j++) {
-		    optstrs[j] = optstrs[j+1];
-		}
-		optstrs[m-1] = NULL;
-		i--;
-		m--;
-	    }
-	}
-	if (m < n) {
-	    optstrs = realloc(optstrs, m * sizeof *optstrs);
-	}
-	*pn = m;
+	strings_array_sort(&optstrs, &n, OPT_U);
+	*pn = n;
     }
     
     return optstrs;
@@ -800,7 +794,7 @@ static int real_push_option_param (int ci, gretlopt opt, char *val,
 	    "err = %d\n", op->val, n_parms, err);
 #endif
 
-    return 0;
+    return err;
 }
 
 /**
@@ -1111,6 +1105,8 @@ static int end_block_ci (const char *s)
 	    return KALMAN;
 	} else if (!strcmp(word, "foreign")) {
 	    return FOREIGN;
+	} else if (!strcmp(word, "system")) {
+	    return SYSTEM;
 	}
     }
 
