@@ -50,6 +50,9 @@ GPT_SPEC *plotspec_new (void)
     spec->labels = NULL;
     spec->n_labels = 0;
 
+    spec->arrows = NULL;
+    spec->n_arrows = 0;
+
     spec->literal = NULL;
     spec->n_literal = 0;
 
@@ -78,6 +81,7 @@ GPT_SPEC *plotspec_new (void)
 
     spec->b_ols = NULL;
     spec->b_quad = NULL;
+    spec->b_cub = NULL;
     spec->b_inv = NULL;
 
     spec->code = PLOT_REGULAR;
@@ -128,6 +132,10 @@ void plotspec_destroy (GPT_SPEC *spec)
 	free(spec->labels);
     }    
 
+    if (spec->arrows != NULL) {
+	free(spec->arrows);
+    } 
+  
     if (spec->data != NULL) {
 	free(spec->data);
     }
@@ -158,6 +166,7 @@ void plotspec_destroy (GPT_SPEC *spec)
 
     gretl_matrix_free(spec->b_ols);
     gretl_matrix_free(spec->b_quad);
+    gretl_matrix_free(spec->b_cub);
     gretl_matrix_free(spec->b_inv);
 
     free(spec);
@@ -529,6 +538,89 @@ GPT_LABEL *plotspec_clone_labels (GPT_SPEC *spec, int *err)
     return labels;
 }
 
+int plotspec_add_arrow (GPT_SPEC *spec)
+{
+    GPT_ARROW *arrows;
+    int n = spec->n_arrows;
+
+    arrows = realloc(spec->arrows, (n + 1) * sizeof *arrows);
+    if (arrows == NULL) {
+	return E_ALLOC;
+    }
+
+    spec->arrows = arrows;
+    spec->n_arrows += 1;
+    arrows[n].x0 = 0;
+    arrows[n].y0 = 0;
+    arrows[n].x1 = 0;
+    arrows[n].y1 = 0;
+    arrows[n].flags = 0;
+
+    return 0;
+}
+
+static void copy_arrow_content (GPT_ARROW *targ, GPT_ARROW *src)
+{
+    targ->x0 = src->x0;
+    targ->y0 = src->y0;
+    targ->x1 = src->x1;
+    targ->y1 = src->y1;
+    targ->flags = src->flags;
+}
+
+int plotspec_delete_arrow (GPT_SPEC *spec, int i)
+{
+    GPT_ARROW *arrows = spec->arrows;
+    int j, n = spec->n_arrows;
+    int err = 0;
+
+    if (i < 0 || i >= n) {
+	return E_DATA;
+    }
+
+    for (j=i; j<n-1; j++) {
+	copy_arrow_content(&arrows[j], &arrows[j+1]);
+    }
+
+    spec->n_arrows -= 1;
+
+    if (spec->n_arrows == 0) {
+	free(spec->arrows);
+	spec->arrows = NULL;
+    } else {
+	arrows = realloc(spec->arrows, (n - 1) * sizeof *arrows);
+	if (arrows == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    spec->arrows = arrows;
+	}
+    }
+
+    return err;
+}
+
+GPT_ARROW *plotspec_clone_arrows (GPT_SPEC *spec, int *err)
+{
+    GPT_ARROW *arrows = NULL;
+    int i;
+
+    if (spec->n_arrows == 0) {
+	/* no-op */
+	return NULL;
+    }
+
+    arrows = malloc(spec->n_arrows * sizeof *arrows);
+    if (arrows == NULL) {
+	*err = E_ALLOC;
+    } else {
+	for (i=0; i<spec->n_arrows; i++) {
+	    copy_arrow_content(&arrows[i], &spec->arrows[i]);
+	}
+    }
+
+    return arrows;
+}
+
 static char *escape_quotes (const char *s)
 {
     if (strchr(s, '"') == NULL) {
@@ -577,8 +669,7 @@ const char *gp_justification_string (int j)
     }
 }
 
-static void 
-print_plot_labelspec (const GPT_LABEL *lbl, FILE *fp)
+static void print_plot_labelspec (const GPT_LABEL *lbl, FILE *fp)
 {
     char *label = escape_quotes(lbl->text);
 
@@ -598,6 +689,19 @@ print_plot_labelspec (const GPT_LABEL *lbl, FILE *fp)
     }
 }
 
+static void print_plot_arrow (const GPT_ARROW *arrow, FILE *fp)
+{
+    gretl_push_c_numeric_locale();
+    fprintf(fp, "set arrow from %g,%g to %g,%g %s", 
+	    arrow->x0, arrow->y0, arrow->x1, arrow->y1,
+	    (arrow->flags & GP_ARROW_HEAD)? "head" : "nohead");
+    if (arrow->flags & GP_ARROW_DOTS) {
+	fputs(" lt 0", fp);
+    }
+    fputc('\n', fp);
+    gretl_pop_c_numeric_locale();
+}
+
 static int print_polar_labels (const GPT_SPEC *spec, FILE *fp)
 {
     double x, y;
@@ -623,9 +727,26 @@ static int print_polar_labels (const GPT_SPEC *spec, FILE *fp)
     return 0;	
 }
 
+static int usable_obs (const double *x, const double *y0,
+		       const double *y1, int t,
+		       const double **py)
+{
+    *py = y0;
+
+    if (!na(x[t]) && !na(y0[t])) {
+	return 1;
+    } else if (!na(x[t]) && y1 != NULL && !na(y1[t])) {
+	*py = y1;
+	return 1;
+    } else {
+	return 0;
+    }
+}
+
 static int print_data_labels (const GPT_SPEC *spec, FILE *fp)
 {
-    const double *x, *y;
+    const double *x, *y, *y0;
+    const double *y1 = NULL;
     double xrange, yrange;
     double yoff;
     int t;
@@ -641,7 +762,11 @@ static int print_data_labels (const GPT_SPEC *spec, FILE *fp)
     }
 
     x = spec->data;
-    y = x + spec->nobs;
+    y0 = x + spec->nobs;
+
+    if (spec->code == PLOT_FACTORIZED) {
+	y1 = y0 + spec->nobs;
+    }
 
     xrange = spec->range[GP_X_RANGE][1] - spec->range[GP_X_RANGE][0];
     yrange = spec->range[GP_Y_RANGE][1] - spec->range[GP_Y_RANGE][0];
@@ -651,7 +776,7 @@ static int print_data_labels (const GPT_SPEC *spec, FILE *fp)
 	double xmin = 1.0e+16, xmax = -1.0e+16;
 
 	for (t=0; t<spec->nobs; t++) {
-	    if (!na(x[t]) && !na(y[t])) {
+	    if (usable_obs(x, y0, y1, t, &y)) {
 		if (yrange == 0.0) {
 		    if (y[t] < ymin) {
 			ymin = y[t];
@@ -690,7 +815,7 @@ static int print_data_labels (const GPT_SPEC *spec, FILE *fp)
 	    continue;
 	}
 
-	if (!na(x[t]) && !na(y[t])) {
+	if (usable_obs(x, y0, y1, t, &y)) {
 	    if (x[t] > .90 * xrange) {
 		xoff = -.02 * xrange;
 	    }
@@ -710,6 +835,8 @@ void print_auto_fit_string (FitType fit, FILE *fp)
 	fputs("# plot includes automatic fit: OLS\n", fp);
     } else if (fit == PLOT_FIT_QUADRATIC) {
 	fputs("# plot includes automatic fit: quadratic\n", fp);
+    } else if (fit == PLOT_FIT_CUBIC) {
+	fputs("# plot includes automatic fit: cubic\n", fp);
     } else if (fit == PLOT_FIT_INVERSE) {
 	fputs("# plot includes automatic fit: inverse\n", fp);
     } else if (fit == PLOT_FIT_LOESS) {
@@ -886,6 +1013,7 @@ static int print_point_type (GPT_LINE *line)
 
 #define show_fit(s) (s->fit == PLOT_FIT_OLS || \
                      s->fit == PLOT_FIT_QUADRATIC || \
+		     s->fit == PLOT_FIT_CUBIC ||     \
                      s->fit == PLOT_FIT_INVERSE || \
                      s->fit == PLOT_FIT_LOESS)
 
@@ -937,6 +1065,10 @@ int plotspec_print (GPT_SPEC *spec, FILE *fp)
 	}
     }
 
+    for (i=0; i<spec->n_arrows; i++) {
+	print_plot_arrow(&spec->arrows[i], fp);
+    }    
+
     if (spec->flags & GPT_XZEROAXIS) {
 	fputs("set xzeroaxis\n", fp);
     }
@@ -1298,31 +1430,50 @@ static int set_loess_fit (GPT_SPEC *spec, int d, double q, gretl_matrix *x,
 }
 
 static void set_plotfit_formula (char *formula, FitType f, const double *b,
-				 double x0, double pd)
+				 double t0, double pd)
 {
     gretl_push_c_numeric_locale();
 
+    /* if t0 is not NA that indicates there we're doing a fitted 
+       plot against time, and we have to transform the
+       coefficients
+    */
+
     if (f == PLOT_FIT_OLS) {
-	if (!na(x0)) {
+	if (!na(t0)) {
 	    double c = b[1] * pd;
 
-	    sprintf(formula, "%.10g + %.10g*x", b[0] - c*x0, c);
+	    sprintf(formula, "%.10g + %.10g*x", b[0] - c*t0, c);
 	} else {
 	    sprintf(formula, "%.10g + %.10g*x", b[0], b[1]);
 	}
     } else if (f == PLOT_FIT_QUADRATIC) {
-	if (!na(x0)) {
+	if (!na(t0)) {
 	    double c = b[1] * pd;
 	    double g = b[2] * pd * pd;
 
 	    sprintf(formula, "%.10g + %.10g*x + %.10g*x**2", 
-		    b[0] - c*x0 + g*x0*x0, c - 2*g*x0, g);
+		    b[0] - c*t0 + g*t0*t0, c - 2*g*t0, g);
 	} else {
 	    sprintf(formula, "%.10g + %.10g*x + %.10g*x**2", b[0], b[1], b[2]);
+	}
+    } else if (f == PLOT_FIT_CUBIC) {	
+	if (!na(t0)) {
+	    double c = b[1] * pd;
+	    double g = b[2] * pd * pd;
+	    double h = b[3] * pd * pd * pd;
+
+	    sprintf(formula, "%.13g + %.10g*x + %.10g*x**2 + %.10g*x**3", 
+		    b[0] - c*t0 + g*t0*t0 - h*t0*t0*t0, 
+		    c - 2*g*t0 + 3*h*t0*t0, 
+		    g - 3*h*t0, h);
+	} else {
+	    sprintf(formula, "%.10g + %.10g*x + %.10g*x**2 + %.10g*x**3", 
+		    b[0], b[1], b[2], b[3]);
 	}	
     } else if (f == PLOT_FIT_INVERSE) {
-	if (!na(x0)) {
-	    double c = x0 * pd;
+	if (!na(t0)) {
+	    double c = t0 * pd;
 
 	    sprintf(formula, "%.10g + %.10g/(%g*x - %.10g)", b[0], b[1], 
 		    pd, c);
@@ -1340,6 +1491,8 @@ void set_plotfit_line (char *title, char *formula,
 {
     char xc = (na(x0)) ? 'X' : 't';
 
+    /* first compose the key string for the fitted line */
+
     if (f == PLOT_FIT_OLS) {
 	sprintf(title, "Y = %#.3g %c %#.3g%c", b[0],
 		(b[1] > 0)? '+' : '-', fabs(b[1]), xc);
@@ -1347,11 +1500,18 @@ void set_plotfit_line (char *title, char *formula,
 	sprintf(title, "Y = %#.3g %c %#.3g%c %c %#.3g%c^2", b[0],
 		(b[1] > 0)? '+' : '-', fabs(b[1]), xc,
 		(b[2] > 0)? '+' : '-', fabs(b[2]), xc);
+    } else if (f == PLOT_FIT_CUBIC) {
+	sprintf(title, "Y = %#.3g %c %#.3g%c %c %#.3g%c^2 %c %#.3g%c^3", 
+		b[0], (b[1] > 0)? '+' : '-', fabs(b[1]), xc,
+		(b[2] > 0)? '+' : '-', fabs(b[2]), xc,
+		(b[3] > 0)? '+' : '-', fabs(b[3]), xc);
     } else if (f == PLOT_FIT_INVERSE) {
 	sprintf(title, "Y = %#.3g %c %#.3g(1/%c)", b[0],
 		(b[1] > 0)? '+' : '-', fabs(b[1]), xc);
     }
 
+    /* then set the formula itself */
+
     set_plotfit_formula(formula, f, b, x0, pd);
 }
 
@@ -1366,6 +1526,8 @@ static void plotspec_set_fitted_line (GPT_SPEC *spec, FitType f, double x0)
 	b = spec->b_ols->val;
     } else if (f == PLOT_FIT_QUADRATIC) {
 	b = spec->b_quad->val;
+    } else if (f == PLOT_FIT_CUBIC) {
+	b = spec->b_cub->val;
     } else if (f == PLOT_FIT_INVERSE) {
 	b = spec->b_inv->val;
     } else {
@@ -1380,6 +1542,10 @@ static void plotspec_set_fitted_line (GPT_SPEC *spec, FitType f, double x0)
     spec->lines[1].ncols = 0;
 }
 
+#define polyfit(f) (f == PLOT_FIT_OLS || \
+		    f == PLOT_FIT_QUADRATIC ||	\
+		    f == PLOT_FIT_CUBIC)
+
 int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 {
     gretl_matrix *y = NULL;
@@ -1395,7 +1561,7 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
     int i, t, k;
     int err = 0;
 
-    if ((spec->flags & GPT_TS) && (f == PLOT_FIT_OLS || f == PLOT_FIT_QUADRATIC)) {
+    if ((spec->flags & GPT_TS) && polyfit(f)) {
 	if (spec->pd == 1 || spec->pd == 4 || spec->pd == 12) {
 	    x0 = px[0];
 	}
@@ -1403,6 +1569,7 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 
     if ((f == PLOT_FIT_OLS && spec->b_ols != NULL) ||
 	(f == PLOT_FIT_QUADRATIC && spec->b_quad != NULL) ||
+	(f == PLOT_FIT_CUBIC && spec->b_cub != NULL) ||
 	(f == PLOT_FIT_INVERSE && spec->b_inv != NULL)) {
 	/* just activate existing setup */
 	plotspec_set_fitted_line(spec, f, x0);
@@ -1413,6 +1580,8 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 	k = 2;
     } else if (f == PLOT_FIT_QUADRATIC) {
 	k = 3;
+    } else if (f == PLOT_FIT_CUBIC) {
+	k = 4;
     } else if (f == PLOT_FIT_LOESS) {
 	k = 1;
     } else {
@@ -1455,9 +1624,12 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 		} else {
 		    gretl_matrix_set(X, i, 1, xt);
 		}
-		if (f == PLOT_FIT_QUADRATIC) {
+		if (f == PLOT_FIT_QUADRATIC || f == PLOT_FIT_CUBIC) {
 		    gretl_matrix_set(X, i, 2, xt * xt);
 		}
+		if (f == PLOT_FIT_CUBIC) {
+		    gretl_matrix_set(X, i, 3, xt * xt * xt);
+		}		
 	    }
 	    i++;
 	}
@@ -1483,6 +1655,9 @@ int plotspec_add_fit (GPT_SPEC *spec, FitType f)
 	} else if (f == PLOT_FIT_QUADRATIC) {
 	    spec->b_quad = b;
 	    b = NULL;
+	} else if (f == PLOT_FIT_CUBIC) {
+	    spec->b_cub = b;
+	    b = NULL;
 	} else if (f == PLOT_FIT_INVERSE) {
 	    spec->b_inv = b;
 	    b = NULL;
