@@ -109,6 +109,16 @@ typedef struct {
     int just;
 } GPT_LABEL;
 
+typedef enum {
+    GP_ARROW_HEAD = 1 << 0,
+    GP_ARROW_DOTS = 1 << 1
+} gp_arrow_flags;
+
+typedef struct {
+    double x0, y0, x1, y1;
+    int flags;
+} GPT_ARROW;
+
 /* "global" information concerning a gnuplot graph specification */
 
 struct GPT_SPEC_ {
@@ -146,9 +156,12 @@ struct GPT_SPEC_ {
     int n_markers;             /* number of such markers */
     GPT_LABEL *labels;         /* textual labels written onto graph */
     int n_labels;              /* number of the above */
+    GPT_ARROW *arrows;         /* arrows drawn onto graph */
+    int n_arrows;              /* number of the above */
     int *reglist;              /* regression list for X-Y plot with fitted line */
     gretl_matrix *b_ols;       /* coeffs for linear fit */
     gretl_matrix *b_quad;      /* coeffs for quadratic fit */
+    gretl_matrix *b_cub;       /* coeffs for cubic fit */
     gretl_matrix *b_inv;       /* coeffs for inverse fit */
     char *labeled;             /* for GUI use */
     void *ptr;                 /* for GUI use */
@@ -166,9 +179,15 @@ int plotspec_add_line (GPT_SPEC *spec);
 
 int plotspec_delete_line (GPT_SPEC *spec, int i);
 
+int plotspec_max_line_width (GPT_SPEC *spec);
+
 GPT_LINE *plotspec_clone_lines (GPT_SPEC *spec, int *err);
 
-int plotspec_max_line_width (GPT_SPEC *spec);
+int plotspec_add_arrow (GPT_SPEC *spec);
+
+int plotspec_delete_arrow (GPT_SPEC *spec, int i);
+
+GPT_ARROW *plotspec_clone_arrows (GPT_SPEC *spec, int *err);
 
 int plotspec_add_label (GPT_SPEC *spec);
 
