@@ -1101,7 +1101,7 @@ static int vmat_maxlen (VMatrix *vmat)
 void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 {
     register int i, j;
-    int nf, li2, p, k, m, idx, ij2;
+    int nf, li2, p, k, idx, ij2;
     int maxlen = 0;
     int fwidth = 14;
     int fields = 5;
@@ -1117,8 +1117,6 @@ void text_print_vmatrix (VMatrix *vmat, PRN *prn)
 	fwidth = 16;
     }
 
-    m = 1;
-
     for (i=0; i<=vmat->dim/fields; i++) {
 	nf = i * fields;
 	li2 = vmat->dim - nf;
@@ -2154,7 +2152,7 @@ static void rtf_print_row_spec (int ncols, int type, PRN *prn)
 #define prn_delimited(p) (prn_format(p) & (GRETL_FORMAT_CSV | GRETL_FORMAT_TAB))
 
 /**
- * print_data_sorted:
+ * print_data_in_columns:
  * @list: list of variables to print.
  * @obsvec: list of observation numbers (or %NULL)
  * @Z: data matrix.
@@ -2172,9 +2170,9 @@ static void rtf_print_row_spec (int ncols, int type, PRN *prn)
  * Returns: 0 on successful completion, non-zero code on error.
  */
 
-int print_data_sorted (const int *list, const int *obsvec, 
-		       const double **Z, const DATAINFO *pdinfo, 
-		       PRN *prn)
+int print_data_in_columns (const int *list, const int *obsvec, 
+			   const double **Z, const DATAINFO *pdinfo, 
+			   PRN *prn)
 {
     int delimited = prn_delimited(prn);
     int rtf = rtf_format(prn);
@@ -2186,25 +2184,22 @@ int print_data_sorted (const int *list, const int *obsvec,
     int colwidth = 0;
     int ncols = 0, obslen = 0;
     int gprec = 6;
-    int T, lmax;
-    int i, s, t;
+    int i, s, t, T;
 
     if (obsvec != NULL) {
 	T = obsvec[0];
-	lmax = 4;
     } else {
 	T = sample_size(pdinfo);
-	lmax = 8;
     }
 
-    /* must have a list of not more than lmax variables... */
-    if (list == NULL || list[0] < 1 || list[0] > lmax) {
+    /* must have a non-empty list of variables */
+    if (list == NULL || list[0] < 1) {
 	return E_DATA;
     }
 
     /* ...with no bad variable numbers */
     for (i=1; i<=list[0]; i++) {
-	if (list[i] >= pdinfo->v) {
+	if (list[i] < 0 || list[i] >= pdinfo->v) {
 	    return E_DATA;
 	}
     }
@@ -2461,14 +2456,18 @@ int text_print_fit_resid (const FITRESID *fr, const DATAINFO *pdinfo,
  * text_print_forecast:
  * @fr: pointer to structure containing forecasts.
  * @pdinfo: dataset information.
- * @opt: if includes %OPT_P, make a plot of the forecasts.
- * @prn: printing structure.
- *
- * Print the forecasts in @fr to @prn, and also plot the
- * forecasts if %OPT_P is given.  If %OPT_F is given in
- * addition to %OPT_P, use fill style for the confidence
- * bands when plotting a forecast with standard errors.
+ * @opt: if includes %OPT_P, make a plot of the forecasts;
+ * if includes %OPT_N, skip printing of the forecast
+ * evaluation statistics.
+ * @prn: printing struct.
  *
+ * Prints the forecasts in @fr to @prn, and also plots the
+ * forecasts if %OPT_P is given. If a plot is requested and
+ * @fr includes forecast standard errors, then the options
+ * %OPT_F or %OPT_L may be given to use "fill" style or lines,
+ * respectively, for the confidence bands (the default style
+ * being vertical bars per observation).
+ * 
  * Returns: 0 on success, non-zero error code on error.
  */
 
@@ -2478,6 +2477,7 @@ int text_print_forecast (const FITRESID *fr, DATAINFO *pdinfo,
     int do_errs = (fr->sderr != NULL);
     int pmax = fr->pmax;
     int errpmax = fr->pmax;
+    int quiet = (opt & OPT_Q);
     double *maxerr = NULL;
     double conf = 100 * (1 - fr->alpha);
     double tval = 0;
@@ -2490,7 +2490,7 @@ int text_print_forecast (const FITRESID *fr, DATAINFO *pdinfo,
 	}
     }
 
-    if (!(opt & OPT_Q)) {
+    if (!quiet) {
 	pputc(prn, '\n');
     }
 
@@ -2499,7 +2499,7 @@ int text_print_forecast (const FITRESID *fr, DATAINFO *pdinfo,
 
 	tval = (fr->asymp)? normal_critval(a2) : student_critval(fr->df, a2);
 
-	if (!(opt & OPT_Q)) {
+	if (!quiet) {
 	    if (fr->asymp) {
 		pprintf(prn, _(" For %g%% confidence intervals, z(%g) = %.2f\n"), 
 			conf, a2, tval);
@@ -2510,7 +2510,7 @@ int text_print_forecast (const FITRESID *fr, DATAINFO *pdinfo,
 	}
     }
 
-    if (!(opt & OPT_Q)) {
+    if (!quiet) {
 	pputc(prn, '\n');
     }
 
