@@ -605,7 +605,7 @@ double student_cdf_inverse (double df, double a)
  * @df: degrees of freedom.
  * @x: the cutoff point in the distribution.
  * 
- * Returns: the integral from 0 to @x of the chi-squared
+ * Returns: the integral from 0 to @x of the chi-square
  * distribution with @df degrees of freedom, or #NADBL
  * on failure.
  */
@@ -629,7 +629,7 @@ double chisq_cdf (int df, double x)
  * @df: degrees of freedom.
  * @x: the cutoff point in the distribution.
  * 
- * Returns: the integral from @x to infinity of the chi-squared
+ * Returns: the integral from @x to infinity of the chi-square
  * distribution with @df degrees of freedom, or #NADBL
  * on failure.
  */
@@ -1314,7 +1314,8 @@ double gamma_cdf (double s1, double s2, double x, int control)
 	shape = s1 / scale; 
     }
 
-    /* cephes expects inverse-scale, shape, I think >8-} */
+    /* for the cephes functions, the parameterization is
+       inverse-scale (or "rate"), shape */
 
     p = gdtr(1.0 / scale, shape, x);
     if (get_cephes_errno()) {
@@ -1760,6 +1761,36 @@ double GED_cdf_comp (double nu, double x)
     }
 }
 
+/**
+ * johansen_trace_pval:
+ * @N: the number of potentially cointegrated variables
+ * minus the cointegration rank under H0.
+ * @det: index of deterministic setup of model (0 to 4).
+ * @T: the sample size, or 0 for an asymptotic result.
+ * @tr: the trace statistic.
+ *
+ * Returns: the p-value of the trace statistic, computed
+ * via Doornik's gamma approximation, or #NADBL on failure.
+ */
+
+double johansen_trace_pval (int N, int det, int T, double tr)
+{
+    double (*pvfunc) (double, int, int, int);
+    void *handle;
+    double pv = NADBL;
+
+    pvfunc = get_plugin_function("trace_pvalue", &handle);
+
+    if (pvfunc == NULL) {
+	fputs(I_("Couldn't load plugin function\n"), stderr);
+    } else {
+	pv = (*pvfunc) (tr, N, det, T);
+	close_plugin(handle);
+    }
+
+    return pv;
+}
+
 /**
  * print_critval:
  * @st: distribution code.
@@ -1781,7 +1812,7 @@ void print_critval (char st, const double *parm, double a, double c, PRN *prn)
 	pprintf(prn, "t(%g)", parm[0]);
 	break;
     case 'X':
-	pprintf(prn, _("Chi-square(%g)"), parm[0]);
+	pprintf(prn, "%s(%g)", _("Chi-square"), parm[0]);
 	break;
     case 'F':
 	pprintf(prn, "F(%g, %g)", parm[0], parm[1]);
@@ -1841,6 +1872,8 @@ static int pdist_check_input (char st, const double *parm,
     } else if (st == 'F' || st == 'G' ||
 	       st == 'B' || st == 'W') {
 	np = 2;
+    } else if (st == 'J') {
+	np = 3;
     }
 
     for (i=0; i<np; i++) {
@@ -2101,6 +2134,9 @@ double gretl_get_pvalue (char st, const double *parm, double x)
 	y = weibull_cdf_comp(parm[0], parm[1], x);
     } else if (st == 'E') {
 	y = GED_cdf_comp(parm[0], x);
+    } else if (st == 'J') {
+	y = johansen_trace_pval((int) parm[0], (int) parm[1], 
+				(int) parm[2], x);
     }
 
     remember_pvalue_args(parm, x);
