@@ -17,23 +17,34 @@
  * 
  */
 
-/* random.c for gretl: RNGs based on GLib */
+/* random.c for gretl: RNGs */
 
 #include "libgretl.h"
 #include <time.h>
 #include <glib.h>
 
+#ifndef NO_SFMT /* flag to cut out all SFMT code */
+# ifdef USE_SSE2
+#  define HAVE_SSE2
+# endif
+# include "../../rng/SFMT.c"
+# if defined(HAVE_POSIX_MEMALIGN) || defined(OSX_BUILD) || defined(WIN32)
+#  define USE_SFMT_ARRAYS 0 /* faster, but needs more testing */
+# else
+#  define USE_SFMT_ARRAYS 0
+# endif
+#endif
+
 /**
  * SECTION:random
  * @short_description: generate pseudo-random values
  * @title: PRNG
  * @include: libgretl.h
  *
- * Libgretl uses the GLib PRNG as its underlying engine
+ * Libgretl uses the Mersenne Twister as its underlying engine
  * for uniform random values, but offers added value in 
  * the form of generators for several distributions commonly 
- * used in econometrics. GLib uses the Mersenne Twister,
- * originally developed by Makoto Matsumoto and Takuji Nishimura.
+ * used in econometrics. 
  *
  * Note that before using the libgretl PRNG you must call
  * either libgretl_init() or the specific initialization
@@ -43,10 +54,125 @@
  */
 
 static GRand *gretl_rand;
-static unsigned int useed;
+static guint32 useed;
+
+#ifdef NO_SFMT
+static int use_sfmt = 0;
+#else
+static int use_sfmt = 1;
+#endif
 
 static guint32 gretl_rand_octet (guint32 *sign);
 
+#if USE_SFMT_ARRAYS
+
+static void *gretl_memalign (size_t size, int *err)
+{
+    void *mem = NULL;
+
+#if defined(HAVE_POSIX_MEMALIGN)
+    *err = posix_memalign((void **) &mem, 16, size);
+    if (*err) {
+	fprintf(stderr, "posix_memalign: err = %d\n", *err);
+	return NULL;
+    }
+#elif defined(WIN32)
+    mem = win32_memalign(size, 16);
+    if (mem == NULL) {
+	fputs("win32_memalign: failed\n", stderr);
+	*err = E_ALLOC;
+    }
+#else /* OS X: should be aligned OK by default */
+    mem = malloc(size);
+    if (mem == NULL) {
+	fputs("osx_malloc: failed\n", stderr);
+	*err = E_ALLOC;
+    }
+#endif
+
+    return mem;
+}
+
+#define RSIZE (MEXP / 128 + 1) * 4
+
+static guint32 *rbuf;
+static int r_i;
+
+static int sfmt_array_setup (void)
+{
+    int err = 0;
+
+    if (rbuf == NULL) {
+	rbuf = gretl_memalign(RSIZE * sizeof *rbuf, &err);
+    }
+
+    if (!err) {
+	fprintf(stderr, "doing sfmt_array_setup, RSIZE=%d\n", RSIZE);
+	fill_array32(rbuf, RSIZE);
+	r_i = 0;
+    }
+
+    return err;
+}
+
+static void sfmt_array_cleanup (void)
+{
+#if defined(WIN32) && !defined(HAVE_POSIX_MEMALIGN)
+    win32_aligned_free(rbuf);
+#else
+    free(rbuf);
+#endif
+    rbuf = NULL;
+}
+
+static int sfmt_fill_U01_array (double *a, int t1, int t2, int n)
+{
+    guint32 *buf;
+    int i, t, err = 0;
+
+    if (n < RSIZE) {
+	n = RSIZE;
+    } else if (n % 4) {
+	n += 4 - (n % 4);
+    }
+
+    buf = gretl_memalign(n * sizeof *buf, &err);
+    if (err) {
+	return err;
+    }
+
+    fill_array32(buf, n);
+
+    i = 0;
+    for (t=t1; t<=t2; t++) {
+	a[t] = to_real2(buf[i++]);
+    }
+
+#if defined(WIN32) && !defined(HAVE_POSIX_MEMALIGN)
+    win32_aligned_free(buf);
+#else
+    free(buf);
+#endif
+
+    return err;
+}
+
+static inline guint32 sfmt_rand32 (void)
+{
+    if (r_i == RSIZE) {
+	fill_array32(rbuf, RSIZE);
+	r_i = 0;
+    }
+
+    return rbuf[r_i++];
+}
+
+#else
+
+#define sfmt_rand32 gen_rand32
+
+#endif /* USE_SFMT_ARRAYS */
+
 /**
  * gretl_rand_init:
  *
@@ -56,10 +182,24 @@ static guint32 gretl_rand_octet (guint32 *sign);
 void gretl_rand_init (void)
 {
     useed = time(NULL);
+
+#ifndef NO_SFMT
+    init_gen_rand(useed);
+# if USE_SFMT_ARRAYS
+    sfmt_array_setup();
+# endif
+#endif
+
     if (gretl_rand == NULL) {
 	gretl_rand = g_rand_new();
     }
-    gretl_rand_set_seed((guint32) useed);
+    g_rand_set_seed(gretl_rand, useed);
+
+    if (getenv("GRETL_RAND_GLIB")) {
+	use_sfmt = 0;
+    }
+
+    gretl_rand_octet(NULL);
 }
 
 /**
@@ -70,6 +210,9 @@ void gretl_rand_init (void)
 
 void gretl_rand_free (void)
 {
+#if USE_SFMT_ARRAYS
+    sfmt_array_cleanup();
+#endif
     g_rand_free(gretl_rand);
     gretl_rand = NULL;
 }
@@ -98,13 +241,85 @@ unsigned int gretl_rand_get_seed (void)
 void gretl_rand_set_seed (unsigned int seed)
 {
     useed = (seed == 0)? time(NULL) : seed;
+#ifndef NO_SFMT
+    init_gen_rand(useed); 
+# if USE_SFMT_ARRAYS
+    sfmt_array_setup();
+# endif
+#endif
     g_rand_set_seed(gretl_rand, useed);
     gretl_rand_octet(NULL);
 }
 
-static double gretl_rand_01 (void)
+#if 0 /* not yet */
+
+/**
+ * gretl_rand_set_multi_seed:
+ * @seed: the chosen seed values.
+ *
+ * Set a specific (and hence reproducible) set of seed values for gretl's 
+ * PRNG, in the form of a matrix containing an array of 4 32-bit values.
+ */
+
+void gretl_rand_set_multi_seed (const gretl_matrix *seed)
 {
+    guint32 useeds[4];
+    int i;
+
+    for (i=0; i<4; i++) {
+	useeds[i] = seed->val[i];
+    }
+
+    init_by_array(useeds, 4);
+    g_rand_set_seed_array(gretl_rand, useeds, 4);
+    gretl_rand_octet(NULL);
+}
+
+#endif
+
+/**
+ * gretl_rand_set_sfmt:
+ * @s: 1 or 0.
+ *
+ * If @s is non-zero, set libgretl's generator for uniform
+ * variates to use SFMT, as opposed to GLib's implementation
+ * of the Mersenne Twister.
+ */
+
+void gretl_rand_set_sfmt (int s)
+{
+    use_sfmt = s;
+}
+
+/**
+ * gretl_rand_get_sfmt:
+ *
+ * Returns: non-zero if libgretl is using the SFMT RNG.
+ */
+
+int gretl_rand_get_sfmt (void)
+{
+    return use_sfmt;
+}
+
+/**
+ * gretl_rand_01:
+ *
+ * Returns: the next random double, equally distributed over
+ * the range [0..1).
+ */
+
+double gretl_rand_01 (void)
+{
+#ifdef NO_SFMT
     return g_rand_double(gretl_rand);
+#else
+    if (use_sfmt) {
+	return to_real2(sfmt_rand32());
+    } else {
+	return g_rand_double(gretl_rand);
+    }
+#endif
 }
 
 /* Below: an implementation of the Marsaglia/Tsang Ziggurat method for
@@ -114,12 +329,12 @@ static double gretl_rand_01 (void)
    The code is based on Jochen Voss's gauss.c, written for use with
    the Gnu Scientific Library -- see http://seehuhn.de/pages/ziggurat.
    It was modified for gretl in two main ways: (a) we use the Mersenne
-   Twister uniform RNG from GLib, and (b) we use a 30-bit unsigned
-   random integer for conversion via the Ziggurat where Voss uses
-   a 24-bit value.  This sacrifices a little speed in exchange for
-   better coverage of the real line: we get over 10^9 distinct 
-   normal values as opposed to around 30 million values from the
-   24-bit input.
+   Twister uniform RNG, and (b) we use a 30-bit unsigned random integer 
+   for conversion via the Ziggurat where Voss uses a 24-bit value.  
+
+   This sacrifices a little speed in exchange for better coverage of the 
+   real line: we get over 10^9 distinct normal values as opposed to around 
+   30 million values from the 24-bit input.
 
    See also Jurgen Doornik's discussion of Ziggurat:
    http://www.doornik.com/research/ziggurat.pdf
@@ -249,7 +464,7 @@ union wraprand {
     gchar c[4];
 };
 
-/* Split a 32-bit random value from GLib into 4 octets: each octet
+/* Split a 32-bit random value into 4 octets: each octet
    provides a 7-bit value for indexing into the Ziggurat plus a
    sign bit.  Load a new guint32 when the material is exhausted.
 
@@ -270,7 +485,15 @@ static guint32 gretl_rand_octet (guint32 *sign)
     }
 
     if (i == 0) {
+#ifdef NO_SFMT
 	wr.u = g_rand_int(gretl_rand);
+#else
+	if (use_sfmt) {
+	    wr.u = sfmt_rand32();
+	} else {
+	    wr.u = g_rand_int(gretl_rand);
+	}
+#endif
 	i = 4;
     }
 
@@ -285,7 +508,11 @@ static double ran_normal_ziggurat (void)
     double x, y;
 
     while (1) {
+#ifdef NO_SFMT
 	j = g_rand_int(gretl_rand);
+#else
+	j = use_sfmt ? sfmt_rand32() : g_rand_int(gretl_rand);
+#endif
 	i = gretl_rand_octet(&sign);
 	j = j >> 2;
 
@@ -435,7 +662,7 @@ double gretl_one_snormal (void)
 {
     if (!env_checked) {
 	box_muller_env_check();
-    }    
+    } 
 
     if (use_box_muller) {
 	return ran_normal_box_muller();
@@ -484,6 +711,40 @@ int gretl_rand_normal_full (double *a, int t1, int t2,
     return 0;
 }
 
+#ifndef NO_SFMT
+
+static guint32 sfmt_int_range (guint32 begin, guint32 end)
+{
+    guint32 dist = end - begin;
+    guint32 rval = 0;
+
+    if (dist > 0) {
+	/* maxval is set to the predecessor of the greatest
+	 * multiple of dist less or equal 2^32. */
+	guint32 maxval;
+
+	if (dist <= 0x80000000u) { /* 2^31 */
+	    /* maxval = 2^32 - 1 - (2^32 % dist) */
+	    guint32 rem = (0x80000000u % dist) * 2;
+
+	    if (rem >= dist) rem -= dist;
+	    maxval = 0xffffffffu - rem;
+	} else {
+	    maxval = dist - 1;
+	}
+	  
+	do {
+	    rval = sfmt_rand32();
+	} while (rval > maxval);
+	  
+	rval %= dist;
+    }
+ 
+    return begin + rval;
+}
+
+#endif
+
 /**
  * gretl_rand_uniform_minmax:
  * @a: target array.
@@ -512,7 +773,16 @@ int gretl_rand_uniform_minmax (double *a, int t1, int t2,
     }
 
     for (t=t1; t<=t2; t++) {
+#ifdef NO_SFMT
 	a[t] = g_rand_double_range(gretl_rand, min, max);
+#else
+	if (use_sfmt) {
+	    /* FIXME use native array functionality? */
+	    a[t] = to_real2(sfmt_rand32()) * (max - min) + min;
+	} else {
+	    a[t] = g_rand_double_range(gretl_rand, min, max);
+	}
+#endif
     }
 
     return 0;
@@ -543,7 +813,15 @@ int gretl_rand_int_minmax (int *a, int n, int min, int max)
     /* note: in g_rand_int_range the upper bound is open */
 
     for (i=0; i<n; i++) {
+#ifdef NO_SFMT
 	a[i] = g_rand_int_range(gretl_rand, min, max + 1);
+#else
+	if (use_sfmt) {
+	    a[i] = sfmt_int_range(min, max + 1);
+	} else {
+	    a[i] = g_rand_int_range(gretl_rand, min, max + 1);
+	} 
+#endif
     }
 
     return 0;
@@ -564,9 +842,34 @@ void gretl_rand_uniform (double *a, int t1, int t2)
 {
     int t;
 
+#ifdef NO_SFMT
     for (t=t1; t<=t2; t++) {
-	a[t] = gretl_rand_01();
+	a[t] = g_rand_double(gretl_rand);
     }
+#else 
+# if USE_SFMT_ARRAYS
+    if (use_sfmt) {
+	int n = t2 - t1 + 1;
+	int err = 0;
+
+	if (n >= 100) {
+	    err = sfmt_fill_U01_array(a, t1, t2, n);
+	    if (!err) {
+		return;
+	    }
+	}
+    }
+# endif
+    if (use_sfmt) {
+	for (t=t1; t<=t2; t++) {
+	   a[t] = to_real2(sfmt_rand32());
+	}
+    } else {
+	for (t=t1; t<=t2; t++) {
+	   a[t] = g_rand_double(gretl_rand);
+	}
+    }
+#endif	
 }
 
 /**
@@ -974,7 +1277,15 @@ int gretl_rand_GED (double *a, int t1, int t2, double nu)
 
 unsigned int gretl_rand_int_max (unsigned int max)
 {
+#ifdef NO_SFMT
     return g_rand_int_range(gretl_rand, 0, max);
+#else
+    if (use_sfmt) {
+	return sfmt_int_range(0, max);
+    } else {
+	return g_rand_int_range(gretl_rand, 0, max);
+    }
+#endif
 }
 
 /**
@@ -986,7 +1297,14 @@ unsigned int gretl_rand_int_max (unsigned int max)
 
 unsigned int gretl_rand_int (void)
 {
+#ifdef NO_SFMT
     return g_rand_int(gretl_rand);
+#else
+    if (use_sfmt) {
+	return sfmt_rand32();
+    } else {
+	return g_rand_int(gretl_rand);
+    }
+#endif
 }
 
-
