@@ -26,12 +26,16 @@ void gretl_rand_free (void);
 
 void gretl_rand_set_seed (unsigned int seed);
 
+void gretl_rand_set_multi_seed (const gretl_matrix *seed);
+
 unsigned int gretl_rand_int (void);
 
 unsigned int gretl_rand_int_max (unsigned int max);
 
 int gretl_rand_int_minmax (int *a, int n, int min, int max);
 
+double gretl_rand_01 (void);
+
 double gretl_one_snormal (void);
 
 void gretl_rand_uniform (double *a, int t1, int t2);
@@ -63,6 +67,10 @@ int gretl_rand_gamma (double *a, int t1, int t2,
 
 int gretl_rand_GED (double *a, int t1, int t2, double nu);
 
+void gretl_rand_set_sfmt (int s);
+
+int gretl_rand_get_sfmt (void);
+
 void gretl_rand_set_box_muller (int s);
 
 int gretl_rand_get_box_muller (void);
