@@ -788,6 +788,74 @@ char **gretl_string_split (const char *s, int *n)
     return S;
 }
 
+/**
+ * gretl_string_split_quoted:
+ * @s: the source string.
+ * @n: location to receive the number of substrings.
+ * @err: location to receive error code.
+ *
+ * Similar to gretl_string_split(), except that for this
+ * function the sub-strings are assumed to be delimited by
+ * ASCII double-quote characters, and may therefore
+ * contain embedded spaces. The quotes are removed in the
+ * members of the returned array. Note that this function
+ * is not fully general in that it doesn't handle escaped
+ * double-quotes.
+ *
+ * Returns: allocated array of substrings or NULL in case of failure.
+ */
+
+char **gretl_string_split_quoted (const char *s, int *n, int *err)
+{
+    const char *p = s;
+    int i, len, m = 0;
+    char *substr;
+    char **S;
+
+    *n = 0;
+
+    while (*p) {
+	if (*p == '"') m++;
+	p++;
+    }
+
+    if (m % 2 != 0) {
+	/* unbalanced quotes */
+	*err = E_PARSE;
+	return NULL;
+    }
+
+    if (m == 0) {
+	return NULL;
+    }
+
+    m /= 2;
+
+    S = strings_array_new(m);
+    if (S == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    for (i=0; i<m; i++) {
+	s = strchr(s, '"') + 1;
+	p = strchr(s, '"');
+	len = p - s;
+	substr = gretl_strndup(s, len);
+	if (substr == NULL) {
+	    *err = E_ALLOC;
+	    free_strings_array(S, m);
+	    return NULL;
+	}
+	S[i] = substr;
+	s = p + 1;
+    }
+
+    *n = m;
+
+    return S;
+}
+
 /**
  * gretl_trunc:
  * @str: the string to truncate.
@@ -951,7 +1019,7 @@ char *chopstr (char *str)
  * Returns: the output string, @targ.
  */
 
-char *switch_ext (char *targ, const char *src, char *ext)
+char *switch_ext (char *targ, const char *src, const char *ext)
 {
     int i = dotpos(src);
 
@@ -966,6 +1034,43 @@ char *switch_ext (char *targ, const char *src, char *ext)
     return targ;
 }
 
+/**
+ * switch_ext_new:
+ * @src: the original string.
+ * @ext: the extension or suffix to attach (without leading '.').
+ *
+ * For processing filenames: creates a copy of @src in which
+ * any existing dot-extension is removed and @ext is appended
+ * (with a dot automatically inserted).
+ *
+ * Returns: the newly allocated string.
+ */
+
+char *switch_ext_new (const char *src, const char *ext)
+{
+    int len = strlen(src) + strlen(ext) + 2;
+    const char *p = strrchr(src, '.');
+    char *ret = NULL;
+
+    if (p != NULL) {
+	len -= strlen(p);
+    } 
+
+    ret = calloc(len, 1);
+
+    if (ret != NULL) {
+	if (p != NULL) {
+	    strncat(ret, src, p - src);
+	} else {
+	    strcat(ret, src);
+	}
+	strcat(ret, ".");
+	strcat(ret, ext);
+    }
+
+    return ret;
+}
+
 /**
  * get_base:
  * @targ: the target or output string (must be pre-allocated).
@@ -1053,7 +1158,7 @@ int top_n_tail (char *str, size_t maxlen, int *err)
 	}
 
 	/* does this line start a comment? */
-	if (strchr(str, '#') || !strncmp(str, "/*", 2)) {
+	if (*str == '#' || !strncmp(str, "/*", 2)) {
 	    ; /* leave well alone */
 	} else {
 	    /* replace backslash, if present */
@@ -1482,6 +1587,62 @@ char **strings_array_dup (char **strs, int n)
     return S;
 }
 
+static int compare_strings (const void *a, const void *b)
+{
+    const char **sa = (const char **) a;
+    const char **sb = (const char **) b;
+     
+    return strcmp(*sa, *sb);
+}
+
+/**
+ * strings_array_sort:
+ * @pS: location of array of strings.
+ * @n: location of the number of strings in the array.
+ * @opt: may contain %OPT_U to trim the sorted array
+ * so that it contains only unique entries.
+ *
+ * Sorts an array of strings in ascending lexicographical
+ * order. If %OPT_U is given, @n holds the number of unique
+ * strings on exit. It is assumed that storage for the
+ * strings array was obtained via strings_array_new() or
+ * a similar libgretl function.
+ */
+
+int strings_array_sort (char ***pS, int *n, gretlopt opt)
+{
+    char **S = *pS;
+    int ns = *n;
+
+    qsort(S, ns, sizeof *S, compare_strings);
+
+    if (opt & OPT_U) {
+	int i, j, m = ns;
+
+	for (i=0; i<m-1; i++) {
+	    if (!strcmp(S[i], S[i+1])) {
+		free(S[i+1]);
+		for (j=i+1; j<m-1; j++) {
+		    S[j] = S[j+1];
+		}
+		S[m-1] = NULL;
+		i--;
+		m--;
+	    }
+	}
+	if (m < ns) {
+	    char **tmp = realloc(S, m * sizeof *S);
+
+	    if (tmp != NULL) {
+		*pS = tmp;
+	    }
+	    *n = m;
+	}
+    }
+
+    return 0;
+}
+
 /**
  * strings_array_cmp:
  * @strs1: first array of strings.
