@@ -83,6 +83,8 @@ char *gretl_quoted_string_strdup (const char *s, const char **ptr);
 
 char **gretl_string_split (const char *s, int *n);
 
+char **gretl_string_split_quoted (const char *s, int *n, int *err);
+
 char *gretl_str_expand (char **orig, const char *add, const char *sep);
 
 char *charsub (char *str, char find, char repl);
@@ -97,7 +99,9 @@ void clear (char *str, int len);
 
 char *chopstr (char *str);
 
-char *switch_ext (char *targ, const char *src, char *ext);
+char *switch_ext (char *targ, const char *src, const char *ext);
+
+char *switch_ext_new (const char *src, const char *ext);
 
 int get_base (char *targ, const char *src, char c);
 
@@ -126,6 +130,8 @@ char **strings_array_new_with_length (int nstrs, int len);
 
 char **strings_array_dup (char **strs, int n);
 
+int strings_array_sort (char ***pS, int *n, gretlopt opt);
+
 int strings_array_cmp (char **strs1, char **strs2, int n);
 
 void free_strings_array (char **strs, int nstrs);
