@@ -333,16 +333,51 @@ int attach_subsample_to_model (MODEL *pmod, const DATAINFO *pdinfo)
     return err;
 }
 
-static int resample_trim_dataset (double ***RZ, DATAINFO *pdinfo)
+/* If series have been added to a resampled dataset, we can't
+   bring these back to the "full" dataset, which may have a
+   longer or shorter series length, and from which there is
+   no definite mapping by row. So we just delete them. In
+   this function we destroy their varnames and varinfo
+   structures; the numerical arrays get deleted later.
+*/
+
+static int resample_sync_dataset (double ***pZ, DATAINFO *pdinfo)
 {
-    int drop = pdinfo->v - fullinfo->v;
-    int err = 0;
+    if (pdinfo->v > fullinfo->v) {
+	char **varname;
+	VARINFO **varinfo;
+	int i, nv = fullinfo->v;
+
+	for (i=fullinfo->v; i<pdinfo->v; i++) {
+	    free(pdinfo->varname[i]);
+	    free(pdinfo->varinfo[i]);
+	}
+
+	varname = realloc(pdinfo->varname, nv * sizeof *varname);
+	if (varname == NULL) {
+	    return E_ALLOC;
+	}
+	pdinfo->varname = varname;
+
+	varinfo = realloc(pdinfo->varinfo, nv * sizeof *varinfo);
+	if (varinfo == NULL) {
+	    return E_ALLOC;
+	}
+	pdinfo->varinfo = varinfo;
+
+#if SUBDEBUG
+	fprintf(stderr, "resample_trim_dataset: drop = %d, pZ=%p, Z=%p\n", 
+		pdinfo->v - fullinfo->v, (void *) pZ, (void *) *pZ);
+#endif
 
-    if (drop > 0) {
-	err = dataset_drop_last_variables(drop, RZ, pdinfo);
     }
 
-    return err;
+    /* sync */
+    fullinfo->varname = pdinfo->varname;
+    fullinfo->varinfo = pdinfo->varinfo;
+    fullinfo->descrip = pdinfo->descrip;
+
+    return 0;
 }
 
 /* Apparatus for updating full dataset when restoring full sample
@@ -570,14 +605,10 @@ int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state)
        subsampled, e.g., by some boolean criterion 
     */
 
-    /* reattach malloc'd elements of subsampled dataset,
-       which may have moved */
-    sync_datainfo_members(pdinfo);
-
     if (pdinfo->submask == RESAMPLED) {
-	/* not regular subsample but resampled dataset */
-	resample_trim_dataset(pZ, pdinfo);
+	err = resample_sync_dataset(pZ, pdinfo);
     } else {
+	sync_datainfo_members(pdinfo);
 	err = sync_data_to_full(pZ, pdinfo);
     }
 
@@ -585,21 +616,21 @@ int restore_full_sample (double ***pZ, DATAINFO *pdinfo, ExecState *state)
         gretl_errmsg_set(_("Missing sub-sample information; can't merge data\n"));
     }
 
+    if (err) {
+	return err;
+    }
+
     /* destroy sub-sampled data array */
 #if SUBDEBUG
-    fprintf(stderr, "restore_full_sample: freeing Z at %p\n", (void *) *pZ);
+    fprintf(stderr, "restore_full_sample: freeing sub-sampled Z at %p (v = %d)\n"
+	    "and clearing pdinfo at %p\n", (void *) *pZ, pdinfo->v, (void *) pdinfo);
 #endif
     free_Z(*pZ, pdinfo);
-
-#if SUBDEBUG
-    fprintf(stderr, "restore_full_sample: clearing pdinfo at %p\n", 
-	    (void *) pdinfo);
-#endif
     clear_datainfo(pdinfo, CLEAR_SUBSAMPLE);
 
     relink_to_full_dataset(pZ, pdinfo);
 
-    if (!err && state != NULL) {
+    if (state != NULL) {
 	/* "full" really means, relative to the original state */
 	if (state->submask != NULL) {
 	    err = restrict_sample_from_mask(state->submask, pZ, pdinfo,
@@ -861,6 +892,9 @@ static int make_random_mask (const char *s, const char *oldmask,
 
 int backup_full_dataset (double **Z, DATAINFO *pdinfo)
 {
+#if SUBDEBUG
+    int newfull = 0;
+#endif
     fullZ = Z;
 
     if (fullinfo == NULL) {
@@ -868,6 +902,9 @@ int backup_full_dataset (double **Z, DATAINFO *pdinfo)
 	if (fullinfo == NULL) {
 	    return E_ALLOC;
 	}
+#if SUBDEBUG
+	newfull = 1;
+#endif
     }
 
     if (pdinfo != NULL) {
@@ -876,8 +913,8 @@ int backup_full_dataset (double **Z, DATAINFO *pdinfo)
     } 
 
 #if SUBDEBUG
-    fprintf(stderr, "backup_full_dataset: fullZ = %p, fullinfo = %p\n",
-	    (void *) fullZ, (void *) fullinfo);
+    fprintf(stderr, "backup_full_dataset: fullZ = %p, fullinfo = %p (%s)\n",
+	    (void *) fullZ, (void *) fullinfo, newfull ? "new" : "old");
 #endif
 
     return 0;
