@@ -403,7 +403,7 @@ int *tsls_make_endolist (const int *reglist, int **instlist,
 
 static int fill_E_matrix (gretl_matrix *E, MODEL *pmod, 
 			  const int *reglist, const int *instlist,
-			  double ***pZ, DATAINFO *pdinfo)
+			  double **Z, DATAINFO *pdinfo)
 {
     MODEL emod;
     int *elist;
@@ -436,7 +436,7 @@ static int fill_E_matrix (gretl_matrix *E, MODEL *pmod,
 	}
 
 	/* regress the given endogenous var on all the instruments */
-	emod = lsq(elist, *pZ, pdinfo, OLS, OPT_A);
+	emod = lsq(elist, Z, pdinfo, OLS, OPT_A);
 	if ((err = emod.errcode)) {
 	    clear_model(&emod);
 	    break;
@@ -477,7 +477,7 @@ static int tsls_loglik (MODEL *pmod,
 			int nendo,
 			const int *reglist,
 			const int *instlist,
-			double ***pZ, 
+			double **Z, 
 			DATAINFO *pdinfo)
 {
     gretl_matrix *E, *W;
@@ -495,7 +495,7 @@ static int tsls_loglik (MODEL *pmod,
     }
 
     if (!err) {
-	err = fill_E_matrix(E, pmod, reglist, instlist, pZ, pdinfo);
+	err = fill_E_matrix(E, pmod, reglist, instlist, Z, pdinfo);
     }
 
     if (!err) {
@@ -602,12 +602,13 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     MODEL hmod;
     double URSS;
     int *HT_list = NULL;
-    int t, df, nobs1;
+    int t, df;
     int nv = pdinfo->v;
     int ku = 0;
     int err = 0;
 
 #if HTDBG
+    int nobs1;
     PRN *dbgprn = NULL;
     dbgprn = gretl_print_new(GRETL_PRINT_STDOUT, NULL);
 #endif
@@ -634,6 +635,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 #if HTDBG
     pprintf(dbgprn, "Hausman: unrestricted model (df = %d)\n", hmod.dfd);
     printmodel(&hmod, pdinfo, OPT_NONE, dbgprn);
+    nobs1 = hmod.nobs;
 #endif
 
     if (hmod.dfd == 0) {
@@ -642,7 +644,6 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     }
 
     URSS = hmod.ess;
-    nobs1 = hmod.nobs;
     ku = hmod.ncoeff;
 
     /* add fitted values from unrestricted model to dataset */
@@ -667,6 +668,8 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
     hmod = lsq(HT_list, *pZ, pdinfo, OLS, OPT_A);
 
 #if HTDBG
+    strcpy(pdinfo->varname[pdinfo->v - 1], "Ufitvals");
+    pprintf(dbgprn, "Hausman: aux regression\n", hmod.dfd);
     printmodel(&hmod, pdinfo, OPT_NONE, dbgprn);
     pprintf(dbgprn, "smpl1 = %d, smpl2 = %d\n", nobs1, hmod.nobs);
     pprintf(dbgprn, "k1 = %d, k2 = %d\n", hmod.ncoeff, ku);
@@ -686,6 +689,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 
 #if HTDBG
 	pprintf(dbgprn, "(RRSS - URSS) = %g, URSS = %g\n", DRSS, URSS);
+	pprintf(dbgprn, "Htest = %g [%.4f]\n", HTest, chisq_cdf_comp(df, HTest));
 #endif
 
 	if (test != NULL) {
@@ -715,7 +719,7 @@ tsls_hausman_test (MODEL *tmod, int *reglist, int *hatlist,
 
 static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
 			     const double **Z, const DATAINFO *pdinfo, 
-			     char *mask, int *err)
+			     char *mask, gretlopt opt, int *err)
 {
     gretl_matrix *Q = NULL;
     gretl_matrix *R = NULL;
@@ -745,7 +749,7 @@ static gretl_matrix *tsls_Q (int *instlist, int *reglist, int **pdlist,
     if (R == NULL) {
 	*err = E_ALLOC;
 	goto bailout;
-    }    
+    } 
 
     *err = gretl_matrix_QR_decomp(Q, R);
     if (*err) {
@@ -862,13 +866,12 @@ static int tsls_form_xhat (gretl_matrix *Q, double *x, double *xhat,
     return 0;
 }
 
-static void tsls_residuals (MODEL *pmod, const int *reglist,
-			    const double **Z, gretlopt opt)
+static void tsls_residuals (MODEL *pmod, const int *reglist, 
+			    const double **Z, const DATAINFO *pdinfo,
+			    int panel_N, gretlopt opt)
 {
-    int den = (opt & OPT_N)? pmod->nobs : pmod->dfd;
-    int yno = pmod->list[1];
-    double sigma0 = pmod->sigma;
-    double yh;
+    int yno = reglist[1];
+    double yh, sigma0 = pmod->sigma;
     int i, t;
 
     pmod->ess = 0.0;
@@ -889,6 +892,8 @@ static void tsls_residuals (MODEL *pmod, const int *reglist,
     if (pmod->ess <= 0.0) {
 	pmod->sigma = 0.0;
     } else {
+	int den = (opt & OPT_N)? pmod->nobs : pmod->dfd;
+
 	pmod->sigma = sqrt(pmod->ess / den);
     }
 
@@ -903,14 +908,13 @@ static void tsls_residuals (MODEL *pmod, const int *reglist,
 
 #define TSLS_CORR_RSQ 1
 
-static void tsls_extra_stats (MODEL *pmod, int overid, const double **Z,
+static void tsls_extra_stats (MODEL *pmod, int yno, int overid, 
+			      const double **Z,
 			      const DATAINFO *pdinfo)
 {
     double r;
 
 #if TSLS_CORR_RSQ
-    int yno = pmod->list[1];
-
     pmod->rsq = gretl_corr_rsq(pmod->t1, pmod->t2, Z[yno], pmod->yhat);
 #else
     pmod->rsq = 1 - pmod->ess / pmod->tss;
@@ -929,7 +933,7 @@ static void tsls_extra_stats (MODEL *pmod, int overid, const double **Z,
 	}
     } 
 
-    /* tsls is not a ML estimator unless it's exactly identified, 
+    /* tsls is not an ML estimator unless it's exactly identified, 
        and in that case we require a special calculation */
     pmod->lnL = NADBL;
     pmod->criterion[C_AIC] = NADBL;
@@ -964,7 +968,7 @@ static void replace_list_element (int *list, int targ, int repl)
 {
     int i;
 
-    for (i=2; i<=list[0]; i++) {
+    for (i=1; i<=list[0]; i++) {
 	if (list[i] == targ) {
 	    list[i] = repl;
 	    break;
@@ -1285,11 +1289,12 @@ compute_first_stage_F (MODEL *pmod, int v, const int *reglist,
     return err;
 }
 
-static int 
-tsls_adjust_sample (const int *list, int *t1, int *t2, 
-		    const double **Z, char **pmask)
+static int tsls_adjust_sample (const int *list, 
+			       const double **Z, DATAINFO *pdinfo,
+			       gretlopt opt, char **pmask,
+			       int *panel_N)
 {
-    int i, t, t1min = *t1, t2max = *t2;
+    int i, t, t1min = pdinfo->t1, t2max = pdinfo->t2;
     char *mask = NULL;
     int T, vi, missobs;
     int err = 0;
@@ -1378,8 +1383,11 @@ tsls_adjust_sample (const int *list, int *t1, int *t2,
 	    t1min, t2max, missobs, t2max - t1min + 1 - missobs);
 #endif
 
-    *t1 = t1min; 
-    *t2 = t2max;
+    if (!err) {
+	pdinfo->t1 = t1min; 
+	pdinfo->t2 = t2max;
+    }
+
     *pmask = mask;
 
     return err;
@@ -1467,12 +1475,14 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     MODEL tsls;
     gretl_matrix *Q = NULL;
     char *missmask = NULL;
+    int panel_N = 0;
     int orig_t1 = pdinfo->t1, orig_t2 = pdinfo->t2;
     int *reglist = NULL, *instlist = NULL;
     int *hatlist = NULL, *s2list = NULL;
     int *exolist = NULL, *endolist = NULL;
     int *droplist = NULL;
-    int addconst = 0;
+    int addconst = 0, addvars = 0;
+    int ninst = 0, nreg = 0;
     int orig_nvar = pdinfo->v;
     int sysest = (opt & OPT_E);
     int nendo, ev = 0;
@@ -1497,8 +1507,8 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     }
 
     /* adjust sample range for missing observations */
-    err = tsls_adjust_sample(list, &pdinfo->t1, &pdinfo->t2, 
-			     (const double **) *pZ, &missmask); 
+    err = tsls_adjust_sample(list, (const double **) *pZ, pdinfo, 
+			     opt, &missmask, &panel_N);
 
     if (!err) {
 	/* allocate second stage regression list */
@@ -1549,7 +1559,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     if (!err) {
 	Q = tsls_Q(instlist, reglist, &droplist,
 		   (const double **) *pZ, pdinfo,
-		   missmask, &err);
+		   missmask, opt, &err);
     }
 
     if (err) {
@@ -1559,7 +1569,9 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     /* check (again) for order condition: we do this after tsls_Q in case
        any vars get dropped at that stage
     */
-    OverIdRank = instlist[0] - reglist[0] + 1;
+    ninst = instlist[0];
+    nreg = reglist[0] - 1;
+    OverIdRank = ninst - nreg;
     if (OverIdRank < 0) {
         gretl_errmsg_sprintf(_("The order condition for identification is not satisfied.\n"
 			       "At least %d more instruments are needed."), -OverIdRank);
@@ -1567,9 +1579,10 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 	goto bailout;
     }
 
-    /* allocate storage for fitted vars, if needed */
-    if (nendo > 0) {
-	err = dataset_add_series(nendo, pZ, pdinfo);
+    /* allocate storage for fitted vars (etc.), if needed */
+    addvars = nendo;
+    if (addvars > 0) {
+	err = dataset_add_series(addvars, pZ, pdinfo);
 	if (err) {
 	    goto bailout;
 	}
@@ -1598,8 +1611,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 	    goto bailout;
 	}
 
-	/* give the fitted series the same name as the original */
-	strcpy(pdinfo->varname[v1], pdinfo->varname[v0]);
+	/* name the fitted series according to the original */
 	strcpy(pdinfo->varname[v1], "h_");
 	strncat(pdinfo->varname[v1], pdinfo->varname[v0], VNAMELEN - 3);
 
@@ -1638,7 +1650,8 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
     if (nendo > 0) {
 	/* special: we need to use the original RHS vars to compute
 	   residuals and associated statistics */
-	tsls_residuals(&tsls, reglist, (const double **) *pZ, opt);
+	tsls_residuals(&tsls, reglist, (const double **) *pZ, 
+		       pdinfo, panel_N, opt);
     }
 
     if (opt & OPT_R) {
@@ -1651,7 +1664,8 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 
     if (nendo > 0) {
 	/* compute additional statistics (R^2, F, etc.) */
-	tsls_extra_stats(&tsls, OverIdRank, (const double **) *pZ, pdinfo);
+	tsls_extra_stats(&tsls, reglist[1], OverIdRank, 
+			 (const double **) *pZ, pdinfo);
     }
 
     if (!sysest && nendo > 0) {
@@ -1661,7 +1675,7 @@ MODEL tsls (const int *list, double ***pZ, DATAINFO *pdinfo,
 	if (OverIdRank > 0) {
 	    ivreg_sargan_test(&tsls, OverIdRank, instlist, pZ, pdinfo);
 	} else {
-	    tsls_loglik(&tsls, nendo, reglist, instlist, pZ, pdinfo);
+	    tsls_loglik(&tsls, nendo, reglist, instlist, *pZ, pdinfo);
 	}
     }
 
