@@ -43,6 +43,7 @@ struct user_matrix_ {
     UMFlags flags;
     char name[VNAMELEN];
     char **colnames;
+    char **rownames;
 };
 
 static user_matrix **matrices;
@@ -98,6 +99,7 @@ static user_matrix *user_matrix_new (gretl_matrix *M, const char *name)
     *u->name = '\0';
     strncat(u->name, name, VNAMELEN - 1);
     u->colnames = NULL;
+    u->rownames = NULL;
 
     return u;
 }
@@ -269,12 +271,18 @@ user_matrix *get_user_matrix_by_index (int idx)
     }
 }
 
-static void usermat_destroy_colnames (user_matrix *u)
+static void usermat_destroy_names (user_matrix *u, int which)
 {
-    if (u->colnames != NULL && u->M != NULL) {
-	free_strings_array(u->colnames, u->M->cols);
-	u->colnames = NULL;
-    }
+    if (u->M != NULL) {
+	if ((which == 2 || which == 0) && u->colnames != NULL) {
+	    free_strings_array(u->colnames, u->M->cols);
+	    u->colnames = NULL;
+	}
+	if ((which == 2 || which == 1) && u->rownames != NULL) {
+	    free_strings_array(u->rownames, u->M->rows);
+	    u->rownames = NULL;
+	}	
+    }	    
 }
 
 static void usermat_destroy_matrix (user_matrix *u)
@@ -296,7 +304,10 @@ int user_matrix_replace_matrix (user_matrix *u, gretl_matrix *M)
 		"matrix at %p\n", u->M, M);
 #endif
 	if (u->colnames != NULL && M->cols != u->M->cols) {
-	    usermat_destroy_colnames(u);
+	    usermat_destroy_names(u, 0);
+	}
+	if (u->rownames != NULL && M->rows != u->M->rows) {
+	    usermat_destroy_names(u, 1);
 	}
 	usermat_destroy_matrix(u);
 	u->M = M;
@@ -402,7 +413,7 @@ gretl_matrix *steal_matrix_by_name (const char *name)
 	user_matrix *u = get_user_matrix_by_name(name);
 
 	if (u != NULL) {
-	    usermat_destroy_colnames(u);
+	    usermat_destroy_names(u, 2);
 	    ret = u->M;
 	    u->M = NULL;
 	}
@@ -863,7 +874,10 @@ gretl_matrix *matrix_get_submatrix (const gretl_matrix *M,
     }
 
     if (S != NULL && S->rows == M->rows) {
-	S->t1 = M->t1;
+	if (M->t1 >= 0 && M->t2 > M->t1) {
+	    S->t1 = M->t1;
+	    S->t2 = M->t2;
+	}
     }
 
     free(rslice);
@@ -951,7 +965,7 @@ static void destroy_user_matrix (user_matrix *u)
 	    (void *) u->M);
 #endif
 
-    usermat_destroy_colnames(u);
+    usermat_destroy_names(u, 2);
     usermat_destroy_matrix(u);
     free(u);
 
@@ -1135,8 +1149,9 @@ int user_matrix_destroy_by_name (const char *name, PRN *prn)
     return err;
 }
 
-int umatrix_set_colnames_from_string (const gretl_matrix *M, 
-				      const char *s)
+int umatrix_set_names_from_string (const gretl_matrix *M, 
+				   const char *s,
+				   int byrow)
 {
     user_matrix *u = get_user_matrix_by_data(M);
     int n, err = 0;
@@ -1145,10 +1160,10 @@ int umatrix_set_colnames_from_string (const gretl_matrix *M,
 	return E_UNKVAR;
     }
 
-    n = M->cols;
+    n = (byrow)? M->rows : M->cols;
 
     if (s == NULL || *s == '\0') {
-	usermat_destroy_colnames(u);
+	usermat_destroy_names(u, byrow);
     } else {
 	char **S;
 	int ns;
@@ -1160,17 +1175,22 @@ int umatrix_set_colnames_from_string (const gretl_matrix *M,
 	    err = E_NONCONF;
 	    free_strings_array(S, ns);
 	} else {
-	    usermat_destroy_colnames(u);
-	    u->colnames = S;
+	    usermat_destroy_names(u, byrow);
+	    if (byrow) {
+		u->rownames = S;
+	    } else {
+		u->colnames = S;
+	    }
 	}
     }
 
     return err;
 }
 
-int umatrix_set_colnames_from_list (const gretl_matrix *M, 
-				    const int *list,
-				    const DATAINFO *pdinfo)
+int umatrix_set_names_from_list (const gretl_matrix *M, 
+				 const int *list,
+				 const DATAINFO *pdinfo,
+				 int byrow)
 {
     user_matrix *u = get_user_matrix_by_data(M);
     int i, n, err = 0;
@@ -1179,10 +1199,10 @@ int umatrix_set_colnames_from_list (const gretl_matrix *M,
 	return E_UNKVAR;
     }
 
-    n = M->cols;
+    n = (byrow)? M->rows : M->cols;
 
     if (list == NULL || list[0] == 0) {
-	usermat_destroy_colnames(u);
+	usermat_destroy_names(u, byrow);
     } else if (list[0] != n) {
 	err = E_NONCONF;
     } else {
@@ -1202,19 +1222,52 @@ int umatrix_set_colnames_from_list (const gretl_matrix *M,
 	if (err) {
 	    free_strings_array(S, n);
 	} else {
-	    usermat_destroy_colnames(u);
-	    u->colnames = S;
+	    usermat_destroy_names(u, byrow);
+	    if (byrow) {
+		u->rownames = S;
+	    } else {
+		u->colnames = S;
+	    }
 	}
     }
 
     return err;
 }
 
-const char **user_matrix_get_column_names (const gretl_matrix *M)
+const char **user_matrix_get_names (const gretl_matrix *M,
+				    int byrow)
 {
     user_matrix *u = get_user_matrix_by_data(M);
 
-    return (u != NULL)? (const char **) u->colnames : NULL;
+    if (u == NULL) {
+	return NULL;
+    } else if (byrow) {
+	return (const char **) u->rownames;
+    } else {
+	return (const char **) u->colnames;
+    }
+}
+
+char *user_matrix_get_column_name (const gretl_matrix *M, int col,
+				   int *err)
+{
+    user_matrix *u = get_user_matrix_by_data(M);
+    char *ret = NULL;
+
+    if (u == NULL || col < 1 || col > M->cols) {
+	*err = E_DATA;
+    } else {
+	if (u->colnames == NULL) {
+	    ret = gretl_strdup("");
+	} else {
+	    ret = gretl_strdup(u->colnames[col-1]);
+	}
+	if (ret == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    return ret;
 }
 
 double 
@@ -1607,16 +1660,13 @@ static gretl_matrix *get_ols_matrix (const char *mname,
     if (m == NULL) {
 	gretl_errmsg_sprintf(_("'%s': no such matrix"), mname);
 	*err = E_UNKVAR;
-	return NULL;
-    } 
-
-    if (newmat != NULL && (m->rows != r || m->cols != c)) {
+    } else if (newmat != NULL && (m->rows != r || m->cols != c)) {
 	m = gretl_matrix_alloc(r, c);
 	if (m == NULL) {
 	    *err = E_ALLOC;
-	    return NULL;
+	} else {
+	    *newmat = 1;
 	}
-	*newmat = 1;
     }
 
     return m;
@@ -1723,6 +1773,81 @@ gretl_matrix *user_matrix_ols (const gretl_matrix *Y,
     return B;
 }
 
+gretl_matrix *user_matrix_rls (const gretl_matrix *Y, 
+			       const gretl_matrix *X,
+			       const gretl_matrix *R,
+			       const gretl_matrix *Q,
+			       const char *Uname, 
+			       const char *Vname, 
+			       int *err)
+{
+    gretl_matrix *B = NULL;
+    gretl_matrix *U = NULL;
+    gretl_matrix *V = NULL;
+    int newU = 0, newV = 0;
+    int g, k, T;
+
+    if (gretl_is_null_matrix(Y) || gretl_is_null_matrix(X)) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    T = Y->rows;
+    k = X->cols;
+    g = Y->cols;
+
+    if (X->rows != T) {
+	*err = E_NONCONF;
+	return NULL;
+    }
+
+    if (!nullarg(Uname)) {
+	U = get_ols_matrix(Uname, T, g, &newU, err);
+	if (*err) {
+	    return NULL;
+	}
+    } 
+
+    if (!nullarg(Vname)) {
+	get_ols_matrix(Vname, 0, 0, NULL, err);
+	if (!*err) {
+	    newV = 1;
+	}
+    }
+
+    if (!*err) {
+	B = gretl_matrix_alloc(k, g);
+	if (B == NULL) {
+	    *err = E_ALLOC;
+	}
+    }
+
+    if (!*err) {
+	if (newV) {
+	    /* note: "V" will actually be M (X'X)^{-1} M' */
+	    *err = gretl_matrix_restricted_multi_ols(Y, X, R, Q, B, U, &V);
+	} else {
+	    *err = gretl_matrix_restricted_multi_ols(Y, X, R, Q, B, U, NULL);
+	}
+    }
+
+    if (*err) {
+	gretl_matrix_free(B);
+	B = NULL;
+	if (newU) gretl_matrix_free(U);
+	if (newV) gretl_matrix_free(V);
+    } else {
+	if (newU) {
+	    user_matrix_replace_matrix_by_name(Uname, U);
+	}
+	if (newV) {
+	    user_matrix_replace_matrix_by_name(Vname, V);
+	}	
+    }
+
+    return B;
+}
+
 static void maybe_eigen_trim (gretl_matrix *E)
 {
     double x;
@@ -1795,6 +1920,50 @@ user_matrix_eigen_analysis (const gretl_matrix *m, const char *rname, int symm,
     return E;
 }
 
+gretl_matrix *user_gensymm_eigenvals (const gretl_matrix *A, 
+				      const gretl_matrix *B,
+				      const char *rname,
+				      int *err)
+{
+    gretl_matrix *E = NULL, *V = NULL;
+
+    if (gretl_is_null_matrix(A) || gretl_is_null_matrix(B)) {
+	*err = E_DATA;
+	return NULL;
+    }
+
+    if (gretl_matrix_xna_check(A) || gretl_matrix_xna_check(B)) {
+	*err = E_NAN;
+	return NULL;
+    }
+
+    if (!nullarg(rname)) {
+	if (get_matrix_by_name(rname) == NULL) {
+	    gretl_errmsg_sprintf(_("'%s': no such matrix"), rname);
+	    *err = E_UNKVAR;
+	    return NULL;
+	} else {
+	    V = gretl_matrix_alloc(B->cols, A->rows);
+	    if (V == NULL) {
+		*err = E_ALLOC;
+		return NULL;
+	    }
+	}
+    }
+
+    E = gretl_gensymm_eigenvals(A, B, V, err);
+
+    if (V != NULL) {
+	if (*err) {
+	    gretl_matrix_free(V);
+	} else {
+	    user_matrix_replace_matrix_by_name(rname, V);
+	}
+    }
+
+    return E;
+}
+
 static void xml_put_user_matrix (user_matrix *u, FILE *fp)
 {
     gretl_matrix *M;
@@ -1817,6 +1986,14 @@ static void xml_put_user_matrix (user_matrix *u, FILE *fp)
 	}
     } 
 
+    if (u->rownames != NULL) {
+	fputs(" rownames=\"", fp);
+	for (j=0; j<M->rows; j++) {
+	    fputs(u->rownames[j], fp);
+	    fputc((j < M->rows - 1)? ' ' : '"', fp);
+	}
+    }     
+
     fputs(">\n", fp);
 
     for (i=0; i<M->rows; i++) {
