@@ -27,6 +27,7 @@
 #include "libset.h"
 #include "transforms.h"
 #include "gretl_xml.h"
+#include "matrix_extra.h"
 
 #define VDEBUG 0
 
@@ -153,7 +154,7 @@ static int VAR_add_XTX_matrix (GRETL_VAR *var)
     return err;
 }
 
-int nrestr (const GRETL_VAR *v) 
+int n_restricted_terms (const GRETL_VAR *v) 
 {
     int n = 0;
 
@@ -181,7 +182,8 @@ void gretl_VAR_clear (GRETL_VAR *var)
     var->detflags = 0;
     var->robust = 0;
     var->LBs = 0;
-    var->k = 0;
+    var->ycols = 0;
+    var->xcols = 0;
 
     var->lags = NULL;
     var->ylist = NULL;
@@ -449,16 +451,24 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 
 	if (v->xlist != NULL && !miss) {
 	    for (i=1; i<=v->xlist[0] && !miss; i++) {
-		if (na(Z[v->xlist[i]][t])) {
+		vi = v->xlist[i];
+		if (na(Z[vi][t])) {
 		    v->t1 += 1;
 		    miss = 1;
 		}
 	    }
 	}
 
+	/* Note that in estimating a VECM we need the first lag of the
+	   level of each restricted exogenous var to serve as
+	   dependent variable in one of the initial OLS equations.
+	*/
+
+	s = t - 1;
 	if (v->rlist != NULL && !miss) {
 	    for (i=1; i<=v->rlist[0] && !miss; i++) {
-		if (na(Z[v->rlist[i]][t])) {
+		vi = v->rlist[i];
+		if (na(Z[vi][t]) || s < 0 || na(Z[vi][s])) {
 		    v->t1 += 1;
 		    miss = 1;
 		}
@@ -476,7 +486,8 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 	int miss = 0;
 
 	for (i=1; i<=v->ylist[0] && !miss; i++) {
-	    if (na(Z[v->ylist[i]][t])) {
+	    vi = v->ylist[i];
+	    if (na(Z[vi][t])) {
 		v->t2 -= 1;
 		miss = 1;
 	    }
@@ -484,7 +495,8 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 
 	if (v->xlist != NULL && !miss) {
 	    for (i=1; i<=v->xlist[0] && !miss; i++) {
-		if (na(Z[v->xlist[i]][t])) {
+		vi = v->xlist[i];
+		if (na(Z[vi][t])) {
 		    v->t2 -= 1;
 		    miss = 1;
 		}
@@ -493,7 +505,8 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 
 	if (v->rlist != NULL && !miss) {
 	    for (i=1; i<=v->rlist[0] && !miss; i++) {
-		if (na(Z[v->rlist[i]][t])) {
+		vi = v->rlist[i];
+		if (na(Z[vi][t])) {
 		    v->t2 -= 1;
 		    miss = 1;
 		}
@@ -510,14 +523,16 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
     for (t=v->t1; t<=v->t2 && !err; t++) {
 
 	for (i=1; i<=v->ylist[0] && !err; i++) {
-	    if (na(Z[v->ylist[i]][t])) {
+	    vi = v->ylist[i];
+	    if (na(Z[vi][t])) {
 		err = E_MISSDATA;
 	    }
 	}
 
 	if (v->xlist != NULL && !err) {
 	    for (i=1; i<=v->xlist[0] && !err; i++) {
-		if (na(Z[v->xlist[i]][t])) {
+		vi = v->xlist[i];
+		if (na(Z[vi][t])) {
 		    err = E_MISSDATA;
 		}
 	    }
@@ -525,13 +540,18 @@ static int VAR_set_sample (GRETL_VAR *v, const double **Z,
 
 	if (v->rlist != NULL && !err) {
 	    for (i=1; i<=v->rlist[0] && !err; i++) {
-		if (na(Z[v->rlist[i]][t])) {
+		vi = v->rlist[i];
+		if (na(Z[vi][t])) {
 		    err = E_MISSDATA;
 		}
 	    }
 	}
     }
 
+#if 0
+    fprintf(stderr, "VAR_set_sample: t1=%d, t2=%d\n", v->t1, v->t2); 
+#endif   
+
     return err;
 }
 
@@ -611,14 +631,16 @@ static int VAR_add_basic_matrices (GRETL_VAR *v, gretlopt opt)
     } 
     
     /* record max. cols of Y matrix */
-    v->k = k;
+    v->ycols = k;
 
     if (v->ncoeff > 0) {
 	v->X = gretl_matrix_alloc(v->T, v->ncoeff);
 	v->B = gretl_matrix_alloc(v->ncoeff, k);
 	if (v->X == NULL || v->B == NULL) { 
 	    return E_ALLOC;
-	}  
+	}
+	/* record max. cols of X matrix */
+	v->xcols = v->ncoeff;
     }
 
     return 0;
@@ -767,6 +789,7 @@ static void johansen_info_free (JohansenInfo *jv)
     gretl_matrix_free(jv->S11);
     gretl_matrix_free(jv->S01);
 
+    gretl_matrix_free(jv->evals);
     gretl_matrix_free(jv->Beta);
     gretl_matrix_free(jv->Alpha);
     gretl_matrix_free(jv->Bvar);
@@ -1388,6 +1411,68 @@ gretl_VAR_get_fitted_matrix (const GRETL_VAR *var)
     return Yh;
 }
 
+gretl_matrix *
+gretl_VAR_get_vma_matrix (const GRETL_VAR *var, const DATAINFO *pdinfo,
+			  int *err)
+{
+    int h = default_VAR_horizon(pdinfo);
+    int ar, n = var->neqns;
+    int n2 = n * n;
+    gretl_matrix *VMA = NULL;
+    gretl_matrix *Tmp1, *Tmp2;
+    double x;
+    int i, j, ii, jj;
+
+    if (var->A == NULL) {
+	/* companion matrix is absent */
+	*err = E_BADSTAT;
+	return NULL;
+    }
+
+    ar = var->A->rows;
+
+    Tmp1 = gretl_identity_matrix_new(ar);
+    Tmp2 = gretl_matrix_alloc(ar, ar);
+    if (Tmp1 == NULL || Tmp2 == NULL) {
+	*err = E_ALLOC;
+	goto bailout;
+    }
+
+    VMA = gretl_zero_matrix_new(h, n2);
+    if (VMA == NULL) {
+	*err = E_ALLOC;
+	goto bailout;
+    }  
+
+    /* compose first row of VMA = vec(I(n))' */
+    for (i=0; i<n2; i+=n+1) {
+	gretl_matrix_set(VMA, 0, i, 1.0);
+    }
+
+    for (i=1; i<h; i++) {
+	/* Tmp <-  A * Tmp */
+	gretl_matrix_multiply(var->A, Tmp1, Tmp2);
+	gretl_matrix_copy_values(Tmp1, Tmp2);
+	/* VMA |= vec(Tmp[1:n,1:n])' */
+	ii = jj = 0;
+	for (j=0; j<n2; j++) {
+	    x = gretl_matrix_get(Tmp1, ii++, jj);
+	    gretl_matrix_set(VMA, i, j, x);
+	    if (ii == n) {
+		jj++;
+		ii = 0;
+	    }
+	}
+    }
+
+ bailout:
+
+    gretl_matrix_free(Tmp1);
+    gretl_matrix_free(Tmp2);
+
+    return VMA;
+}
+
 int gretl_VAR_get_variable_number (const GRETL_VAR *var, int k)
 {
     int vnum = 0;
@@ -1454,7 +1539,7 @@ int default_VAR_horizon (const DATAINFO *pdinfo)
     return h;
 }
 
-gretl_matrix *reorder_responses (GRETL_VAR *var, int *err)
+gretl_matrix *reorder_responses (const GRETL_VAR *var, int *err)
 {
     gretl_matrix *S, *C;
     int i, j, r, c;
@@ -1497,7 +1582,7 @@ gretl_matrix *reorder_responses (GRETL_VAR *var, int *err)
 
 static gretl_matrix *
 gretl_VAR_get_point_responses (GRETL_VAR *var, int targ, int shock,
-			       int periods) 
+			       int periods, int *err) 
 {
     int rows = var->neqns * effective_order(var);
     gretl_matrix *rtmp = NULL;
@@ -1505,26 +1590,29 @@ gretl_VAR_get_point_responses (GRETL_VAR *var, int targ, int shock,
     gretl_matrix *resp = NULL;
     gretl_matrix *C = var->C;
     double x;
-    int t, err = 0;
+    int t;
 
     if (shock >= var->neqns) {
 	fprintf(stderr, "Shock variable out of bounds\n");
+	*err = E_DATA;
 	return NULL;
     }  
 
     if (targ >= var->neqns) {
 	fprintf(stderr, "Target variable out of bounds\n");
+	*err = E_DATA;
 	return NULL;
     } 
 
     if (periods <= 0) {
 	fprintf(stderr, "Invalid number of periods\n");
+	*err = E_DATA;
 	return NULL;
     }
 
     if (var->ord != NULL) {
-	C = reorder_responses(var, &err);
-	if (err) {
+	C = reorder_responses(var, err);
+	if (*err) {
 	    return NULL;
 	}
     } 
@@ -1534,7 +1622,7 @@ gretl_VAR_get_point_responses (GRETL_VAR *var, int targ, int shock,
     ctmp = gretl_matrix_alloc(rows, var->neqns);
 
     if (resp == NULL || rtmp == NULL || ctmp == NULL) {
-	err = E_ALLOC;
+	*err = E_ALLOC;
 	goto bailout;
     }
 
@@ -1560,7 +1648,7 @@ gretl_VAR_get_point_responses (GRETL_VAR *var, int targ, int shock,
 	gretl_matrix_free(C);
     }
 
-    if (err && resp != NULL) {
+    if (*err && resp != NULL) {
 	gretl_matrix_free(resp);
 	resp = NULL;
     }
@@ -1603,21 +1691,36 @@ gretl_VAR_get_impulse_response (GRETL_VAR *var,
 				int targ, int shock, 
 				int periods, double alpha,
 				const double **Z,
-				const DATAINFO *pdinfo)
+				const DATAINFO *pdinfo,
+				int *err)
 {
     gretl_matrix *point = NULL;
     gretl_matrix *full = NULL;
     gretl_matrix *ret = NULL;
     int i;
 
-    point = gretl_VAR_get_point_responses(var, targ, shock, periods);
+    if (periods == 0) {
+	if (pdinfo != NULL) {
+	    periods = default_VAR_horizon(pdinfo);
+	} else {
+	    *err = E_DATA;
+	    return NULL;
+	}
+    }
 
-    if (Z == NULL) {
-	/* no data matrix provided: just return point estimate */
+    if (alpha != 0 && (alpha < 0.01 || alpha > 0.6)) {
+	*err = E_DATA;
+    }
+
+    point = gretl_VAR_get_point_responses(var, targ, shock, periods, err);
+
+    if (Z == NULL || alpha == 0.0) {
+	/* no data matrix provided, or no alpha given: 
+	   just return point estimate */
 	ret = point;
     } else if (point != NULL) {
 	full = irf_bootstrap(var, targ, shock, periods, 
-			     alpha, Z, pdinfo);
+			     alpha, Z, pdinfo, err);
 	if (full != NULL) {
 	    double p;
 
@@ -1689,7 +1792,8 @@ gretl_VAR_get_fcast_se (GRETL_VAR *var, int periods)
 }
 
 gretl_matrix *
-gretl_VAR_get_fcast_decomp (GRETL_VAR *var, int targ, int periods,
+gretl_VAR_get_fcast_decomp (const GRETL_VAR *var, 
+			    int targ, int periods,
 			    int *err)
 {
     int n = var->neqns;
@@ -1801,6 +1905,71 @@ gretl_VAR_get_fcast_decomp (GRETL_VAR *var, int targ, int periods,
     return vd;
 }
 
+gretl_matrix *
+gretl_VAR_get_FEVD_matrix (const GRETL_VAR *var, 
+			   int targ, int horizon, 
+			   const DATAINFO *pdinfo,
+			   int *err)
+{
+    gretl_matrix *vd, *V;
+    double vjk;
+    int h = horizon;
+    int n = var->neqns;
+    int imin, imax;
+    int i, j, k, kk;
+
+    if (h <= 0) {
+	h = default_VAR_horizon(pdinfo);
+    }
+
+    if (targ < 0) {
+	/* doing the whole thing */
+	k = n * n;
+	imin = 0;
+	imax = n;
+    } else {
+	/* doing one specific equation */
+	k = n;
+	imin = targ;
+	imax = targ + 1;
+    }
+    
+    V = gretl_matrix_alloc(h, k);
+    if (V == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    kk = 0;
+    for (i=imin; i<imax && !*err; i++) {
+	vd = gretl_VAR_get_fcast_decomp(var, i, h, err);
+	if (!*err) {
+	    for (k=0; k<n; k++) {
+		for (j=0; j<h; j++) {
+		    vjk = gretl_matrix_get(vd, j, k);
+		    gretl_matrix_set(V, j, kk, vjk / 100.0);
+		}
+		kk++;
+	    }
+	    gretl_matrix_free(vd);
+	}
+    }
+
+    if (*err) {
+	gretl_matrix_free(V);
+	V = NULL;
+    }
+    
+    return V;
+}
+
+gretl_matrix *
+gretl_VAR_get_full_FEVD_matrix (const GRETL_VAR *var, const DATAINFO *pdinfo,
+				int *err)
+{
+    return gretl_VAR_get_FEVD_matrix(var, -1, -1, pdinfo, err);
+}
+
 int var_max_order (const int *list, const DATAINFO *pdinfo)
 {
     int T = pdinfo->t2 - pdinfo->t1 + 1;
@@ -2099,6 +2268,8 @@ void gretl_VAR_param_names (GRETL_VAR *v, char **params,
     }
 }
 
+/* public because called from irfboot.c */
+
 void VAR_write_A_matrix (GRETL_VAR *v)
 {
     int i, ii, j, k, lag;
@@ -2158,7 +2329,6 @@ int transcribe_VAR_models (GRETL_VAR *var,
     char **params = NULL;
     int yno, N = pdinfo->n;
     int ecm = (var->ci == VECM);
-    const double *y;
     double x;
     int i, j, jmax;
     int err = 0;
@@ -2174,7 +2344,6 @@ int transcribe_VAR_models (GRETL_VAR *var,
 
     for (i=0; i<var->neqns && !err; i++) {
 	yno = var->ylist[i+1];
-	y = Z[yno];
 
 	pmod = var->models[i];
 	pmod->ID = i + 1;
@@ -2220,8 +2389,15 @@ int transcribe_VAR_models (GRETL_VAR *var,
 	    for (j=0; j<jmax; j++) {
 		pmod->coeff[j] = gretl_matrix_get(var->B, j, i);
 		if (XTX != NULL) {
-		    x = gretl_matrix_get(XTX, j, j);
-		    pmod->sderr[j] = pmod->sigma * sqrt(x);
+		    if (XTX->rows <= var->ncoeff) {
+			x = gretl_matrix_get(XTX, j, j);
+			pmod->sderr[j] = pmod->sigma * sqrt(x);
+		    } else {
+			int jj = i * var->ncoeff + j;
+
+			x = gretl_matrix_get(XTX, jj, jj);
+			pmod->sderr[j] = pmod->sigma * sqrt(x);
+		    }
 		}
 	    }
 	}
@@ -2363,9 +2539,9 @@ print_johansen_sigmas (const JohansenInfo *jv, PRN *prn)
 {
     int i, j;
 
-    pprintf(prn, "\n%s\n\n", _("Sample variance-covariance matrices for residuals"));
+    pprintf(prn, "%s\n\n", _("Sample variance-covariance matrices for residuals"));
 
-    pprintf(prn, " %s\n\n", _("VAR system in first differences"));
+    pprintf(prn, " %s (S00)\n\n", _("VAR system in first differences"));
     for (i=0; i<jv->S00->rows; i++) {
 	for (j=0; j<jv->S00->rows; j++) {
 	    pprintf(prn, "%#12.5g", gretl_matrix_get(jv->S00, i, j));
@@ -2373,7 +2549,7 @@ print_johansen_sigmas (const JohansenInfo *jv, PRN *prn)
 	pputc(prn, '\n');
     }
 
-    pprintf(prn, "\n %s\n\n", _("System with levels as dependent variable"));
+    pprintf(prn, "\n %s (S11)\n\n", _("System with levels as dependent variable"));
     for (i=0; i<jv->S11->rows; i++) {
 	for (j=0; j<jv->S11->rows; j++) {
 	    pprintf(prn, "%#12.5g", gretl_matrix_get(jv->S11, i, j));
@@ -2381,13 +2557,15 @@ print_johansen_sigmas (const JohansenInfo *jv, PRN *prn)
 	pputc(prn, '\n');
     } 
     
-    pprintf(prn, "\n %s\n\n", _("Cross-products"));
+    pprintf(prn, "\n %s (S01)\n\n", _("Cross-products"));
     for (i=0; i<jv->S01->rows; i++) {
 	for (j=0; j<jv->S01->cols; j++) {
 	    pprintf(prn, "%#12.5g", gretl_matrix_get(jv->S01, i, j));
 	}
 	pputc(prn, '\n');
-    }     
+    }
+
+    pputc(prn, '\n');
 }
 
 /* called from gretl_restriction_finalize() if the target
@@ -2445,8 +2623,7 @@ johansen_test_complete (GRETL_VAR *jvar, const DATAINFO *pdinfo,
 }
 
 static int 
-johansen_estimate_complete (GRETL_VAR *jvar, 
-			    gretl_restriction *rset,
+johansen_estimate_complete (GRETL_VAR *jvar, gretl_restriction *rset,
 			    const double **Z, const DATAINFO *pdinfo, 
 			    PRN *prn)
 {
@@ -2456,7 +2633,7 @@ johansen_estimate_complete (GRETL_VAR *jvar,
     int err = 0;
 
     gretl_error_clear();
-    
+
     jfun = get_plugin_function("johansen_estimate", &handle);
 
     if (jfun == NULL) {
@@ -2473,7 +2650,7 @@ static int
 allocate_johansen_residual_matrices (GRETL_VAR *v)
 {
     int p = v->neqns;
-    int p1 = p + nrestr(v);
+    int p1 = p + n_restricted_terms(v);
 
     clear_gretl_matrix_err();
 
@@ -2537,82 +2714,95 @@ static void johansen_degenerate_stage_1 (GRETL_VAR *v,
    Then compute S00, S11, S01.
 */
 
-int johansen_stage_1 (GRETL_VAR *jvar, const double **Z, 
-		      const DATAINFO *pdinfo)
+int johansen_stage_1 (GRETL_VAR *v, const double **Z, 
+		      const DATAINFO *pdinfo, 
+		      gretlopt opt, PRN *prn)
 {
-    int nr = nrestr(jvar);
     int err;
 
-    err = allocate_johansen_residual_matrices(jvar); 
+    err = allocate_johansen_residual_matrices(v); 
     if (err) {
 	return err;
     }
 
-    if (jvar->ncoeff == 0) {
+    if (v->ncoeff == 0) {
 	/* nothing to concentrate out */
-	johansen_degenerate_stage_1(jvar, Z);
+	if (opt & OPT_V) {
+	    pputs(prn, "\nNo initial VAR estimation is required\n\n");
+	}
+	johansen_degenerate_stage_1(v, Z);
 	goto finish;
     }
 
-    if (nr > 0) {
+    if (v->Y->cols > v->neqns) {
 	/* shrink to size */
-	gretl_matrix_reuse(jvar->Y, -1, jvar->neqns);
-	gretl_matrix_reuse(jvar->B, -1, jvar->neqns);
+	gretl_matrix_reuse(v->Y, -1, v->neqns);
+	gretl_matrix_reuse(v->B, -1, v->neqns);
     }
 
-    VAR_fill_Y(jvar, DIFF, Z);
+    VAR_fill_Y(v, DIFF, Z);
 
     /* (1) VAR in first differences */
 
 #if JVAR_USE_SVD
-    err = gretl_matrix_multi_SVD_ols(jvar->Y, jvar->X, jvar->B, 
-				     jvar->jinfo->R0, NULL);
+    err = gretl_matrix_multi_SVD_ols(v->Y, v->X, v->B, 
+				     v->jinfo->R0, NULL);
 #else
-    err = gretl_matrix_multi_ols(jvar->Y, jvar->X, jvar->B, 
-				 jvar->jinfo->R0, NULL);
+    err = gretl_matrix_multi_ols(v->Y, v->X, v->B, 
+				 v->jinfo->R0, NULL);
 #endif
 
-    if (nr > 0) {
+    if (!err && (opt & OPT_V)) {
+	gretl_matrix_print_to_prn(v->B, "\nCoefficients, VAR in differences", 
+				  prn);
+    }
+
+    if (v->ycols > v->Y->cols) {
 	/* re-expand to full size */
-	gretl_matrix_reuse(jvar->Y, -1, jvar->neqns + nr);
-	gretl_matrix_reuse(jvar->B, -1, jvar->neqns + nr);
+	gretl_matrix_reuse(v->Y, -1, v->ycols);
+	gretl_matrix_reuse(v->B, -1, v->ycols);
     }
 
     /* (2) System with lagged levels on LHS (may include
-       restricted const or trend in last column)
+       restricted terms in trailing columns)
     */
 
-    VAR_fill_Y(jvar, LAGS, Z);
+    VAR_fill_Y(v, LAGS, Z);
 
 #if JVAR_USE_SVD
-    err = gretl_matrix_multi_SVD_ols(jvar->Y, jvar->X, jvar->B, 
-				     jvar->jinfo->R1, NULL);
+    err = gretl_matrix_multi_SVD_ols(v->Y, v->X, v->B, 
+				     v->jinfo->R1, NULL);
 #else
-    err = gretl_matrix_multi_ols(jvar->Y, jvar->X, jvar->B, 
-				 jvar->jinfo->R1, NULL);
-#endif 
+    err = gretl_matrix_multi_ols(v->Y, v->X, v->B, 
+				 v->jinfo->R1, NULL);
+#endif
 
-    if (nr > 0) {
+    if (!err && (opt & OPT_V)) {
+	gretl_matrix_print_to_prn(v->B, "Coefficients, eqns in lagged levels", 
+				  prn);
+    }
+
+    if (v->Y->cols > v->neqns) {
 	/* shrink to "final" size */
-	gretl_matrix_reuse(jvar->Y, -1, jvar->neqns);
-	gretl_matrix_reuse(jvar->B, -1, jvar->neqns);
+	gretl_matrix_reuse(v->Y, -1, v->neqns);
+	gretl_matrix_reuse(v->B, -1, v->neqns);
     }
 
  finish:
 
-    gretl_matrix_multiply_mod(jvar->jinfo->R0, GRETL_MOD_TRANSPOSE,
-			      jvar->jinfo->R0, GRETL_MOD_NONE,
-			      jvar->jinfo->S00, GRETL_MOD_NONE);
-    gretl_matrix_multiply_mod(jvar->jinfo->R1, GRETL_MOD_TRANSPOSE,
-			      jvar->jinfo->R1, GRETL_MOD_NONE,
-			      jvar->jinfo->S11, GRETL_MOD_NONE);
-    gretl_matrix_multiply_mod(jvar->jinfo->R0, GRETL_MOD_TRANSPOSE,
-			      jvar->jinfo->R1, GRETL_MOD_NONE,
-			      jvar->jinfo->S01, GRETL_MOD_NONE);
+    gretl_matrix_multiply_mod(v->jinfo->R0, GRETL_MOD_TRANSPOSE,
+			      v->jinfo->R0, GRETL_MOD_NONE,
+			      v->jinfo->S00, GRETL_MOD_NONE);
+    gretl_matrix_multiply_mod(v->jinfo->R1, GRETL_MOD_TRANSPOSE,
+			      v->jinfo->R1, GRETL_MOD_NONE,
+			      v->jinfo->S11, GRETL_MOD_NONE);
+    gretl_matrix_multiply_mod(v->jinfo->R0, GRETL_MOD_TRANSPOSE,
+			      v->jinfo->R1, GRETL_MOD_NONE,
+			      v->jinfo->S01, GRETL_MOD_NONE);
 
-    gretl_matrix_divide_by_scalar(jvar->jinfo->S00, jvar->T);
-    gretl_matrix_divide_by_scalar(jvar->jinfo->S11, jvar->T);
-    gretl_matrix_divide_by_scalar(jvar->jinfo->S01, jvar->T);
+    gretl_matrix_divide_by_scalar(v->jinfo->S00, v->T);
+    gretl_matrix_divide_by_scalar(v->jinfo->S11, v->T);
+    gretl_matrix_divide_by_scalar(v->jinfo->S01, v->T);
 
 #if VDEBUG
     fprintf(stderr, "johansen_stage_1: returning err = %d\n", err);
@@ -2674,6 +2864,7 @@ johansen_info_new (GRETL_VAR *var, int rank, gretlopt opt)
     jv->S00 = NULL;
     jv->S11 = NULL;
     jv->S01 = NULL;
+    jv->evals = NULL;
     jv->Beta = NULL;
     jv->Alpha = NULL;
     jv->Bse = NULL;
@@ -2704,7 +2895,7 @@ static void coint_test_header (const GRETL_VAR *v,
 {
     char stobs[OBSLEN], endobs[OBSLEN];
 
-    pprintf(prn, "%s:\n", _("Johansen test"));
+    pprintf(prn, "\n%s:\n", _("Johansen test"));
     pprintf(prn, "%s = %d\n", _("Number of equations"), v->neqns);
     pprintf(prn, "%s = %d\n", _("Lag order"), v->order + 1);
     pprintf(prn, "%s: %s - %s (T = %d)\n", _("Estimation period"),
@@ -2741,17 +2932,17 @@ johansen_driver (GRETL_VAR *jvar,
 		 gretlopt opt, PRN *prn)
 {
     int r = jrank(jvar);
-    
-    jvar->err = johansen_stage_1(jvar, Z, pdinfo); 
-    if (jvar->err) {
-	return jvar->err;
-    }
 
     if (r == 0) {
 	/* doing cointegration test */
 	coint_test_header(jvar, pdinfo, prn);
     }
 
+    jvar->err = johansen_stage_1(jvar, Z, pdinfo, opt, prn); 
+    if (jvar->err) {
+	return jvar->err;
+    }
+
     if (opt & OPT_V) {
 	print_johansen_sigmas(jvar->jinfo, prn);
     }
@@ -2831,7 +3022,8 @@ GRETL_VAR *johansen_test (int order, const int *list,
  * include centered seasonals; %OPT_N: no constant; %OPT_R:
  * restricted constant; %OPT_T: constant and unrestricted trend
  * (note: default "case" is unrestricted constant);
- * %OPT_V: produce verbose results; %OPT_Q: just print the tests.
+ * %OPT_V: produce verbose results; %OPT_Q: just print the tests;
+ * %OPT_S: don't print anything.
  * @prn: gretl printing struct.
  *
  * Carries out the Johansen test for cointegration and prints the
@@ -2842,14 +3034,15 @@ GRETL_VAR *johansen_test (int order, const int *list,
  */
 
 int johansen_test_simple (int order, const int *list, 
-			  const double **Z, const DATAINFO *pdinfo, 
+			  const double **Z, const DATAINFO *pdinfo,
 			  gretlopt opt, PRN *prn)
 {
     GRETL_VAR *jvar = NULL;
     int err = 0;
 
     jvar = johansen_wrapper(VECM_CTEST, order, 0, NULL, list, NULL, 
-			    Z, pdinfo, opt, prn, &err);
+			    Z, pdinfo, opt, (opt & OPT_S)? NULL : prn, 
+			    &err);
 
     if (jvar != NULL) {
 	gretl_VAR_free(jvar);
@@ -3259,38 +3452,22 @@ int restricted_VECM (const GRETL_VAR *vecm)
 
 const gretl_matrix *gretl_VECM_R_matrix (const GRETL_VAR *vecm)
 {
-    if (vecm->jinfo != NULL && vecm->jinfo->R != NULL) {
-	return vecm->jinfo->R;
-    }
-
-    return NULL;
+    return (vecm->jinfo != NULL)? vecm->jinfo->R : NULL; 
 }
 
 const gretl_matrix *gretl_VECM_q_matrix (const GRETL_VAR *vecm)
 {
-    if (vecm->jinfo != NULL && vecm->jinfo->q != NULL) {
-	return vecm->jinfo->q;
-    }
-
-    return NULL;
+    return (vecm->jinfo != NULL)? vecm->jinfo->q : NULL;
 }   
 
 const gretl_matrix *gretl_VECM_Ra_matrix (const GRETL_VAR *vecm)
 {
-    if (vecm->jinfo != NULL && vecm->jinfo->Ra != NULL) {
-	return vecm->jinfo->Ra;
-    }
-
-    return NULL;
+    return (vecm->jinfo != NULL)? vecm->jinfo->Ra : NULL;    
 }
 
 const gretl_matrix *gretl_VECM_qa_matrix (const GRETL_VAR *vecm)
 {
-    if (vecm->jinfo != NULL && vecm->jinfo->qa != NULL) {
-	return vecm->jinfo->qa;
-    }
-
-    return NULL;
+    return (vecm->jinfo != NULL)? vecm->jinfo->qa : NULL;
 }   
 
 double *gretl_VAR_get_series (const GRETL_VAR *var, const DATAINFO *pdinfo, 
@@ -3347,41 +3524,163 @@ VAR_matrix_from_models (const GRETL_VAR *var, int idx, int *err)
     double x;
     int i, j;
 
-    m = gretl_matrix_alloc(var->models[0]->ncoeff, var->neqns);
+    if (idx == M_VECG && var->ci != VECM) {
+	*err = E_BADSTAT;
+	return NULL;
+    }
+
+    if (idx == M_VECG) {
+	m = gretl_matrix_alloc(var->neqns, var->neqns * var->order);
+    } else {
+	m = gretl_matrix_alloc(var->models[0]->ncoeff, var->neqns);
+    }
+
     if (m == NULL) {
 	*err = E_ALLOC;
 	return NULL;
     }
 
-    for (j=0; j<var->neqns; j++) {
-	pmod = var->models[j];
-	for (i=0; i<pmod->ncoeff; i++) {
-	    if (idx == M_COEFF) {
-		x = pmod->coeff[i];
-	    } else {
-		x = pmod->sderr[i];
+    if (idx == M_VECG) {
+	int k, mcol, cnum;
+
+	for (j=0; j<var->neqns; j++) {
+	    pmod = var->models[j];
+	    mcol = 0;
+	    for (i=0; i<var->order; i++) {
+		cnum = pmod->ifc + i;
+		for (k=0; k<var->neqns; k++) {
+		    x = pmod->coeff[cnum];
+		    gretl_matrix_set(m, j, mcol++, x);
+		    cnum += var->order;
+		}
+	    }
+	}
+    } else {	
+	for (j=0; j<var->neqns; j++) {
+	    pmod = var->models[j];
+	    for (i=0; i<pmod->ncoeff; i++) {
+		if (idx == M_COEFF) {
+		    x = pmod->coeff[i];
+		} else {
+		    x = pmod->sderr[i];
+		}
+		gretl_matrix_set(m, i, j, x);
 	    }
-	    gretl_matrix_set(m, i, j, x);
 	}
     }
 
     return m;
 }
 
-static gretl_matrix *VECM_get_EC_matrix (const GRETL_VAR *v, int *err)
+static gretl_matrix *alt_VECM_get_EC_matrix (const GRETL_VAR *vecm,
+					     const double **Z, 
+					     const DATAINFO *pdinfo, 
+					     int *err)
+{
+    const gretl_matrix *B = vecm->jinfo->Beta;
+    int r = jrank(vecm);
+    gretl_matrix *EC = NULL;
+    double xti, xj;
+    int s, t, T;
+    int i, j, k;
+
+    if (Z == NULL || pdinfo == NULL) {
+	*err = E_BADSTAT;
+	return NULL;
+    }	
+
+    for (i=1; i<=vecm->ylist[0]; i++) {
+	if (vecm->ylist[i] >= pdinfo->v) {
+	    *err = E_DATA;
+	    return NULL;
+	}
+    }
+
+    T = vecm->t2 - vecm->t1 + 1;
+
+    EC = gretl_matrix_alloc(T, r);
+    if (EC == NULL) {
+	*err = E_ALLOC;
+	return NULL;
+    }
+
+    s = 0;
+
+    for (t=vecm->t1; t<=vecm->t2; t++) {
+	for (j=0; j<r; j++) { 
+	    xj = 0.0;
+	    /* beta * X(t-1) */
+	    k = 0;
+	    for (i=0; i<vecm->neqns; i++) {
+		xti = Z[vecm->ylist[i+1]][t-1];
+		if (na(xti)) {
+		    xj = NADBL;
+		    break;
+		}
+		xj += xti * gretl_matrix_get(B, k++, j);
+	    }
+
+	    /* restricted const or trend */
+	    if (auto_restr(vecm) && !na(xj)) {
+		xti = gretl_matrix_get(B, k++, j);
+		if (jcode(vecm) == J_REST_TREND) {
+		    xti *= t;
+		}
+		xj += xti;
+	    }
+
+	    /* restricted exog vars */
+	    if (vecm->rlist != NULL && !na(xj)) {
+		for (i=0; i<vecm->rlist[0]; i++) {
+		    xti = Z[vecm->rlist[i+1]][t-1];
+		    if (na(xti)) {
+			xj = NADBL;
+			break;
+		    }
+		    xj += xti * gretl_matrix_get(B, k++, j);
+		}
+	    }
+
+	    if (na(xj)) {
+		gretl_matrix_set(EC, s, j, M_NA);
+	    } else {
+		gretl_matrix_set(EC, s, j, xj);
+	    }
+	}
+	s++;
+    }
+
+    gretl_matrix_set_t1(EC, vecm->t1);
+    gretl_matrix_set_t2(EC, vecm->t2);
+	
+    return EC;
+}
+
+gretl_matrix *VECM_get_EC_matrix (const GRETL_VAR *v, 
+				  const double **Z, 
+				  const DATAINFO *pdinfo, 
+				  int *err)
 {
     gretl_matrix *EC = NULL;
     double x;
     int rank, k, k0;
     int j, t, T;
 
+    rank = jrank(v);
+    if (rank == 0) {
+	*err = E_BADSTAT;
+	return NULL;
+    }	
+
     if (v->X == NULL) {
 	fprintf(stderr, "VECM_get_EC_matrix: v->X is NULL\n");
 	*err = E_BADSTAT;
 	return NULL;
-    }
+    } else if (v->X->cols < v->ncoeff) {
+	fprintf(stderr, "VECM_get_EC_matrix: v->X is short of cols\n");
+	return alt_VECM_get_EC_matrix(v, Z, pdinfo, err);
+    }	
 
-    rank = jrank(v);
     T = v->X->rows;
 
     EC = gretl_matrix_alloc(T, rank);
@@ -3407,7 +3706,8 @@ static gretl_matrix *VECM_get_EC_matrix (const GRETL_VAR *v, int *err)
 
 #define vecm_matrix(i) (i == M_JALPHA || i == M_JBETA || \
                         i == M_JVBETA || i == M_JS00 || \
-                        i == M_JS11 || i == M_JS01 || i == M_EC)
+                        i == M_JS11 || i == M_JS01 || \
+			i == M_EC || i == M_EVALS)
 
 gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx, 
 				    int *err)
@@ -3428,7 +3728,7 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
 	copy = 0;
     } else if (idx == M_COMPAN) {
 	src = var->A;
-    } else if (idx == M_COEFF || idx == M_SE) {
+    } else if (idx == M_COEFF || idx == M_SE || idx == M_VECG) {
 	M = VAR_matrix_from_models(var, idx, err);
 	copy = 0;
     } else if (idx == M_XTXINV) {
@@ -3438,6 +3738,9 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
     } else if (vecm_matrix(idx)) {
 	if (var->jinfo != NULL) {
 	    switch (idx) {
+	    case M_EVALS: 
+		src = var->jinfo->evals;
+		break;
 	    case M_JALPHA: 
 		src = var->jinfo->Alpha;
 		break;
@@ -3457,7 +3760,7 @@ gretl_matrix *gretl_VAR_get_matrix (const GRETL_VAR *var, int idx,
 		src = var->jinfo->S01;
 		break;
 	    case M_EC:
-		M = VECM_get_EC_matrix(var, err);
+		M = VECM_get_EC_matrix(var, NULL, NULL, err);
 		copy = 0;
 		break;
 	    }
@@ -3542,7 +3845,7 @@ double *gretl_VECM_get_EC (GRETL_VAR *vecm, int j, const double **Z,
 	}
 
 	/* restricted exog vars */
-	if (vecm->rlist != NULL) {
+	if (vecm->rlist != NULL && !na(x[t])) {
 	    for (i=0; i<vecm->rlist[0]; i++) {
 		xti = Z[vecm->rlist[i+1]][t-1];
 		if (na(xti)) {
@@ -3624,6 +3927,8 @@ static int VAR_retrieve_jinfo (xmlNodePtr node, xmlDocPtr doc,
 		    jinfo->S11 = gretl_xml_get_matrix(cur, doc, &err);
 		} else if (!strcmp(mname, "Suv")) {
 		    jinfo->S01 = gretl_xml_get_matrix(cur, doc, &err);
+		} else if (!strcmp(mname, "evals")) {
+		    jinfo->evals = gretl_xml_get_matrix(cur, doc, &err);
 		} else if (!strcmp(mname, "Beta")) {
 		    jinfo->Beta = gretl_xml_get_matrix(cur, doc, &err);
 		} else if (!strcmp(mname, "Alpha")) {
@@ -3949,6 +4254,7 @@ static void johansen_serialize (JohansenInfo *j, FILE *fp)
     gretl_xml_put_matrix(j->S00, "Suu", fp);
     gretl_xml_put_matrix(j->S11, "Svv", fp);
     gretl_xml_put_matrix(j->S01, "Suv", fp);
+    gretl_xml_put_matrix(j->evals, "evals", fp);
     gretl_xml_put_matrix(j->Beta, "Beta", fp);
     gretl_xml_put_matrix(j->Alpha, "Alpha", fp);
     gretl_xml_put_matrix(j->Bvar, "Bvar", fp);
