@@ -43,7 +43,8 @@ struct GRETL_VAR_ {
     int df;              /* T - average coeffs per equation */
     int ifc;             /* equations include a constant (1) or not (0) */
     int ncoeff;          /* total coefficients per equation */
-    int k;               /* maximum columns of Y (VECM special) */
+    int ycols;           /* maximum columns of Y (VECM special) */
+    int xcols;           /* maximum columns of X (VECM special) */
     int *lags;           /* list of specific lags */
     int *ylist;          /* list of stochastic vars */
     int *xlist;          /* list of exogenous variables */
@@ -98,9 +99,30 @@ const gretl_matrix *
 gretl_VAR_get_residual_matrix (const GRETL_VAR *var);
 
 gretl_matrix *
-gretl_VAR_get_fcast_decomp (GRETL_VAR *var, int targ, int periods,
+gretl_VAR_get_fcast_decomp (const GRETL_VAR *var, 
+			    int targ, int periods,
 			    int *errp);
 
+gretl_matrix *
+gretl_VAR_get_vma_matrix (const GRETL_VAR *var, const DATAINFO *pdinfo,
+			  int *err);
+
+gretl_matrix *gretl_VAR_get_FEVD_matrix (const GRETL_VAR *var, 
+					 int targ, int horizon, 
+					 const DATAINFO *pdinfo,
+					 int *err);
+
+gretl_matrix *
+gretl_VAR_get_full_FEVD_matrix (const GRETL_VAR *var, 
+				const DATAINFO *pdinfo,
+				int *err);
+
+gretl_matrix *
+VECM_get_EC_matrix (const GRETL_VAR *v, 
+		    const double **Z, 
+		    const DATAINFO *pdinfo, 
+		    int *err);
+
 int 
 gretl_VAR_do_error_decomp (const gretl_matrix *S, gretl_matrix *C,
 			   const gretl_matrix *ord);
@@ -136,7 +158,8 @@ gretl_VAR_get_impulse_response (GRETL_VAR *var,
 				int targ, int shock,
 				int periods, double alpha,
 				const double **Z,
-				const DATAINFO *pdinfo);
+				const DATAINFO *pdinfo,
+				int *err);
 
 void gretl_VAR_set_name (GRETL_VAR *var, const char *name);
 
