@@ -687,7 +687,6 @@ arbond_sample_check_unit (dpdinfo *dpd, const double **Z, int i,
     unit_info *unit = &dpd->ui[i];
     int t1i = dpd->T - 1, t2i = 0; 
     int tmin = dpd->p + 1;
-    int Ti = 0;
     int t;
 
 #if ADEBUG
@@ -721,8 +720,6 @@ arbond_sample_check_unit (dpdinfo *dpd, const double **Z, int i,
 	return 0;
     }
 
-    Ti = t2i - t1i + 1;
-
     if (unit->nobs > dpd->maxTi) {
 	dpd->maxTi = unit->nobs;
     }
@@ -742,7 +739,7 @@ arbond_sample_check_unit (dpdinfo *dpd, const double **Z, int i,
 
 #if ADEBUG
     fprintf(stderr, "t1 = %d, t2 = %d, Ti = %d, usable obs = %d\n", 
-	    t1i, t2i, Ti, unit->nobs);
+	    t1i, t2i, t2i - t1i + 1, unit->nobs);
 #endif
 
     return 0;
@@ -1682,7 +1679,6 @@ static int dpd_finalize_model (MODEL *pmod, dpdinfo *dpd,
 			       gretlopt opt)
 {
     char tmp[32];
-    char prefix;
     int i, j;
     int err = 0;
 
@@ -1718,21 +1714,39 @@ static int dpd_finalize_model (MODEL *pmod, dpdinfo *dpd,
 	return pmod->errcode;
     }
 
-    prefix = (dpd->flags & DPD_ORTHDEV)? 'O' : 'D';
-
-    j = 0;
-    if (dpd->laglist != NULL) {
-	for (i=1; i<=dpd->laglist[0]; i++) {
-	    pmod->params[j][0] = '\0';
-	    sprintf(tmp, "%c%.10s(-%d)", prefix, pdinfo->varname[dpd->yno], 
-		    dpd->laglist[i]);
-	    strncat(pmod->params[j++], tmp, 15);
-	}	
+    if (pmod->ci == DPANEL) {
+	j = 0;
+	if (dpd->laglist != NULL) {
+	    for (i=1; i<=dpd->laglist[0]; i++) {
+		pmod->params[j][0] = '\0';
+		sprintf(tmp, "%.10s(-%d)", pdinfo->varname[dpd->yno], 
+			dpd->laglist[i]);
+		strncat(pmod->params[j++], tmp, 15);
+	    }	
+	} else {
+	    for (i=0; i<dpd->p; i++) {
+		pmod->params[j][0] = '\0';
+		sprintf(tmp, "%.10s(-%d)", pdinfo->varname[dpd->yno], i+1);
+		strncat(pmod->params[j++], tmp, 15);
+	    }
+	}
     } else {
-	for (i=0; i<dpd->p; i++) {
-	    pmod->params[j][0] = '\0';
-	    sprintf(tmp, "%c%.10s(-%d)", prefix, pdinfo->varname[dpd->yno], i+1);
-	    strncat(pmod->params[j++], tmp, 15);
+	char prefix = (dpd->flags & DPD_ORTHDEV)? 'O' : 'D';
+
+	j = 0;
+	if (dpd->laglist != NULL) {
+	    for (i=1; i<=dpd->laglist[0]; i++) {
+		pmod->params[j][0] = '\0';
+		sprintf(tmp, "%c%.10s(-%d)", prefix, pdinfo->varname[dpd->yno], 
+			dpd->laglist[i]);
+		strncat(pmod->params[j++], tmp, 15);
+	    }	
+	} else {
+	    for (i=0; i<dpd->p; i++) {
+		pmod->params[j][0] = '\0';
+		sprintf(tmp, "%c%.10s(-%d)", prefix, pdinfo->varname[dpd->yno], i+1);
+		strncat(pmod->params[j++], tmp, 15);
+	    }
 	}
     }
 
@@ -1760,7 +1774,7 @@ static int dpd_finalize_model (MODEL *pmod, dpdinfo *dpd,
 	for (i=0; i<dpd->k; i++) {
 	    pmod->coeff[i] = dpd->beta->val[i];
 	}
-	err = gretl_model_write_vcv(pmod, dpd->vbeta);
+	err = gretl_model_write_vcv(pmod, dpd->vbeta, -1);
     }
 
     /* add uhat, yhat */
