@@ -828,8 +828,8 @@ static int arma_OPG_vcv (MODEL *pmod, kalman *K, double *b,
     double rcond;
     int err = 0;
 
-    G = build_score_matrix(b, k, T, kalman_arma_llt_callback, 
-			   K, &err);
+    G = numerical_score_matrix(b, k, T, kalman_arma_llt_callback, 
+			       K, &err);
     if (err) {
 	goto bailout;
     }
@@ -856,7 +856,7 @@ static int arma_OPG_vcv (MODEL *pmod, kalman *K, double *b,
 
     if (!err) {
 	gretl_matrix_multiply_by_scalar(V, s2);
-	err = gretl_model_write_vcv(pmod, V);
+	err = gretl_model_write_vcv(pmod, V, -1);
     }
 
  bailout:
@@ -1042,13 +1042,14 @@ static int kalman_arma_finish (MODEL *pmod, arma_info *ainfo,
 	gretl_matrix *Hinv;
 
 	kalman_do_ma_check = 0;
-	Hinv = numerical_hessian(b, ainfo->nc, kalman_arma_ll, K, &err);
+	Hinv = numerical_hessian_inverse(b, ainfo->nc, kalman_arma_ll, 
+					 K, &err);
 	kalman_do_ma_check = 1;
 	if (!err) {
 	    if (kopt & KALMAN_AVG_LL) {
 		gretl_matrix_divide_by_scalar(Hinv, ainfo->T);
 	    }
-	    err = gretl_model_write_vcv(pmod, Hinv);
+	    err = gretl_model_write_vcv(pmod, Hinv, -1);
 	    if (!err) {
 		gretl_model_set_vcv_info(pmod, VCV_ML, VCV_HESSIAN);
 	    }
@@ -1263,6 +1264,7 @@ static int kalman_arma (double *coeff,
     gretl_matrix *X = NULL;
     int r, k = 1 + ainfo->nexo; /* number of exog vars plus space for const */
     int fncount = 0, grcount = 0;
+    int use_newton = 0;
     double *b;
     int i, err = 0;
     
@@ -1330,8 +1332,8 @@ static int kalman_arma (double *coeff,
     if (err) {
 	fprintf(stderr, "kalman_new(): err = %d\n", err);
     } else {
-	int maxit, save_lbfgs = 0;
 	double toler;
+	int maxit;
 
 	kalman_attach_printer(K, ainfo->prn);
 	kalman_attach_data(K, kh);
@@ -1348,28 +1350,44 @@ static int kalman_arma (double *coeff,
 	    kalman_set_options(K, KALMAN_ARMA_LL);
 	}
 
-	save_lbfgs = libset_get_bool(USE_LBFGS);
-
-	if (save_lbfgs) {
-	    ainfo->pflags |= ARMA_LBFGS;
-	} else if (opt & OPT_L) {
-	    libset_set_bool(USE_LBFGS, 1);
-	    ainfo->pflags |= ARMA_LBFGS;
-	}
-
 	BFGS_defaults(&maxit, &toler, ARMA);
 
-	err = BFGS_max(b, ainfo->nc, maxit, toler, 
-		       &fncount, &grcount, kalman_arma_ll, C_LOGLIK,
-		       NULL, K, NULL, opt, ainfo->prn);
+	if (libset_get_int(GRETL_OPTIM) == OPTIM_NEWTON ||
+	    getenv("ARMA_USE_NEWTON") != NULL) {
+	    use_newton = 1;
+	}
+
+	if (use_newton) {
+	    double crittol = 1.0e-7;
+	    double gradtol = 1.0e-7;
+
+	    err = newton_raphson_max(b, ainfo->nc, maxit, 
+				     crittol, gradtol, &fncount, 
+				     C_LOGLIK, kalman_arma_ll, 
+				     NULL, NULL, K, opt, 
+				     ainfo->prn);
+	} else {
+	    int save_lbfgs = libset_get_bool(USE_LBFGS);
+
+	    if (save_lbfgs) {
+		ainfo->pflags |= ARMA_LBFGS;
+	    } else if (opt & OPT_L) {
+		libset_set_bool(USE_LBFGS, 1);
+		ainfo->pflags |= ARMA_LBFGS;
+	    }
+	    
+	    err = BFGS_max(b, ainfo->nc, maxit, toler, 
+			   &fncount, &grcount, kalman_arma_ll, C_LOGLIK,
+			   NULL, K, NULL, opt, ainfo->prn);
+
+	    if (save_lbfgs == 0 && (opt & OPT_L)) {
+		libset_set_bool(USE_LBFGS, 0);
+	    }
+	}
 
 	if (err) {
-	    fprintf(stderr, "BFGS_max returned %d\n", err);
+	    fprintf(stderr, "kalman_arma: optimizer returned %d\n", err);
 	} 
-
-	if (!save_lbfgs && (opt & OPT_L)) {
-	    libset_set_bool(USE_LBFGS, 0);
-	}
     }
 
     if (!err && ainfo->yscale != 1.0) {
@@ -1377,8 +1395,12 @@ static int kalman_arma (double *coeff,
     }
 
     if (!err) {
-	gretl_model_set_int(pmod, "fncount", fncount);
-	gretl_model_set_int(pmod, "grcount", grcount);
+	if (use_newton) {
+	    gretl_model_set_int(pmod, "iters", fncount);
+	} else {
+	    gretl_model_set_int(pmod, "fncount", fncount);
+	    gretl_model_set_int(pmod, "grcount", grcount);
+	}
 	err = kalman_arma_finish(pmod, ainfo, 
 				 Z, pdinfo, K, b, 
 				 opt, ainfo->prn);
@@ -1416,7 +1438,7 @@ static int user_arma_init (double *coeff, arma_info *ainfo, int *init_done)
     }
 
     if (arma_exact_ml(ainfo)) {
-	/* initialization is handled within BFGS */
+	/* initialization is handled within BFGS/Newton-Raphson */
 	for (i=0; i<ainfo->nc; i++) {
 	    coeff[i] = 0.0;
 	}	
@@ -1546,7 +1568,7 @@ static int arima_by_ls (const double **Z, const DATAINFO *pdinfo,
 	for (t=0; t<ainfo->T; t++) {
 	    pmod->uhat[t + ainfo->t1] = u->val[t];
 	}	
-	err = gretl_model_write_vcv(pmod, V);
+	err = gretl_model_write_vcv(pmod, V, -1);
     }
 
     if (!err) {
