@@ -1,21 +1,47 @@
+/* 
+ *  gretl -- Gnu Regression, Econometrics and Time-series Library
+ *  Copyright (C) 2001 Allin Cottrell and Riccardo "Jack" Lucchetti
+ * 
+ *  This program is free software: you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation, either version 3 of the License, or
+ *  (at your option) any later version.
+ * 
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ * 
+ *  You should have received a copy of the GNU General Public License
+ *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ */
+
 #include "usermat.h"
 
 #define MAX_ARMA_ORDER 128
 #define MAX_ARIMA_DIFF 2
 
+static void 
+real_arima_difference_series (double *dx, const double *x,
+			      int t1, int t2, int *delta, 
+			      int k);
+
 static void 
 arma_info_init (arma_info *ainfo, gretlopt opt, 
-		const char *pqspec, const DATAINFO *pdinfo)
+		const int *pqspec, const DATAINFO *pdinfo)
 {
     ainfo->yno = 0;
     ainfo->flags = 0;
+    ainfo->pflags = 0;
     ainfo->alist = NULL;
 
     if (opt & OPT_X) {
+	/* we got --x-12-arima */
 	ainfo->flags |= ARMA_X12A;
     }    
 
     if (!(opt & OPT_C)) {
+	/* we didn't get --conditional */
 	ainfo->flags |= ARMA_EXACT;
     }
 
@@ -44,6 +70,7 @@ arma_info_init (arma_info *ainfo, gretlopt opt,
     ainfo->t2 = pdinfo->t2;
     ainfo->pd = pdinfo->pd;
     ainfo->T = 0;
+    ainfo->r0 = 0;
 
     ainfo->y = NULL;
     ainfo->e = NULL;
@@ -51,6 +78,7 @@ arma_info_init (arma_info *ainfo, gretlopt opt,
     ainfo->yscale = 1.0;
 
     ainfo->xlist = NULL;
+    ainfo->misslist = NULL;
     ainfo->dX = NULL;
     ainfo->G = NULL;
     ainfo->V = NULL;
@@ -66,15 +94,19 @@ static void arma_info_cleanup (arma_info *ainfo)
     free(ainfo->alist);
     free(ainfo->pmask);
     free(ainfo->qmask);
-    free(ainfo->y);
     free(ainfo->e);
     free(ainfo->Z);
     free(ainfo->xlist);
+    free(ainfo->misslist);
 
     gretl_matrix_free(ainfo->dX);
     gretl_matrix_free(ainfo->G);
     gretl_matrix_free(ainfo->V);
 
+    if (arima_ydiff(ainfo)) {
+	free(ainfo->y);
+    }
+
     doubles_array_free(ainfo->aux, ainfo->n_aux);
 }
 
@@ -89,11 +121,10 @@ enum {
    lags used.
 */
 
-static char *mask_from_vec (const gretl_vector *v, 
-			    arma_info *ainfo,
-			    int m, int *err)
+static char *mask_from_list (const int *list, 
+			     arma_info *ainfo,
+			     int m, int *err)
 {
-    int vlen = gretl_vector_get_length(v);
     int mlen = (m == AR_MASK)? ainfo->p : ainfo->q;
     int nv = 0, nmax = 0;
     char *mask;
@@ -110,13 +141,13 @@ static char *mask_from_vec (const gretl_vector *v,
     }
     mask[mlen] = '\0';
 
-    for (i=0; i<vlen; i++) {
-	k = v->val[i] - 1; /* convert to zero-based */
-	if (k >= 0 && k < mlen) {
-	    mask[k] = '1'; 
+    for (i=1; i<=list[0]; i++) {
+	k = list[i];
+	if (k > 0) {
+	    mask[k-1] = '1'; 
 	    nv++;
-	    if (k + 1 > nmax) {
-		nmax = k + 1;
+	    if (k > nmax) {
+		nmax = k;
 	    }
 	}
     }
@@ -137,108 +168,40 @@ static char *mask_from_vec (const gretl_vector *v,
     return mask;
 }
 
-/* construct vector of specific lags from string spec */
-
-static gretl_matrix *
-matrix_from_spec (const char *s, int *tmp, int *err)
+static int arma_make_masks (arma_info *ainfo)
 {
-    gretl_matrix *m = NULL;
-    const char *p = s;
-    char *rem;
-    int i, k, n = 0;
+    int *plist = NULL, *qlist = NULL;
+    int err = 0;
 
-    while (*s != '\0' && *s != '}' && n < 20) {
-	strtol(s, &rem, 10);
-	n++;
-	s = rem;
-    }
-
-    m = gretl_vector_alloc(n);
-    if (m == NULL) {
-	*err = E_ALLOC;
-	return NULL;
-    }
-
-    s = p;
-    i = 0;
-    while (*s != '\0' && *s != '}' && i < 20) {
-	k = (int) strtol(s, &rem, 10);
-	m->val[i++] = k;
-	s = rem;
-    }
-
-    *tmp = 1;
-
-    return m;
-}
-
-static gretl_matrix *get_arma_pq_vec (arma_info *ainfo,
-				      int t, int *tmp, int *err)
-{
-    gretl_matrix *m = NULL;
-    const char *test = (t == AR_MASK)? "p=" : "q=";
-    const char *s = strstr(ainfo->pqspec, test);
-
-    *tmp = 0;
-
-    if (s != NULL) {
-	s += 2;
-	if (*s == '{') {
-	    m = matrix_from_spec(s + 1, tmp, err);
+    if (ainfo->pqspec != NULL) {
+	if (gretl_list_has_separator(ainfo->pqspec)) {
+	    gretl_list_split_on_separator(ainfo->pqspec, &plist, &qlist);
 	} else {
-	    char *p, mname[VNAMELEN];
-
-	    *mname = '\0';
-	    strncat(mname, s, VNAMELEN - 1);
-	    p = strchr(mname, ',');
-	    if (p != NULL) {
-		*p = '\0';
-	    }
-	    m = get_matrix_by_name(mname);
-	    if (m == NULL) {
-		*err = E_UNKVAR;
-	    }
-	} 
+	    plist = gretl_list_copy(ainfo->pqspec);
+	}
     }
 
-    return m;
-}
-
-static int arma_make_masks (arma_info *ainfo, int *list)
-{
-    gretl_matrix *m;
-    int tmp, err = 0;
-
     if (ainfo->p > 0) {
 	ainfo->np = ainfo->p;
-	if (ainfo->pqspec != NULL && *ainfo->pqspec != '\0') {
-	    m = get_arma_pq_vec(ainfo, AR_MASK, &tmp, &err);
-	    if (m != NULL) {
-		ainfo->pmask = mask_from_vec(m, ainfo, AR_MASK, &err);
-		if (tmp) {
-		    gretl_matrix_free(m);
-		}
-	    }
+	if (plist != NULL && plist[0] > 0) {
+	    ainfo->pmask = mask_from_list(plist, ainfo, AR_MASK, &err);
 	}
     }
 
     if (ainfo->q > 0 && !err) {
 	ainfo->nq = ainfo->q;
-	if (ainfo->pqspec != NULL && *ainfo->pqspec != '\0') {
-	    m = get_arma_pq_vec(ainfo, MA_MASK, &tmp, &err);
-	    if (m != NULL) {
-		ainfo->qmask = mask_from_vec(m, ainfo, MA_MASK, &err);
-		if (tmp) {
-		    gretl_matrix_free(m);
-		}
-	    }
+	if (qlist != NULL && qlist[0] > 0) {
+	    ainfo->qmask = mask_from_list(qlist, ainfo, MA_MASK, &err);
 	}
     }
 
+    free(plist);
+    free(qlist);
+
     return err;
 }
 
-static int arma_list_y_position (arma_info *ainfo)
+int arma_list_y_position (arma_info *ainfo)
 {
     int ypos;
 
@@ -251,64 +214,38 @@ static int arma_list_y_position (arma_info *ainfo)
     return ypos;
 }
 
-#define INT_DEBUG 0
-
 static int arima_integrate (double *dx, const double *x,
 			    int t1, int t2, int d, int D, int s)
 {
     double *ix;
-    int t;
-
-#if INT_DEBUG
-    fprintf(stderr, "arima_integrate: t1=%d, t2=%d, d=%d, D=%d, s=%d\n",
-	    t1, t2, d, D, s);
-#endif
+    int *c;
+    int k = d + s * D;
+    int i, t;
 
     ix = malloc((t2 + 1) * sizeof *ix);
     if (ix == NULL) {
 	return E_ALLOC;
     }
 
+    c = arima_delta_coeffs(d, D, s);
+    if (c == NULL) {
+	free(ix);
+	return E_ALLOC;
+    }    
+
     for (t=0; t<t1; t++) {
 	ix[t] = 0.0;
     }
 
     for (t=t1; t<=t2; t++) {
 	ix[t] = dx[t];
-	if (d > 0) {
-	    ix[t] += x[t-1];
-	} 
-	if (d == 2) {
-	    ix[t] += x[t-1] - x[t-2];
-	}
-	if (D > 0) {
-	    ix[t] += x[t-s];
-	    if (d > 0) {
-		ix[t] -= x[t-s-1];
-	    }
-	    if (d == 2) {
-		ix[t] -= x[t-s-1] - x[t-s-2];
-	    }	    
-	} 
-	if (D == 2) {
-	    ix[t] += x[t-s] - x[t-2*s];
-	    if (d > 0) {
-		ix[t] -= x[t-s-1] - x[t-2*s-1];
-	    }
-	    if (d == 2) {
-		ix[t] -= x[t-s-1] - x[t-2*s-1];
-		ix[t] += x[t-s-2] - x[t-2*s-2];
+	for (i=0; i<k; i++) {
+	    if (c[i] != 0) {
+		ix[t] += c[i] * x[t-i-1];
 	    }
 	}
     }
 
-#if INT_DEBUG
-    for (t=0; t<=t2; t++) {
-	fprintf(stderr, "%2d: %12.5g %12.5g %12.5g\n",
-		t, x[t], dx[t], ix[t]);
-    }
-#endif
-
     /* transcribe integrated result back into "dx" */
     for (t=0; t<=t2; t++) {
 	if (t < t1) {
@@ -319,6 +256,7 @@ static int arima_integrate (double *dx, const double *x,
     }
 
     free(ix);
+    free(c);
 
     return 0;
 }
@@ -356,14 +294,41 @@ static void ainfo_data_to_model (arma_info *ainfo, MODEL *pmod)
     }
 }
 
+static void arma_depvar_stats (MODEL *pmod, arma_info *ainfo,
+			       const double **Z)
+{
+    if (arma_is_arima(ainfo) && !arima_ydiff(ainfo)) {
+	/* calculate differenced y for stats */
+	int d = ainfo->d, D = ainfo->D;
+	int T = pmod->t2 - pmod->t1 + 1;
+	double *dy = malloc(T * sizeof *dy);
+	int *delta = arima_delta_coeffs(d, D, ainfo->pd);
+
+	if (dy != NULL && delta != NULL) {
+	    int k = d + ainfo->pd * D;
+
+	    real_arima_difference_series(dy, Z[ainfo->yno], pmod->t1,
+					 pmod->t2, delta, k);
+	    pmod->ybar = gretl_mean(0, T - 1, dy);
+	    pmod->sdy = gretl_stddev(0, T - 1, dy);
+	}
+	free(dy);
+	free(delta);
+    } else {
+	pmod->ybar = gretl_mean(pmod->t1, pmod->t2, ainfo->y);
+	pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, ainfo->y);
+    }
+}
+
+#define USE_ARIMA_INTEGRATE 1
+
 /* write the various statistics from ARMA estimation into
    a gretl MODEL struct */
 
-static void write_arma_model_stats (MODEL *pmod, arma_info *ainfo,
-				    const double **Z, 
-				    const DATAINFO *pdinfo)
+void write_arma_model_stats (MODEL *pmod, arma_info *ainfo,
+			     const double **Z, 
+			     const DATAINFO *pdinfo)
 {
-    const double *y = NULL;
     double mean_error;
     int do_criteria = 1;
     int t;
@@ -375,37 +340,38 @@ static void write_arma_model_stats (MODEL *pmod, arma_info *ainfo,
     free(pmod->list);
     pmod->list = gretl_list_copy(ainfo->alist);
 
-    if (arma_is_arima(ainfo)) {
-	y = ainfo->y;
-    } else {
-	y = Z[ainfo->yno];
-    }
-
     if (!arma_least_squares(ainfo)) {
-	pmod->ybar = gretl_mean(pmod->t1, pmod->t2, y);
-	pmod->sdy = gretl_stddev(pmod->t1, pmod->t2, y);
+	arma_depvar_stats(pmod, ainfo, Z);
     }
 
     mean_error = pmod->ess = 0.0;
 
     for (t=pmod->t1; t<=pmod->t2; t++) {
-	if (!na(y[t]) && !na(pmod->uhat[t])) {
-	    pmod->yhat[t] = y[t] - pmod->uhat[t];
+	if (!na(ainfo->y[t]) && !na(pmod->uhat[t])) {
+#if USE_ARIMA_INTEGRATE == 0
+	    if (arma_is_arima(ainfo) && arima_ydiff(ainfo)) {
+		pmod->yhat[t] = Z[ainfo->yno][t] - pmod->uhat[t];
+	    }
+#else
+	    pmod->yhat[t] = ainfo->y[t] - pmod->uhat[t];
+#endif
 	    pmod->ess += pmod->uhat[t] * pmod->uhat[t];
 	    mean_error += pmod->uhat[t];
-	}
+	} 
     }
 
-    if (arma_is_arima(ainfo)) {
+#if USE_ARIMA_INTEGRATE
+    if (arma_is_arima(ainfo) && arima_ydiff(ainfo)) {
 	arima_integrate(pmod->yhat, Z[ainfo->yno], pmod->t1, pmod->t2, 
 			ainfo->d, ainfo->D, ainfo->pd);
     }
+#endif
 
     mean_error /= pmod->nobs;
     gretl_model_set_double(pmod, "mean_error", mean_error);
 
     if (na(pmod->sigma)) {
-	/* in X12A or native exact cases this is already done */
+	/* in x12a or native exact cases this is already done */
 	pmod->sigma = sqrt(pmod->ess / pmod->nobs);
     } 
 
@@ -433,7 +399,7 @@ static void write_arma_model_stats (MODEL *pmod, arma_info *ainfo,
 
 static void calc_max_lag (arma_info *ainfo)
 {
-    if (ainfo->flags & ARMA_EXACT) {
+    if (arma_exact_ml(ainfo)) {
 	ainfo->maxlag = ainfo->d + ainfo->D * ainfo->pd;
     } else {
 	/* conditional ML */
@@ -448,19 +414,19 @@ static void calc_max_lag (arma_info *ainfo)
 #endif
 }
 
-#define SAMPLE_DEBUG 0
-
 static int arma_adjust_sample (arma_info *ainfo, 
 			       const double **Z,
-			       const DATAINFO *pdinfo)
+			       const DATAINFO *pdinfo,
+			       int *missv, int *misst)
 {
     int *list = ainfo->alist;
     int ypos = arma_list_y_position(ainfo);
     int t0, t1 = pdinfo->t1, t2 = pdinfo->t2;
     int i, vi, vlmax, k, t;
-    int anymiss;
+    int missing;
+    int err = 0;
 
-#if SAMPLE_DEBUG
+#if ARMA_DEBUG
     fprintf(stderr, "arma_adjust_sample: at start, t1=%d, t2=%d, maxlag = %d\n",
 	    t1, t2, ainfo->maxlag);
 #endif
@@ -471,7 +437,7 @@ static int arma_adjust_sample (arma_info *ainfo,
     }
 
     /* list position of last var to check for lags */
-    if (ainfo->flags & ARMA_XDIFF) {
+    if (arma_xdiff(ainfo)) {
 	vlmax = list[0];
     } else {
 	vlmax = ypos;
@@ -480,23 +446,23 @@ static int arma_adjust_sample (arma_info *ainfo,
     /* advance the starting point if need be */
 
     for (t=t1; t<=t2; t++) {
-	anymiss = 0;
-	for (i=ypos; i<=list[0] && !anymiss; i++) {
+	missing = 0;
+	for (i=ypos; i<=list[0] && !missing; i++) {
 	    vi = list[i];
 	    if (na(Z[vi][t])) {
 		/* current value missing */
-		anymiss = 1;
+		missing = 1;
 	    }
 	    if (i <= vlmax) {
-		for (k=1; k<=ainfo->maxlag && !anymiss; k++) {
+		for (k=1; k<=ainfo->maxlag && !missing; k++) {
 		    if (na(Z[vi][t-k])) {
 			/* lagged value missing */
-			anymiss = 1;
+			missing = 1;
 		    }
 		}
 	    }
 	}
-	if (anymiss) {
+	if (missing) {
 	    t1++;
 	} else {
 	    break;
@@ -506,51 +472,67 @@ static int arma_adjust_sample (arma_info *ainfo,
     /* retard the ending point if need be */
 
     for (t=t2; t>=t1; t--) {
-	anymiss = 0;
-	for (i=ypos; i<=list[0] && !anymiss; i++) {
+	missing = 0;
+	for (i=ypos; i<=list[0] && !missing; i++) {
 	    vi = list[i];
 	    if (na(Z[vi][t])) {
-		anymiss = 1;
+		missing = 1;
 	    }
 	}
-	if (anymiss) {
+	if (missing) {
 	    t2--;
 	} else {
 	    break;
 	}
     }
-	
-    /* check for missing obs within the sample range */
+
+    missing = 0;
+
+    /* check for missing obs within the adjusted sample range */
     for (t=t1; t<t2; t++) {
+	int tmiss = 0;
+
 	for (i=ypos; i<=list[0]; i++) {
 	    vi = list[i];
 	    if (na(Z[vi][t])) {
-		gretl_errmsg_sprintf(_("Missing value encountered for "
-				       "variable %d, obs %d"), vi, t + 1);
-		return E_MISSDATA;
+		if (missv != NULL && misst != NULL && *missv == 0) {
+		    /* record info on first missing obs */
+		    *missv = vi;
+		    *misst = t + 1;
+		}
+		tmiss = 1;
 	    }
 	}
+	if (tmiss) {
+	    missing++;
+	}
     }
 
-    ainfo->T = t2 - t1 + 1;
-    if (ainfo->T <= ainfo->nc) {
-	/* insufficient observations */
-	return E_DF; 
+    if (missing > 0 && !arma_na_ok(ainfo)) {
+	err = E_MISSDATA;
     }
 
-#if SAMPLE_DEBUG
-    fprintf(stderr, "arma_adjust_sample: at end, t1=%d, t2=%d\n",
-	    t1, t2);
+    if (!err) {
+	ainfo->fullT = t2 - t1 + 1;
+	ainfo->T = ainfo->fullT - missing;
+	if (ainfo->T <= ainfo->nc) {
+	    /* insufficient observations */
+	    err = E_DF; 
+	}
+    }
+
+    if (!err) {
+#if ARMA_DEBUG
+	fprintf(stderr, "arma_adjust_sample: at end, t1=%d, t2=%d\n",
+		t1, t2);
 #endif
+	ainfo->t1 = t1;
+	ainfo->t2 = t2;
+    }
 
-    ainfo->t1 = t1;
-    ainfo->t2 = t2;
-
-    return 0;
+    return err;
 }
 
-#define ARIMA_DEBUG 0
-
 /* remove the intercept from list of regressors */
 
 static int arma_remove_const (int *list, int seasonal, int diffs,
@@ -612,10 +594,6 @@ static int check_arma_sep (int *list, int sep1, arma_info *ainfo)
 	}
     }
 
-#if ARIMA_DEBUG
-    fprintf(stderr, "check_arma_sep: returning %d\n", err);
-#endif
-
     return err;
 }
 
@@ -677,7 +655,7 @@ static int check_arma_list (int *list, gretlopt opt,
     /* now that we have p and q we can check for masked lags */
 
     if (!err) {
-	err = arma_make_masks(ainfo, list);
+	err = arma_make_masks(ainfo);
     }
 
     /* If there's an explicit constant in the list here, we'll remove
@@ -723,10 +701,6 @@ static int check_arima_list (int *list, gretlopt opt,
     int hadconst = 0;
     int err = 0;
 
-#if ARIMA_DEBUG
-    printlist(list, "check_arima_list");
-#endif
-
     if (list[1] < 0 || list[1] > MAX_ARMA_ORDER) {
 	err = 1;
     } else if (list[2] < 0 || list[2] > MAX_ARIMA_DIFF) {
@@ -762,7 +736,7 @@ static int check_arima_list (int *list, gretlopt opt,
     /* now that we have p and q we can check for masked lags */
 
     if (!err) {
-	err = arma_make_masks(ainfo, list);
+	err = arma_make_masks(ainfo);
     }
 
     /* If there's an explicit constant in the list here, we'll remove
@@ -808,11 +782,6 @@ static int arma_check_list (arma_info *ainfo,
     int sep1 = gretl_list_separator_position(list);
     int err = 0;
 
-#if ARIMA_DEBUG
-    fprintf(stderr, "arma_check_list: sep1 = %d\n", sep1);
-    printlist(list, "incoming list");
-#endif
-
     if (sep1 == 3) {
 	if (list[0] < 4) {
 	    err = E_PARSE;
@@ -846,57 +815,51 @@ static int arma_check_list (arma_info *ainfo,
 	err = E_ARGS;
     }
 
-#if ARIMA_DEBUG
-    printlist(list, "ar(i)ma list after checking");
-    fprintf(stderr, "err = %d\n", err);
-#endif
-
     return err;
 }
 
 static void 
 real_arima_difference_series (double *dx, const double *x,
-			      int t1, arma_info *ainfo)
+			      int t1, int t2, int *delta, 
+			      int k)
 {
-    int t, i, s = ainfo->pd;
+    int i, p, t, s = 0;
     
-    for (i=0, t=t1; t<=ainfo->t2; i++, t++) {
-	dx[i] = x[t];
-	if (ainfo->d > 0) {
-	    dx[i] -= x[t-1];
-	} 
-	if (ainfo->d == 2) {
-	    dx[i] -= x[t-1] - x[t-2];
-	}
-	if (ainfo->D > 0) {
-	    dx[i] -= x[t-s];
-	    if (ainfo->d > 0) {
-		dx[i] += x[t-s-1];
-	    }
-	    if (ainfo->d == 2) {
-		dx[i] += x[t-s-1] - x[t-s-2];
-	    }	    
-	} 
-	if (ainfo->D == 2) {
-	    dx[i] -= x[t-s] - x[t-2*s];
-	    if (ainfo->d > 0) {
-		dx[i] += x[t-s-1] - x[t-2*s-1];
-	    }
-	    if (ainfo->d == 2) {
-		dx[i] += x[t-s-1] - x[t-2*s-1];
-		dx[i] -= x[t-s-2] - x[t-2*s-2];
+    for (t=t1; t<=t2; t++) {
+	dx[s] = x[t];
+	for (i=0; i<k && !na(dx[s]); i++) {
+	    if (delta[i] != 0) {
+		p = t - i - 1;
+		if (p < 0 || na(x[p])) {
+		    dx[s]  = NADBL;
+		} else {
+		    dx[s] -= delta[i] * x[p];
+		}
 	    }
 	}
+	s++;
     }
 }
 
-static int arima_difference (arma_info *ainfo, const double **Z,
-			     const DATAINFO *pdinfo)
+#ifndef X12A_CODE
+
+/* Add to the ainfo struct a full-length series y holding 
+   the differenced version of the dependent variable.
+   If the "xdiff" flag is set on ainfo, in addition 
+   create a matrix dX holding the differenced regressors;
+   in that case the time-series length of dX depends on
+   the @fullX flag -- if fullX = 0, this equals
+   ainfo->T but if fullX = 0 it equals ainfo->t2 + 1.
+*/
+
+int arima_difference (arma_info *ainfo, const double **Z,
+		      const DATAINFO *pdinfo, int fullX)
 {
     const double *y = Z[ainfo->yno];
     double *dy = NULL;
+    int *delta = NULL;
     int s = ainfo->pd;
-    int t, t1 = 0;
+    int k, t, t1 = 0;
     int err = 0;
 
 #if ARMA_DEBUG
@@ -913,6 +876,12 @@ static int arima_difference (arma_info *ainfo, const double **Z,
 	return E_ALLOC;
     }
 
+    delta = arima_delta_coeffs(ainfo->d, ainfo->D, s);
+    if (delta == NULL) {
+	free(dy);
+	return E_ALLOC;
+    }    
+
     for (t=0; t<pdinfo->n; t++) {
 	dy[t] = NADBL;
     }
@@ -926,8 +895,9 @@ static int arima_difference (arma_info *ainfo, const double **Z,
     }
 
     t1 += ainfo->d + ainfo->D * s;
+    k = ainfo->d + s * ainfo->D;
 
-    real_arima_difference_series(dy + t1, y, t1, ainfo);
+    real_arima_difference_series(dy + t1, y, t1, ainfo->t2, delta, k);
 
 #if ARMA_DEBUG > 1
     for (t=0; t<pdinfo->n; t++) {
@@ -936,10 +906,19 @@ static int arima_difference (arma_info *ainfo, const double **Z,
 #endif    
 
     ainfo->y = dy;
+    set_arima_ydiff(ainfo);
 
-    if (ainfo->flags & ARMA_XDIFF) {
+    if (arma_xdiff(ainfo)) {
 	/* also difference the ARIMAX regressors */
-	ainfo->dX = gretl_matrix_alloc(ainfo->T, ainfo->nexo);
+	int xt1 = ainfo->t1, xT = ainfo->T;
+
+	if (fullX) {
+	    xt1 = 0;
+	    xT = ainfo->t2 + 1;
+	} 
+
+	ainfo->dX = gretl_matrix_alloc(xT, ainfo->nexo);
+
 	if (ainfo->dX == NULL) {
 	    err = E_ALLOC;
 	} else {
@@ -948,12 +927,31 @@ static int arima_difference (arma_info *ainfo, const double **Z,
 
 	    for (i=0; i<ainfo->nexo; i++) {
 		vi = ainfo->xlist[i+1];
-		real_arima_difference_series(val, Z[vi], ainfo->t1, ainfo);
-		val += ainfo->T;
+		real_arima_difference_series(val, Z[vi], xt1, ainfo->t2,
+					     delta, k);
+		val += xT;
 	    }
 	}
     }
 
+    free(delta);
+
     return err;
 }
 
+void arima_difference_undo (arma_info *ainfo, const double **Z)
+{
+    free(ainfo->y);
+    ainfo->y = (double *) Z[ainfo->yno];
+
+    if (ainfo->dX != NULL) {
+	gretl_matrix_free(ainfo->dX);
+	ainfo->dX = NULL;
+    }
+
+    unset_arima_ydiff(ainfo);
+}
+
+#endif /* X12A_CODE not defined */
+
+
