@@ -20,6 +20,7 @@
 #include "libgretl.h"
 #include "matrix_extra.h"
 #include "gretl_bfgs.h"
+#include "libset.h"
 #include "missing_private.h"
 
 #define BIPDEBUG 0
@@ -697,7 +698,7 @@ static int biprob_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll,
 
 /* analytical hessian */
 
-int biprobit_ahessian (double *theta, gretl_matrix *H, void *ptr)
+int biprobit_hessian (double *theta, gretl_matrix *H, void *ptr)
 {
     bp_container *bp = (bp_container *) ptr;
     double a, b, P, f, d1, d2, da, tmp, u_ab, u_ba;
@@ -855,7 +856,16 @@ int biprobit_ahessian (double *theta, gretl_matrix *H, void *ptr)
     tmp = gretl_matrix_get(H, k1+k2, k1+k2) - h33;
     gretl_matrix_set(H, k1+k2, k1+k2, tmp);
 
-    err = gretl_invert_symmetric_matrix(H);
+    return err;
+}
+
+int biprobit_hessian_inverse (double *theta, gretl_matrix *H, void *ptr)
+{
+    int err = biprobit_hessian(theta, H, ptr);
+
+    if (!err) {
+	err = gretl_invert_symmetric_matrix(H);
+    }
 
     return err;
 }
@@ -867,7 +877,7 @@ int biprobit_ahessian (double *theta, gretl_matrix *H, void *ptr)
    around, just in case 
 */
 
-int biprobit_nhessian (double *b, gretl_matrix *H, void *ptr)
+int biprobit_numerical_hessian (double *b, gretl_matrix *H, void *ptr)
 {
     bp_container *bp = (bp_container *) ptr;
     double x, eps = 1.0e-06;
@@ -959,13 +969,12 @@ static double *make_bp_theta (bp_container *bp, int *err)
 /* Actual likelihood maximization and post-estimation                     */
 /* ---------------------------------------------------------------------- */
 
-#define USE_NEWTON 1
-
 static int bp_do_maxlik (bp_container *bp, gretlopt opt, PRN *prn)
 {
     double crittol = 1.0e-06;
-    double gradtol = 1.0e-03;
+    double gradtol = 1.0e-05;
     int fncount, maxit = 1000;
+    gretlopt maxopt = opt & OPT_V;
     double *theta;
     int err = 0;
 
@@ -974,20 +983,20 @@ static int bp_do_maxlik (bp_container *bp, gretlopt opt, PRN *prn)
 	return err;
     }
 
-#if USE_NEWTON
-    err = newton_raphson_max(theta, bp->npar, maxit, crittol, gradtol,
-			     &fncount, C_LOGLIK, biprob_loglik, 
-			     biprob_score, biprobit_ahessian,
-			     bp, opt & OPT_V, prn);
-#else
-    int grcount;
+    if (libset_get_int(GRETL_OPTIM) == OPTIM_BFGS) {
+	int grcount = 0;
 
-    crittol = 1.0e-11;
-    err = BFGS_max(theta, bp->npar, maxit, crittol, &fncount, 
-		   &grcount, biprob_loglik, C_LOGLIK,
-		   biprob_score, bp, NULL,
-		   opt & OPT_V, prn);
-#endif
+	crittol = 1.0e-11;
+	err = BFGS_max(theta, bp->npar, maxit, crittol, &fncount, 
+		       &grcount, biprob_loglik, C_LOGLIK,
+		       biprob_score, bp, NULL,
+		       maxopt, prn);
+    } else {	
+	err = newton_raphson_max(theta, bp->npar, maxit, crittol, gradtol,
+				 &fncount, C_LOGLIK, biprob_loglik, 
+				 biprob_score, biprobit_hessian,
+				 bp, maxopt, prn);
+    }
 
     free(theta);
 
@@ -1013,7 +1022,7 @@ static gretl_matrix *biprobit_vcv (bp_container *bp, gretlopt opt, int *err)
 	}
 
 	if (!*err) {
-	    *err = biprobit_ahessian(theta, Hess, bp);
+	    *err = biprobit_hessian_inverse(theta, Hess, bp);
 	}
 #if 0
 	gretl_matrix_print(Hess, "iHess");
@@ -1193,26 +1202,17 @@ static int biprobit_fill_model (MODEL *pmod, bp_container *bp, DATAINFO *pdinfo,
 				gretlopt opt)
 {
     gretl_matrix *V;
-    double *coeff, *sderr, *vcv;
-    int nvc, npar = bp->npar - 1;
-    int i, j, err = 0;
+    double *tmp;
+    int npar = bp->npar - 1;
+    int i, err = 0;
 
-    nvc = (npar * npar + npar) / 2;
+    tmp = realloc(pmod->coeff, npar * sizeof *tmp);
 
-    coeff = realloc(pmod->coeff, npar * sizeof *coeff);
-    sderr = realloc(pmod->sderr, npar * sizeof *sderr);
-    vcv = realloc(pmod->vcv, nvc * sizeof *vcv);
-
-    if (coeff == NULL || sderr == NULL || vcv == NULL) {
+    if (tmp == NULL) {
 	err = E_ALLOC;
     } else {
-	pmod->coeff = coeff;
-	pmod->sderr = sderr;
-	pmod->vcv = vcv;
+	pmod->coeff = tmp;
 	pmod->ncoeff = npar;
-    }
-
-    if (!err) {
 	err = gretl_model_allocate_params(pmod, npar);
     }
 
@@ -1254,22 +1254,10 @@ static int biprobit_fill_model (MODEL *pmod, bp_container *bp, DATAINFO *pdinfo,
     V = biprobit_vcv(bp, opt, &err);
 
     if (!err) {
-	double x;
-	int k = 0;
-
-	/* note that the V matrix is "over-sized", so we can't use
-	   gretl_model_write_vcv() here */
-
-	for (i=0; i<npar; i++) {
-	    for (j=i; j<npar; j++) {
-		x = gretl_matrix_get(V, i, j);
-		pmod->vcv[k++] = x;
-		if (i == j) {
-		    pmod->sderr[i] = sqrt(x);
-		}
-	    }
-	}
+	err = gretl_model_write_vcv(pmod, V, npar);
+    }
 
+    if (!err) {
 	if (opt & OPT_R) {
 	    gretl_model_set_vcv_info(pmod, VCV_ML, VCV_QML);
 	} else if (opt & OPT_G) {
