@@ -20,6 +20,7 @@
 #include "libgretl.h"
 #include "matrix_extra.h"
 #include "gretl_bfgs.h"
+#include "libset.h"
 
 #include <errno.h>
 
@@ -360,7 +361,7 @@ static int duration_hessian (double *theta,
     if (dinfo->dist == DUR_EXPON) {
 	s2 = s = 1;
     } else {
-	s = dinfo->theta[np - 1];
+	s = theta[np - 1];
 	s2 = s * s;
     }
 
@@ -407,9 +408,21 @@ static int duration_hessian (double *theta,
 	}
     }
 
-    /* fill out upper triangle and invert */
+    /* fill out upper triangle */
     gretl_matrix_mirror(H, 'L');
-    err = gretl_invert_symmetric_matrix(H);
+
+    return err;
+}
+
+static int duration_hessian_inverse (double *theta, 
+				     gretl_matrix *H,
+				     void *data)
+{
+    int err = duration_hessian(theta, H, data);
+
+    if (!err) {
+	err = gretl_invert_symmetric_matrix(H);
+    }
 
     return err;
 }
@@ -426,7 +439,7 @@ static gretl_matrix *duration_init_H (duration_info *dinfo)
     dinfo->flags = DUR_UPDATE_XB;
 
     err = duration_score(dinfo->theta, NULL, dinfo->npar, 
-			     NULL, dinfo);
+			 NULL, dinfo);
     if (!err) {
 	H = gretl_matrix_GG_inverse(dinfo->G, &err);
     }
@@ -470,7 +483,7 @@ static int duration_robust_vcv (duration_info *dinfo)
 			      dinfo->G, GRETL_MOD_NONE,
 			      GG, GRETL_MOD_NONE);
 
-    err = duration_hessian(NULL, H, dinfo);
+    err = duration_hessian_inverse(dinfo->theta, H, dinfo);
 
     if (!err) {
 	err = gretl_matrix_qform(H, GRETL_MOD_NONE,
@@ -490,7 +503,8 @@ static int duration_robust_vcv (duration_info *dinfo)
 */
 
 static void
-duration_overall_LR_test (MODEL *pmod, duration_info *dinfo)
+duration_overall_LR_test (MODEL *pmod, duration_info *dinfo,
+			  int use_bfgs)
 {
     double llu = dinfo->ll;
     int err = 0;
@@ -505,16 +519,26 @@ duration_overall_LR_test (MODEL *pmod, duration_info *dinfo)
     dinfo->flags |= DUR_CONST_ONLY;
     err = duration_estimates_init(dinfo);
 
-    if (!err) {
+    /* now estimate constant-only model */
+
+    if (!err && use_bfgs) {
 	int maxit, fncount = 0, grcount = 0;
 	double toler;
 
-	/* estimate constant-only model */
-
 	BFGS_defaults(&maxit, &toler, DURATION); 
 	err = BFGS_max(dinfo->theta, dinfo->npar, maxit, toler, 
 		       &fncount, &grcount, duration_loglik, C_LOGLIK,
 		       duration_score, dinfo, NULL, OPT_NONE, NULL);
+    } else if (!err) {	
+	double crittol = 1.0e-7;
+	double gradtol = 1.0e-7;
+	int iters = 0, maxit = 100;
+
+	err = newton_raphson_max(dinfo->theta, dinfo->npar, maxit, 
+				 crittol, gradtol, &iters, 
+				 C_LOGLIK, duration_loglik, 
+				 duration_score, duration_hessian, 
+				 dinfo, OPT_NONE, NULL);
     }
 
     if (!err && llu > dinfo->ll) {
@@ -624,7 +648,7 @@ static void duration_set_predictions (MODEL *pmod, duration_info *dinfo,
 static int 
 transcribe_duration_results (MODEL *pmod, duration_info *dinfo, 
 			     const double **Z, const DATAINFO *pdinfo, 
-			     int fncount, int grcount,
+			     int fncount, int grcount, int use_bfgs,
 			     int censvar, gretlopt opt)
 {
     int np = dinfo->npar;
@@ -639,15 +663,19 @@ transcribe_duration_results (MODEL *pmod, duration_info *dinfo,
     } else if (dinfo->dist == DUR_LOGNORM) {
 	pmod->opt |= OPT_Z;
     } else {
-	pmod->opt |= OPT_W;
+	pmod->opt |= OPT_W; /* Weibull */
     }
 
     if (censvar > 0) {
 	gretl_model_set_int(pmod, "cens_var", censvar);
     }
 
-    gretl_model_set_int(pmod, "fncount", fncount);
-    gretl_model_set_int(pmod, "grcount", grcount);
+    if (use_bfgs) {
+	gretl_model_set_int(pmod, "fncount", fncount);
+	gretl_model_set_int(pmod, "grcount", grcount);
+    } else {
+	gretl_model_set_int(pmod, "iters", fncount);
+    } 
 
     if (!err) {
 	err = gretl_model_allocate_params(pmod, np);
@@ -678,12 +706,12 @@ transcribe_duration_results (MODEL *pmod, duration_info *dinfo,
 	    pmod->opt |= OPT_G;
 	    err = duration_OPG_vcv(dinfo);
 	} else {
-	    err = duration_hessian(NULL, dinfo->V, dinfo);
+	    err = duration_hessian_inverse(dinfo->theta, dinfo->V, dinfo);
 	} 
     }
 
     if (!err) {
-	err = gretl_model_write_vcv(pmod, dinfo->V);
+	err = gretl_model_write_vcv(pmod, dinfo->V, -1);
 	if (!err) {
 	    int vtype = (opt & OPT_G) ? VCV_OP :
 		(opt & OPT_R)? VCV_QML : VCV_HESSIAN;
@@ -701,7 +729,9 @@ transcribe_duration_results (MODEL *pmod, duration_info *dinfo,
 	pmod->rsq = pmod->adjrsq = NADBL;
 	pmod->ess = NADBL;
 	/* but add overall LR test if possible */
-	duration_overall_LR_test(pmod, dinfo);
+	if (np > 1 + (dinfo->dist != DUR_EXPON)) {
+	    duration_overall_LR_test(pmod, dinfo, use_bfgs);
+	}
     }
 
     return err;
@@ -711,15 +741,18 @@ int duration_estimate (MODEL *pmod, int censvar, const double **Z,
 		       const DATAINFO *pdinfo, gretlopt opt, 
 		       PRN *prn)
 {
-    gretlopt max_opt = (opt & OPT_V)? OPT_V : OPT_NONE;
+    gretlopt maxopt = opt & OPT_V;
     duration_info dinfo;
-    double toler;
-    int maxit;
+    int maxit = 200;
     int fncount = 0;
     int grcount = 0;
+    int use_bfgs = 0;
     int err = 0;
 
     err = duration_init(&dinfo, pmod, censvar, Z, opt, prn);
+    if (err) {
+	goto bailout;
+    }
 
 #if DDEBUG
     if (!err && censvar > 0) {
@@ -728,24 +761,40 @@ int duration_estimate (MODEL *pmod, int censvar, const double **Z,
     }
 #endif
 
-    if (!err) {
+    if (libset_get_int(GRETL_OPTIM) == OPTIM_BFGS) {
+	use_bfgs = 1;
+    }
+
+    if (use_bfgs) {
 	/* initialize BFGS curvature */
 	gretl_matrix *H = duration_init_H(&dinfo);
+	double toler;
 
 	BFGS_defaults(&maxit, &toler, DURATION); 
 	err = BFGS_max(dinfo.theta, dinfo.npar, maxit, toler, 
 		       &fncount, &grcount, duration_loglik, C_LOGLIK,
-		       duration_score, &dinfo, H, max_opt, 
+		       duration_score, &dinfo, H, maxopt, 
 		       dinfo.prn);
 	gretl_matrix_free(H);
+    } else {	
+	double crittol = 1.0e-7;
+	double gradtol = 1.0e-7;
+
+	err = newton_raphson_max(dinfo.theta, dinfo.npar, maxit, 
+				 crittol, gradtol, &fncount,
+				 C_LOGLIK, duration_loglik, 
+				 duration_score, duration_hessian, 
+				 &dinfo, maxopt, dinfo.prn);
     }
 
     if (!err) {
 	err = transcribe_duration_results(pmod, &dinfo, Z, pdinfo, 
-					  fncount, grcount, 
+					  fncount, grcount, use_bfgs,
 					  censvar, opt);
     }
 
+ bailout:
+
     duration_free(&dinfo);
 
     if (err && !pmod->errcode) {
