@@ -111,7 +111,7 @@ write_garch_stats (MODEL *pmod, const int *list,
     err = gretl_model_write_coeffs(pmod, theta, npar);
 
     if (!err) {
-	gretl_model_write_vcv(pmod, V);
+	gretl_model_write_vcv(pmod, V, -1);
     }
 
     if (err) {
@@ -434,9 +434,11 @@ garch_driver (const int *list, double **Z, double scale,
 			  ifc, prn);
 	if (iters > 0) {
 	    gretl_model_set_int(pmod, "iters", iters);
-	} else if (fnc > 0) {
+	} else if (grc > 0) {
 	    gretl_model_set_int(pmod, "fncount", fnc);
 	    gretl_model_set_int(pmod, "grcount", grc);
+	} else {
+	    gretl_model_set_int(pmod, "iters", fnc);
 	}
 	gretl_model_set_vcv_info(pmod, VCV_ML, vopt);
     }
