@@ -33,9 +33,7 @@ struct h_container_ {
     int kmain;		     /* no. of params in the main eq. */
     int ksel;		     /* no. of params in the selection eq. */
     double ll;		     /* log-likelihood */
-    gretl_matrix *score;     /* score matrix */
-    gretl_matrix *sscore;    /* score vector (sum) */
-
+   
     int ntot, nunc;	     /* total and uncensored obs */
     int depvar;		     /* location of y in array Z */
     int selvar;		     /* location of selection variable in array Z */
@@ -50,9 +48,13 @@ struct h_container_ {
     gretl_matrix *d;	     /* selection dummy variable */
     gretl_matrix *selreg;    /* selection eq. regressors */ 
     gretl_matrix *selreg_u;  /* selection eq. regressors (subsample d==1) */
+
+    gretl_matrix_block *B;   /* workspace for the follwing */
     gretl_vector *fitted;
     gretl_vector *u;
     gretl_vector *ndx;
+    gretl_matrix *score;     /* score matrix */
+    gretl_matrix *sscore;    /* score vector (sum) */
 
     gretl_vector *beta;	     /* main eq. parameters */
     gretl_vector *gama;	     /* selection eq. parameters */
@@ -66,6 +68,12 @@ struct h_container_ {
     char *probmask;	     /* mask NAs for initial probit */
     char *fullmask;	     /* mask NAs */
     char *uncmask;	     /* mask NAs and (d==0) */
+
+    gretl_matrix_block *BH;  /* workspace for the analytical Hessian */
+    gretl_matrix *H11; 
+    gretl_matrix *H12;
+    gretl_matrix *H22;
+    gretl_matrix *H13;
 };
 
 static void h_container_destroy (h_container *HC)
@@ -84,11 +92,6 @@ static void h_container_destroy (h_container *HC)
     gretl_matrix_free(HC->d);
     gretl_matrix_free(HC->selreg);
     gretl_matrix_free(HC->selreg_u);
-    gretl_vector_free(HC->fitted);
-    gretl_vector_free(HC->u);
-    gretl_vector_free(HC->ndx);
-    gretl_matrix_free(HC->score);
-    gretl_vector_free(HC->sscore);
 
     gretl_vector_free(HC->beta);
     gretl_vector_free(HC->gama);
@@ -100,6 +103,9 @@ static void h_container_destroy (h_container *HC)
     free(HC->fullmask);
     free(HC->uncmask);
 
+    gretl_matrix_block_destroy(HC->B);
+    gretl_matrix_block_destroy(HC->BH);
+
     free(HC);
 }
 
@@ -125,11 +131,6 @@ static h_container *h_container_new (const int *list)
     HC->d = NULL;
     HC->selreg = NULL;
     HC->selreg_u = NULL;
-    HC->fitted = NULL;
-    HC->u = NULL;
-    HC->ndx = NULL;
-    HC->score = NULL;
-    HC->sscore = NULL;
 
     HC->beta = NULL;
     HC->gama = NULL;
@@ -140,6 +141,9 @@ static h_container *h_container_new (const int *list)
     HC->vcv = NULL;
     HC->VProbit = NULL;
 
+    HC->B = NULL;
+    HC->BH = NULL;
+
     HC->probmask = NULL;
     HC->fullmask = NULL;
     HC->uncmask = NULL;
@@ -470,63 +474,38 @@ static int h_container_fill (h_container *HC, const int *Xl,
     }
 
     if (!err) {
-	HC->fitted = gretl_matrix_alloc(HC->nunc, 1);
-	HC->u      = gretl_matrix_alloc(HC->nunc, 1);
-	HC->ndx    = gretl_matrix_alloc(HC->ntot, 1);
-	HC->score  = gretl_matrix_alloc(HC->ntot, npar); 
-	HC->sscore = gretl_vector_alloc(npar); 
-
-	if (HC->fitted == NULL || HC->u == NULL || HC->ndx == NULL
-	    || HC->score == NULL || HC->sscore == NULL) {
+	HC->B = gretl_matrix_block_new(&HC->fitted, HC->nunc, 1,
+				       &HC->u,      HC->nunc, 1,
+				       &HC->ndx,    HC->ntot, 1,
+				       &HC->score,  HC->ntot, npar,
+				       &HC->sscore, 1, npar,
+				       NULL);
+	if (HC->B == NULL) {
 	    err = E_ALLOC;
 	}
+		
+    }
+
+    if (!err){
+	/* workspace for the analytical Hessian */
+        HC->BH = gretl_matrix_block_new(&HC->H11, HC->kmain, HC->kmain,
+					&HC->H12, HC->kmain, HC->ksel,
+					&HC->H13, HC->kmain + HC->ksel, 2,
+					&HC->H22, HC->ksel, HC->ksel,
+					NULL);
+	if (HC->BH == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    gretl_matrix_block_zero(HC->BH);
+	}
     }
 
     return err;
 }
 
-static double h_loglik (const double *param, void *ptr)
+static int setup_ndx_functions (h_container *HC)
 {
-    h_container *HC = (h_container *) ptr;
-    double lnsig, x, ll = NADBL;
-    double ca, sa, arho;
-    int i, j, err = 0;
-    int kmain, ksel, kmax, npar;
-
-    kmain = HC->kmain;
-    ksel  = HC->ksel;
-    kmax  = kmain + ksel;
-    npar  = kmax + 2;
-    
-    for (i=0; i<kmain; i++) {
-	gretl_vector_set(HC->beta, i, param[i]);
-    }
-
-    j = 0;
-    for (i=kmain; i<kmax; i++) {
-	gretl_vector_set(HC->gama, j++, param[i]);
-    }
-
-    HC->sigma = param[npar-2];
-    lnsig = log(HC->sigma);
-    
-    arho = param[npar-1];
-    HC->rho = tanh(arho);
-    ca = cosh(arho);
-    sa = sinh(arho);
-
-#if HDEBUG > 1
-    gretl_matrix_print(HC->beta, "beta");
-    gretl_matrix_print(HC->gama, "gama");
-    fprintf(stderr, "sigma = %12.6f, rho = %12.6f", HC->sigma, HC->rho);
-    fputc('\n', stderr);
-#endif
-
-    if (HC->sigma <= 0) {
-	return NADBL;
-    } 
-
-    err = gretl_matrix_multiply(HC->reg, HC->beta, HC->fitted);
+    int err = gretl_matrix_multiply(HC->reg, HC->beta, HC->fitted);
 
     if (!err) {
 	gretl_matrix_copy_values(HC->u, HC->y);
@@ -541,77 +520,134 @@ static double h_loglik (const double *param, void *ptr)
 	err = gretl_matrix_multiply(HC->selreg, HC->gama, HC->ndx);
     }
 
-    if (!err) {
-	double ll0 = 0, ll1 = 0, ll2 = 0;
-	double ndxt, ut = 0;
-	double P, mills, tmp, psum;
-	int sel, k;
+    return err;
+}
 
-	gretl_matrix_zero(HC->score);
-	gretl_matrix_zero(HC->sscore);
+static int h_common_setup (h_container *HC, const double *param,
+			   double *ca, double *sa)
+{
+    int kmax = HC->kmain + HC->ksel;
+    int i, j = 0, err = 0;
+    double arho;
 
-	/* i goes through all obs, while j keeps track of the uncensored
-	   ones */
-	j = 0;
-	for (i=0; i<HC->ntot; i++) {
-	    sel = (gretl_vector_get(HC->d, i) == 1.0);
-	    ndxt = gretl_vector_get(HC->ndx, i);
-	    if (sel) {
-		ut = gretl_vector_get(HC->u, j);
-		x = ca * (ndxt + HC->rho*ut);
-		ll1 -= LN_SQRT_2_PI + 0.5*ut*ut + lnsig;
-		P = normal_cdf(x);
-		mills = invmills(-x);
-		ll2 += log(P);
-	    } else {
-		P = normal_cdf(-ndxt);
-		mills = -invmills(ndxt);
-		ll0 += log(P);
-	    }
+    for (i=0; i<=kmax; i++) {
+	if (i < HC->kmain) {
+	    gretl_vector_set(HC->beta, i, param[i]);
+	} else if (i < kmax) {
+	    gretl_vector_set(HC->gama, j++, param[i]);
+	} else {
+	    HC->sigma = param[i];
+	}
+    }
 
-	    /* score for beta */
-	    if (sel) {
-		tmp = (ut - sa*mills)/HC->sigma;
-		for (k=0; k<kmain; k++) {
-		    x = tmp * gretl_matrix_get(HC->reg, j, k);
-		    gretl_matrix_set(HC->score, i, k, x); 
-		    psum = x + gretl_vector_get(HC->sscore, k);
-		    gretl_vector_set(HC->sscore, k, psum);
-		}
-	    }
+    arho = param[kmax+1];
 
-	    /* score for gamma */
-	    tmp = sel ? ca*mills : mills;
-	    for (k=0; k<ksel; k++) {
-		x = tmp * gretl_matrix_get(HC->selreg, i, k);
-		gretl_matrix_set(HC->score, i, kmain+k, x); 
-		psum = x + gretl_vector_get(HC->sscore, kmain+k);
-		gretl_vector_set(HC->sscore, kmain+k, psum);
-	    }
+    if (HC->sigma <= 0 || fabs(arho) > 3.5) {
+	err = E_NAN;
+    } else {
+	HC->rho = tanh(arho);
+	*ca = cosh(arho);
+	*sa = sinh(arho);
+	err = setup_ndx_functions(HC);
+    }
 
-	    /* score for sigma and arho */
-	    if (sel) {
-		x = (ut * (ut - sa*mills) - 1) / HC->sigma;
-		gretl_matrix_set(HC->score, i, npar-2, x); 
-		psum = x + gretl_vector_get(HC->sscore, npar-2);
-		gretl_vector_set(HC->sscore, npar-2, psum);
+    return err;
+}
 
-		x = mills * ca * (ut + HC->rho*ndxt);
-		gretl_matrix_set(HC->score, i, npar-1, x); 
-		psum = x + gretl_vector_get(HC->sscore, npar-1);
-		gretl_vector_set(HC->sscore, npar-1, psum);
-	    }
+static double h_loglik (const double *param, void *ptr)
+{
+    h_container *HC = (h_container *) ptr;
+    double ll0 = 0, ll1 = 0, ll2 = 0;
+    double ca, sa, ndxt, ut = 0;
+    double P, mills, tmp, psum;
+    double lnsig, x, ll = NADBL;
+    int sel, k, kmax, npar; 
+    int i, j, jj;
+    int err;
 
-	    if (sel) {
-		j++;
+    err = h_common_setup(HC, param, &ca, &sa);
+    if (err) {
+	return NADBL;
+    }
+
+    kmax = HC->kmain + HC->ksel;
+    npar = kmax + 2;
+
+    lnsig = log(HC->sigma);
+    
+#if HDEBUG > 1
+    gretl_matrix_print(HC->beta, "beta");
+    gretl_matrix_print(HC->gama, "gama");
+    fprintf(stderr, "sigma = %12.6f, rho = %12.6f\n", HC->sigma, HC->rho);
+#endif
+
+    gretl_matrix_zero(HC->score);
+    gretl_matrix_zero(HC->sscore);
+
+    /* i goes through all obs, while j keeps track of the uncensored
+       ones */
+    j = 0;
+    for (i=0; i<HC->ntot; i++) {
+	sel = (gretl_vector_get(HC->d, i) == 1.0);
+	ndxt = gretl_vector_get(HC->ndx, i);
+	if (sel) {
+	    ut = gretl_vector_get(HC->u, j);
+	    x = ca * (ndxt + HC->rho*ut);
+	    ll1 -= LN_SQRT_2_PI + 0.5*ut*ut + lnsig;
+	    P = normal_cdf(x);
+	    mills = invmills(-x);
+	    ll2 += log(P);
+	} else {
+	    P = normal_cdf(-ndxt);
+	    mills = -invmills(ndxt);
+	    ll0 += log(P);
+	}
+
+	/* score for beta */
+	if (sel) {
+	    tmp = (ut - sa*mills)/HC->sigma;
+	    for (k=0; k<HC->kmain; k++) {
+		x = tmp * gretl_matrix_get(HC->reg, j, k);
+		gretl_matrix_set(HC->score, i, k, x); 
+		psum = x + gretl_vector_get(HC->sscore, k);
+		gretl_vector_set(HC->sscore, k, psum);
 	    }
 	}
-	ll = ll0 + ll1 + ll2;
-#if HDEBUG
-	fprintf(stderr, "ll0 = %g, ll1 = %g, ll2 = %g: ll = %g (ntot = %d)\n", 
-		ll0, ll1, ll2, ll, HC->ntot);
-#endif
+
+	/* score for gamma */
+	tmp = sel ? ca*mills : mills;
+	for (k=0; k<HC->ksel; k++) {
+	    jj = HC->kmain + k;
+	    x = tmp * gretl_matrix_get(HC->selreg, i, k);
+	    gretl_matrix_set(HC->score, i, jj, x); 
+	    psum = x + gretl_vector_get(HC->sscore, HC->kmain+k);
+	    gretl_vector_set(HC->sscore, jj, psum);
+	}
+
+	/* score for sigma and arho */
+	if (sel) {
+	    x = (ut * (ut - sa*mills) - 1) / HC->sigma;
+	    gretl_matrix_set(HC->score, i, npar-2, x); 
+	    psum = x + gretl_vector_get(HC->sscore, npar-2);
+	    gretl_vector_set(HC->sscore, npar-2, psum);
+
+	    x = mills * ca * (ut + HC->rho*ndxt);
+	    gretl_matrix_set(HC->score, i, npar-1, x); 
+	    psum = x + gretl_vector_get(HC->sscore, npar-1);
+	    gretl_vector_set(HC->sscore, npar-1, psum);
+	}
+
+	if (sel) {
+	    j++;
+	}
     }
+
+    ll = ll0 + ll1 + ll2;
+
+#if HDEBUG
+    fprintf(stderr, "ll0 = %g, ll1 = %g, ll2 = %g: ll = %g (ntot = %d)\n", 
+	    ll0, ll1, ll2, ll, HC->ntot);
+#endif
     
     return ll;
 }
@@ -626,10 +662,214 @@ static int heckit_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll,
 	s[i] = gretl_vector_get(HC->sscore,i);
     }
 
-    return 1;
+    return 0;
 }
 
-double *heckit_hessian (const double *b, int n, BFGS_CRIT_FUNC func, 
+double *heckit_nhessian (const double *b, int n, BFGS_CRIT_FUNC func, 
+			 h_container *HC, int *err);
+
+#define h(i,j) h[4*(i-1)+j-1]
+
+/* analytical Hessian */
+
+int heckit_hessian (double *theta, gretl_matrix *H, void *ptr)
+{
+    h_container *HC = (h_container *) ptr;
+    double mills, dmills, x, z;
+    double ca, sa, sa2, ca2, invs, invs2;
+    double ndxt, ut, tmp = 0;
+    double h[16] = {0.0};
+    double c33 = 0;
+    double c34 = 0;
+    double c44 = 0;
+    int i, ii, j, jj, k, m, sel;
+    int kmax, npar;
+    int err;
+
+    err = h_common_setup(HC, theta, &ca, &sa);
+    if (err) {
+	return err;
+    }
+
+    kmax = HC->kmain + HC->ksel;
+    npar = kmax + 2;
+
+    invs = 1/HC->sigma;
+    invs2 = invs*invs;	
+   
+    sa2 = sa*sa;
+    ca2 = ca*ca;	
+
+    /* i goes through all obs, while j keeps track of the uncensored
+       ones */
+
+    gretl_matrix_zero(HC->H11);
+    gretl_matrix_zero(HC->H12);
+    gretl_matrix_zero(HC->H13);
+    gretl_matrix_zero(HC->H22);
+
+    j = 0;
+    for (i=0; i<HC->ntot; i++) {
+	sel = (gretl_vector_get(HC->d, i) == 1.0);
+	ndxt = gretl_vector_get(HC->ndx, i);
+	if (sel) {
+	    ut = gretl_vector_get(HC->u, j);
+	    x = ca * ndxt + sa * ut;
+	    z = sa * ndxt + ca * ut;
+	    mills = invmills(-x);
+	    dmills = -mills*(x + mills);
+
+	    h(1,1) = invs2 * (sa2*dmills - 1);	   
+	    h(1,3) = ut*h(1,1) - invs2*(ut - sa*mills); 
+	    h(1,4) = -invs * (ca*mills + sa*dmills*z);
+	    h(1,2) = -ca * sa * invs * dmills;
+	    h(2,2) = dmills * ca2;
+
+	    /* h23 = -invs * ca * sa * dmills * ut; */
+	    h(2,3) = h(1,2) * ut;
+	    h(2,4) = (dmills*ca*z + mills*sa);
+	    h(3,3) = invs2 * (-3*ut*ut + 2*ut*sa*mills + sa2*ut*ut*dmills +1);
+
+	    /* h34 = -invs*ut*(ca*mills + sa*dmills*z); */
+	    h(3,4) = h(1,4) * ut;
+	    h(4,4) = dmills*z*z + mills*x;
+	} else {	
+	    mills = -invmills(ndxt);
+	    dmills = -mills*(ndxt + mills);
+	    for (ii=0; ii<16; ii++) {
+		h[ii] = 0.0;
+	    }
+	    h(2,2) = dmills;
+	}
+
+	if (sel) {
+	    for (m=0; m<HC->kmain; m++) {
+		x = gretl_matrix_get(HC->reg, j, m);
+		    
+		for (k=0; k<=m; k++) {
+		    tmp = gretl_matrix_get(HC->H11, m, k); 
+		    tmp += h(1,1) * x * gretl_matrix_get(HC->reg, j, k);
+		    gretl_matrix_set(HC->H11, m, k, tmp);
+		    gretl_matrix_set(HC->H11, k, m, tmp);
+		}
+	      
+		for (k=0; k<HC->ksel; k++) {
+		    tmp = gretl_matrix_get(HC->H12, m, k);
+		    tmp += h(1,2) * x * gretl_matrix_get(HC->selreg, i, k);
+		    gretl_matrix_set(HC->H12, m, k, tmp);
+		}
+
+		tmp = gretl_matrix_get(HC->H13, m, 0);
+		tmp += h(1,3) * x;
+		gretl_matrix_set(HC->H13, m, 0, tmp);
+		    
+		tmp = gretl_matrix_get(HC->H13, m, 1);
+		tmp += h(1,4) * x;
+		gretl_matrix_set(HC->H13, m, 1, tmp);
+	    }
+	}
+
+	for (m=0; m<HC->ksel; m++) {
+	    x = gretl_matrix_get(HC->selreg, i, m);
+		
+	    for (k=0; k<=m; k++) {
+		tmp = gretl_matrix_get(HC->H22, m, k);
+		tmp += h(2,2) * x * gretl_matrix_get(HC->selreg, i, k);
+		gretl_matrix_set(HC->H22, m, k, tmp);
+		gretl_matrix_set(HC->H22, k, m, tmp);
+	    }
+
+	    if (sel) {
+		ii = HC->kmain + m;
+		tmp = gretl_matrix_get(HC->H13, ii, 0);
+		tmp += h(2,3) * x;
+		gretl_matrix_set(HC->H13, ii, 0, tmp);
+		
+		tmp = gretl_matrix_get(HC->H13, ii, 1);
+		tmp += h(2,4) * x;
+		gretl_matrix_set(HC->H13, ii, 1, tmp);
+	    }
+	}
+	    
+	if (sel) { 
+	    c33 += h(3,3);
+	    c34 += h(3,4);
+	    c44 += h(4,4);
+	    j++;
+	}
+    }
+
+#if 0
+    gretl_matrix_print(HC->H11, "H11");
+    gretl_matrix_print(HC->H12, "H12");
+    gretl_matrix_print(HC->H22, "H22");
+    gretl_matrix_print(HC->H13, "H13");
+#endif
+
+    /* fill H up (and flip sign while we're at it) */
+
+    for (i=0; i<HC->kmain; i++) {
+	for (j=i; j<HC->kmain; j++) {
+	    tmp = -gretl_matrix_get(HC->H11, i, j);
+	    gretl_matrix_set(H, i, j, tmp);
+	    gretl_matrix_set(H, j, i, tmp);
+	}
+	
+	for (j=0; j<HC->ksel; j++) {
+	    jj = HC->kmain + j;
+	    tmp = -gretl_matrix_get(HC->H12, i, j);
+	    gretl_matrix_set(H, i, jj, tmp);
+	    gretl_matrix_set(H, jj, i, tmp);
+			
+	}	
+    }
+
+    for (i=0; i<HC->ksel; i++) {
+	ii = HC->kmain + i;
+	for (j=i; j<HC->ksel; j++) {
+	    jj = HC->kmain + j;
+	    tmp = -gretl_matrix_get(HC->H22, i, j);
+	    gretl_matrix_set(H, ii, jj, tmp);
+	    gretl_matrix_set(H, jj, ii, tmp);
+	}
+	    
+    }
+
+    for (i=0; i<kmax; i++) {
+	for (j=0; j<2; j++) {
+	    jj = kmax + j;
+	    tmp = -gretl_matrix_get(HC->H13, i, j);
+	    gretl_matrix_set(H, i, jj, tmp);
+	    gretl_matrix_set(H, jj, i, tmp);
+	}	    
+    }
+
+    ii = npar - 2;
+    jj = npar - 1;
+
+    gretl_matrix_set(H, ii, ii, -c33);
+    gretl_matrix_set(H, ii, jj, -c34);
+    gretl_matrix_set(H, jj, ii, -c34);
+    gretl_matrix_set(H, jj, jj, -c44);
+
+    return err;
+}
+
+static int heckit_hessian_inverse (double *theta, gretl_matrix *H, 
+				   void *ptr)
+{
+    int err = heckit_hessian(theta, H, ptr);
+
+    if (!err) {
+	err = gretl_invert_symmetric_matrix(H);
+    }
+
+    return err;
+}
+
+#undef h
+ 
+double *heckit_nhessian (const double *b, int n, BFGS_CRIT_FUNC func, 
 			h_container *HC, int *err)
 {
     double x, eps = 1.0e-05;
@@ -682,6 +922,7 @@ double *heckit_hessian (const double *b, int n, BFGS_CRIT_FUNC func,
 	}
     }
 
+
     gretl_matrix_xtr_symmetric(H);
     gretl_invert_symmetric_matrix(H);
 
@@ -703,13 +944,13 @@ double *heckit_hessian (const double *b, int n, BFGS_CRIT_FUNC func,
 }
 
 /*
-  What we should do here is not entirely clear: we set yhat to the
-  linear predictor, computed over uncensored AND censored
-  observations.  
+   What we should do here is not entirely clear: we set yhat to the
+   linear predictor, computed over uncensored AND censored
+   observations.  
 
-  On the contrary, uhat is defined as 
-  y - yhat	for uncensored observations, and as
-  E(u|d=0)	for censored observations
+   On the contrary, uhat is defined as 
+   y - yhat	for uncensored observations, and as
+   E(u|d=0)	for censored observations
 */
 
 static void heckit_yhat_uhat (MODEL *hm, h_container *HC, 
@@ -789,14 +1030,6 @@ static void heckit_yhat_uhat (MODEL *hm, h_container *HC,
 			     GRETL_TYPE_DOUBLE_ARRAY,
 			     pdinfo->n * sizeof *llt);
     }
-
-#if 0
-    /* R-squared based on correlation? */
-    x = gretl_corr(hm->t1, hm->t2, Z[depvar], hm->yhat, NULL);
-    if (!na(x)) {
-	hm->rsq = x * x;
-    }
-#endif
 }
 
 /*
@@ -837,7 +1070,7 @@ static int transcribe_heckit_params (MODEL *hm, h_container *HC, DATAINFO *pdinf
     }
 
     hm->lnL = HC->ll;
-    mle_criteria(hm, 0); /* FIXME? */
+    mle_criteria(hm, 0);
     hm->sigma = HC->sigma;
     hm->rho = HC->rho;
 
@@ -857,7 +1090,7 @@ static int transcribe_heckit_params (MODEL *hm, h_container *HC, DATAINFO *pdinf
 }
 
 static int transcribe_2step_vcv (MODEL *pmod, const gretl_matrix *S, 
-				  const gretl_matrix *Vp)
+				 const gretl_matrix *Vp)
 {
     int m = S->rows;
     int n = Vp->rows;
@@ -1030,8 +1263,11 @@ static int adjust_ml_vcv_hyperbolic (h_container *HC)
     /*
       Here we modify $vcv as if rho was used in BFGS instead 
       of atanh(rho); we use the fact that 
+
       \frac{\partial tanh(a)}{\partial a} = (cosh(a))^{-2}
+
       which translates into
+
       \frac{\partial \rho}{\partial a} = 1 - \rho^2
     */
 
@@ -1080,12 +1316,14 @@ static gretl_matrix *heckit_init_H (double *theta,
 
 int heckit_ml (MODEL *hm, h_container *HC, gretlopt opt, PRN *prn)
 {
+    gretl_matrix *H = NULL;
     gretl_matrix *init_H = NULL;
     int maxit, fncount, grcount;
-    double hij, rho, toler;
+    double rho, gradtol = 1.0e-06, toler = 1.0e-8;
     double *hess = NULL;
     double *theta = NULL;
-    int i, j, k, np = HC->kmain + HC->ksel + 2;
+    int i, j, np = HC->kmain + HC->ksel + 2;
+    int use_bfgs = 0;
     int err = 0;
 
     theta = malloc(np * sizeof *theta);
@@ -1111,25 +1349,53 @@ int heckit_ml (MODEL *hm, h_container *HC, gretlopt opt, PRN *prn)
 
     theta[np-1] = atanh(rho);
 
+    if (libset_get_int(GRETL_OPTIM) == OPTIM_BFGS) {
+	use_bfgs = 1;
+    }
+
     BFGS_defaults(&maxit, &toler, HECKIT);
 
 #if INITH_OPG
-    init_H = heckit_init_H(theta, HC, np);
+    if (use_bfgs) {
+	init_H = heckit_init_H(theta, HC, np);
+    }
 #endif
 
-    err = BFGS_max(theta, np, maxit, toler, &fncount, 
-		   &grcount, h_loglik, C_LOGLIK,
-		   heckit_score, HC, init_H,
-		   (prn != NULL)? OPT_V : OPT_NONE, prn);
+    if (use_bfgs) {
+	err = BFGS_max(theta, np, maxit, toler, &fncount, 
+		       &grcount, h_loglik, C_LOGLIK,
+		       heckit_score, HC, init_H,
+		       (prn != NULL)? OPT_V : OPT_NONE, prn);
+    } else {	
+	gretlopt maxopt = opt & OPT_V;
+
+	err = newton_raphson_max(theta, np, maxit, toler, gradtol,
+				 &fncount, C_LOGLIK, h_loglik, 
+				 heckit_score, heckit_hessian,
+				 HC, maxopt, prn);
+    } 
 
     gretl_matrix_free(init_H);
 
     if (!err) {
 	HC->ll = hm->lnL = h_loglik(theta, HC);
-	gretl_model_set_int(hm, "fncount", fncount);	
-	gretl_model_set_int(hm, "grcount", grcount);	
+	if (use_bfgs) {
+	    gretl_model_set_int(hm, "fncount", fncount);	
+	    gretl_model_set_int(hm, "grcount", grcount);
+	} else {	    
+	    gretl_model_set_int(hm, "iters", fncount);
+	} 	
 	HC->lambda = HC->sigma * HC->rho;
-	hess = heckit_hessian(theta, np, h_loglik, HC, &err);
+
+	H = gretl_matrix_alloc(np, np);
+	if (H == NULL) {
+	    err = E_ALLOC;
+	} else {
+	    err = heckit_hessian_inverse(theta, H, HC);
+	}
+#if 0
+	hess = heckit_nhessian(theta, np, h_loglik, HC, &err);
+#endif
     }
 
     if (!err) {
@@ -1140,16 +1406,7 @@ int heckit_ml (MODEL *hm, h_container *HC, gretlopt opt, PRN *prn)
     }
 
     if (!err) {
-	k = 0;
-	for (i=0; i<np; i++) {
-	    for (j=i; j<np; j++) {
-		hij = hess[k++];
-		gretl_matrix_set(HC->vcv, i, j, hij);
-		if (i != j) {
-		    gretl_matrix_set(HC->vcv, j, i, hij);
-		}
-	    }
-	}
+	gretl_matrix_copy_values(HC->vcv, H);
 
 	if (opt & OPT_R) {
 	    gretl_matrix *GG = gretl_matrix_XTX_new(HC->score);
@@ -1161,6 +1418,7 @@ int heckit_ml (MODEL *hm, h_container *HC, gretlopt opt, PRN *prn)
 		gretl_matrix_qform(HC->vcv, GRETL_MOD_NONE, GG, tmp, GRETL_MOD_NONE);
 		gretl_matrix_copy_values(HC->vcv, tmp);
 	    }
+
 	    gretl_matrix_free(tmp);
 	    gretl_matrix_free(GG);
 	}
@@ -1171,60 +1429,24 @@ int heckit_ml (MODEL *hm, h_container *HC, gretlopt opt, PRN *prn)
 	add_lambda_to_ml_vcv(HC);
 #if HDEBUG
 	for (i=0; i<np; i++) {
-	    hij = gretl_matrix_get(HC->vcv, i, i);
+	    double hij = gretl_matrix_get(HC->vcv, i, i);
 	    fprintf(stderr, "theta[%d] = %12.6f, (%12.6f)\n", i, theta[i], sqrt(hij));
 	}
 #endif
     }
 
+    if (!err) {
+	np = HC->vcv->rows - 2;
+	err = gretl_model_write_vcv(hm, HC->vcv, np);
+    }
+
     free(hess);
     free(theta);
+    gretl_matrix_free(H);
 
     return err;
 }
 
-/*
-  This function copies the VCV matrix for the ML estimator into
-  the model struct.  Note that we don't transcribe the variances for
-  sigma and rho into the model.
-*/
-
-static int transcribe_ml_vcv (MODEL *pmod, h_container *HC)
-{
-    int nvc, npar = HC->vcv->rows - 2;
-    int i, j, k = 0;
-    double vij;
-
-    if (pmod->vcv != NULL) {
-	free(pmod->vcv);
-    }
-
-    if (pmod->sderr != NULL) {
-	free(pmod->sderr);
-    }
-    
-    nvc = (npar * npar + npar) / 2;
-
-    pmod->vcv = malloc(nvc * sizeof *pmod->vcv);
-    pmod->sderr = malloc(npar * sizeof *pmod->sderr);
-
-    if (pmod->vcv == NULL || pmod->sderr == NULL) {
-	return E_ALLOC;
-    }
-
-    for (i=0; i<npar; i++) {
-	for (j=i; j<npar; j++) {
-	    vij = gretl_matrix_get(HC->vcv, i, j);
-	    pmod->vcv[k++] = vij;
-	    if (i == j) {
-		pmod->sderr[i] = sqrt(vij);
-	    }
-	}
-    }
-
-    return 0;
-}
-
 static MODEL heckit_init (h_container *HC, double ***pZ, DATAINFO *pdinfo)
 {
 #if HDEBUG
@@ -1363,9 +1585,6 @@ MODEL heckit_estimate (const int *list, double ***pZ, DATAINFO *pdinfo,
     } else {
 	/* use MLE */
 	err = heckit_ml(&hm, HC, opt, vprn);
-	if (!err) {
-	    err = transcribe_ml_vcv(&hm, HC);
-	}
     } 
 
     if (err) {
