@@ -175,11 +175,14 @@ static int_container *int_container_new (int *list, double **Z,
     }
     IC->theta[i] = log(mod->sigma);
 
-#if INTDEBUG > 1
+#if INTDEBUG > 2
     fprintf(stderr, "nobs = %d\n", IC->nobs);
     fprintf(stderr, "t1-t2 = %d-%d\n", IC->t1, IC->t2);
     fprintf(stderr, "k = %d\n", IC->k);
     fprintf(stderr, "nx = %d\n", IC->nx);
+    for (i=0; i<=IC->nx; i++) {
+	fprintf(stderr, "theta[%2d] = %10.6f\n", i, IC->theta[i]);
+    }
 #endif
 
     return IC;
@@ -188,9 +191,8 @@ static int_container *int_container_new (int *list, double **Z,
 static int create_midpoint_y (int *list, double ***pZ, DATAINFO *pdinfo, 
 			      int **initlist)
 {
-    int n = list[0];
     int mpy = pdinfo->v;
-    int lv, hv;
+    double *lo, *hi, *mid;
     double x0, x1;
     int i, t, err;
 
@@ -199,24 +201,25 @@ static int create_midpoint_y (int *list, double ***pZ, DATAINFO *pdinfo,
 	return err;
     }
 
-    lv = list[1];
-    hv = list[2];
+    lo = (*pZ)[list[1]];
+    hi = (*pZ)[list[2]];
+    mid = (*pZ)[mpy];
 
     for (t=pdinfo->t1; t<=pdinfo->t2 && !err; t++) {
-	x0 = (*pZ)[lv][t];
-	x1 = (*pZ)[hv][t];
+	x0 = lo[t];
+	x1 = hi[t];
 
 	if (na(x0)) {
-	    (*pZ)[mpy][t] = x1;
+	    mid[t] = x1;
 	} else if (na(x1)) {
-	    (*pZ)[mpy][t] = x0;
+	    mid[t] = x0;
 	} else if (x0 > x1) {
 	    gretl_errmsg_sprintf(_("Obs %d: lower bound (%g) "
 				   "exceeds upper (%g)"), t + 1,
 				 x0, x1);
 	    err = E_DATA;
 	} else {
-	    (*pZ)[mpy][t] = 0.5 * (x0 + x1);
+	    mid[t] = 0.5 * (x0 + x1);
 	}
     }
 
@@ -224,19 +227,19 @@ static int create_midpoint_y (int *list, double ***pZ, DATAINFO *pdinfo,
 	return err;
     }
 
-    *initlist = gretl_list_new(n - 1);
+    *initlist = gretl_list_new(list[0] - 1);
 
     if (*initlist == NULL) {
 	err = E_ALLOC;
     } else {
 	(*initlist)[1] = mpy;
-	for (i=3; i<=n; i++) {
+	for (i=3; i<=list[0]; i++) {
 	    (*initlist)[i-1] = list[i];
 	}
 
 #if INTDEBUG > 1
 	for (t=pdinfo->t1; t<=pdinfo->t2; t++) {
-	    fprintf(stderr, "%2d: %g\n", t, (*pZ)[mpy][t]);
+	    fprintf(stderr, "%2d: %g\n", t, mid[t]);
 	}
 #endif
     }
@@ -244,58 +247,91 @@ static int create_midpoint_y (int *list, double ***pZ, DATAINFO *pdinfo,
     return err;
 }
 
+static void loglik_prelim (const double *theta, int_container *IC)
+{
+    int i, t, k = IC->k;
+    double ndxt, z0, z1, x0, x1;
+    double sigma = exp(theta[k-1]);
+
+    for (t=0; t<IC->nobs; t++) {
+	ndxt = 0.0;
+	for (i=0; i<IC->nx; i++) {
+	    ndxt += theta[i] * IC->X[i][t];
+	}
+
+	IC->ndx[t] = ndxt;
+
+	x0 = IC->lo[t];
+	x1 = IC->hi[t];
+
+	switch (IC->obstype[t]) {
+	case INT_LOW:
+	    z1 = (x1 - ndxt)/sigma;
+	    IC->dP[t] = normal_cdf(z1);
+	    IC->f0[t] = 0;
+	    IC->f1[t] = normal_pdf(z1) / IC->dP[t];
+	    break;
+	case INT_HIGH:
+	    z0 = (x0 - ndxt)/sigma;
+	    IC->dP[t] = normal_cdf_comp(z0);
+	    IC->f0[t] = normal_pdf(z0) / IC->dP[t];
+	    IC->f1[t] = 0;
+	    break;
+	case INT_MID:
+	    z0 = (x0 - ndxt)/sigma;
+	    z1 = (x1 - ndxt)/sigma;
+	    IC->dP[t] = normal_cdf(z1) - normal_cdf(z0);
+	    IC->f0[t] = normal_pdf(z0) / IC->dP[t];
+	    IC->f1[t] = normal_pdf(z1) / IC->dP[t];
+	    break;
+	case INT_POINT:
+	    z0 = (x0 - ndxt)/sigma;
+	    IC->dP[t] = normal_pdf(z0)/sigma;
+	    IC->f0[t] = IC->f1[t] = 0;
+	}
+    }
+}
+
 static double int_loglik (const double *theta, void *ptr)
 {
     int_container *IC = (int_container *) ptr;
     double x0, x1, z0, z1;
-    double ndx, ll = 0.0;
     double derivs = 0.0, derivb = 0.0;
-    int i, t;
-    int k = IC->k;
-    double sigma = exp(theta[k-1]);
+    double sigma, ndxt, ll = 0.0;
+    int i, t, k = IC->k;
+
+    sigma = exp(theta[k-1]);
 
     for (i=0; i<k; i++) {
 	IC->g[i] = 0;
     }
 
+    loglik_prelim(theta, IC);
+
     for (t=0; t<IC->nobs; t++) {
 	x0 = IC->lo[t];
 	x1 = IC->hi[t];
-
-	ndx = 0;
-	for (i=0; i<IC->nx; i++) {
-	    ndx += theta[i] * IC->X[i][t];
-	}
-
-	IC->ndx[t] = ndx;
+	ndxt = IC->ndx[t];
 
 	switch (IC->obstype[t]) {
 	case INT_LOW:
-	    z1 = (x1 - ndx)/sigma;
-	    IC->dP[t] = normal_cdf(z1);
-	    IC->f1[t] = normal_pdf(z1) / IC->dP[t];
+	    z1 = (x1 - ndxt)/sigma;
 	    derivb = -IC->f1[t]/sigma;
 	    derivs = -IC->f1[t]*z1;
 	    break;
 	case INT_HIGH:
-	    z0 = (x0 - ndx)/sigma;
-	    IC->dP[t] = normal_cdf_comp(z0);
-	    IC->f0[t] = normal_pdf(z0) / IC->dP[t];
+	    z0 = (x0 - ndxt)/sigma;
 	    derivb = IC->f0[t]/sigma;
 	    derivs = IC->f0[t]*z0;
 	    break;
 	case INT_MID:
-	    z0 = (x0 - ndx)/sigma;
-	    z1 = (x1 - ndx)/sigma;
-	    IC->dP[t] = normal_cdf(z1) - normal_cdf(z0);
-	    IC->f0[t] = normal_pdf(z0) / IC->dP[t];
-	    IC->f1[t] = normal_pdf(z1) / IC->dP[t];
+	    z0 = (x0 - ndxt)/sigma;
+	    z1 = (x1 - ndxt)/sigma;
 	    derivb = (IC->f0[t] - IC->f1[t])/sigma;
 	    derivs = (IC->f0[t]*z0 - IC->f1[t]*z1);
 	    break;
 	case INT_POINT:
-	    z0 = (x0 - ndx)/sigma;
-	    IC->dP[t] = normal_pdf(z0)/sigma;
+	    z0 = (x0 - ndxt)/sigma;
 	    derivb = z0/sigma;
 	    derivs = z0*z0 - 1;
 	}
@@ -304,12 +340,11 @@ static double int_loglik (const double *theta, void *ptr)
 
 	for (i=0; i<IC->nx; i++) {
 	    IC->G[i][t] = derivb * IC->X[i][t];
-	    IC->g[i] += IC->G[i][t];
+	    IC->g[i] += IC->G[i][t]; 
 	}
 
 	IC->G[k-1][t] = derivs;
-	IC->g[k-1] += IC->G[k-1][t];
-
+	IC->g[k-1] += derivs;
     }
     
 #if INTDEBUG > 1
@@ -334,81 +369,136 @@ static int int_score (double *theta, double *s, int npar, BFGS_CRIT_FUNC ll,
 	s[i] = IC->g[i];
     }
 
-    return 1;
+    return 0;
 }
 
-static gretl_matrix *int_hess (int_container *IC, int *err)
+int int_ahess (double *theta, gretl_matrix *V, void *ptr)
 {
-    double x, smal = 1.0e-07;
-    gretl_matrix *V = NULL;
-    double *q, *g, *gplus, *gminus, *hss;
-    int k = IC->k;
-    int i, j, n;
+    int_container *IC = (int_container *) ptr;
+    double z0 = 0, z1 = 0, mu = 0;
+    double q0 = 0, q1 = 0, nu = 0;
+    double x, vij, x0, x1;    
+    double f0, f1, Hss = 0;
+    double sigma, ndxt, lambda = 0;
+    int i, j, t, k = IC->k;
+    int err = 0;
 
-    q = malloc(k * sizeof *q); 
-    g = malloc(k * sizeof *g); 
-    gplus = malloc(k * sizeof *gplus); 
-    gminus = malloc(k * sizeof *gminus);
-    hss = malloc(k * k * sizeof *hss);
-
-    if (q == NULL || g == NULL || gplus == NULL ||
-	gminus == NULL || hss == NULL) {
-	*err = E_ALLOC;
-	goto bailout;
-    }
-
-    V = gretl_matrix_alloc(k, k);
-    if (V == NULL) {
-	*err = E_ALLOC;
-	goto bailout;
-    }	
+    sigma = exp(theta[k-1]);
+    loglik_prelim(theta, IC);
 
     for (i=0; i<k; i++) {
-	g[i] = IC->g[i];
-	q[i] = IC->theta[i];
+	for (j=0; j<k; j++) {
+	    gretl_matrix_set(V, i, j, 0);
+	}
     }
 
-    n = 0;
+#if INTDEBUG > 1
+    fprintf(stderr, "int_ahess:\n");
     for (i=0; i<k; i++) {
-	q[i] += smal;
-	int_loglik(q, IC);
-	for (j=0; j<k; j++) {
-	    gplus[j] = IC->g[j];
+	fprintf(stderr, "theta[%2d] = %20.16f\n", i, theta[i]);
+    }
+#endif
+
+    for (t=0; t<IC->nobs; t++) {
+	/* ndx, dP, f0, f1 etc should already be there */
+	x0 = IC->lo[t];
+	x1 = IC->hi[t];
+	ndxt = IC->ndx[t];
+	f0 = IC->f0[t];
+	f1 = IC->f1[t];
+
+	switch (IC->obstype[t]) {
+	case INT_LOW:
+	    z1 = (x1 - ndxt)/sigma;
+	    mu     = -f1/sigma;
+	    lambda = mu * z1;
+	    q1     = (z1*z1 - 1.0);
+	    nu     = mu * q1;
+	    break;
+	case INT_HIGH:
+	    z0 = (x0 - ndxt)/sigma;
+	    mu     = f0/sigma;
+	    lambda = mu * z0;
+	    q0     = (z0*z0 - 1.0);
+	    nu     = mu * q0;
+	    break;
+	case INT_MID:
+	    z0 = (x0 - ndxt)/sigma;
+	    z1 = (x1 - ndxt)/sigma;
+	    mu     = (f0 - f1)/sigma;
+	    lambda = (f0*z0 - f1*z1)/sigma;
+	    q0     = (z0*z0 - 1.0);
+	    q1     = (z1*z1 - 1.0);
+	    nu     = (f0 * q0 - f1 * q1)/sigma;
+	    break;
+	case INT_POINT:
+	    z0 = (x0 - ndxt)/sigma;
 	}
 
-	q[i] -= 2 * smal;
-	int_loglik(q, IC);
-	for (j=0; j<k; j++) {
-	    gminus[j] = IC->g[j];
+	if (IC->obstype[t] == INT_POINT) {
+	    x = 1.0 / (sigma*sigma);
+	} else {
+	    x = (mu*mu - lambda/sigma);
 	}
 
-	for (j=0; j<k; j++) {
-	    hss[n++] = (gplus[j] - gminus[j]) / (2 * smal);
+	for (i=0; i<IC->nx; i++) {
+	    x0 = IC->X[i][t];
+	    for (j=i; j<IC->nx; j++) {
+		x1 = IC->X[j][t];
+		vij = gretl_matrix_get(V, i, j);
+		vij += x * x0 * x1;
+		gretl_matrix_set(V, i, j, vij);
+	    }
+	}
+
+	if (IC->obstype[t] == INT_POINT) {
+	    x = 2 * z0 / sigma;
+	} else {
+	    x = (mu*lambda*sigma - nu);
+	}
+
+	for (i=0; i<IC->nx; i++) {
+	    x0 = IC->X[i][t];
+	    vij = gretl_matrix_get(V, i, k-1);
+	    vij += x * x0;
+	    gretl_matrix_set(V, i, k-1, vij);
+	}
+
+	if (IC->obstype[t] == INT_POINT) {
+	    x = 2 * (z0*z0);
+	    Hss += x;
+	} else {
+	    x = lambda*sigma;
+	    Hss += x*(x+1) - (f0 * q0 * z0 - f1 * q1 *z1);
 	}
     }
 
+    gretl_matrix_set(V, k-1, k-1, Hss);
+
     for (i=0; i<k; i++) {
 	for (j=i; j<k; j++) {
-	    x = -0.5 * (hss[i+k*j] + hss[i*k+j]);
-	    gretl_matrix_set(V, i, j, x);
-	    gretl_matrix_set(V, j, i, x);
+	    vij = gretl_matrix_get(V, i, j);
+	    gretl_matrix_set(V, j, i, vij);
 	}
     }
 
-    *err = gretl_invert_symmetric_matrix(V);
-#if INTDEBUG
-    gretl_matrix_print(V, "V = -H^{-1}");
+#if INTDEBUG > 1
+    gretl_matrix_print(V, "Hessian (analytical)");
 #endif
 
- bailout:
+    return err;
+}
 
-    free(q);
-    free(g);
-    free(gplus);
-    free(gminus);
-    free(hss);
+static int 
+int_ahess_inverse (double *theta, gretl_matrix *V, void *ptr)
+{
+    int err = int_ahess(theta, V, ptr);
 
-    return V;
+    if (!err) {
+	err = gretl_invert_symmetric_matrix(V);
+    }
+
+    return err;
 }
 
 static gretl_matrix *intreg_sandwich (int_container *IC,
@@ -462,7 +552,13 @@ static gretl_matrix *intreg_sandwich (int_container *IC,
 static gretl_matrix *intreg_VCV (int_container *IC, gretlopt opt,
 				 int *err)
 {
-    gretl_matrix *H = int_hess(IC, err);
+    gretl_matrix *H = gretl_zero_matrix_new(IC->k, IC->k);
+
+    if (H == NULL) {
+	*err = E_ALLOC;
+    } else {
+	*err = int_ahess_inverse(IC->theta, H, IC);
+    }
 
     if (!*err && (opt & OPT_R)) {
 	return intreg_sandwich(IC, H, err);
@@ -474,11 +570,9 @@ static gretl_matrix *intreg_VCV (int_container *IC, gretlopt opt,
 static void int_compute_gresids (int_container *IC)
 {
     int t, k = IC->k;
-    double ndx, sigma = exp(IC->theta[k-1]);
+    double sigma = exp(IC->theta[k-1]);
 
     for (t=0; t<IC->nobs; t++) {
-	ndx = IC->ndx[t];
-
 	switch (IC->obstype[t]) {
 	case INT_LOW:
 	    IC->uhat[t] = -sigma * IC->f1[t];
@@ -490,18 +584,18 @@ static void int_compute_gresids (int_container *IC)
 	    IC->uhat[t] = sigma * (IC->f0[t] - IC->f1[t]);
 	    break;
 	case INT_POINT:
-	    IC->uhat[t] = IC->lo[t] - ndx;
+	    IC->uhat[t] = IC->lo[t] - IC->ndx[t];
 	}
     }
 } 
 
+/*
+  Wald test for zeroing all coefficient apart from the constant,
+  which is assumed to be the first explanatory variable
+*/
+
 static double chisq_overall_test (int_container *IC)
 {
-    /*
-      Wald test for zeroing all coefficient apart from the constant,
-      which is assumed to be the first explanatory variable
-    */
-
     gretl_vector *b;
     gretl_matrix *V;
     int k = IC->nx - 1;
@@ -539,37 +633,32 @@ static double chisq_overall_test (int_container *IC)
     return ret;
 } 
 
+/* 
+   Here we exploit the following properties of sub-support normal 
+   moments: if x ~ N(0,1) and A = [a,b], then
+
+   E(x^n | x \in A) = (n-1) E(x^{n-2} | x \in A) + 
+   + \frac{a^{n-1}\phi(a) - b^{n-1}\phi(b)}{\Phi(b)-\Phi(a)}
+
+   We have:
+
+   E(x^0 | x \in A) = 1 (trivially) 
+   E(x^1 | x \in A) = f0 - f1
+   E(x^2 | x \in A) = 1 + lo*f0 - hi*f1
+   ...
+
+   Returns the matrix of the orthogonality conditions (conditional
+   moments); the non-zero ones are stored in the two pointers sm3
+   and sm4.
+*/
+
 static gretl_matrix *cond_moments (int_container *IC, double *sm3, 
 				   double *sm4)
 {
-    /* 
-       Here we exploit the following properties of sub-support normal 
-       moments: if x ~ N(0,1) and A = [a,b], then
-
-       E(x^n | x \in A) = (n-1) E(x^{n-2} | x \in A) + 
-       + \frac{a^{n-1}\phi(a) - b^{n-1}\phi(b)}{\Phi(b)-\Phi(a)}
-
-       We have:
-
-       E(x^0 | x \in A) = 1 (trivially) 
-       E(x^1 | x \in A) = f0 - f1
-       E(x^2 | x \in A) = 1 + lo*f0 - hi*f1
-       ...
-
-       Returns the matrix of the orthogonality conditions (conditional
-       moments); the non-zero ones are stored in the two pointers sm3
-       and sm4.
-    */
-
+    gretl_matrix *ret = NULL;
     int n = IC->nobs;  
     int k = IC->k;  
     int noc = k + 2;  
-    double *ndx = IC->ndx;
-    double *lo = IC->lo; 
-    double *hi = IC->hi;
-    double *f0 = IC->f0;
-    double *f1 = IC->f1;
-    gretl_matrix *ret = NULL;
     double a, b, phi0, phi1, m1, m2, u, x, y;
     double m3 = 0.0, m4 = 0.0;
     double sigma = exp(IC->theta[k - 1]);
@@ -586,26 +675,25 @@ static gretl_matrix *cond_moments (int_container *IC, double *sm3,
     for (t=0; t<n; t++) {
 	switch (IC->obstype[t]) {
 	case INT_LOW:
-	    b = (hi[t] - ndx[t])/sigma;
-	    phi1 = y = f1[t];
+	    b = (IC->hi[t] - IC->ndx[t])/sigma;
+	    phi1 = y = IC->f1[t];
 	    m1 = -phi1;
 	    m2 = 1 - (y *= b);
 	    m3 = 2*m1 - (y *= b);
 	    m4 = 3*m2 - (y *= b);
 	    break;
 	case INT_HIGH:
-	    a = (lo[t] - ndx[t])/sigma;
-	    m1 = phi0 = x = f0[t];
+	    a = (IC->lo[t] - IC->ndx[t])/sigma;
+	    m1 = phi0 = x = IC->f0[t];
 	    m2 = 1 + (x *= a);
 	    m3 = 2*m1 + (x *= a);
 	    m4 = 3*m2 + (x *= a);
 	    break;
 	case INT_MID:
-	    a = (lo[t] - ndx[t])/sigma;
-	    b = (hi[t] - ndx[t])/sigma;
-
-	    phi0 = x = f0[t];
-	    phi1 = y = f1[t];
+	    a = (IC->lo[t] - IC->ndx[t])/sigma;
+	    b = (IC->hi[t] - IC->ndx[t])/sigma;
+	    phi0 = x = IC->f0[t];
+	    phi1 = y = IC->f1[t];
 	    m1 = phi0 - phi1;
 	    m2 = 1 + (x *= a) - (y *= b);
 	    m3 = 2*m1 + (x *= a) - (y *= b);
@@ -621,7 +709,7 @@ static gretl_matrix *cond_moments (int_container *IC, double *sm3,
 	*sm3 += m3;
 	*sm4 += m4;
 
-	for(i=0; i<k; i++) {
+	for (i=0; i<k; i++) {
 	    gretl_matrix_set(ret, t, i, IC->G[i][t]);
 	}
 
@@ -632,24 +720,24 @@ static gretl_matrix *cond_moments (int_container *IC, double *sm3,
     return ret;
 }
 
+/*
+  Conditional moment test for normality: in practice, a regression
+  where the dep. var is 1 and the matrix of explanatory variables
+  is [ G | sk | ku ], where:
+
+  * G is the by-obs score matrix (should already be in IC by now)
+  * sk[t] is an unbiased estimator of \epsilon_t^3
+  * ku[t] is an unbiased estimator of (\epsilon_t^4 - 3)
+
+  The test statistic is the explained sum of squares.
+*/
+
 static int intreg_normtest (int_container *IC, double *teststat)
 {
-    /*
-      Conditonal moment test for normality: in practice, a regression
-      where the dep. var is 1 and the matrix of explanatory variables
-      is [ G | sk | ku ], where:
-
-      * G is the by-obs score matrix (should already be in IC by now)
-      * sk[t] is an unbiased estimator of \epsilon_t^3
-      * ku[t] is an unbiased estimator of (\epsilon_t^4 - 3)
-
-      The test statistic is the explained sum of squares.
-    */
-
     gretl_matrix *condmom;
-    double skew, kurt;
     gretl_matrix *GG = NULL;
     gretl_matrix *g = NULL;
+    double skew, kurt;
     int noc = IC->k + 2;
     int err = 0;
 
@@ -663,21 +751,18 @@ static int intreg_normtest (int_container *IC, double *teststat)
 
     if (GG == NULL || g == NULL) {
 	err = E_ALLOC;
-	goto bailout;
-    }
-
-    err = gretl_invert_symmetric_matrix(GG);
-    if (!err) {
-	gretl_vector_set(g, noc-2, skew);
-	gretl_vector_set(g, noc-1, kurt);
-	*teststat = gretl_scalar_qform(g, GG, &err);
     } else {
-	gretl_matrix_print(condmom, "conditional moment matrix");
-	*teststat = NADBL;
+	err = gretl_invert_symmetric_matrix(GG);
+	if (!err) {
+	    gretl_vector_set(g, noc-2, skew);
+	    gretl_vector_set(g, noc-1, kurt);
+	    *teststat = gretl_scalar_qform(g, GG, &err);
+	} else {
+	    gretl_matrix_print(condmom, "conditional moment matrix");
+	    *teststat = NADBL;
+	}
     }
 
- bailout:
-
     gretl_matrix_free(condmom);
     gretl_matrix_free(GG);
     gretl_matrix_free(g);
@@ -685,62 +770,39 @@ static int intreg_normtest (int_container *IC, double *teststat)
     return err;
 }
 
-static int add_norm_test_to_model (MODEL *pmod, double X2)
-{
-    ModelTest *test = model_test_new(GRETL_TEST_NORMAL);
-    int err = 0;
-
-    if (test != NULL) {
-        model_test_set_teststat(test, GRETL_STAT_NORMAL_CHISQ);
-        model_test_set_dfn(test, 2);
-        model_test_set_value(test, X2);
-        model_test_set_pvalue(test, chisq_cdf_comp(2, X2));
-        maybe_add_test_to_model(pmod, test);
-    } else {
-        err = 1;
-    }
-
-    return err;
-}
-
 static int fill_intreg_model (int_container *IC, gretl_matrix *V,
-			      gretlopt opt, const DATAINFO *pdinfo)
+			      int fncount, int grcount,
+			      const DATAINFO *pdinfo,
+			      gretlopt opt)
 {
     MODEL *pmod = IC->pmod;
     double x, ndx, u;
-    int i, j, n, m, k = IC->k, nx = IC->nx;
-    int obstype;
-
-    pmod->ci = INTREG;
+    int i, j, k = IC->k;
+    int obstype, vtype;
+    int err = 0;
 
+    pmod->ci = (opt & OPT_T)? TOBIT : INTREG;
     pmod->lnL = IC->ll;
     mle_criteria(pmod, 1);
-
-    if (pmod->vcv != NULL) {
-	free(pmod->vcv);
-    }
-
-    m =  k * (k - 1) / 2;
-
-    pmod->vcv = malloc(m * sizeof *pmod->vcv);
-    if (pmod->vcv == NULL) {
-	return E_ALLOC;
-    }
-
-    n = 0;
-    for (i=0; i<nx; i++) {
-	pmod->coeff[i] = IC->theta[i];
-	for (j=i; j<nx; j++) {
-	    x = gretl_matrix_get(V, i, j);
-	    if (i == j) {
-		pmod->sderr[i] = sqrt(x);
-	    }
-	    pmod->vcv[n++] = x;
-	}
-    }
-
     pmod->sigma = exp(IC->theta[k-1]);
 
+    for (i=0; i<IC->nx; i++) {
+	pmod->coeff[i] = IC->theta[i];
+    }
+
+    err = gretl_model_write_vcv(pmod, V, IC->nx);
+    if (err) {
+	return err;
+    }
+
+    vtype = (opt & OPT_R)? VCV_QML : VCV_HESSIAN;
+    gretl_model_set_vcv_info(pmod, VCV_ML, vtype);
+
+    /* get the s.e. of sigma via the delta method */
+    x = gretl_matrix_get(V, k-1, k-1);
+    x *= pmod->sigma * pmod->sigma;
+    gretl_model_set_double(pmod, "se_sigma", sqrt(x));
+
     int_compute_gresids(IC);
 
     j = 0;
@@ -751,27 +813,38 @@ static int fill_intreg_model (int_container *IC, gretl_matrix *V,
 	    IC->typecount[obstype] += 1;
 	    ndx = IC->ndx[j];
 	    u = IC->uhat[j];
-
 	    pmod->uhat[i] = u;
 	    pmod->yhat[i] = ndx;
 	    j++;
 	}
     }
 
+    /* mask invalid statistics */
     pmod->ybar = pmod->sdy = NADBL;
     pmod->ess = pmod->tss = NADBL;
     pmod->fstt = NADBL;
     pmod->rsq = pmod->adjrsq = NADBL;
 
-    gretl_model_set_int(pmod, "lovar", IC->lov);
-    gretl_model_set_int(pmod, "hivar", IC->hiv);
+    if (grcount > 0) {
+	/* BFGS */
+	gretl_model_set_int(pmod, "fncount", fncount);
+	gretl_model_set_int(pmod, "grcount", grcount);
+    } else {
+	/* Newton */
+	gretl_model_set_int(pmod, "iters", fncount);
+    }
+
     gretl_model_set_int(pmod, "n_left", IC->typecount[0]);
-    gretl_model_set_int(pmod, "n_both", IC->typecount[1]);
     gretl_model_set_int(pmod, "n_right", IC->typecount[2]);
-    gretl_model_set_int(pmod, "n_point", IC->typecount[3]);
+
+    if (!(opt & OPT_T)) {
+	gretl_model_set_int(pmod, "lovar", IC->lov);
+	gretl_model_set_int(pmod, "hivar", IC->hiv);
+	gretl_model_set_int(pmod, "n_both", IC->typecount[1]);
+	gretl_model_set_int(pmod, "n_point", IC->typecount[3]);
+    }
 
     if (opt & OPT_R) {
-	gretl_model_set_vcv_info(pmod, VCV_ML, VCV_QML);
 	pmod->opt |= OPT_R;
     }
 
@@ -781,8 +854,10 @@ static int fill_intreg_model (int_container *IC, gretl_matrix *V,
 	pmod->chisq = NADBL;
     }
 
-    /* the variable at list[1] will be deleted */
-    pmod->list[1] = 0;
+    if (!(opt & OPT_T)) {
+	/* the variable at list[1] will be deleted */
+	pmod->list[1] = 0;
+    }
 
     return 0;
 }
@@ -792,8 +867,10 @@ static int do_interval (int *list, double **Z, DATAINFO *pdinfo,
 {
     int_container *IC;
     gretl_matrix *V = NULL;
-    int maxit, fncount, grcount;
+    int maxit, fncount, grcount = 0;
     double toler, normtest = NADBL;
+    gretlopt maxopt = opt & OPT_V;
+    int use_bfgs = 0;
     int err = 0;
 
     IC = int_container_new(list, Z, pdinfo, mod);
@@ -803,9 +880,22 @@ static int do_interval (int *list, double **Z, DATAINFO *pdinfo,
 
     BFGS_defaults(&maxit, &toler, INTREG);
 
-    err = BFGS_max(IC->theta, IC->k, maxit, toler, 
-		   &fncount, &grcount, int_loglik, C_LOGLIK,
-		   int_score, IC, NULL, opt & OPT_V, prn);
+    if (libset_get_int(GRETL_OPTIM) == OPTIM_BFGS) {
+	use_bfgs = 1;
+    }    
+
+    if (use_bfgs) {
+	err = BFGS_max(IC->theta, IC->k, maxit, toler, 
+		       &fncount, &grcount, int_loglik, C_LOGLIK,
+		       int_score, IC, NULL, maxopt, prn);
+    } else {
+	double crittol = 1.0e-07;
+	double gradtol = 1.0e-07;
+	
+	err = newton_raphson_max(IC->theta, IC->k, maxit, crittol, gradtol,
+				 &fncount, C_LOGLIK, int_loglik, 
+				 int_score, int_ahess, IC, maxopt, prn);
+    }
 
     if (!err) {
 	IC->ll = int_loglik(IC->theta, IC);
@@ -816,13 +906,13 @@ static int do_interval (int *list, double **Z, DATAINFO *pdinfo,
     }
 
     if (!err) {
-	err = fill_intreg_model(IC, V, opt, pdinfo);
+	err = fill_intreg_model(IC, V, fncount, grcount, pdinfo, opt);
     }
 
     if (!err) {
 	err = intreg_normtest(IC, &normtest);
 	if (!err) {
-	    err = add_norm_test_to_model(IC->pmod, normtest);
+	    err = gretl_model_add_normality_test(IC->pmod, normtest);
 	}
 	/* don't let this be a show-stopper */
 	err = 0;
@@ -863,8 +953,8 @@ MODEL interval_estimate (int *list, double ***pZ, DATAINFO *pdinfo,
 	maybe_reposition_const(list, (const double **) *pZ, pdinfo);
     }
 
-    /* create extra variable for model initialization and
-       corresponding list
+    /* create extra series for model initialization and
+       corresponding regression list
     */
     model.errcode = create_midpoint_y(list, pZ, pdinfo, &initlist);
     if (model.errcode) {
@@ -895,3 +985,103 @@ MODEL interval_estimate (int *list, double ***pZ, DATAINFO *pdinfo,
 
     return model;
 }
+
+static int tobit_add_lo_hi (MODEL *pmod, double llim, double rlim,
+			    double ***pZ, DATAINFO *pdinfo, 
+			    int **plist)
+{
+    double *y, *lo, *hi;
+    int *list;
+    int lv, hv;
+    int i, t, err;
+
+    err = dataset_add_series(2, pZ, pdinfo);
+    if (err) {
+	return err;
+    }
+
+    lv = pdinfo->v - 2;
+    hv = pdinfo->v - 1;
+
+    lo = (*pZ)[lv];
+    hi = (*pZ)[hv];
+    y = (*pZ)[pmod->list[1]];
+
+    for (t=pmod->t1; t<=pmod->t2 && !err; t++) {
+	if (na(y[t])) {
+	    lo[t] = hi[t] = NADBL;
+	} else if (y[t] > llim && y[t] < rlim) {
+	    lo[t] = hi[t] = y[t];
+	} else if (y[t] >= rlim) {
+	    lo[t] = rlim;
+	    hi[t] = NADBL;
+	} else if (y[t] <= llim) {
+	    lo[t] = NADBL;
+	    hi[t] = llim;
+	} 
+    }
+
+    list = gretl_list_new(pmod->list[0] + 1);
+
+    if (list == NULL) {
+	err = E_ALLOC;
+    } else {
+	list[1] = lv;
+	list[2] = hv;
+	for (i=3; i<=list[0]; i++) {
+	    list[i] = pmod->list[i-1];
+	}
+	*plist = list;
+    }
+
+    return err;
+}
+
+MODEL tobit_via_intreg (int *list, double llim, double rlim,
+			double ***pZ, DATAINFO *pdinfo,
+			gretlopt opt, PRN *prn) 
+{
+    MODEL model;
+    int *ilist = NULL;
+    int origv = pdinfo->v;
+
+    /* run initial OLS */
+    model = lsq(list, *pZ, pdinfo, OLS, OPT_A);
+    if (model.errcode) {
+	fprintf(stderr, "intreg: initial OLS failed\n");
+	return model;
+    }
+
+#if INTDEBUG
+    pprintf(prn, "tobit_via_intreg: initial OLS\n");
+    printmodel(&model, pdinfo, OPT_S, prn);
+#endif
+
+    model.errcode = tobit_add_lo_hi(&model, llim, rlim,
+				    pZ, pdinfo, &ilist);
+
+    if (!model.errcode) {
+	/* do the actual analysis */
+	model.errcode = do_interval(ilist, *pZ, pdinfo, &model, 
+				    opt | OPT_T, prn);
+    }
+
+    clear_model_xpx(&model);
+
+    if (!model.errcode) {
+	if (opt & OPT_L) {
+	    model.opt |= OPT_L;
+	    gretl_model_set_double(&model, "llimit", llim);
+	}
+	if ((opt & OPT_M) && !na(rlim)) {
+	    model.opt |= OPT_M;
+	    gretl_model_set_double(&model, "rlimit", rlim);
+	}	
+    }
+
+    /* clean up extra data */
+    dataset_drop_last_variables(pdinfo->v - origv, pZ, pdinfo);
+    free(ilist);
+
+    return model;
+}
