@@ -730,11 +730,29 @@ static int update_phi (Jwrap *J, switcher *s)
 	gretl_matrix_copy_values(s->TmpL, s->K2);
     }
 
+#if 0 /* just testing */
+    gretl_matrix *Tmp1 = gretl_matrix_copy(J->I11);
+    gretl_matrix *Tmp2 = gretl_matrix_copy(s->TmpL);
+
+    err = gretl_cholesky_decomp_solve(J->I11, s->TmpL);
+    if (err) {
+	fprintf(stderr, "cholesky decomp failed in update_phi\n");
+	gretl_matrix_copy_values(J->I11, Tmp1);
+	gretl_matrix_copy_values(s->TmpL, Tmp2);
+	err = gretl_LU_solve(J->I11, s->TmpL);
+	if (err) {
+	    fprintf(stderr, " and LU_solve failed too\n");
+	}
+    } 
+    gretl_matrix_free(Tmp1);
+    gretl_matrix_free(Tmp2);
+#else
     /* combine first and second chunks */
     err = gretl_cholesky_decomp_solve(J->I11, s->TmpL);
     if (err) {
 	fprintf(stderr, "cholesky decomp failed in update_phi\n");
     }
+#endif
 
     if (!err) {
 	/* right-hand chunk */
@@ -1841,7 +1859,8 @@ static int check_jacobian (Jwrap *J)
 
 /* Doornik's approach to checking identification */
 
-static int vecm_id_check (Jwrap *J, GRETL_VAR *jvar, PRN *prn)
+static int vecm_id_check (Jwrap *J, GRETL_VAR *jvar, 
+			  gretlopt opt, PRN *prn)
 {
     int npar = J->alen + J->blen;
     int err = 0;
@@ -1871,11 +1890,13 @@ static int vecm_id_check (Jwrap *J, GRETL_VAR *jvar, PRN *prn)
 	fprintf(stderr, "Jacobian: got df = %d\n", J->df);
 #endif
 
-	/* system was subject to a prior restriction? */
-	if (jvar->jinfo->lrdf > 0) {
-	    J->df -= jvar->jinfo->lrdf;
-	    pprintf(prn, _("Allowing for prior restriction, df = %d\n"), 
-		    J->df);
+	if (!(opt & OPT_B)) {
+	    /* system was subject to a prior restriction? */
+	    if (jvar->jinfo->lrdf > 0) {
+		J->df -= jvar->jinfo->lrdf;
+		pprintf(prn, _("Allowing for prior restriction, df = %d\n"), 
+			J->df);
+	    }
 	}
     }
 
@@ -2802,7 +2823,7 @@ static int printres (Jwrap *J, GRETL_VAR *jvar,
 	
 
 	pprintf(prn, "2 * (lu - lr) = %g\n", x);
-	pprintf(prn, _("P(Chi-Square(%d) > %g) = %g\n"), J->df, x, pv);
+	pprintf(prn, "P(%s(%d) > %g) = %g\n", _("Chi-square"), J->df, x, pv);
 	rset_add_results(rset, x, pv, J->ll);
     }
 
@@ -3084,11 +3105,14 @@ static int make_theta (Jwrap *J)
 /* write info from the temporary Jwrap structure into
    the "permanent" GRETL_VAR structure */
 
-static void transcribe_to_jvar (Jwrap *J, GRETL_VAR *jvar)
+static void transcribe_to_jvar (Jwrap *J, GRETL_VAR *jvar,
+				gretlopt opt)
 {
-    jvar->jinfo->ll0 = jvar->ll;
-    jvar->ll = J->ll;
-    jvar->jinfo->lrdf += J->df; /* ? */
+    if (!(opt & OPT_B)) {
+	jvar->jinfo->ll0 = jvar->ll;
+	jvar->ll = J->ll;
+	jvar->jinfo->lrdf += J->df; /* ? */
+    }
 
     gretl_matrix_replace(&jvar->S, J->Omega);
     J->Omega = NULL;
@@ -3099,6 +3123,11 @@ static void transcribe_to_jvar (Jwrap *J, GRETL_VAR *jvar)
     gretl_matrix_replace(&jvar->jinfo->Alpha, J->alpha);
     J->alpha = NULL;
 
+    if (opt & OPT_B) {
+	/* bootstrapping: we don't need the variances */
+	return;
+    }
+
     gretl_matrix_replace(&jvar->jinfo->Bvar, J->Vb);
     J->Vb = NULL;
 
@@ -3106,7 +3135,7 @@ static void transcribe_to_jvar (Jwrap *J, GRETL_VAR *jvar)
     J->bse = NULL;
 
     gretl_matrix_replace(&jvar->jinfo->Ase, J->ase);
-    J->ase = NULL;    
+    J->ase = NULL; 
 }
 
 static int do_bfgs (Jwrap *J, gretlopt opt, PRN *prn)
@@ -3133,6 +3162,7 @@ static int do_bfgs (Jwrap *J, gretlopt opt, PRN *prn)
    OPT_F: doing full estimation of restricted system, not just 
           testing the restriction.
    OPT_J: ("jitter") use simulated annealing in initialization.
+   OPT_B: bootstrapping impulse response
 */
 
 int general_vecm_analysis (GRETL_VAR *jvar, 
@@ -3168,7 +3198,7 @@ int general_vecm_analysis (GRETL_VAR *jvar,
     }   
 
     if (!err) {
-	err = vecm_id_check(J, jvar, prn);
+	err = vecm_id_check(J, jvar, opt, prn);
 	if (J->df == 0) {
 	    fprintf(stderr, "warning: test df = 0\n");
 	    J->flags &= ~J_USE_LBFGS;
@@ -3217,13 +3247,15 @@ int general_vecm_analysis (GRETL_VAR *jvar,
 
  skipest:
 
-    if (!err) {
+    if (!err && !(opt & OPT_B)) {
 	err = variance_from_info_matrix(J, jvar);
     }
 
     if (!err) {
 	if (full) {
-	    transcribe_to_jvar(J, jvar);
+	    transcribe_to_jvar(J, jvar, OPT_F);
+	} else if (opt & OPT_B) {
+	    transcribe_to_jvar(J, jvar, OPT_B);
 	} else {
 	    printres(J, jvar, rset, pdinfo, prn);
 	}
