@@ -29,6 +29,10 @@
 #include <glib.h>
 #include <errno.h>
 
+#ifndef G_OS_WIN32
+# include <unistd.h>
+#endif
+
 #define ODS_IMPORTER
 
 #include "import_common.c"
@@ -1213,6 +1217,11 @@ static int finalize_ods_import (double ***pZ, DATAINFO *pdinfo,
 
     err = ods_prune_columns(sheet);
 
+    if (!err && sheet->dinfo->v == 1) {
+	gretl_errmsg_set(_("No numeric data were found"));
+	err = E_DATA;
+    }
+
     if (!err) {
 	tprn = gretl_print_new(GRETL_PRINT_STDERR, NULL);
 	ts_check(sheet, tprn);
