@@ -305,10 +305,11 @@ static gretl_matrix *negbin_nhessian (negbin_info *nbinfo, int *err)
 {
     gretl_matrix *H;
     int np = nbinfo->k + 1;
-    
+
     nbinfo->flags = SCORE_UPDATE_MU;
-    H = hessian_from_score(nbinfo->theta, np, negbin_score, 
-			   nbinfo, err);
+    H = hessian_inverse_from_score(nbinfo->theta, np,
+				   negbin_score, NULL,
+				   nbinfo, err);
     nbinfo->flags = 0;
 
     return H;
@@ -381,8 +382,12 @@ transcribe_negbin_results (MODEL *pmod, negbin_info *nbinfo,
 
     pmod->ci = NEGBIN;
 
-    gretl_model_set_int(pmod, "fncount", fncount);
-    gretl_model_set_int(pmod, "grcount", grcount);
+    if (grcount > 0) {
+	gretl_model_set_int(pmod, "fncount", fncount);
+	gretl_model_set_int(pmod, "grcount", grcount);
+    } else {
+	gretl_model_set_int(pmod, "iters", fncount);
+    }
 
     if (oinfo != NULL) {
 	gretl_model_set_int(pmod, "offset_var", oinfo->vnum);
@@ -431,7 +436,7 @@ transcribe_negbin_results (MODEL *pmod, negbin_info *nbinfo,
 		(opt & OPT_R)? VCV_QML : VCV_HESSIAN;
 
 	    gretl_model_set_vcv_info(pmod, VCV_ML, vtype);
-	    err = gretl_model_write_vcv(pmod, nbinfo->V);
+	    err = gretl_model_write_vcv(pmod, nbinfo->V, -1);
 	}
     }
 
@@ -460,28 +465,43 @@ static int do_negbin (MODEL *pmod, offset_info *oinfo,
     gretl_matrix *H = NULL;
     negbin_info nbinfo;
     double toler;
-    int maxit;
+    int maxit = 100;
     int fncount = 0;
     int grcount = 0;
+    int use_newton = 0;
     int err = 0;
 
+    if (libset_get_int(GRETL_OPTIM) == OPTIM_NEWTON) {
+	use_newton = 1;
+    }
+
     err = negbin_init(&nbinfo, pmod, Z, oinfo, opt, prn);
 
-    if (!err) {
-	/* to initialize BFGS curvature */
+    if (!err && !use_newton) {
+	/* initialize BFGS curvature */
 	H = negbin_init_H(&nbinfo);
     }
 
-    if (!err) {
+    if (!err && use_newton) {
+	double crittol = 1.0e-7;
+	double gradtol = 1.0e-7;
+
+	nbinfo.flags = SCORE_UPDATE_MU;
+	err = newton_raphson_max(nbinfo.theta, nbinfo.k + 1, maxit, 
+				 crittol, gradtol, &fncount, 
+				 C_LOGLIK, negbin_loglik, 
+				 negbin_score, NULL, 
+				 &nbinfo, max_opt, nbinfo.prn);
+	nbinfo.flags = 0;
+    } else if (!err) {
 	BFGS_defaults(&maxit, &toler, NEGBIN);
 	err = BFGS_max(nbinfo.theta, nbinfo.k + 1, maxit, toler, 
 		       &fncount, &grcount, negbin_loglik, C_LOGLIK,
 		       negbin_score, &nbinfo, H, max_opt, 
 		       nbinfo.prn);
+	gretl_matrix_free(H);
     }
 
-    gretl_matrix_free(H);
-
     if (!err) {
 	err = transcribe_negbin_results(pmod, &nbinfo, Z, pdinfo, 
 					oinfo, fncount, grcount, 
@@ -527,7 +547,7 @@ static int poisson_robust_vcv (MODEL *pmod, gretl_matrix *G)
     }
 
     if (!err) {
-	err = gretl_model_write_vcv(pmod, V);
+	err = gretl_model_write_vcv(pmod, V, -1);
     }
 
     if (!err) {
