@@ -808,7 +808,7 @@ static int rq_write_variance (const gretl_matrix *V,
 	    }
 	}
     } else {
-	err = gretl_model_write_vcv(pmod, V);
+	err = gretl_model_write_vcv(pmod, V, -1);
     }
 
     return err;
