@@ -4,7 +4,7 @@
    Based on stataread.c from the GNU R "foreign" package with the 
    following original info:
 
-     * $Id: stata_import.c,v 1.41 2010/03/01 16:46:08 allin Exp $
+     * $Id: stata_import.c,v 1.45 2010/09/01 18:00:01 allin Exp $
   
      (c) 1999, 2000, 2001, 2002 Thomas Lumley. 
      2000 Saikat DebRoy
@@ -420,6 +420,8 @@ static int set_time_info (int t1, int pd, DATAINFO *dinfo)
 {
     int yr, mo, qt;
 
+    *dinfo->stobs = '\0';
+
     if (pd == 12) {
 	yr = (t1 / 12) + 1960;
 	mo = t1 % 12 + 1;
@@ -430,14 +432,20 @@ static int set_time_info (int t1, int pd, DATAINFO *dinfo)
 	sprintf(dinfo->stobs, "%d:%d", yr, qt);
     } else {
 	yr = t1 + 1960;
-	sprintf(dinfo->stobs, "%d", yr);
+	if (yr > 2050) {
+	    ; /* Can't be a year? (FIXME: how did we get here?) */
+	} else {
+	    sprintf(dinfo->stobs, "%d", yr);
+	}
     }
 
-    printf("starting obs seems to be %s\n", dinfo->stobs);
-    
     dinfo->pd = pd;
-    dinfo->structure = TIME_SERIES;
-    dinfo->sd0 = get_date_x(dinfo->pd, dinfo->stobs);
+
+    if (*dinfo->stobs != '\0') {
+	printf("starting obs seems to be %s\n", dinfo->stobs);
+	dinfo->structure = TIME_SERIES;
+	dinfo->sd0 = get_date_x(dinfo->pd, dinfo->stobs);
+    } 
 
     return 0;
 }
@@ -508,6 +516,26 @@ static int label_array_header (const int *list, char **names,
     return 1;
 }
 
+/* when printing stata format strings, make sure we double any
+   '%' characters */
+
+static void print_var_format (int k, const char *s, PRN *prn)
+{
+    char tmp[64];
+    int i = 0;
+
+    while (*s) {
+	tmp[i++] = *s;
+	if (*s == '%') {
+	    tmp[i++] = '%';
+	}
+	s++;
+    }
+
+    tmp[i] = '\0';
+    pprintf(prn, "variable %d: format = '%s'\n", k, tmp);    
+}
+
 static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
 			  gretl_string_table **pst, int namelen,
 			  int *nvread, PRN *prn, PRN *vprn)
@@ -585,8 +613,12 @@ static int read_dta_data (FILE *fp, double **Z, DATAINFO *dinfo,
     /* format list (use it to identify date variables?) */
     for (i=0; i<nvar && !err; i++){
         stata_read_string(fp, fmtlen, c50, &err);
+	if (stata_type_string(types[i])) {
+	    /* the format is of no interest */
+	    continue;
+	}
 	if (*c50 != '\0' && c50[strlen(c50)-1] != 'g') {
-	    pprintf(vprn, "variable %d: format = '%s'\n", i+1, c50);
+	    print_var_format(i+1, c50, vprn);
 	    if (!strcmp(c50, "%tm")) {
 		pd = 12;
 		tnum = i;
