@@ -182,9 +182,26 @@ static void set_logtrans (GtkButton *b, tx_request *request)
     request->xopt.logtrans = GPOINTER_TO_INT(p);
 }
 
+static void toggle_edit_script (GtkToggleButton *b, tx_request *request)
+{
+    GtkWidget **chk = g_object_get_data(G_OBJECT(b), "checks");
+    gboolean s = gtk_toggle_button_get_active(b);
+    int i;
+
+    if (s) {
+	*request->popt |= OPT_S;
+    } else {
+	*request->popt &= ~OPT_S;
+    }
+
+    for (i=0; i<4; i++) {
+	gtk_widget_set_sensitive(chk[i], !s);
+    }
+}
+
 static void show_x12a_options (tx_request *request, GtkBox *vbox)
 {
-    GtkWidget *tmp, *b[3];
+    GtkWidget *tmp, *b[3], *chk[4];
     GSList *group;
 
     tmp = gtk_label_new(_("X-12-ARIMA options"));
@@ -243,36 +260,36 @@ static void show_x12a_options (tx_request *request, GtkBox *vbox)
     gtk_widget_show(tmp);
     gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
 
-    tmp = gtk_check_button_new_with_label(_("Seasonally adjusted series"));
-    gtk_widget_show(tmp);
-    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
-    request->opts[TX_SA].check = tmp;
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
+    chk[0] = gtk_check_button_new_with_label(_("Seasonally adjusted series"));
+    gtk_widget_show(chk[0]);
+    gtk_box_pack_start(vbox, chk[0], FALSE, FALSE, 0);
+    request->opts[TX_SA].check = chk[0];
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chk[0]), 
 				 (*request->popt & OPT_A)? TRUE : FALSE);
 
-    tmp = gtk_check_button_new_with_label(_("Trend/cycle"));
-    gtk_widget_show(tmp);
-    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
-    request->opts[TX_TR].check = tmp;
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
+    chk[1] = gtk_check_button_new_with_label(_("Trend/cycle"));
+    gtk_widget_show(chk[1]);
+    gtk_box_pack_start(vbox, chk[1], FALSE, FALSE, 0);
+    request->opts[TX_TR].check = chk[1];
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chk[1]), 
 				 (*request->popt & OPT_B)? TRUE : FALSE);
 
-    tmp = gtk_check_button_new_with_label(_("Irregular"));
-    gtk_widget_show(tmp);
-    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
-    request->opts[TX_IR].check = tmp;
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
+    chk[2] = gtk_check_button_new_with_label(_("Irregular"));
+    gtk_widget_show(chk[2]);
+    gtk_box_pack_start(vbox, chk[2], FALSE, FALSE, 0);
+    request->opts[TX_IR].check = chk[2];
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chk[2]), 
 				 (*request->popt & OPT_C)? TRUE : FALSE);
 
     tmp = gtk_hseparator_new();
     gtk_widget_show(tmp);
     gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 5);
     
-    tmp = gtk_check_button_new_with_label(_("Generate graph"));
-    gtk_widget_show(tmp);
-    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 0);
-    request->opts[TRIGRAPH].check = tmp;
-    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp), 
+    chk[3] = gtk_check_button_new_with_label(_("Generate graph"));
+    gtk_widget_show(chk[3]);
+    gtk_box_pack_start(vbox, chk[3], FALSE, FALSE, 0);
+    request->opts[TRIGRAPH].check = chk[3];
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(chk[3]), 
 				 (*request->popt & OPT_G)? TRUE : FALSE);
 
     tmp = gtk_check_button_new_with_label(_("Show full output"));
@@ -281,10 +298,58 @@ static void show_x12a_options (tx_request *request, GtkBox *vbox)
     request->opts[TEXTOUT].check = tmp;
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(tmp),
 				 (*request->popt & OPT_Q)? FALSE : TRUE);
+
+    tmp = gtk_hseparator_new();
+    gtk_widget_show(tmp);
+    gtk_box_pack_start(vbox, tmp, FALSE, FALSE, 5);
+
+    b[0] = gtk_radio_button_new_with_label(NULL, _("Execute X-12-ARIMA directly"));
+    gtk_widget_show(b[0]);
+    gtk_box_pack_start(vbox, b[0], FALSE, FALSE, 0);
+
+    group = gtk_radio_button_get_group(GTK_RADIO_BUTTON(b[0]));
+    b[1] = gtk_radio_button_new_with_label(group, _("Make X-12-ARIMA command file"));
+    gtk_widget_show(b[1]);
+    gtk_box_pack_start(vbox, b[1], FALSE, FALSE, 0);
+    g_object_set_data(G_OBJECT(b[1]), "checks", chk);
+    g_signal_connect(GTK_TOGGLE_BUTTON(b[1]), "toggled",
+		     G_CALLBACK(toggle_edit_script), request);
+
+    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(b[0]), TRUE); 
 }
 
+static GtkWidget *x12a_help_button (GtkWidget *hbox, tx_request *request)
+{
+    GtkWidget *button;
+
+    button = gtk_button_new_from_stock(GTK_STOCK_HELP);
+    gtk_widget_set_can_default(button, TRUE);
+    gtk_container_add(GTK_CONTAINER(hbox), button);
+    gtk_button_box_set_child_secondary(GTK_BUTTON_BOX(hbox),
+				       button, TRUE);
+    gtk_widget_show(button);
+
+    g_signal_connect(G_OBJECT(button), "clicked", 
+		     G_CALLBACK(request->helpfunc), 
+		     NULL);
+    
+    return button;
+}
+
+static void tx_dialog_callback (GtkDialog *dlg, gint id, int *ret)
+{
+    if (id == GTK_RESPONSE_REJECT || id == GTK_RESPONSE_ACCEPT) {
+	*ret = id;
+    } else if (id == GTK_RESPONSE_DELETE_EVENT) {
+	*ret = GTK_RESPONSE_REJECT;
+    }
+
+    gtk_main_quit();
+} 
+
 static int tx_dialog (tx_request *request)
 {
+    GtkDialogFlags dflags = GTK_DIALOG_DESTROY_WITH_PARENT;
     GtkWidget *hbox, *vbox;
     gint i, ret = 0;
 
@@ -292,12 +357,14 @@ static int tx_dialog (tx_request *request)
 	request->opts[i].check = NULL;
     }
 
+    if (request->prog != X12A) {
+	dflags |= GTK_DIALOG_MODAL;
+    }
+
     request->dialog = 
 	gtk_dialog_new_with_buttons ((request->prog == TRAMO_SEATS)?
 				     "TRAMO/SEATS" : "X-12-ARIMA",
-				     NULL,
-				     GTK_DIALOG_MODAL | 
-				     GTK_DIALOG_DESTROY_WITH_PARENT,
+				     NULL, dflags,
 				     GTK_STOCK_CANCEL,
 				     GTK_RESPONSE_REJECT,
 				     GTK_STOCK_OK,
@@ -307,7 +374,9 @@ static int tx_dialog (tx_request *request)
     vbox = gtk_vbox_new(FALSE, 0);    
 
     if (request->prog == TRAMO_SEATS) {
+#if GTK_MAJOR_VERSION < 3    
 	gtk_dialog_set_has_separator(GTK_DIALOG(request->dialog), FALSE);
+#endif	
 	show_tramo_options(request, vbox);
     } else {
 	show_x12a_options(request, GTK_BOX(vbox));
@@ -321,7 +390,16 @@ static int tx_dialog (tx_request *request)
     vbox = gtk_dialog_get_content_area(GTK_DIALOG(request->dialog));
     gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 5);
 
-    ret = gtk_dialog_run(GTK_DIALOG(request->dialog));
+    if (request->prog == X12A) {
+	hbox = gtk_dialog_get_action_area(GTK_DIALOG(request->dialog));
+	x12a_help_button(hbox, request);
+	g_signal_connect(G_OBJECT(request->dialog), "response", 
+			 G_CALLBACK(tx_dialog_callback), &ret);
+	gtk_widget_show_all(request->dialog);
+	gtk_main(); /* block */
+    } else {
+	ret = gtk_dialog_run(GTK_DIALOG(request->dialog));
+    }
 
     return (ret == GTK_RESPONSE_ACCEPT)? 1 : 0;
 }
@@ -542,8 +620,7 @@ static void clear_x12a_files (const char *path, const char *vname)
 
 static int add_series_from_file (const char *path, int src,
 				 double **Z, DATAINFO *pdinfo,
-				 int targv, tx_request *request,
-				 char *errmsg)
+				 int targv, tx_request *request)
 {
     FILE *fp;
     char line[128], sfname[MAXLEN];
@@ -600,8 +677,7 @@ static int add_series_from_file (const char *path, int src,
 	}
 
 	if (!gotit) {
-	    fprintf(stderr, "Couldn't open %s\n", sfname);
-	    sprintf(errmsg, _("Couldn't open %s"), sfname);
+	    gretl_errmsg_sprintf(_("Couldn't open %s"), sfname);
 	    return 1;
 	}
     }
@@ -748,11 +824,13 @@ static int grab_deseasonal_series (double *y, const DATAINFO *pdinfo,
     return err;
 }
 
-static void request_opts_init (tx_request *request, const DATAINFO *pdinfo)
+static void request_opts_init (tx_request *request, const DATAINFO *pdinfo,
+			       void (*helpfunc))
 {
     int i;
 
     request->savevars = 0;
+    request->helpfunc = helpfunc;
 
     request->xopt.logtrans = 3; /* x12a: automatic logs or not */
     request->xopt.outliers = 1; /* x12a: detect outliers */
@@ -986,8 +1064,7 @@ static void copy_basic_data_info (DATAINFO *targ, DATAINFO *src)
 
 static int save_vars_to_dataset (double ***pZ, DATAINFO *pdinfo,
 				 double **tmpZ, DATAINFO *tmpinfo,
-				 int *varlist, tx_request *request,
-				 char *errmsg)
+				 int *varlist, tx_request *request)
 {
     int i, v, j, addvars = 0;
 
@@ -1069,7 +1146,7 @@ static int helper_spawn (const char *path, const char *vname,
 
 #endif
 
-/* The 12a .err file is always produced, but often contains no
+/* The x12a .err file is always produced, but often contains no
    warning or error messages: in that case it contains 2 lines
    of boiler-plate text followed by two blank lines. Here we
    check if we got more than 4 lines of output.
@@ -1099,12 +1176,13 @@ static int got_x12a_warning (const char *fname)
 
 /* make a default x12a.mdl file if it doesn't already exist */
 
-static int check_x12a_model_file (const char *workdir, char *fname)
+static int check_x12a_model_file (const char *workdir)
 {
+    gchar *fname;
     FILE *fp;
     int err = 0;
 
-    sprintf(fname, "%s%cx12a.mdl", workdir, SLASH);
+    fname = g_strdup_printf("%s%cx12a.mdl", workdir, SLASH);
     fp = gretl_fopen(fname, "r");
 
     if (fp != NULL) {
@@ -1119,24 +1197,25 @@ static int check_x12a_model_file (const char *workdir, char *fname)
 	}
     } 
 
+    g_free(fname);
+
     return err;
 }
 
-static int check_sample_bound (int prog, const DATAINFO *pdinfo,
-			       char *errmsg)
+static int check_sample_bound (int prog, const DATAINFO *pdinfo)
 {
     int T = pdinfo->t2 - pdinfo->t1 + 1;
 
     if (prog == TRAMO_SEATS && T > 600) {
-	strcpy(errmsg, _("TRAMO can't handle more than 600 observations.\n"
+	gretl_errmsg_set(_("TRAMO can't handle more than 600 observations.\n"
 			 "Please select a smaller sample."));
 	return E_EXTERNAL;
     } else if (prog == X12A) {
 	int pdmax = get_x12a_maxpd();
 
 	if (T > 50 * pdmax) {
-	    sprintf(errmsg, _("X-12-ARIMA can't handle more than %d observations.\n"
-			      "Please select a smaller sample."), 50 * pdmax);
+	    gretl_errmsg_sprintf(_("X-12-ARIMA can't handle more than %d observations.\n"
+				   "Please select a smaller sample."), 50 * pdmax);
 	    return E_EXTERNAL;
 	}
     }
@@ -1144,22 +1223,84 @@ static int check_sample_bound (int prog, const DATAINFO *pdinfo,
     return 0;
 }
 
+/* Callback for the "Run" button in a gretl editor window displaying
+   an x12a command file: run the commands (which are provided in @buf),
+   and fill @outname with the name of the x12a output file.
+*/
+
+int exec_tx_script (char *outname, const gchar *buf)
+{
+    const char *exepath;
+    const char *workdir = NULL;
+    const char *tmpname = "x12atmp";
+    gchar *tmppath;
+    FILE *fp;
+    int err = 0;
+
+    *outname = '\0';
+
+    exepath = gretl_x12_arima();
+    workdir = gretl_x12_arima_dir();
+
+    tmppath = g_strdup_printf("%s%c%s.spc", workdir, SLASH, tmpname);
+    fp = gretl_fopen(tmppath, "w");
+    if (fp == NULL) {
+	g_free(tmppath);
+	return E_FOPEN;
+    }
+
+    fputs(buf, fp);
+    fclose(fp);
+    g_free(tmppath);
+	
+    clear_x12a_files(workdir, tmpname);
+    err = helper_spawn(exepath, tmpname, workdir, X12A);
+
+    if (err == E_EXTERNAL) {
+	; /* fatal: couldn't run program */
+    } else if (err) {
+	sprintf(outname, "%s%c%s.err", workdir, SLASH, tmpname);
+    } else {
+	/* set the output filename */
+	sprintf(outname, "%s%c%s.out", workdir, SLASH, tmpname); 
+    }
+
+    return err;
+}
+
+/* Driver for access to TRAMO/SEATS (if @tramo is non-zero) or
+   X-12-ARIMA, for analysis of the series with ID number @varnum. The
+   @opt pointer is filled out based on selections from a dialog box.
+
+   The @fname argument is filled out to tell the caller the name of
+   the output file from the third-party program, if available; it will
+   be an empty string if the user cancels, or if we fail to execute
+   the program in question.
+
+   At present the @help_func argument is used only for X-12-ARIMA,
+   for which we offer a special option, namely writing a spec file
+   for display in an editor window rather than directly executing
+   x12a commands. If that option is selected we signal the caller
+   by putting OPT_S into @opt, and we return the name of the gretl-
+   generated command file in @fname.
+*/
+
 int write_tx_data (char *fname, int varnum, 
-		   double ***pZ, DATAINFO *pdinfo, gretlopt *opt, 
-		   int tramo, int *graph_ok, char *errmsg)
+		   double ***pZ, DATAINFO *pdinfo, 
+		   gretlopt *opt, int tramo,
+		   void (*help_func))
 {
     const char *exepath;
     const char *workdir;
     char vname[VNAMELEN];
     int savelist[4];
     tx_request request;
-    double **tmpZ;
-    DATAINFO *tmpinfo;
+    double **tmpZ = NULL;
+    DATAINFO *tmpinfo = NULL;
+    int savescript = 0;
     int i, doit;
     int err = 0;
 
-    *errmsg = '\0';
-
     if (tramo) {
 	request.prog = TRAMO_SEATS;
 	exepath = gretl_tramo();
@@ -1170,9 +1311,9 @@ int write_tx_data (char *fname, int varnum,
 	workdir = gretl_x12_arima_dir();
     }	
 	
-    request_opts_init(&request, pdinfo);
+    request_opts_init(&request, pdinfo, help_func);
 
-    err = check_sample_bound(request.prog, pdinfo, errmsg);
+    err = check_sample_bound(request.prog, pdinfo);
     if (err) {
 	return err;
     }
@@ -1184,6 +1325,7 @@ int write_tx_data (char *fname, int varnum,
     doit = tx_dialog(&request); 
     if (!doit) {
 	gtk_widget_destroy(request.dialog);
+	*fname = '\0';
 	return 0;
     }
     set_opts(&request);
@@ -1196,18 +1338,22 @@ int write_tx_data (char *fname, int varnum,
     }
 #endif
 
-    /* create little temporary dataset */
-    tmpinfo = create_auxiliary_dataset(&tmpZ, 4, pdinfo->n);
-    if (tmpinfo == NULL) {
-	return E_ALLOC;
-    }
+    if (*opt & OPT_S) {
+	savescript = 1;
+    } else {
+	/* create little temporary dataset */
+	tmpinfo = create_auxiliary_dataset(&tmpZ, 4, pdinfo->n);
+	if (tmpinfo == NULL) {
+	    return E_ALLOC;
+	}
 
-    copy_basic_data_info(tmpinfo, pdinfo);
+	copy_basic_data_info(tmpinfo, pdinfo);
 
-    if (request.prog == X12A) { 
-	err = check_x12a_model_file(workdir, fname);
-	if (err) {
-	    goto bailout;
+	if (request.prog == X12A) { 
+	    err = check_x12a_model_file(workdir);
+	    if (err) {
+		goto bailout;
+	    }
 	}
     } 
 
@@ -1233,6 +1379,11 @@ int write_tx_data (char *fname, int varnum,
 	}
     }
 
+    if (savescript) {
+	/* x12a file written; we're done */
+	return 0;
+    }
+
     /* now run the program(s): we try to ensure that any
        old output files get deleted first 
     */
@@ -1275,7 +1426,7 @@ int write_tx_data (char *fname, int varnum,
 	    if (request.prog == TRAMO_ONLY) {
 		/* no graph offered */
 		request.opts[TRIGRAPH].save = 0;
-		*graph_ok = 0;
+		*opt |= OPT_T;
 	    }
 	} 
 
@@ -1287,7 +1438,7 @@ int write_tx_data (char *fname, int varnum,
 
 	    for (i=1; i<=savelist[0]; i++) {
 		err = add_series_from_file(path, savelist[i], tmpZ, tmpinfo,
-					   i, &request, errmsg);
+					   i, &request);
 		if (err) {
 		    fprintf(stderr, "i = %d: add_series_from_file() failed\n", i);
 		    if (request.prog == X12A) {
@@ -1323,7 +1474,7 @@ int write_tx_data (char *fname, int varnum,
 	/* now save the local vars to main dataset, if wanted */
 	if (!err && request.savevars > 0) {
 	    err = save_vars_to_dataset(pZ, pdinfo, tmpZ, tmpinfo, savelist, 
-				       &request, errmsg);
+				       &request);
 	}
     }
 
@@ -1354,7 +1505,7 @@ int adjust_series (const double *x, double *y, const DATAINFO *pdinfo,
     }    
 
     if (prog == X12A) { 
-	err = check_x12a_model_file(workdir, fname);
+	err = check_x12a_model_file(workdir);
 	if (err) {
 	    return err;
 	}
