@@ -8,6 +8,15 @@
 /* Extra floating-point GMP routines? */
 #define HAVE_MPFR 1
 
+/* disable SFMT RNG? */
+/* #undef NO_SFMT */
+
+/* openmp: not for now */
+/* #undef USE_OPENMP */
+
+/* sse2: assume this is OK */
+#define USE_SSE2 1
+
 /* X-12-ARIMA support? */
 #define HAVE_X12A 1
 
@@ -35,6 +44,9 @@
 /* Do we have paths.h? */
 /* #define HAVE_PATHS_H 1 */
 
+/* Do we have stdint.h? */
+#define HAVE_STDINT_H 1 
+
 /* Do we have unistd.h? */
 #define HAVE_UNISTD_H 1 
 
